@@ -60,6 +60,7 @@ static int lpc_cmd_chat(LinphoneCore *, char *);
 static int lpc_cmd_answer(LinphoneCore *, char *);
 static int lpc_cmd_autoanswer(LinphoneCore *, char *);
 static int lpc_cmd_terminate(LinphoneCore *, char *);
+static int lpc_cmd_redirect(LinphoneCore *, char *);
 static int lpc_cmd_call_logs(LinphoneCore *, char *);
 static int lpc_cmd_ipv6(LinphoneCore *, char *);
 static int lpc_cmd_transfer(LinphoneCore *, char *);
@@ -89,6 +90,7 @@ static int lpc_cmd_pause(LinphoneCore *lc, char *args);
 static int lpc_cmd_resume(LinphoneCore *lc, char *args);
 static int lpc_cmd_mute_mic(LinphoneCore *lc, char *args);
 static int lpc_cmd_unmute_mic(LinphoneCore *lc, char *args);
+static int lpc_cmd_playback_gain(LinphoneCore *lc, char *args);
 static int lpc_cmd_rtp_no_xmit_on_audio_mute(LinphoneCore *lc, char *args);
 #ifdef VIDEO_ENABLED
 static int lpc_cmd_camera(LinphoneCore *lc, char *args);
@@ -100,6 +102,7 @@ static int lpc_cmd_vfureq(LinphoneCore *lc, char *arg);
 static int lpc_cmd_states(LinphoneCore *lc, char *args);
 static int lpc_cmd_identify(LinphoneCore *lc, char *args);
 static int lpc_cmd_ringback(LinphoneCore *lc, char *args);
+static int lpc_cmd_conference(LinphoneCore *lc, char *args);
 
 /* Command handler helpers */
 static void linphonec_proxy_add(LinphoneCore *lc);
@@ -120,8 +123,7 @@ static int linphonec_friend_delete(LinphoneCore *lc, int num);
 static void linphonec_codec_list(int type, LinphoneCore *lc);
 static void linphonec_codec_enable(int type, LinphoneCore *lc, int index);
 static void linphonec_codec_disable(int type, LinphoneCore *lc, int index);
-
-
+static void lpc_display_call_states(LinphoneCore *lc);
 
 /* Command table management */
 static LPC_COMMAND *lpc_find_command(const char *name);
@@ -182,6 +184,10 @@ static LPC_COMMAND commands[] = {
 		"'transfer <call id> <sip-uri>': transfers the call with 'id' to the destination sip-uri\n"
 		"'transfer <call id1> --to-call <call id2>': transfers the call with 'id1' to the destination of call 'id2' (attended transfer)\n"
 	},
+	{ "conference", lpc_cmd_conference, "Create and manage an audio conference.",
+		"'conference add <call id> : join the call with id 'call id' into the audio conference."
+		"'conference rm <call id> : remove the call with id 'call id' from the audio conference."
+	},
 	{ "mute", lpc_cmd_mute_mic, 
 	  "Mute microphone and suspend voice transmission."},
 #ifdef VIDEO_ENABLED
@@ -191,6 +197,8 @@ static LPC_COMMAND commands[] = {
 #endif
 	{ "unmute", lpc_cmd_unmute_mic, 
 		  "Unmute microphone and resume voice transmission."},
+	{ "playbackgain", lpc_cmd_playback_gain, 
+		  "Adjust playback gain."},
 	{ "duration", lpc_cmd_duration, "Print duration in seconds of the last call.", NULL },
 	
 	{ "autoanswer", lpc_cmd_autoanswer, "Show/set auto-answer mode",
@@ -337,6 +345,9 @@ static LPC_COMMAND advanced_commands[] = {
 		"'ringback <path of mono .wav file>'\t: Specifies a ringback tone to be played to remote end during incoming calls\n"
 		"'ringback disable'\t: Disable playing of ringback tone to callers\n"
 	},
+	{ "redirect", lpc_cmd_redirect, "Redirect an incoming call",
+		"'redirect <redirect-uri>'\t: Redirect all pending incoming calls to the <redirect-uri>\n"
+	},
 	{	NULL,NULL,NULL,NULL}
 };
 
@@ -516,37 +527,6 @@ lpc_cmd_help(LinphoneCore *lc, char *arg)
 static char callee_name[256]={0};
 static char caller_name[256]={0};
 
-static const char *get_call_status(LinphoneCall *call){
-	switch(linphone_call_get_state(call)){
-		case LinphoneCallPaused:
-			if (linphone_call_get_refer_to (call)!=NULL){
-				return "Paused (transfered)";
-			}else{
-				return "Paused";
-			}
-		break;
-		case LinphoneCallPausedByRemote:
-			return "Paused by remote";
-		break;
-		case LinphoneCallIncomingReceived:
-			return "Pending";
-		break;
-		case LinphoneCallOutgoingInit:
-		case LinphoneCallOutgoingProgress:
-			return "Dialing out";
-		break;
-		case LinphoneCallOutgoingEarlyMedia:
-		case LinphoneCallOutgoingRinging:
-			return "Remote ringing";
-		break;
-		default:
-			if (linphone_call_has_transfer_pending(call)){
-				return "Running (transfer pending)";
-			}else
-				return "Running";
-	}
-	return "";
-}
 
 static int
 lpc_cmd_call(LinphoneCore *lc, char *args)
@@ -592,19 +572,7 @@ lpc_cmd_calls(LinphoneCore *lc, char *args){
 	const MSList *calls = linphone_core_get_calls(lc);
 	if(calls)
 	{
-		const MSList *p_calls = calls;
-		linphonec_out("ID\t\tDestination\t\t\t\tStatus\n---------------------------------------------------------------------\n");
-		while(p_calls != NULL)			
-		{
-			LinphoneCall *call=(LinphoneCall*)p_calls->data;
-			char *tmp=linphone_call_get_remote_address_as_string(call);
-			linphonec_out("%li\t%s\t\t\t%s\r\n",
-						  (long)linphone_call_get_user_pointer (call),
-					tmp,
-					get_call_status(call));
-			p_calls = p_calls->next;
-			ms_free(tmp);
-		}
+		lpc_display_call_states(lc);
 	}else
 	{
 		linphonec_out("No active call.\n");
@@ -734,6 +702,30 @@ lpc_cmd_terminate(LinphoneCore *lc, char *args)
 	
 }
 
+static int
+lpc_cmd_redirect(LinphoneCore *lc, char *args){
+	const MSList *elem;
+	int didit=0;
+	if (!args) return 0;
+	if ((elem=linphone_core_get_calls(lc))==NULL){
+		linphonec_out("No active calls.\n");
+		return 1;
+	}
+	while(elem!=NULL){
+		LinphoneCall *call=(LinphoneCall*)elem->data;
+		if (linphone_call_get_state(call)==LinphoneCallIncomingReceived){
+			linphone_core_redirect_call(lc,call,args);
+			didit=1;
+			/*as the redirection closes the call, we need to re-check the call list that is invalidated.*/
+			elem=linphone_core_get_calls(lc);
+		}else elem=elem->next;
+	}
+	if (didit==0){
+		linphonec_out("There is no pending incoming call to redirect.");
+	}
+	return 1;
+}
+
 static int
 lpc_cmd_answer(LinphoneCore *lc, char *args){
 	if (!args)
@@ -1430,6 +1422,32 @@ static int lpc_cmd_resume(LinphoneCore *lc, char *args){
     
 }
 
+static int lpc_cmd_conference(LinphoneCore *lc, char *args){
+	long id;
+	char subcommand[32]={0};
+	int n;
+	if (args==NULL) return 0;
+	n=sscanf(args, "%31s %li", subcommand,&id);
+	if (n == 2){
+		LinphoneCall *call=linphonec_get_call(id);
+		if (call==NULL) return 1;
+		if (strcmp(subcommand,"add")==0){
+			linphone_core_add_to_conference(lc,call);
+			return 1;
+		}else if (strcmp(subcommand,"rm")==0){
+			linphone_core_remove_from_conference(lc,call);
+			return 1;
+		}else if (strcmp(subcommand,"enter")==0){
+			linphone_core_enter_conference(lc);
+			return 1;
+		}else if (strcmp(subcommand,"leave")==0){
+			linphone_core_leave_conference(lc);
+			return 1;
+		}
+	}
+	return 0;
+}
+
 /***************************************************************************
  *
  *  Commands helper functions
@@ -2275,6 +2293,15 @@ static int lpc_cmd_unmute_mic(LinphoneCore *lc, char *args){
 	return 1;
 }
 
+static int lpc_cmd_playback_gain(LinphoneCore *lc, char *args)
+{
+	if (args){
+	        linphone_core_set_playback_gain_db(lc, atof(args));
+        	return 1;
+	}
+	return 0;
+}
+
 static int lpc_cmd_rtp_no_xmit_on_audio_mute(LinphoneCore *lc, char *args)
 {
 	bool_t rtp_xmit_off=FALSE;
@@ -2363,17 +2390,21 @@ static void lpc_display_call_states(LinphoneCore *lc){
 	const MSList *elem;
 	char *tmp;
 	linphonec_out("Call states\n"
-	              "Id |            Destination              |      State\n"
-	              "---------------------------------------------------------------\n");
+	              "Id |            Destination              |      State      |    Flags   |\n"
+	              "------------------------------------------------------------------------\n");
 	elem=linphone_core_get_calls(lc);
 	if (elem==NULL){
 		linphonec_out("(empty)\n");
 	}else{
 		for(;elem!=NULL;elem=elem->next){
+			const char *flag;
 			call=(LinphoneCall*)elem->data;
+			bool_t in_conference=linphone_call_params_local_conference_mode(linphone_call_get_current_params(call));
 			tmp=linphone_call_get_remote_address_as_string (call);
-			linphonec_out("%-2i | %-35s | %s\n",(int)(long)linphone_call_get_user_pointer(call),
-						  tmp,linphone_call_state_to_string(linphone_call_get_state(call)));
+			flag=in_conference ? "conferencing" : "";
+			flag=linphone_call_has_transfer_pending(call) ? "transfer pending" : flag;
+			linphonec_out("%-2i | %-35s | %-15s | %s\n",(int)(long)linphone_call_get_user_pointer(call),
+						  tmp,linphone_call_state_to_string(linphone_call_get_state(call))+strlen("LinphoneCall"),flag);
 			ms_free(tmp);
 		}
 	}
