@@ -107,7 +107,7 @@ static int copy_file(const char *from, const char *to);
 #endif /*_WIN32_WCE*/
 static int linphonec_parse_cmdline(int argc, char **argv);
 static int linphonec_init(int argc, char **argv);
-static int linphonec_main_loop (LinphoneCore * opm, char * sipAddr);
+static int linphonec_main_loop (LinphoneCore * opm);
 static int linphonec_idle_call (void);
 #ifdef HAVE_READLINE
 static int linphonec_initialize_readline(void);
@@ -161,7 +161,7 @@ static int trace_level = 0;
 static char *logfile_name = NULL;
 static char configfile_name[PATH_MAX];
 static const char *factory_configfile_name=NULL;
-static char *sipAddr = NULL; /* for autocall */
+static char *sip_addr_to_call = NULL; /* for autocall */
 static int window_id = 0; /* 0=standalone window, or window id for embedding video */
 #if !defined(_WIN32_WCE)
 static ortp_pipe_t client_sock=ORTP_PIPE_INVALID;
@@ -323,12 +323,23 @@ static void linphonec_call_updated(LinphoneCall *call){
 	}
 }
 
+static void linphonec_call_encryption_changed(LinphoneCore *lc, LinphoneCall *call, bool_t encrypted, const char *auth_token) {
+	long id=(long)linphone_call_get_user_pointer (call);
+	if (!encrypted) {
+		linphonec_out("Call %i is not fully encrypted and auth token is %s.\n", id,
+				(auth_token != NULL) ? auth_token : "absent");
+	} else {
+		linphonec_out("Call %i is fully encrypted and auth token is %s.\n", id,
+				(auth_token != NULL) ? auth_token : "absent");
+	}
+}
+
 static void linphonec_call_state_changed(LinphoneCore *lc, LinphoneCall *call, LinphoneCallState st, const char *msg){
 	char *from=linphone_call_get_remote_address_as_string(call);
 	long id=(long)linphone_call_get_user_pointer (call);
 	switch(st){
 		case LinphoneCallEnd:
-			linphonec_out("Call %i with %s ended.\n", id, from);
+			linphonec_out("Call %i with %s ended (%s).\n", id, from, linphone_reason_to_string(linphone_call_get_reason(call)));
 		break;
 		case LinphoneCallResuming:
 			linphonec_out("Resuming call %i with %s.\n", id, from);
@@ -358,7 +369,6 @@ static void linphonec_call_state_changed(LinphoneCore *lc, LinphoneCall *call, L
 		case LinphoneCallOutgoingInit:
 			linphonec_call_identify(call);
 			id=(long)linphone_call_get_user_pointer (call);
-			from=linphone_call_get_remote_address_as_string(call);
 			linphonec_out("Establishing call id to %s, assigned id %i\n", from,id);
 		break;
 		case LinphoneCallUpdatedByRemote:
@@ -543,7 +553,8 @@ char *linphonec_readline(char *prompt){
 		}
 	}else{
 #ifdef HAVE_READLINE
-		return readline(prompt);
+		char* ret=readline(prompt);
+		return ret;
 #endif
 	}
 }
@@ -627,10 +638,11 @@ main (int argc, char *argv[]) {
 	linphonec_vtable.dtmf_received=linphonec_dtmf_received;
 	linphonec_vtable.refer_received=linphonec_display_refer;
 	linphonec_vtable.notify_recv=linphonec_notify_received;
+	linphonec_vtable.call_encryption_changed=linphonec_call_encryption_changed;
 	
 	if (! linphonec_init(argc, argv) ) exit(EXIT_FAILURE);
 
-	linphonec_main_loop (linphonec, sipAddr);
+	linphonec_main_loop (linphonec);
 
 	linphonec_finish(EXIT_SUCCESS);
 
@@ -794,11 +806,17 @@ linphonec_finish(int exit_status)
 int
 linphonec_prompt_for_auth_final(LinphoneCore *lc)
 {
+	static int reentrancy=0;
 	char *input, *iptr;
 	char auth_prompt[256];
 #ifdef HAVE_READLINE
 	rl_hook_func_t *old_event_hook;
 #endif
+
+	if (reentrancy!=0) return 0;
+	
+	reentrancy++;
+	
 	LinphoneAuthInfo *pending_auth=auth_stack.elem[auth_stack.nitems-1];
 
 	snprintf(auth_prompt, 256, "Password for %s on %s: ",
@@ -901,6 +919,7 @@ static void x11_apply_video_params(VideoParams *params, Window window){
 	const char *dname=getenv("DISPLAY");
 
 	if (display==NULL && dname!=NULL){
+		XInitThreads();
 		display=XOpenDisplay(dname);
 	}
 
@@ -985,6 +1004,14 @@ linphonec_idle_call ()
 		linphone_core_accept_call(opm,NULL);
 		answer_call=FALSE;
 	}
+	/* auto call handling */
+	if (sip_addr_to_call != NULL )
+	{
+		char buf[512];
+		snprintf (buf, sizeof(buf),"call %s", sip_addr_to_call);
+		sip_addr_to_call=NULL;
+		linphonec_parse_command_line(linphonec, buf);
+	}
 
 	if ( auth_stack.nitems )
 	{
@@ -1073,21 +1100,13 @@ static void print_prompt(LinphoneCore *opm){
 }
 
 static int
-linphonec_main_loop (LinphoneCore * opm, char * sipAddr)
+linphonec_main_loop (LinphoneCore * opm)
 {
-	char buf[LINE_MAX_LEN]; /* auto call handling */
 	char *input;
 
 	print_prompt(opm);
 
 
-	/* auto call handling */
-	if (sipAddr != NULL )
-	{
-		snprintf (buf, sizeof(buf),"call %s", sipAddr);
-		linphonec_parse_command_line(linphonec, buf);
-	}
-
 	while (linphonec_running && (input=linphonec_readline(prompt)))
 	{
 		char *iptr; /* input and input pointer */
@@ -1195,7 +1214,7 @@ linphonec_parse_cmdline(int argc, char **argv)
 		{
 			arg_num++;
 			if (arg_num < argc)
-				sipAddr = argv[arg_num];
+				sip_addr_to_call = argv[arg_num];
 		}
                 else if (strncmp ("-a", argv[arg_num], 2) == 0)
                 {
