@@ -28,6 +28,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 static void register_failure(SalOp *op, SalError error, SalReason reason, const char *details);
 
 static bool_t media_parameters_changed(LinphoneCall *call, SalMediaDescription *oldmd, SalMediaDescription *newmd){
+	if (call->params.in_conference!=call->current_params.in_conference) return TRUE;
 	return !sal_media_description_equals(oldmd,newmd)  || call->up_bw!=linphone_core_get_upload_bandwidth(call->core);
 }
 
@@ -49,7 +50,9 @@ void linphone_core_update_streams(LinphoneCore *lc, LinphoneCall *call, SalMedia
 		/* we already started media: check if we really need to restart it*/
 		if (oldmd){
 			if (!media_parameters_changed(call,oldmd,new_md) && !call->playing_ringbacktone){
-				sal_media_description_unref(oldmd);
+				/*as nothing has changed, keep the oldmd */
+				call->resultdesc=oldmd;
+				sal_media_description_unref(new_md);
 				if (call->all_muted){
 					ms_message("Early media finished, unmuting inputs...");
 					/*we were in early media, now we want to enable real media */
@@ -91,7 +94,7 @@ void linphone_core_update_streams(LinphoneCore *lc, LinphoneCall *call, SalMedia
 		linphone_call_start_media_streams(call,all_muted,send_ringbacktone);
 	}
 }
-
+#if 0
 static bool_t is_duplicate_call(LinphoneCore *lc, const LinphoneAddress *from, const LinphoneAddress *to){
 	MSList *elem;
 	for(elem=lc->calls;elem!=NULL;elem=elem->next){
@@ -103,6 +106,37 @@ static bool_t is_duplicate_call(LinphoneCore *lc, const LinphoneAddress *from, c
 	}
 	return FALSE;
 }
+#endif
+
+static bool_t already_a_call_with_remote_address(const LinphoneCore *lc, const LinphoneAddress *remote) {
+	ms_warning(" searching for already_a_call_with_remote_address.");
+
+	MSList *elem;
+	for(elem=lc->calls;elem!=NULL;elem=elem->next){
+		const LinphoneCall *call=(LinphoneCall*)elem->data;
+		const LinphoneAddress *cRemote=linphone_call_get_remote_address(call);
+		if (linphone_address_weak_equal(cRemote,remote)) {
+			ms_warning("already_a_call_with_remote_address found.");
+			return TRUE;
+		}
+	}
+	return FALSE;
+}
+
+static bool_t already_a_call_pending(LinphoneCore *lc){
+	MSList *elem;
+	for(elem=lc->calls;elem!=NULL;elem=elem->next){
+		LinphoneCall *call=(LinphoneCall*)elem->data;
+		if (call->state==LinphoneCallIncomingReceived
+		    || call->state==LinphoneCallOutgoingInit
+		    || call->state==LinphoneCallOutgoingProgress
+		    || call->state==LinphoneCallOutgoingEarlyMedia
+		    || call->state==LinphoneCallOutgoingRinging){
+			return TRUE;
+		}
+	}
+	return FALSE;
+}
 
 static void call_received(SalOp *h){
 	LinphoneCore *lc=(LinphoneCore *)sal_get_user_pointer(sal_op_get_sal(h));
@@ -142,9 +176,10 @@ static void call_received(SalOp *h){
 	from_addr=linphone_address_new(from);
 	to_addr=linphone_address_new(to);
 
-	if (is_duplicate_call(lc,from_addr,to_addr)){
-		ms_warning("Receiving duplicated call, refusing this one.");
+	if (already_a_call_with_remote_address(lc,from_addr) || already_a_call_pending(lc)){
+		ms_warning("Receiving another call while one is ringing or initiated, refusing this one with busy message.");
 		sal_call_decline(h,SalReasonBusy,NULL);
+		sal_op_release(h);
 		linphone_address_destroy(from_addr);
 		linphone_address_destroy(to_addr);
 		return;
@@ -174,24 +209,28 @@ static void call_received(SalOp *h){
 	    lc->vtable.display_status(lc,barmesg);
 
 	/* play the ring if this is the only call*/
-	if (lc->sound_conf.ring_sndcard!=NULL && ms_list_size(lc->calls)==1){
+	if (ms_list_size(lc->calls)==1){
 		lc->current_call=call;
 		if (lc->ringstream && lc->dmfs_playing_start_time!=0){
 			ring_stop(lc->ringstream);
 			lc->ringstream=NULL;
 			lc->dmfs_playing_start_time=0;
 		}
-		if(lc->ringstream==NULL && lc->sound_conf.local_ring){
-			MSSndCard *ringcard=lc->sound_conf.lsd_card ?lc->sound_conf.lsd_card : lc->sound_conf.ring_sndcard;
-			ms_message("Starting local ring...");
-			lc->ringstream=ring_start(lc->sound_conf.local_ring,2000,ringcard);
-		}
-		else
-		{
-			ms_message("the local ring is already started");
+		if (lc->sound_conf.ring_sndcard!=NULL){
+			if(lc->ringstream==NULL && lc->sound_conf.local_ring){
+				MSSndCard *ringcard=lc->sound_conf.lsd_card ?lc->sound_conf.lsd_card : lc->sound_conf.ring_sndcard;
+				ms_message("Starting local ring...");
+				lc->ringstream=ring_start(lc->sound_conf.local_ring,2000,ringcard);
+			}
+			else
+			{
+				ms_message("the local ring is already started");
+			}
 		}
 	}else{
-		/*TODO : play a tone within the context of the current call */
+		/* else play a tone within the context of the current call */
+		call->ringing_beep=TRUE;
+		linphone_core_play_tone(lc);
 	}
 
 	
@@ -328,7 +367,8 @@ static void call_accepted(SalOp *op){
 			}
 			linphone_core_update_streams (lc,call,md);
 			linphone_call_set_state(call, LinphoneCallStreamsRunning, "Streams running");
-			lc->current_call=call;
+			if (!call->current_params.in_conference)
+				lc->current_call=call;
 		}
 	}else{
 		/*send a bye*/
@@ -370,11 +410,19 @@ static void call_updating(SalOp *op){
 	LinphoneCall *call=(LinphoneCall*)sal_op_get_user_pointer(op);
 	LinphoneCallState prevstate=LinphoneCallIdle;
 	SalMediaDescription *md;
+	SalMediaDescription *old_md=call->resultdesc;
+
+	sal_media_description_ref(old_md);
 	
 	md=sal_call_get_final_media_description(op);
+
+	/*accept the modification (sends a 200Ok)*/
+	sal_call_accept(op);
 	
 	if (md && !sal_media_description_empty(md))
 	{
+		linphone_core_update_streams (lc,call,md);
+
 		if (sal_media_description_has_dir(call->localdesc,SalStreamSendRecv)){
 			ms_message("Our local status is SalStreamSendRecv");
 			if (sal_media_description_has_dir (md,SalStreamRecvOnly) || sal_media_description_has_dir(md,SalStreamInactive)){
@@ -382,23 +430,23 @@ static void call_updating(SalOp *op){
 				if(lc->vtable.display_status)
 					lc->vtable.display_status(lc,_("We are being paused..."));
 				linphone_call_set_state (call,LinphoneCallPausedByRemote,"Call paused by remote");
-			}else if (!sal_media_description_has_dir(call->resultdesc,SalStreamSendRecv) && sal_media_description_has_dir(md,SalStreamSendRecv)){
+			}else if (!sal_media_description_has_dir(old_md,SalStreamSendRecv) && sal_media_description_has_dir(md,SalStreamSendRecv)){
 				if(lc->vtable.display_status)
 					lc->vtable.display_status(lc,_("We have been resumed..."));
 				linphone_call_set_state (call,LinphoneCallStreamsRunning,"Connected (streams running)");
-				lc->current_call=call;
 			}else{
 				prevstate=call->state;
+				if(lc->vtable.display_status)
+					lc->vtable.display_status(lc,_("Call has been updated by remote..."));
 				linphone_call_set_state(call, LinphoneCallUpdatedByRemote,"Call updated by remote");
 			}
 		}
-		/*accept the modification (sends a 200Ok)*/
-		sal_call_accept(op);
-		linphone_core_update_streams (lc,call,md);
+
 		if (prevstate!=LinphoneCallIdle){
 			linphone_call_set_state (call,prevstate,"Connected (streams running)");
 		}
 	}
+	sal_media_description_unref(old_md);
 }
 
 static void call_terminated(SalOp *op, const char *from){
@@ -517,6 +565,15 @@ static void auth_requested(SalOp *h, const char *realm, const char *username){
 	LinphoneCore *lc=(LinphoneCore *)sal_get_user_pointer(sal_op_get_sal(h));
 	LinphoneAuthInfo *ai=(LinphoneAuthInfo*)linphone_core_find_auth_info(lc,realm,username);
 	ms_message("auth_requested() for realm=%s, username=%s",realm,username);
+
+	if (ai && ai->works==FALSE && ai->usecount>=3){
+		/*case we tried 3 times to authenticate, without success */
+		/*Better is to stop (implemeted below in else statement), and retry later*/
+		if (ms_time(NULL)-ai->last_use_time>30){
+			ai->usecount=0; /*so that we can allow to retry */
+		}
+	}
+	
 	if (ai && (ai->works || ai->usecount<3)){
 		SalAuthInfo sai;
 		sai.username=ai->username;
@@ -526,6 +583,7 @@ static void auth_requested(SalOp *h, const char *realm, const char *username){
 		ms_message("auth_requested(): authenticating realm=%s, username=%s",realm,username);
 		sal_op_authenticate(h,&sai);
 		ai->usecount++;
+		ai->last_use_time=ms_time(NULL);
 	}else{
 		if (ai && ai->works==FALSE) {
 			sal_op_cancel_authentication(h);
@@ -584,6 +642,13 @@ static void register_failure(SalOp *op, SalError error, SalReason reason, const
 		linphone_proxy_config_set_error(cfg, LinphoneReasonNoResponse);
 	}
 	linphone_proxy_config_set_state(cfg,LinphoneRegistrationFailed,details);
+	if (error== SalErrorFailure && reason == SalReasonForbidden) {
+		const char *realm=NULL,*username=NULL;
+		if (sal_op_get_auth_requested(op,&realm,&username)==0){
+			if (lc->vtable.auth_info_requested)
+				lc->vtable.auth_info_requested(lc,realm,username);
+		}
+	}
 }
 
 static void vfu_request(SalOp *op){
@@ -660,12 +725,6 @@ static void subscribe_closed(SalOp *op, const char *from){
 	linphone_subscription_closed(lc,op);
 }
 
-static void internal_message(Sal *sal, const char *msg){
-	LinphoneCore *lc=(LinphoneCore *)sal_get_user_pointer(sal);
-	if (lc->vtable.show)
-		lc->vtable.show(lc);
-}
-
 static void ping_reply(SalOp *op){
 	LinphoneCall *call=(LinphoneCall*) sal_op_get_user_pointer(op);
 	ms_message("ping reply !");
@@ -701,7 +760,6 @@ SalCallbacks linphone_sal_callbacks={
 	notify_presence,
 	subscribe_received,
 	subscribe_closed,
-	internal_message,
 	ping_reply
 };
 
