@@ -1,7 +1,7 @@
 
 /*
 linphone
-Copyright (C) 2010  Belledonne Communications SARL 
+Copyright (C) 2010  Belledonne Communications SARL
  (simon.morlat@linphone.org)
 
 This program is free software; you can redistribute it and/or
@@ -25,6 +25,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "sipsetup.h"
 #include "lpconfig.h"
 #include "private.h"
+#include <ortp/event.h>
 
 
 #include "mediastreamer2/mediastream.h"
@@ -40,6 +41,108 @@ static MSWebCam *get_nowebcam_device(){
 }
 #endif
 
+LinphoneCore *linphone_call_get_core(const LinphoneCall *call){
+	return call->core;
+}
+
+static const char* get_hexa_zrtp_identifier(LinphoneCore *lc){
+	const char *confZid=lp_config_get_string(lc->config,"rtp","zid",NULL);
+	if (confZid != NULL) {
+		return confZid;
+	} else {
+        char zidstr[128];
+        snprintf(zidstr,sizeof(zidstr),"%x-%x-%x",rand(),rand(),rand());
+		lp_config_set_string(lc->config,"rtp","zid",zidstr);
+		return lp_config_get_string(lc->config,"rtp","zid",NULL);
+	}
+}
+
+const char* linphone_call_get_authentication_token(LinphoneCall *call){
+	return call->auth_token;
+}
+
+bool_t linphone_call_get_authentication_token_verified(LinphoneCall *call){
+	return call->auth_token_verified;
+}
+bool_t linphone_call_are_all_streams_encrypted(LinphoneCall *call) {
+	// Check ZRTP encryption in audiostream
+	if (!call->audiostream_encrypted) {
+		return FALSE;
+	}
+
+#ifdef VIDEO_ENABLED
+	// If video enabled, check ZRTP encryption in videostream
+	const LinphoneCallParams *params=linphone_call_get_current_params(call);
+	if (params->has_video && !call->videostream_encrypted) {
+		return FALSE;
+	}
+#endif
+
+	return TRUE;
+}
+
+void propagate_encryption_changed(LinphoneCall *call){
+	if (call->core->vtable.call_encryption_changed == NULL) return;
+
+	if (!linphone_call_are_all_streams_encrypted(call)) {
+		ms_message("Some streams are not encrypted");
+		call->core->vtable.call_encryption_changed(call->core, call, FALSE, call->auth_token);
+	} else {
+		ms_message("All streams are encrypted");
+		call->core->vtable.call_encryption_changed(call->core, call, TRUE, call->auth_token);
+	}
+}
+
+#ifdef VIDEO_ENABLED
+static void linphone_call_videostream_encryption_changed(void *data, bool_t encrypted){
+	ms_message("Video stream is %s", encrypted ? "encrypted" : "not encrypted");
+
+	LinphoneCall *call = (LinphoneCall *)data;
+	call->videostream_encrypted=encrypted;
+	propagate_encryption_changed(call);
+}
+#endif
+
+static void linphone_call_audiostream_encryption_changed(void *data, bool_t encrypted) {
+	char status[255]={0};
+	ms_message("Audio stream is %s ", encrypted ? "encrypted" : "not encrypted");
+
+	LinphoneCall *call = (LinphoneCall *)data;
+	call->audiostream_encrypted=encrypted;
+
+	if (encrypted && call->core->vtable.display_status != NULL) {
+		snprintf(status,sizeof(status)-1,_("Authentication token is %s"),call->auth_token);
+		 call->core->vtable.display_status(call->core, status);
+	}
+
+	propagate_encryption_changed(call);
+
+
+#ifdef VIDEO_ENABLED
+	// Enable video encryption
+	const LinphoneCallParams *params=linphone_call_get_current_params(call);
+	if (params->has_video) {
+		ms_message("Trying to enable encryption on video stream");
+		OrtpZrtpParams params;
+		params.zid=get_hexa_zrtp_identifier(call->core);
+		params.zid_file=NULL; //unused
+		video_stream_enable_zrtp(call->videostream,call->audiostream,&params);
+	}
+#endif
+}
+
+
+static void linphone_call_audiostream_auth_token_ready(void *data, const char* auth_token, bool_t verified) {
+	LinphoneCall *call=(LinphoneCall *)data;
+	if (call->auth_token != NULL)
+		ms_free(call->auth_token);
+
+	call->auth_token=ms_strdup(auth_token);
+	call->auth_token_verified=verified;
+
+	ms_message("Authentication token is %s (%s)", auth_token, verified?"verified":"unverified");
+}
+
 
 static MSList *make_codec_list(LinphoneCore *lc, const MSList *codecs, int bandwidth_limit){
 	MSList *l=NULL;
@@ -66,14 +169,14 @@ static SalMediaDescription *_create_local_media_description(LinphoneCore *lc, Li
 	LinphoneAddress *addr=linphone_address_new(me);
 	const char *username=linphone_address_get_username (addr);
 	SalMediaDescription *md=sal_media_description_new();
-	
+
 	md->session_id=session_id;
 	md->session_ver=session_ver;
 	md->nstreams=1;
 	strncpy(md->addr,call->localip,sizeof(md->addr));
 	strncpy(md->username,username,sizeof(md->username));
 	md->bandwidth=linphone_core_get_download_bandwidth(lc);
-	
+
 	/*set audio capabilities */
 	strncpy(md->streams[0].addr,call->localip,sizeof(md->streams[0].addr));
 	md->streams[0].port=call->audio_port;
@@ -84,7 +187,7 @@ static SalMediaDescription *_create_local_media_description(LinphoneCore *lc, Li
 	pt=payload_type_clone(rtp_profile_get_payload_from_mime(&av_profile,"telephone-event"));
 	l=ms_list_append(l,pt);
 	md->streams[0].payloads=l;
-	
+
 
 	if (call->params.has_video){
 		md->nstreams++;
@@ -151,7 +254,7 @@ static void linphone_call_init_common(LinphoneCall *call, LinphoneAddress *from,
 	if (port_offset==-1) return;
 	call->audio_port=linphone_core_get_audio_port(call->core)+port_offset;
 	call->video_port=linphone_core_get_video_port(call->core)+port_offset;
-	
+
 }
 
 static void discover_mtu(LinphoneCore *lc, const char *remote){
@@ -190,7 +293,6 @@ LinphoneCall * linphone_call_new_outgoing(struct _LinphoneCore *lc, LinphoneAddr
 
 LinphoneCall * linphone_call_new_incoming(LinphoneCore *lc, LinphoneAddress *from, LinphoneAddress *to, SalOp *op){
 	LinphoneCall *call=ms_new0(LinphoneCall,1);
-	char *to_str;
 	char *from_str;
 
 	call->dir=LinphoneCallIncoming;
@@ -202,15 +304,13 @@ LinphoneCall * linphone_call_new_incoming(LinphoneCore *lc, LinphoneAddress *fro
 		/*the following sends an option request back to the caller so that
 		 we get a chance to discover our nat'd address before answering.*/
 		call->ping_op=sal_op_new(lc->sal);
-		to_str=linphone_address_as_string(to);
 		from_str=linphone_address_as_string(from);
 		sal_op_set_route(call->ping_op,sal_op_get_network_origin(call->op));
 		sal_op_set_user_pointer(call->ping_op,call);
-		sal_ping(call->ping_op,to_str,from_str);
-		ms_free(to_str);
+		sal_ping(call->ping_op,linphone_core_find_best_identity(lc,from,NULL),from_str);
 		ms_free(from_str);
 	}
-	
+
 	linphone_address_clean(from);
 	linphone_core_get_local_ip(lc,linphone_address_get_domain(from),call->localip);
 	linphone_call_init_common(call, from, to);
@@ -226,26 +326,18 @@ LinphoneCall * linphone_call_new_incoming(LinphoneCore *lc, LinphoneAddress *fro
 /* this function is called internally to get rid of a call.
  It performs the following tasks:
  - remove the call from the internal list of calls
- - unref the LinphoneCall object
  - update the call logs accordingly
 */
 
 static void linphone_call_set_terminated(LinphoneCall *call){
-	LinphoneCallStatus status=LinphoneCallAborted;
 	LinphoneCore *lc=call->core;
-	
+
 	linphone_core_update_allocated_audio_bandwidth(lc);
-	if (call->state==LinphoneCallEnd){
-		if (call->reason==LinphoneReasonDeclined){
-			status=LinphoneCallDeclined;
-		}
-		else status=LinphoneCallSuccess;
-		
-	}
+
 	call->owns_call_log=FALSE;
-	linphone_call_log_completed(call->log,call, status);
-	
-	
+	linphone_call_log_completed(call);
+
+
 	if (call == lc->current_call){
 		ms_message("Resetting the current call");
 		lc->current_call=NULL;
@@ -254,10 +346,15 @@ static void linphone_call_set_terminated(LinphoneCall *call){
 	if (linphone_core_del_call(lc,call) != 0){
 		ms_error("Could not remove the call from the list !!!");
 	}
-	
+
 	if (ms_list_size(lc->calls)==0)
 		linphone_core_notify_all_friends(lc,lc->presence_mode);
-	
+
+	linphone_core_conference_check_uninit(&lc->conf_ctx);
+	if (call->ringing_beep){
+		linphone_core_stop_dtmf(lc);
+		call->ringing_beep=FALSE;
+	}
 }
 
 const char *linphone_call_state_to_string(LinphoneCallState cs){
@@ -323,9 +420,15 @@ void linphone_call_set_state(LinphoneCall *call, LinphoneCallState cstate, const
 			call->state=cstate;
 		}
 		if (cstate==LinphoneCallEnd || cstate==LinphoneCallError){
-			linphone_call_set_terminated (call);
+             if (call->reason==LinphoneReasonDeclined){
+                 call->log->status=LinphoneCallDeclined;
+             }
+            linphone_call_set_terminated (call);
 		}
-		
+        if (cstate == LinphoneCallConnected) {
+            call->log->status=LinphoneCallSuccess;
+        }
+
 		if (lc->vtable.call_state_changed)
 			lc->vtable.call_state_changed(lc,call,cstate,message);
 		if (cstate==LinphoneCallReleased){
@@ -362,6 +465,10 @@ static void linphone_call_destroy(LinphoneCall *obj)
 	}
 	if (obj->owns_call_log)
 		linphone_call_log_destroy(obj->log);
+	if (obj->auth_token) {
+		ms_free(obj->auth_token);
+	}
+
 	ms_free(obj);
 }
 
@@ -377,8 +484,9 @@ static void linphone_call_destroy(LinphoneCall *obj)
  * valid. Once the application no more needs this pointer,
  * it must call linphone_call_unref().
 **/
-void linphone_call_ref(LinphoneCall *obj){
+LinphoneCall * linphone_call_ref(LinphoneCall *obj){
 	obj->refcnt++;
+	return obj;
 }
 
 /**
@@ -490,7 +598,7 @@ const char *linphone_call_get_remote_user_agent(LinphoneCall *call){
  * executed yet.
  * Pending transfers are executed when this call is being paused or closed,
  * locally or by remote endpoint.
- * If the call is already paused while receiving the transfer request, the 
+ * If the call is already paused while receiving the transfer request, the
  * transfer immediately occurs.
 **/
 bool_t linphone_call_has_transfer_pending(const LinphoneCall *call){
@@ -582,6 +690,13 @@ bool_t linphone_call_params_early_media_sending_enabled(const LinphoneCallParams
 	return cp->real_early_media;
 }
 
+/**
+ * Returns true if the call is part of the locally managed conference.
+**/
+bool_t linphone_call_params_local_conference_mode(const LinphoneCallParams *cp){
+	return cp->in_conference;
+}
+
 /**
  * Refine bandwidth settings for this call by setting a bandwidth limit for audio streams.
  * As a consequence, codecs whose bitrates are not compatible with this limit won't be used.
@@ -590,6 +705,17 @@ void linphone_call_params_set_audio_bandwidth_limit(LinphoneCallParams *cp, int
 	cp->audio_bw=bandwidth;
 }
 
+#ifdef VIDEO_ENABLED
+/**
+ * Request remote side to send us a Video Fast Update.
+**/
+void linphone_call_send_vfu_request(LinphoneCall *call)
+{
+	if (LinphoneCallStreamsRunning == linphone_call_get_state(call))
+		sal_call_send_vfu_request(call->op);
+}
+#endif
+
 /**
  *
 **/
@@ -637,7 +763,7 @@ void linphone_call_init_media_streams(LinphoneCall *call){
 	LinphoneCore *lc=call->core;
 	SalMediaDescription *md=call->localdesc;
 	AudioStream *audiostream;
-	
+
 	call->audiostream=audiostream=audio_stream_new(md->streams[0].port,linphone_core_ipv6_enabled(lc));
 	if (linphone_core_echo_limiter_enabled(lc)){
 		const char *type=lp_config_get_string(lc->config,"sound","el_type","mic");
@@ -649,25 +775,38 @@ void linphone_call_init_media_streams(LinphoneCall *call){
 	audio_stream_enable_gain_control(audiostream,TRUE);
 	if (linphone_core_echo_cancellation_enabled(lc)){
 		int len,delay,framesize;
+		const char *statestr=lp_config_get_string(lc->config,"sound","ec_state",NULL);
 		len=lp_config_get_int(lc->config,"sound","ec_tail_len",0);
 		delay=lp_config_get_int(lc->config,"sound","ec_delay",0);
 		framesize=lp_config_get_int(lc->config,"sound","ec_framesize",0);
 		audio_stream_set_echo_canceller_params(audiostream,len,delay,framesize);
+		if (statestr && audiostream->ec){
+			ms_filter_call_method(audiostream->ec,MS_ECHO_CANCELLER_SET_STATE_STRING,(void*)statestr);
+		}
 	}
 	audio_stream_enable_automatic_gain_control(audiostream,linphone_core_agc_enabled(lc));
 	{
 		int enabled=lp_config_get_int(lc->config,"sound","noisegate",0);
 		audio_stream_enable_noise_gate(audiostream,enabled);
 	}
+
 	if (lc->a_rtp)
 		rtp_session_set_transports(audiostream->session,lc->a_rtp,lc->a_rtcp);
 
+	call->audiostream_app_evq = ortp_ev_queue_new();
+	rtp_session_register_event_queue(audiostream->session,call->audiostream_app_evq);
+
 #ifdef VIDEO_ENABLED
+
 	if ((lc->video_conf.display || lc->video_conf.capture) && md->streams[1].port>0){
 		call->videostream=video_stream_new(md->streams[1].port,linphone_core_ipv6_enabled(lc));
 	if( lc->video_conf.displaytype != NULL)
 		video_stream_set_display_filter_name(call->videostream,lc->video_conf.displaytype);
 	video_stream_set_event_callback(call->videostream,video_stream_event_cb, call);
+	if (lc->v_rtp)
+		rtp_session_set_transports(call->videostream->session,lc->v_rtp,lc->v_rtcp);
+	call->videostream_app_evq = ortp_ev_queue_new();
+	rtp_session_register_event_queue(call->videostream->session,call->videostream_app_evq);
 #ifdef TEST_EXT_RENDERER
 		video_stream_set_render_callback(call->videostream,rendercb,NULL);
 #endif
@@ -712,26 +851,24 @@ static void parametrize_equalizer(LinphoneCore *lc, AudioStream *st){
 	}
 }
 
-
-static void post_configure_audio_streams(LinphoneCall*call){
-	AudioStream *st=call->audiostream;
-	LinphoneCore *lc=call->core;
+void _post_configure_audio_stream(AudioStream *st, LinphoneCore *lc, bool_t muted){
 	float mic_gain=lp_config_get_float(lc->config,"sound","mic_gain",1);
 	float thres = 0;
 	float recv_gain;
 	float ng_thres=lp_config_get_float(lc->config,"sound","ng_thres",0.05);
 	float ng_floorgain=lp_config_get_float(lc->config,"sound","ng_floorgain",0);
 	int dc_removal=lp_config_get_int(lc->config,"sound","dc_removal",0);
-	
-	if (!call->audio_muted)
+
+	if (!muted)
 		audio_stream_set_mic_gain(st,mic_gain);
-	else 
+	else
 		audio_stream_set_mic_gain(st,0);
 
 	recv_gain = lc->sound_conf.soft_play_lev;
 	if (recv_gain != 0) {
 		linphone_core_set_playback_gain_db (lc,recv_gain);
 	}
+	
 	if (st->volsend){
 		ms_filter_call_method(st->volsend,MS_VOLUME_REMOVE_DC,&dc_removal);
 		float speed=lp_config_get_float(lc->config,"sound","el_speed",-1);
@@ -758,10 +895,18 @@ static void post_configure_audio_streams(LinphoneCall*call){
 	if (st->volrecv){
 		/* parameters for a limited noise-gate effect, using echo limiter threshold */
 		float floorgain = 1/mic_gain;
-		ms_filter_call_method(st->volrecv,MS_VOLUME_SET_NOISE_GATE_THRESHOLD,&thres);
+		int spk_agc=lp_config_get_int(lc->config,"sound","speaker_agc_enabled",0);
+		ms_filter_call_method(st->volrecv, MS_VOLUME_ENABLE_AGC, &spk_agc);
+		ms_filter_call_method(st->volrecv,MS_VOLUME_SET_NOISE_GATE_THRESHOLD,&ng_thres);
 		ms_filter_call_method(st->volrecv,MS_VOLUME_SET_NOISE_GATE_FLOORGAIN,&floorgain);
 	}
 	parametrize_equalizer(lc,st);
+}
+
+static void post_configure_audio_streams(LinphoneCall*call){
+	AudioStream *st=call->audiostream;
+	LinphoneCore *lc=call->core;
+	_post_configure_audio_stream(st,lc,call->audio_muted);
 	if (lc->vtable.dtmf_received!=NULL){
 		/* replace by our default action*/
 		audio_stream_play_received_dtmfs(call->audiostream,FALSE);
@@ -769,9 +914,6 @@ static void post_configure_audio_streams(LinphoneCall*call){
 	}
 }
 
-
-
-
 static RtpProfile *make_profile(LinphoneCall *call, const SalMediaDescription *md, const SalStreamDescription *desc, int *used_pt){
 	int bw;
 	const MSList *elem;
@@ -779,15 +921,17 @@ static RtpProfile *make_profile(LinphoneCall *call, const SalMediaDescription *m
 	bool_t first=TRUE;
 	int remote_bw=0;
 	LinphoneCore *lc=call->core;
+	int up_ptime=0;
 	*used_pt=-1;
-	
+
 	for(elem=desc->payloads;elem!=NULL;elem=elem->next){
 		PayloadType *pt=(PayloadType*)elem->data;
 		int number;
-		
-		if (first) {
+
+		if ((pt->flags & PAYLOAD_TYPE_FLAG_CAN_SEND) && first) {
 			if (desc->type==SalAudio){
 				linphone_core_update_allocated_audio_bandwidth_in_call(call,pt);
+				up_ptime=linphone_core_get_upload_ptime(lc);
 			}
 			*used_pt=payload_type_get_number(pt);
 			first=FALSE;
@@ -800,8 +944,8 @@ static RtpProfile *make_profile(LinphoneCall *call, const SalMediaDescription *m
 				remote_bw=get_video_bandwidth(remote_bw,call->audio_bw);
 			}
 		}
-		
-		if (desc->type==SalAudio){			
+
+		if (desc->type==SalAudio){
 				bw=get_min_bandwidth(call->audio_bw,remote_bw);
 		}else bw=get_min_bandwidth(get_video_bandwidth(linphone_core_get_upload_bandwidth (lc),call->audio_bw),remote_bw);
 		if (bw>0) pt->normal_bitrate=bw*1000;
@@ -809,8 +953,11 @@ static RtpProfile *make_profile(LinphoneCall *call, const SalMediaDescription *m
 			pt->normal_bitrate=-1;
 		}
 		if (desc->ptime>0){
+			up_ptime=desc->ptime;
+		}
+		if (up_ptime>0){
 			char tmp[40];
-			snprintf(tmp,sizeof(tmp),"ptime=%i",desc->ptime);
+			snprintf(tmp,sizeof(tmp),"ptime=%i",up_ptime);
 			payload_type_append_send_fmtp(pt,tmp);
 		}
 		number=payload_type_get_number(pt);
@@ -829,171 +976,214 @@ static void setup_ring_player(LinphoneCore *lc, LinphoneCall *call){
 	ms_filter_call_method(call->audiostream->soundread,MS_FILE_PLAYER_LOOP,&pause_time);
 }
 
+#define LINPHONE_RTCP_SDES_TOOL "Linphone-" LINPHONE_VERSION
+
+static bool_t linphone_call_sound_resources_available(LinphoneCall *call){
+	LinphoneCore *lc=call->core;
+	LinphoneCall *current=linphone_core_get_current_call(lc);
+	return !linphone_core_is_in_conference(lc) && 
+		(current==NULL || current==call);
+}
+
+static void linphone_call_start_audio_stream(LinphoneCall *call, const char *cname, bool_t muted, bool_t send_ringbacktone, bool_t use_arc){
+	LinphoneCore *lc=call->core;
+	int jitt_comp=lc->rtp_conf.audio_jitt_comp;
+	int used_pt=-1;
+	const SalStreamDescription *stream=sal_media_description_find_stream(call->resultdesc,
+	    					SalProtoRtpAvp,SalAudio);
+
+	if (stream && stream->dir!=SalStreamInactive && stream->port!=0){
+		MSSndCard *playcard=lc->sound_conf.lsd_card ?
+			lc->sound_conf.lsd_card : lc->sound_conf.play_sndcard;
+		MSSndCard *captcard=lc->sound_conf.capt_sndcard;
+		const char *playfile=lc->play_file;
+		const char *recfile=lc->rec_file;
+		call->audio_profile=make_profile(call,call->resultdesc,stream,&used_pt);
+		bool_t use_ec;
+
+		if (used_pt!=-1){
+			if (playcard==NULL) {
+				ms_warning("No card defined for playback !");
+			}
+			if (captcard==NULL) {
+				ms_warning("No card defined for capture !");
+			}
+			/*Replace soundcard filters by inactive file players or recorders
+			 when placed in recvonly or sendonly mode*/
+			if (stream->port==0 || stream->dir==SalStreamRecvOnly){
+				captcard=NULL;
+				playfile=NULL;
+			}else if (stream->dir==SalStreamSendOnly){
+				playcard=NULL;
+				captcard=NULL;
+				recfile=NULL;
+				/*And we will eventually play "playfile" if set by the user*/
+				/*playfile=NULL;*/
+			}
+			if (send_ringbacktone){
+				captcard=NULL;
+				playfile=NULL;/* it is setup later*/
+			}
+			/*if playfile are supplied don't use soundcards*/
+			if (lc->use_files) {
+				captcard=NULL;
+				playcard=NULL;
+			}
+			if (call->params.in_conference){
+				/* first create the graph without soundcard resources*/
+				captcard=playcard=NULL;
+			}
+			if (!linphone_call_sound_resources_available(call)){
+				ms_message("Sound resources are used by another call, not using soundcard.");
+				captcard=playcard=NULL;
+			}
+			use_ec=captcard==NULL ? FALSE : linphone_core_echo_cancellation_enabled(lc);
+
+			audio_stream_enable_adaptive_bitrate_control(call->audiostream,use_arc);
+			audio_stream_start_full(
+				call->audiostream,
+				call->audio_profile,
+				stream->addr[0]!='\0' ? stream->addr : call->resultdesc->addr,
+				stream->port,
+				linphone_core_rtcp_enabled(lc) ? (stream->port+1) : 0,
+				used_pt,
+				jitt_comp,
+				playfile,
+				recfile,
+				playcard,
+				captcard,
+				use_ec
+				);
+			post_configure_audio_streams(call);
+			if (muted && !send_ringbacktone){
+				audio_stream_set_mic_gain(call->audiostream,0);
+			}
+			if (stream->dir==SalStreamSendOnly && playfile!=NULL){
+				int pause_time=500;
+				ms_filter_call_method(call->audiostream->soundread,MS_FILE_PLAYER_LOOP,&pause_time);
+			}
+			if (send_ringbacktone){
+				setup_ring_player(lc,call);
+			}
+			audio_stream_set_rtcp_information(call->audiostream, cname, LINPHONE_RTCP_SDES_TOOL);
+			if (call->params.in_conference){
+				/*transform the graph to connect it to the conference filter */
+				linphone_call_add_to_conf(call);
+			}
+		}else ms_warning("No audio stream accepted ?");
+	}
+}
+
+static void linphone_call_start_video_stream(LinphoneCall *call, const char *cname,bool_t all_inputs_muted){
+#ifdef VIDEO_ENABLED
+	LinphoneCore *lc=call->core;
+	int used_pt=-1;
+	const SalStreamDescription *vstream=sal_media_description_find_stream(call->resultdesc,
+		    					SalProtoRtpAvp,SalVideo);
+	/* shutdown preview */
+	if (lc->previewstream!=NULL) {
+		video_preview_stop(lc->previewstream);
+		lc->previewstream=NULL;
+	}
+	call->current_params.has_video=FALSE;
+	if (vstream!=NULL && vstream->dir!=SalStreamInactive && vstream->port!=0) {
+		const char *addr=vstream->addr[0]!='\0' ? vstream->addr : call->resultdesc->addr;
+		call->video_profile=make_profile(call,call->resultdesc,vstream,&used_pt);
+		if (used_pt!=-1){
+			VideoStreamDir dir=VideoStreamSendRecv;
+			MSWebCam *cam=lc->video_conf.device;
+			bool_t is_inactive=FALSE;
+
+			call->current_params.has_video=TRUE;
+
+			video_stream_enable_adaptive_bitrate_control(call->videostream,
+			                                          linphone_core_adaptive_rate_control_enabled(lc));
+			video_stream_set_sent_video_size(call->videostream,linphone_core_get_preferred_video_size(lc));
+			video_stream_enable_self_view(call->videostream,lc->video_conf.selfview);
+			if (lc->video_window_id!=0)
+				video_stream_set_native_window_id(call->videostream,lc->video_window_id);
+			if (lc->preview_window_id!=0)
+				video_stream_set_native_preview_window_id (call->videostream,lc->preview_window_id);
+			video_stream_use_preview_video_window (call->videostream,lc->use_preview_window);
+			
+			if (vstream->dir==SalStreamSendOnly && lc->video_conf.capture ){
+				cam=get_nowebcam_device();
+				dir=VideoStreamSendOnly;
+			}else if (vstream->dir==SalStreamRecvOnly && lc->video_conf.display ){
+				dir=VideoStreamRecvOnly;
+			}else if (vstream->dir==SalStreamSendRecv){
+				if (lc->video_conf.display && lc->video_conf.capture)
+					dir=VideoStreamSendRecv;
+				else if (lc->video_conf.display)
+					dir=VideoStreamRecvOnly;
+				else
+					dir=VideoStreamSendOnly;
+			}else{
+				ms_warning("video stream is inactive.");
+				/*either inactive or incompatible with local capabilities*/
+				is_inactive=TRUE;
+			}
+			if (call->camera_active==FALSE || all_inputs_muted){
+				cam=get_nowebcam_device();
+			}
+			if (!is_inactive){
+				video_stream_set_direction (call->videostream, dir);
+				ms_message("%s lc rotation:%d\n", __FUNCTION__, lc->device_rotation);
+				video_stream_set_device_rotation(call->videostream, lc->device_rotation);
+				video_stream_start(call->videostream,
+					call->video_profile, addr, vstream->port,
+					linphone_core_rtcp_enabled(lc) ? (vstream->port+1) : 0,
+					used_pt, lc->rtp_conf.audio_jitt_comp, cam);
+				video_stream_set_rtcp_information(call->videostream, cname,LINPHONE_RTCP_SDES_TOOL);
+			}
+		}else ms_warning("No video stream accepted.");
+	}else{
+		ms_warning("No valid video stream defined.");
+	}
+#endif
+}
 
 void linphone_call_start_media_streams(LinphoneCall *call, bool_t all_inputs_muted, bool_t send_ringbacktone){
 	LinphoneCore *lc=call->core;
 	LinphoneAddress *me=linphone_core_get_primary_contact_parsed(lc);
-	const char *tool="linphone-" LINPHONE_VERSION;
 	char *cname;
-	int used_pt=-1;
+	bool_t use_arc;
 #ifdef VIDEO_ENABLED
 	const SalStreamDescription *vstream=sal_media_description_find_stream(call->resultdesc,
 		    					SalProtoRtpAvp,SalVideo);
 #endif
-	
+
 	if(call->audiostream == NULL)
 	{
 		ms_fatal("start_media_stream() called without prior init !");
 		return;
 	}
 	call->current_params = call->params;
-	/* adjust rtp jitter compensation. It must be at least the latency of the sound card */
-	int jitt_comp=MAX(lc->sound_conf.latency,lc->rtp_conf.audio_jitt_comp);
-
 	if (call->media_start_time==0) call->media_start_time=time(NULL);
-
 	cname=linphone_address_as_string_uri_only(me);
-	{
-		const SalStreamDescription *stream=sal_media_description_find_stream(call->resultdesc,
-		    					SalProtoRtpAvp,SalAudio);
-		if (stream && stream->dir!=SalStreamInactive){
-			MSSndCard *playcard=lc->sound_conf.lsd_card ? 
-				lc->sound_conf.lsd_card : lc->sound_conf.play_sndcard;
-			MSSndCard *captcard=lc->sound_conf.capt_sndcard;
-			const char *playfile=lc->play_file;
-			const char *recfile=lc->rec_file;
-			call->audio_profile=make_profile(call,call->resultdesc,stream,&used_pt);
-			bool_t use_ec;
 
-			if (used_pt!=-1){
-				if (playcard==NULL) {
-					ms_warning("No card defined for playback !");
-				}
-				if (captcard==NULL) {
-					ms_warning("No card defined for capture !");
-				}
-				/*Replace soundcard filters by inactive file players or recorders
-				 when placed in recvonly or sendonly mode*/
-				if (stream->port==0 || stream->dir==SalStreamRecvOnly){
-					captcard=NULL;
-					playfile=NULL;
-				}else if (stream->dir==SalStreamSendOnly){
-					playcard=NULL;
-					captcard=NULL;
-					recfile=NULL;
-					/*And we will eventually play "playfile" if set by the user*/
-					/*playfile=NULL;*/
-				}
-				if (send_ringbacktone){
-					captcard=NULL;
-					playfile=NULL;/* it is setup later*/
-				}
-				/*if playfile are supplied don't use soundcards*/
-				if (lc->use_files) {
-					captcard=NULL;
-					playcard=NULL;
-				}
-				use_ec=captcard==NULL ? FALSE : linphone_core_echo_cancellation_enabled(lc);
-#if defined(VIDEO_ENABLED) && defined(ANDROID)
-				/*On android we have to disable the echo canceller to preserve CPU for video codecs */
-				if (vstream && vstream->dir!=SalStreamInactive && vstream->payloads!=NULL)
-					use_ec=FALSE;
-#endif
-				audio_stream_start_full(
-					call->audiostream,
-					call->audio_profile,
-					stream->addr[0]!='\0' ? stream->addr : call->resultdesc->addr,
-					stream->port,
-					stream->port+1,
-					used_pt,
-					jitt_comp,
-					playfile,
-					recfile,
-					playcard,
-					captcard,
-					use_ec
-					);
-				post_configure_audio_streams(call);
-				if (all_inputs_muted && !send_ringbacktone){
-					audio_stream_set_mic_gain(call->audiostream,0);
-				}
-				if (stream->dir==SalStreamSendOnly && playfile!=NULL){
-					int pause_time=500;
-					ms_filter_call_method(call->audiostream->soundread,MS_FILE_PLAYER_LOOP,&pause_time);
-				}
-				if (send_ringbacktone){
-					setup_ring_player(lc,call);
-				}
-				audio_stream_set_rtcp_information(call->audiostream, cname, tool);
-			}else ms_warning("No audio stream accepted ?");
-		}
-	}
-#ifdef VIDEO_ENABLED
-	{
-		
-		used_pt=-1;
-		/* shutdown preview */
-		if (lc->previewstream!=NULL) {
-			video_preview_stop(lc->previewstream);
-			lc->previewstream=NULL;
-		}
-		call->current_params.has_video=FALSE;
-		if (vstream && vstream->dir!=SalStreamInactive) {
-			const char *addr=vstream->addr[0]!='\0' ? vstream->addr : call->resultdesc->addr;
-			call->video_profile=make_profile(call,call->resultdesc,vstream,&used_pt);
-			if (used_pt!=-1){
-				VideoStreamDir dir=VideoStreamSendRecv;
-				MSWebCam *cam=lc->video_conf.device;
-				bool_t is_inactive=FALSE;
-
-				call->current_params.has_video=TRUE;
-				
-				video_stream_set_sent_video_size(call->videostream,linphone_core_get_preferred_video_size(lc));
-				video_stream_enable_self_view(call->videostream,lc->video_conf.selfview);
-				if (lc->video_window_id!=0)
-					video_stream_set_native_window_id(call->videostream,lc->video_window_id);
-				if (lc->preview_window_id!=0)
-					video_stream_set_native_preview_window_id (call->videostream,lc->preview_window_id);
-				video_stream_use_preview_video_window (call->videostream,lc->use_preview_window);
-				
-				if (vstream->dir==SalStreamSendOnly && lc->video_conf.capture ){
-					cam=get_nowebcam_device();
-					dir=VideoStreamSendOnly;
-				}else if (vstream->dir==SalStreamRecvOnly && lc->video_conf.display ){
-					dir=VideoStreamRecvOnly;
-				}else if (vstream->dir==SalStreamSendRecv){
-					if (lc->video_conf.display && lc->video_conf.capture)
-						dir=VideoStreamSendRecv;
-					else if (lc->video_conf.display)
-						dir=VideoStreamRecvOnly;
-					else
-						dir=VideoStreamSendOnly;
-				}else{
-					ms_warning("video stream is inactive.");
-					/*either inactive or incompatible with local capabilities*/
-					is_inactive=TRUE;
-				}
-				if (call->camera_active==FALSE || all_inputs_muted){
-					cam=get_nowebcam_device();
-				}
-				if (!is_inactive){
-					video_stream_set_direction (call->videostream, dir);
-					video_stream_start(call->videostream,
-						call->video_profile, addr, vstream->port,
-						vstream->port+1,
-						used_pt, jitt_comp, cam);
-					video_stream_set_rtcp_information(call->videostream, cname,tool);
-				}
-			}else ms_warning("No video stream accepted.");
-		}else{
-			ms_warning("No valid video stream defined.");
-		}
+#if defined(VIDEO_ENABLED)
+	if (vstream!=NULL && vstream->dir!=SalStreamInactive && vstream->payloads!=NULL){
+		/*when video is used, do not make adaptive rate control on audio, it is stupid.*/
+		use_arc=FALSE;
 	}
 #endif
+	linphone_call_start_audio_stream(call,cname,all_inputs_muted,send_ringbacktone,use_arc);
+	if (call->videostream!=NULL) {
+		linphone_call_start_video_stream(call,cname,all_inputs_muted);
+	}
+
 	call->all_muted=all_inputs_muted;
 	call->playing_ringbacktone=send_ringbacktone;
 	call->up_bw=linphone_core_get_upload_bandwidth(lc);
-	
+
+	if (ortp_zrtp_available()) {
+		OrtpZrtpParams params;
+		params.zid=get_hexa_zrtp_identifier(lc);
+		params.zid_file=lc->zrtp_secrets_cache;
+		audio_stream_enable_zrtp(call->audiostream,&params);
+	}
+
 	goto end;
 	end:
 		ms_free(cname);
@@ -1002,22 +1192,43 @@ void linphone_call_start_media_streams(LinphoneCall *call, bool_t all_inputs_mut
 
 static void linphone_call_log_fill_stats(LinphoneCallLog *log, AudioStream *st){
 	audio_stream_get_local_rtp_stats (st,&log->local_stats);
+	log->quality=audio_stream_get_average_quality_rating(st);
 }
 
 void linphone_call_stop_media_streams(LinphoneCall *call){
 	if (call->audiostream!=NULL) {
+		rtp_session_unregister_event_queue(call->audiostream->session,call->audiostream_app_evq);
+		ortp_ev_queue_flush(call->audiostream_app_evq);
+		ortp_ev_queue_destroy(call->audiostream_app_evq);
+
+		if (call->audiostream->ec){
+			const char *state_str=NULL;
+			ms_filter_call_method(call->audiostream->ec,MS_ECHO_CANCELLER_GET_STATE_STRING,&state_str);
+			if (state_str){
+				ms_message("Writing echo canceller state, %i bytes",(int)strlen(state_str));
+				lp_config_set_string(call->core->config,"sound","ec_state",state_str);
+			}
+		}
 		linphone_call_log_fill_stats (call->log,call->audiostream);
+		if (call->endpoint){
+			linphone_call_remove_from_conf(call);
+		}
 		audio_stream_stop(call->audiostream);
 		call->audiostream=NULL;
 	}
+
+
 #ifdef VIDEO_ENABLED
 	if (call->videostream!=NULL){
+		rtp_session_unregister_event_queue(call->videostream->session,call->videostream_app_evq);
+		ortp_ev_queue_flush(call->videostream_app_evq);
+		ortp_ev_queue_destroy(call->videostream_app_evq);
 		video_stream_stop(call->videostream);
 		call->videostream=NULL;
 	}
+#endif
 	ms_event_queue_skip(call->core->msevq);
 	
-#endif
 	if (call->audio_profile){
 		rtp_profile_clear_all(call->audio_profile);
 		rtp_profile_destroy(call->audio_profile);
@@ -1030,16 +1241,7 @@ void linphone_call_stop_media_streams(LinphoneCall *call){
 	}
 }
 
-#ifdef VIDEO_ENABLED
-/**
- * Request remote side to send us VFU.
-**/
-void linphone_call_send_vfu_request(LinphoneCall *call)
-{
-	if (LinphoneCallStreamsRunning == linphone_call_get_state(call))
-		sal_call_send_vfu_request(call->op);
-}
-#endif
+
 
 void linphone_call_enable_echo_cancellation(LinphoneCall *call, bool_t enable) {
 	if (call!=NULL && call->audiostream!=NULL && call->audiostream->ec){
@@ -1079,33 +1281,203 @@ bool_t linphone_call_echo_limiter_enabled(const LinphoneCall *call){
 	}
 }
 
+/**
+ * @addtogroup call_misc
+ * @{
+**/
+
 /**
  * Returns the measured sound volume played locally (received from remote)
- * It is expressed in dbm0. 
+ * It is expressed in dbm0.
 **/
 float linphone_call_get_play_volume(LinphoneCall *call){
 	AudioStream *st=call->audiostream;
-	if (st && st->volsend){
+	if (st && st->volrecv){
 		float vol=0;
-		ms_filter_call_method(st->volsend,MS_VOLUME_GET,&vol);
+		ms_filter_call_method(st->volrecv,MS_VOLUME_GET,&vol);
 		return vol;
-		
+
 	}
 	return LINPHONE_VOLUME_DB_LOWEST;
 }
 
 /**
  * Returns the measured sound volume recorded locally (sent to remote)
- * It is expressed in dbm0. 
+ * It is expressed in dbm0.
 **/
 float linphone_call_get_record_volume(LinphoneCall *call){
 	AudioStream *st=call->audiostream;
-	if (st && st->volrecv){
+	if (st && st->volsend && !call->audio_muted && call->state==LinphoneCallStreamsRunning){
 		float vol=0;
-		ms_filter_call_method(st->volrecv,MS_VOLUME_GET,&vol);
+		ms_filter_call_method(st->volsend,MS_VOLUME_GET,&vol);
 		return vol;
-		
+
 	}
 	return LINPHONE_VOLUME_DB_LOWEST;
 }
 
+/**
+ * Obtain real-time quality rating of the call
+ *
+ * Based on local RTP statistics and RTCP feedback, a quality rating is computed and updated
+ * during all the duration of the call. This function returns its value at the time of the function call.
+ * It is expected that the rating is updated at least every 5 seconds or so.
+ * The rating is a floating point number comprised between 0 and 5.
+ *
+ * 4-5 = good quality <br>
+ * 3-4 = average quality <br>
+ * 2-3 = poor quality <br>
+ * 1-2 = very poor quality <br>
+ * 0-1 = can't be worse, mostly unusable <br>
+ *
+ * @returns The function returns -1 if no quality measurement is available, for example if no
+ * active audio stream exist. Otherwise it returns the quality rating.
+**/
+float linphone_call_get_current_quality(LinphoneCall *call){
+	if (call->audiostream){
+		return audio_stream_get_quality_rating(call->audiostream);
+	}
+	return -1;
+}
+
+/**
+ * Returns call quality averaged over all the duration of the call.
+ *
+ * See linphone_call_get_current_quality() for more details about quality measurement.
+**/
+float linphone_call_get_average_quality(LinphoneCall *call){
+	if (call->audiostream){
+		return audio_stream_get_average_quality_rating(call->audiostream);
+	}
+	return -1;
+}
+
+/**
+ * @}
+**/
+
+static void display_bandwidth(RtpSession *as, RtpSession *vs){
+	ms_message("bandwidth usage: audio=[d=%.1f,u=%.1f] video=[d=%.1f,u=%.1f] kbit/sec",
+	(as!=NULL) ? (rtp_session_compute_recv_bandwidth(as)*1e-3) : 0,
+	(as!=NULL) ? (rtp_session_compute_send_bandwidth(as)*1e-3) : 0,
+	(vs!=NULL) ? (rtp_session_compute_recv_bandwidth(vs)*1e-3) : 0,
+	(vs!=NULL) ? (rtp_session_compute_send_bandwidth(vs)*1e-3) : 0);
+}
+
+static void linphone_core_disconnected(LinphoneCore *lc, LinphoneCall *call){
+	char temp[256];
+	char *from=NULL;
+	if(call)
+		from = linphone_call_get_remote_address_as_string(call);
+	if (from)
+	{
+		snprintf(temp,sizeof(temp),"Remote end %s seems to have disconnected, the call is going to be closed.",from);
+		free(from);
+	}
+	else
+	{
+		snprintf(temp,sizeof(temp),"Remote end seems to have disconnected, the call is going to be closed.");
+	}
+	if (lc->vtable.display_warning!=NULL)
+		lc->vtable.display_warning(lc,temp);
+	linphone_core_terminate_call(lc,call);
+}
+
+void linphone_call_background_tasks(LinphoneCall *call, bool_t one_second_elapsed){
+	int disconnect_timeout = linphone_core_get_nortp_timeout(call->core);
+	bool_t disconnected=FALSE;
+
+	if (call->state==LinphoneCallStreamsRunning && one_second_elapsed){
+		RtpSession *as=NULL,*vs=NULL;
+		float audio_load=0, video_load=0;
+		if (call->audiostream!=NULL){
+			as=call->audiostream->session;
+			if (call->audiostream->ticker)
+				audio_load=ms_ticker_get_average_load(call->audiostream->ticker);
+		}
+		if (call->videostream!=NULL){
+			if (call->videostream->ticker)
+				video_load=ms_ticker_get_average_load(call->videostream->ticker);
+			vs=call->videostream->session;
+		}
+		display_bandwidth(as,vs);
+		ms_message("Thread processing load: audio=%f\tvideo=%f",audio_load,video_load);
+	}
+#ifdef VIDEO_ENABLED
+	if (call->videostream!=NULL) {
+		// Beware that the application queue should not depend on treatments fron the
+		// mediastreamer queue.
+		video_stream_iterate(call->videostream);
+
+		if (call->videostream_app_evq){
+			OrtpEvent *ev;
+			while (NULL != (ev=ortp_ev_queue_get(call->videostream_app_evq))){
+				OrtpEventType evt=ortp_event_get_type(ev);
+				if (evt == ORTP_EVENT_ZRTP_ENCRYPTION_CHANGED){
+					OrtpEventData *evd=ortp_event_get_data(ev);
+					linphone_call_videostream_encryption_changed(call, evd->info.zrtp_stream_encrypted);
+				}
+				ortp_event_destroy(ev);
+			}
+		}
+	}
+#endif
+	if (call->audiostream!=NULL) {
+		// Beware that the application queue should not depend on treatments fron the
+		// mediastreamer queue.
+		audio_stream_iterate(call->audiostream);
+
+		if (call->audiostream->evq){
+			OrtpEvent *ev;
+			while (NULL != (ev=ortp_ev_queue_get(call->audiostream_app_evq))){
+				OrtpEventType evt=ortp_event_get_type(ev);
+				if (evt == ORTP_EVENT_ZRTP_ENCRYPTION_CHANGED){
+					OrtpEventData *evd=ortp_event_get_data(ev);
+					linphone_call_audiostream_encryption_changed(call, evd->info.zrtp_stream_encrypted);
+				} else if (evt == ORTP_EVENT_ZRTP_SAS_READY) {
+					OrtpEventData *evd=ortp_event_get_data(ev);
+					linphone_call_audiostream_auth_token_ready(call, evd->info.zrtp_sas.sas, evd->info.zrtp_sas.verified);
+				}
+				ortp_event_destroy(ev);
+			}
+		}
+	}
+	if (call->state==LinphoneCallStreamsRunning && one_second_elapsed && call->audiostream!=NULL && disconnect_timeout>0 )
+		disconnected=!audio_stream_alive(call->audiostream,disconnect_timeout);
+	if (disconnected)
+		linphone_core_disconnected(call->core,call);
+}
+
+void linphone_call_log_completed(LinphoneCall *call){
+	LinphoneCore *lc=call->core;
+
+	call->log->duration=time(NULL)-call->start_time;
+
+	if (call->log->status==LinphoneCallMissed){
+		char *info;
+		lc->missed_calls++;
+		info=ortp_strdup_printf(ngettext("You have missed %i call.",
+                                         "You have missed %i calls.", lc->missed_calls),
+                                lc->missed_calls);
+        if (lc->vtable.display_status!=NULL)
+            lc->vtable.display_status(lc,info);
+		ms_free(info);
+	}
+	lc->call_logs=ms_list_prepend(lc->call_logs,(void *)call->log);
+	if (ms_list_size(lc->call_logs)>lc->max_call_logs){
+		MSList *elem,*prevelem=NULL;
+		/*find the last element*/
+		for(elem=lc->call_logs;elem!=NULL;elem=elem->next){
+			prevelem=elem;
+		}
+		elem=prevelem;
+		linphone_call_log_destroy((LinphoneCallLog*)elem->data);
+		lc->call_logs=ms_list_remove_link(lc->call_logs,elem);
+	}
+	if (lc->vtable.call_log_updated!=NULL){
+		lc->vtable.call_log_updated(lc,call->log);
+	}
+	call_logs_write_to_config_file(lc);
+}
+
+
