@@ -68,7 +68,7 @@ typedef struct _LCSipTransports LCSipTransports;
  * The LinphoneAddress has methods to extract and manipulate all parts of the address.
  * When some part of the address (for example the username) is empty, the accessor methods
  * return NULL.
- * 
+ *
  * @ingroup linphone_address
  * @var LinphoneAddress
  */
@@ -128,7 +128,7 @@ typedef enum _LinphoneCallDir LinphoneCallDir;
  * Enum representing the status of a call
  * @ingroup call_logs
 **/
-typedef enum _LinphoneCallStatus { 
+typedef enum _LinphoneCallStatus {
 	LinphoneCallSuccess, /**< The call was sucessful*/
 	LinphoneCallAborted, /**< The call was aborted */
 	LinphoneCallMissed, /**< The call was missed (unanswered)*/
@@ -139,7 +139,7 @@ typedef enum _LinphoneCallStatus {
  * Structure representing a call log.
  *
  * @ingroup call_logs
- * 
+ *
 **/
 typedef struct _LinphoneCallLog{
 	LinphoneCallDir dir; /**< The direction of the call*/
@@ -152,6 +152,7 @@ typedef struct _LinphoneCallLog{
 	void *user_pointer;
 	rtp_stats_t local_stats;
 	rtp_stats_t remote_stats;
+	float quality;
 	struct _LinphoneCore *lc;
 } LinphoneCallLog;
 
@@ -169,7 +170,7 @@ char * linphone_call_log_to_str(LinphoneCallLog *cl);
 
 /**
  * The LinphoneCallParams is an object containing various call related parameters.
- * It can be used to retrieve parameters from a currently running call or modify the call's characteristics 
+ * It can be used to retrieve parameters from a currently running call or modify the call's characteristics
  * dynamically.
 **/
 struct _LinphoneCallParams;
@@ -180,6 +181,7 @@ void linphone_call_params_enable_video(LinphoneCallParams *cp, bool_t enabled);
 bool_t linphone_call_params_video_enabled(const LinphoneCallParams *cp);
 void linphone_call_params_enable_early_media_sending(LinphoneCallParams *cp, bool_t enabled);
 bool_t linphone_call_params_early_media_sending_enabled(const LinphoneCallParams *cp);
+bool_t linphone_call_params_local_conference_mode(const LinphoneCallParams *cp);
 void linphone_call_params_set_audio_bandwidth_limit(LinphoneCallParams *cp, int bw);
 void linphone_call_params_destroy(LinphoneCallParams *cp);
 
@@ -233,14 +235,14 @@ typedef enum _LinphoneCallState{
 
 const char *linphone_call_state_to_string(LinphoneCallState cs);
 
-
+LinphoneCore *linphone_call_get_core(const LinphoneCall *call);
 LinphoneCallState linphone_call_get_state(const LinphoneCall *call);
 bool_t linphone_call_asked_to_autoanswer(LinphoneCall *call);
 const LinphoneAddress * linphone_core_get_current_call_remote_address(struct _LinphoneCore *lc);
 const LinphoneAddress * linphone_call_get_remote_address(const LinphoneCall *call);
 char *linphone_call_get_remote_address_as_string(const LinphoneCall *call);
 LinphoneCallDir linphone_call_get_dir(const LinphoneCall *call);
-void linphone_call_ref(LinphoneCall *call);
+LinphoneCall * linphone_call_ref(LinphoneCall *call);
 void linphone_call_unref(LinphoneCall *call);
 LinphoneCallLog *linphone_call_get_call_log(const LinphoneCall *call);
 const char *linphone_call_get_refer_to(const LinphoneCall *call);
@@ -255,6 +257,8 @@ LinphoneReason linphone_call_get_reason(const LinphoneCall *call);
 const char *linphone_call_get_remote_user_agent(LinphoneCall *call);
 float linphone_call_get_play_volume(LinphoneCall *call);
 float linphone_call_get_record_volume(LinphoneCall *call);
+float linphone_call_get_current_quality(LinphoneCall *call);
+float linphone_call_get_average_quality(LinphoneCall *call);
 void *linphone_call_get_user_pointer(LinphoneCall *call);
 void linphone_call_set_user_pointer(LinphoneCall *call, void *user_pointer);
 /**
@@ -364,6 +368,7 @@ bool_t linphone_proxy_config_publish_enabled(const LinphoneProxyConfig *obj);
 const char *linphone_proxy_config_get_addr(const LinphoneProxyConfig *obj);
 int linphone_proxy_config_get_expires(const LinphoneProxyConfig *obj);
 bool_t linphone_proxy_config_register_enabled(const LinphoneProxyConfig *obj);
+void linphone_proxy_config_refresh_register(LinphoneProxyConfig *obj);
 struct _LinphoneCore * linphone_proxy_config_get_core(const LinphoneProxyConfig *obj);
 
 bool_t linphone_proxy_config_get_dial_escape_plus(const LinphoneProxyConfig *cfg);
@@ -426,15 +431,15 @@ struct _LinphoneAuthInfo;
  * different SIP domains.
  *
  * Once created and filled, a LinphoneAuthInfo must be added to the LinphoneCore in
- * order to become known and used automatically when needed. 
+ * order to become known and used automatically when needed.
  * Use linphone_core_add_auth_info() for that purpose.
  *
  * The LinphoneCore object can take the initiative to request authentication information
  * when needed to the application through the auth_info_requested callback of the
  * LinphoneCoreVTable structure.
  *
- * The application can respond to this information request later using 
- * linphone_core_add_auth_info(). This will unblock all pending authentication 
+ * The application can respond to this information request later using
+ * linphone_core_add_auth_info(). This will unblock all pending authentication
  * transactions and retry them with authentication headers.
  *
 **/
@@ -523,6 +528,9 @@ const char *linphone_global_state_to_string(LinphoneGlobalState gs);
 typedef void (*LinphoneGlobalStateCb)(struct _LinphoneCore *lc, LinphoneGlobalState gstate, const char *message);
 /**Call state notification callback prototype*/
 typedef void (*LinphoneCallStateCb)(struct _LinphoneCore *lc, LinphoneCall *call, LinphoneCallState cstate, const char *message);
+/**Call encryption changed callback prototype*/
+typedef void (*CallEncryptionChangedCb)(struct _LinphoneCore *lc, LinphoneCall *call, bool_t on, const char *authentication_token);
+
 /** @ingroup Proxies
  * Registration state notification callback prototype
  * */
@@ -596,6 +604,7 @@ typedef struct _LinphoneVTable{
 	DisplayMessageCb display_warning;/** Callback to display a warning to the user */
 	DisplayUrlCb display_url;
 	ShowInterfaceCb show; /**< Notifies the application that it should show up*/
+	CallEncryptionChangedCb call_encryption_changed; /**<Notifies on change in the encryption of call streams */
 } LinphoneCoreVTable;
 
 /**
@@ -663,6 +672,8 @@ int linphone_core_accept_call(LinphoneCore *lc, LinphoneCall *call);
 
 int linphone_core_terminate_call(LinphoneCore *lc, LinphoneCall *call);
 
+int linphone_core_redirect_call(LinphoneCore *lc, LinphoneCall *call, const char *redirect_uri);
+
 int linphone_core_terminate_all_calls(LinphoneCore *lc);
 
 int linphone_core_pause_call(LinphoneCore *lc, LinphoneCall *call);
@@ -699,18 +710,25 @@ void linphone_core_set_upload_bandwidth(LinphoneCore *lc, int bw);
 
 int linphone_core_get_download_bandwidth(const LinphoneCore *lc);
 int linphone_core_get_upload_bandwidth(const LinphoneCore *lc);
+
+void linphone_core_enable_adaptive_rate_control(LinphoneCore *lc, bool_t enabled);
+bool_t linphone_core_adaptive_rate_control_enabled(const LinphoneCore *lc);
 /**
- * set audio packetization time linphone expect to received from peer
+ * set audio packetization time linphone expect to receive from peer
  * @ingroup media_parameters
  *
  */
 void linphone_core_set_download_ptime(LinphoneCore *lc, int ptime);
 /**
- * get audio packetization time linphone expect to received from peer, 0 means unspecified
+ * get audio packetization time linphone expect to receive from peer, 0 means unspecified
  * @ingroup media_parameters
  */
 int  linphone_core_get_download_ptime(LinphoneCore *lc);
 
+void linphone_core_set_upload_ptime(LinphoneCore *lc, int ptime);
+
+int linphone_core_get_upload_ptime(LinphoneCore *lc);
+
 /* returns a MSList of PayloadType */
 const MSList *linphone_core_get_audio_codecs(const LinphoneCore *lc);
 
@@ -787,7 +805,12 @@ int linphone_core_get_sip_port(LinphoneCore *lc);
 int linphone_core_set_sip_transports(LinphoneCore *lc, const LCSipTransports *transports);
 
 int linphone_core_get_sip_transports(LinphoneCore *lc, LCSipTransports *transports);
-
+/**
+ * @ingroup IOS
+ * Give access to the UDP sip socket. Can be useful to configure this socket as persistent I.E kCFStreamNetworkServiceType set to kCFStreamNetworkServiceTypeVoIP)
+ * @param lc #LinphoneCore
+ * @return socket file descriptor
+ */
 ortp_socket_t linphone_core_get_sip_socket(LinphoneCore *lc);
 
 void linphone_core_set_inc_timeout(LinphoneCore *lc, int seconds);
@@ -811,7 +834,7 @@ const char * linphone_core_get_relay_addr(const LinphoneCore *lc);
 int linphone_core_set_relay_addr(LinphoneCore *lc, const char *addr);
 
 /* sound functions */
-/* returns a null terminated static array of string describing the sound devices */ 
+/* returns a null terminated static array of string describing the sound devices */
 const char**  linphone_core_get_sound_devices(LinphoneCore *lc);
 bool_t linphone_core_sound_device_can_capture(LinphoneCore *lc, const char *device);
 bool_t linphone_core_sound_device_can_playback(LinphoneCore *lc, const char *device);
@@ -835,6 +858,7 @@ char linphone_core_get_sound_source(LinphoneCore *lc);
 void linphone_core_set_sound_source(LinphoneCore *lc, char source);
 void linphone_core_set_ring(LinphoneCore *lc, const char *path);
 const char *linphone_core_get_ring(const LinphoneCore *lc);
+void linphone_core_set_root_ca(LinphoneCore *lc, const char *path);
 void linphone_core_set_ringback(LinphoneCore *lc, const char *path);
 const char * linphone_core_get_ringback(const LinphoneCore *lc);
 
@@ -891,7 +915,7 @@ void linphone_core_enable_self_view(LinphoneCore *lc, bool_t val);
 bool_t linphone_core_self_view_enabled(const LinphoneCore *lc);
 
 
-/* returns a null terminated static array of string describing the webcams */ 
+/* returns a null terminated static array of string describing the webcams */
 const char**  linphone_core_get_video_devices(const LinphoneCore *lc);
 int linphone_core_set_video_device(LinphoneCore *lc, const char *id);
 const char *linphone_core_get_video_device(const LinphoneCore *lc);
@@ -911,6 +935,7 @@ unsigned long linphone_core_get_native_preview_window_id(const LinphoneCore *lc)
 void linphone_core_set_native_preview_window_id(LinphoneCore *lc, unsigned long id);
 
 void linphone_core_use_preview_window(LinphoneCore *lc, bool_t yesno);
+void linphone_core_set_device_rotation(LinphoneCore *lc, int rotation);
 
 /*play/record support: use files instead of soundcard*/
 void linphone_core_use_files(LinphoneCore *lc, bool_t yesno);
@@ -927,6 +952,7 @@ int linphone_core_get_mtu(const LinphoneCore *lc);
 void linphone_core_set_mtu(LinphoneCore *lc, int mtu);
 
 /**
+ * @ingroup network_parameters
  * This method is called by the application to notify the linphone core library when network is reachable.
  * Calling this method with true trigger linphone to initiate a registration process for all proxy
  * configuration with parameter register set to enable.
@@ -935,15 +961,18 @@ void linphone_core_set_mtu(LinphoneCore *lc, int mtu);
  */
 void linphone_core_set_network_reachable(LinphoneCore* lc,bool_t value);
 /**
+ * @ingroup network_parameters
  * return network state either as positioned by the application or by linphone
  */
 bool_t linphone_core_is_network_reachabled(LinphoneCore* lc);
 
 /**
+ *  @ingroup network_parameters
  *  enable signaling keep alive. small udp packet sent periodically to keep udp NAT association
  */
 void linphone_core_enable_keep_alive(LinphoneCore* lc,bool_t enable);
 /**
+ *  @ingroup network_parameters
  * Is signaling keep alive
  */
 bool_t linphone_core_keep_alive_enabled(LinphoneCore* lc);
@@ -954,14 +983,6 @@ void *linphone_core_get_user_data(LinphoneCore *lc);
 the config file with your own sections */
 struct _LpConfig *linphone_core_get_config(LinphoneCore *lc);
 
-/* attempts to wake up another linphone engine already running.
-The "show" callback is called for the other linphone, causing gui to show up.
-call_addr is an optional sip-uri to call immediately after waking up.
-The method returns 0 if an already running linphone was found*/
-
-int linphone_core_wake_up_possible_already_running_instance(
-    const char * config_file, const char * call_addr);
-
 /*set a callback for some blocking operations, it takes you informed of the progress of the operation*/
 void linphone_core_set_waiting_callback(LinphoneCore *lc, LinphoneWaitingCallback cb, void *user_context);
 
@@ -972,6 +993,8 @@ void linphone_core_destroy(LinphoneCore *lc);
 
 /*for advanced users:*/
 void linphone_core_set_audio_transports(LinphoneCore *lc, RtpTransport *rtp, RtpTransport *rtcp);
+void linphone_core_set_video_transports(LinphoneCore *lc, RtpTransport *rtp, RtpTransport *rtcp);
+
 
 int linphone_core_get_current_call_stats(LinphoneCore *lc, rtp_stats_t *local, rtp_stats_t *remote);
 
@@ -982,11 +1005,37 @@ LinphoneGlobalState linphone_core_get_global_state(const LinphoneCore *lc);
  * force registration refresh to be initiated upon next iterate
  * @ingroup proxies
  */
-void linphone_core_refresh_registers(LinphoneCore* lc);	
+void linphone_core_refresh_registers(LinphoneCore* lc);
+
+
+void linphone_call_send_vfu_request(LinphoneCall *call);
+
+/* Path to the file storing secrets cache */
+void linphone_core_set_zrtp_secrets_file(LinphoneCore *lc, const char* file);
+
+bool_t linphone_call_are_all_streams_encrypted(LinphoneCall *call);
+const char* linphone_call_get_authentication_token(LinphoneCall *call);
+bool_t linphone_call_get_authentication_token_verified(LinphoneCall *call);
+
+const LinphoneCall* linphone_core_find_call_from_uri(LinphoneCore *lc, const char *uri);
+
+int linphone_core_add_to_conference(LinphoneCore *lc, LinphoneCall *call);
+int linphone_core_add_all_to_conference(LinphoneCore *lc);
+int linphone_core_remove_from_conference(LinphoneCore *lc, LinphoneCall *call);
+bool_t linphone_core_is_in_conference(const LinphoneCore *lc);
+int linphone_core_enter_conference(LinphoneCore *lc);
+int linphone_core_leave_conference(LinphoneCore *lc);
+float linphone_core_get_conference_local_input_volume(LinphoneCore *lc);
+
+int linphone_core_terminate_conference(LinphoneCore *lc);
+int linphone_core_get_conference_size(LinphoneCore *lc);
+
+int linphone_core_get_max_calls(LinphoneCore *lc);
+bool_t linphone_core_sound_resources_locked(LinphoneCore *lc);
+
 #ifdef __cplusplus
 }
 #endif
 
-void linphone_call_send_vfu_request(LinphoneCall *call);
 
 #endif
