@@ -18,13 +18,27 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 #include <jni.h>
 #include "linphonecore_utils.h"
+#include <ortp/zrtp.h>
 
+extern "C" {
+#include "mediastreamer2/mediastream.h"
+}
 #include "mediastreamer2/msjava.h"
+#include "private.h"
+#include <cpu-features.h>
 
 #ifdef ANDROID
 #include <android/log.h>
 extern "C" void libmsilbc_init();
+#ifdef HAVE_X264
 extern "C" void libmsx264_init();
+#endif
+#ifdef HAVE_AMR
+extern "C" void libmsamr_init();
+#endif
+#ifdef HAVE_SILK
+extern "C" void libmssilk_init();
+#endif
 #endif /*ANDROID*/
 
 static JavaVM *jvm=0;
@@ -42,6 +56,10 @@ static void linphone_android_log_handler(OrtpLogLevel lev, const char *fmt, va_l
 	}
 	__android_log_vprint(prio, LOG_DOMAIN, fmt, args);
 }
+
+int dumbMethodForAllowingUsageOfCpuFeaturesFromStaticLibMediastream() {
+	return (android_getCpuFamily() == ANDROID_CPU_FAMILY_ARM && (android_getCpuFeatures() & ANDROID_CPU_ARM_FEATURE_NEON) != 0);
+}
 #endif /*ANDROID*/
 
 JNIEXPORT jint JNICALL  JNI_OnLoad(JavaVM *ajvm, void *reserved)
@@ -82,37 +100,47 @@ public:
 		vTable.global_state_changed = globalStateChange;
 		vTable.registration_state_changed = registrationStateChange;
 		vTable.call_state_changed = callStateChange;
+		vTable.call_encryption_changed = callEncryptionChange;
 		vTable.text_received = text_received;
 		vTable.new_subscription_request = new_subscription_request;
 		vTable.notify_presence_recv = notify_presence_recv;
 
-		listernerClass = (jclass)env->NewGlobalRef(env->GetObjectClass( alistener));
+		listenerClass = (jclass)env->NewGlobalRef(env->GetObjectClass( alistener));
+
 		/*displayStatus(LinphoneCore lc,String message);*/
-		displayStatusId = env->GetMethodID(listernerClass,"displayStatus","(Lorg/linphone/core/LinphoneCore;Ljava/lang/String;)V");
+		displayStatusId = env->GetMethodID(listenerClass,"displayStatus","(Lorg/linphone/core/LinphoneCore;Ljava/lang/String;)V");
+
 		/*void generalState(LinphoneCore lc,int state); */
-		globalStateId = env->GetMethodID(listernerClass,"globalState","(Lorg/linphone/core/LinphoneCore;Lorg/linphone/core/LinphoneCore$GlobalState;Ljava/lang/String;)V");
+		globalStateId = env->GetMethodID(listenerClass,"globalState","(Lorg/linphone/core/LinphoneCore;Lorg/linphone/core/LinphoneCore$GlobalState;Ljava/lang/String;)V");
 		globalStateClass = (jclass)env->NewGlobalRef(env->FindClass("org/linphone/core/LinphoneCore$GlobalState"));
 		globalStateFromIntId = env->GetStaticMethodID(globalStateClass,"fromInt","(I)Lorg/linphone/core/LinphoneCore$GlobalState;");
+
 		/*registrationState(LinphoneCore lc, LinphoneProxyConfig cfg, LinphoneCore.RegistrationState cstate, String smessage);*/
-		registrationStateId = env->GetMethodID(listernerClass,"registrationState","(Lorg/linphone/core/LinphoneCore;Lorg/linphone/core/LinphoneProxyConfig;Lorg/linphone/core/LinphoneCore$RegistrationState;Ljava/lang/String;)V");
+		registrationStateId = env->GetMethodID(listenerClass,"registrationState","(Lorg/linphone/core/LinphoneCore;Lorg/linphone/core/LinphoneProxyConfig;Lorg/linphone/core/LinphoneCore$RegistrationState;Ljava/lang/String;)V");
 		registrationStateClass = (jclass)env->NewGlobalRef(env->FindClass("org/linphone/core/LinphoneCore$RegistrationState"));
 		registrationStateFromIntId = env->GetStaticMethodID(registrationStateClass,"fromInt","(I)Lorg/linphone/core/LinphoneCore$RegistrationState;");
+
 		/*callState(LinphoneCore lc, LinphoneCall call, LinphoneCall.State cstate,String message);*/
-		callStateId = env->GetMethodID(listernerClass,"callState","(Lorg/linphone/core/LinphoneCore;Lorg/linphone/core/LinphoneCall;Lorg/linphone/core/LinphoneCall$State;Ljava/lang/String;)V");
+		callStateId = env->GetMethodID(listenerClass,"callState","(Lorg/linphone/core/LinphoneCore;Lorg/linphone/core/LinphoneCall;Lorg/linphone/core/LinphoneCall$State;Ljava/lang/String;)V");
 		callStateClass = (jclass)env->NewGlobalRef(env->FindClass("org/linphone/core/LinphoneCall$State"));
 		callStateFromIntId = env->GetStaticMethodID(callStateClass,"fromInt","(I)Lorg/linphone/core/LinphoneCall$State;");
+
+		/*callEncryption(LinphoneCore lc, LinphoneCall call, boolean encrypted,String auth_token);*/
+		callEncryptionChangedId=env->GetMethodID(listenerClass,"callEncryptionChanged","(Lorg/linphone/core/LinphoneCore;Lorg/linphone/core/LinphoneCall;ZLjava/lang/String;)V");
+
 		/*void ecCalibrationStatus(LinphoneCore.EcCalibratorStatus status, int delay_ms, Object data);*/
-		ecCalibrationStatusId = env->GetMethodID(listernerClass,"ecCalibrationStatus","(Lorg/linphone/core/LinphoneCore;Lorg/linphone/core/LinphoneCore$EcCalibratorStatus;ILjava/lang/Object;)V");
+		ecCalibrationStatusId = env->GetMethodID(listenerClass,"ecCalibrationStatus","(Lorg/linphone/core/LinphoneCore;Lorg/linphone/core/LinphoneCore$EcCalibratorStatus;ILjava/lang/Object;)V");
 		ecCalibratorStatusClass = (jclass)env->NewGlobalRef(env->FindClass("org/linphone/core/LinphoneCore$EcCalibratorStatus"));
 		ecCalibratorStatusFromIntId = env->GetStaticMethodID(ecCalibratorStatusClass,"fromInt","(I)Lorg/linphone/core/LinphoneCore$EcCalibratorStatus;");
+
 		/*void newSubscriptionRequest(LinphoneCore lc, LinphoneFriend lf, String url)*/
-		newSubscriptionRequestId = env->GetMethodID(listernerClass,"newSubscriptionRequest","(Lorg/linphone/core/LinphoneCore;Lorg/linphone/core/LinphoneFriend;Ljava/lang/String;)V");
+		newSubscriptionRequestId = env->GetMethodID(listenerClass,"newSubscriptionRequest","(Lorg/linphone/core/LinphoneCore;Lorg/linphone/core/LinphoneFriend;Ljava/lang/String;)V");
 
 		/*void notifyPresenceReceived(LinphoneCore lc, LinphoneFriend lf);*/
-		notifyPresenceReceivedId = env->GetMethodID(listernerClass,"notifyPresenceReceived","(Lorg/linphone/core/LinphoneCore;Lorg/linphone/core/LinphoneFriend;)V");
+		notifyPresenceReceivedId = env->GetMethodID(listenerClass,"notifyPresenceReceived","(Lorg/linphone/core/LinphoneCore;Lorg/linphone/core/LinphoneFriend;)V");
 
 		/*void textReceived(LinphoneCore lc, LinphoneChatRoom cr,LinphoneAddress from,String message);*/
-		textReceivedId = env->GetMethodID(listernerClass,"textReceived","(Lorg/linphone/core/LinphoneCore;Lorg/linphone/core/LinphoneChatRoom;Lorg/linphone/core/LinphoneAddress;Ljava/lang/String;)V");
+		textReceivedId = env->GetMethodID(listenerClass,"textReceived","(Lorg/linphone/core/LinphoneCore;Lorg/linphone/core/LinphoneChatRoom;Lorg/linphone/core/LinphoneAddress;Ljava/lang/String;)V");
 
 		proxyClass = (jclass)env->NewGlobalRef(env->FindClass("org/linphone/core/LinphoneProxyConfigImpl"));
 		proxyCtrId = env->GetMethodID(proxyClass,"<init>", "(J)V");
@@ -137,7 +165,7 @@ public:
 		env->DeleteGlobalRef(core);
 		env->DeleteGlobalRef(listener);
 		if (userdata) env->DeleteGlobalRef(userdata);
-		env->DeleteGlobalRef(listernerClass);
+		env->DeleteGlobalRef(listenerClass);
 		env->DeleteGlobalRef(globalStateClass);
 		env->DeleteGlobalRef(registrationStateClass);
 		env->DeleteGlobalRef(callStateClass);
@@ -151,7 +179,7 @@ public:
 	jobject listener;
 	jobject userdata;
 
-	jclass listernerClass;
+	jclass listenerClass;
 	jmethodID displayStatusId;
 	jmethodID newSubscriptionRequestId;
 	jmethodID notifyPresenceReceivedId;
@@ -169,6 +197,8 @@ public:
 	jmethodID callStateId;
 	jmethodID callStateFromIntId;
 
+	jmethodID callEncryptionChangedId;
+
 	jclass ecCalibratorStatusClass;
 	jmethodID ecCalibrationStatusId;
 	jmethodID ecCalibratorStatusFromIntId;
@@ -241,9 +271,28 @@ public:
 							,env->CallStaticObjectMethod(lcData->registrationStateClass,lcData->registrationStateFromIntId,(jint)state),
 							message ? env->NewStringUTF(message) : NULL);
 	}
+	jobject getCall(JNIEnv *env , LinphoneCall *call){
+		jobject jobj=0;
+
+		if (call!=NULL){
+			void *up=linphone_call_get_user_pointer(call);
+			
+			if (up==NULL){
+				jobj=env->NewObject(callClass,callCtrId,(jlong)call);
+				jobj=env->NewGlobalRef(jobj);
+				linphone_call_set_user_pointer(call,(void*)jobj);
+				linphone_call_ref(call);
+			}else{
+				jobj=(jobject)up;
+			}
+		}
+		return jobj;
+	}
+
 	static void callStateChange(LinphoneCore *lc, LinphoneCall* call,LinphoneCallState state,const char* message) {
 		JNIEnv *env = 0;
 		jint result = jvm->AttachCurrentThread(&env,NULL);
+		jobject jcall;
 		if (result != 0) {
 			ms_error("cannot attach VM\n");
 			return;
@@ -252,9 +301,28 @@ public:
 		env->CallVoidMethod(lcData->listener
 							,lcData->callStateId
 							,lcData->core
-							,env->NewObject(lcData->callClass,lcData->callCtrId,(jlong)call)
+							,(jcall=lcData->getCall(env,call))
 							,env->CallStaticObjectMethod(lcData->callStateClass,lcData->callStateFromIntId,(jint)state),
 							message ? env->NewStringUTF(message) : NULL);
+		if (state==LinphoneCallReleased){
+			linphone_call_set_user_pointer(call,NULL);
+			env->DeleteGlobalRef(jcall);
+		}
+	}
+	static void callEncryptionChange(LinphoneCore *lc, LinphoneCall* call, bool_t encrypted,const char* authentication_token) {
+		JNIEnv *env = 0;
+		jint result = jvm->AttachCurrentThread(&env,NULL);
+		if (result != 0) {
+			ms_error("cannot attach VM\n");
+			return;
+		}
+		LinphoneCoreData* lcData = (LinphoneCoreData*)linphone_core_get_user_data(lc);
+		env->CallVoidMethod(lcData->listener
+							,lcData->callEncryptionChangedId
+							,lcData->core
+							,lcData->getCall(env,call)
+							,encrypted
+							,authentication_token ? env->NewStringUTF(authentication_token) : NULL);
 	}
 	static void notify_presence_recv (LinphoneCore *lc,  LinphoneFriend *my_friend) {
 		JNIEnv *env = 0;
@@ -335,8 +403,14 @@ extern "C" jlong Java_org_linphone_core_LinphoneCoreImpl_newLinphoneCore(JNIEnv*
 #ifdef HAVE_ILBC
 	libmsilbc_init(); // requires an fpu
 #endif
-#ifdef VIDEO_ENABLED
+#ifdef HAVE_X264
 	libmsx264_init();
+#endif
+#ifdef HAVE_AMR
+	libmsamr_init();
+#endif
+#ifdef HAVE_SILK
+	libmssilk_init();
 #endif
 	jlong nativePtr = (jlong)linphone_core_new(	&ldata->vTable
 			,userConfig
@@ -403,20 +477,22 @@ extern "C" void Java_org_linphone_core_LinphoneCoreImpl_iterate(	JNIEnv*  env
 		,jlong lc) {
 	linphone_core_iterate((LinphoneCore*)lc);
 }
-extern "C" jlong Java_org_linphone_core_LinphoneCoreImpl_invite(	JNIEnv*  env
+extern "C" jobject Java_org_linphone_core_LinphoneCoreImpl_invite(	JNIEnv*  env
 		,jobject  thiz
 		,jlong lc
 		,jstring juri) {
 	const char* uri = env->GetStringUTFChars(juri, NULL);
+	LinphoneCoreData *lcd=(LinphoneCoreData*)linphone_core_get_user_data((LinphoneCore*)lc);
 	LinphoneCall* lCall = linphone_core_invite((LinphoneCore*)lc,uri);
 	env->ReleaseStringUTFChars(juri, uri);
-	return (jlong)lCall;
+	return lcd->getCall(env,lCall);
 }
-extern "C" jlong Java_org_linphone_core_LinphoneCoreImpl_inviteAddress(	JNIEnv*  env
+extern "C" jobject Java_org_linphone_core_LinphoneCoreImpl_inviteAddress(	JNIEnv*  env
 		,jobject  thiz
 		,jlong lc
 		,jlong to) {
-	return (jlong) linphone_core_invite_address((LinphoneCore*)lc,(LinphoneAddress*)to);
+	LinphoneCoreData *lcd=(LinphoneCoreData*)linphone_core_get_user_data((LinphoneCore*)lc);
+	return lcd->getCall(env, linphone_core_invite_address((LinphoneCore*)lc,(LinphoneAddress*)to));
 }
 
 extern "C" void Java_org_linphone_core_LinphoneCoreImpl_terminateCall(	JNIEnv*  env
@@ -555,6 +631,24 @@ extern "C" jlongArray Java_org_linphone_core_LinphoneCoreImpl_listVideoPayloadTy
 	return jCodecs;
 }
 
+extern "C" jlongArray Java_org_linphone_core_LinphoneCoreImpl_listAudioPayloadTypes(JNIEnv*  env
+																			,jobject  thiz
+																			,jlong lc) {
+	const MSList* codecs = linphone_core_get_audio_codecs((LinphoneCore*)lc);
+	int codecsCount = ms_list_size(codecs);
+	jlongArray jCodecs = env->NewLongArray(codecsCount);
+	jlong *jInternalArray = env->GetLongArrayElements(jCodecs, NULL);
+
+	for (int i = 0; i < codecsCount; i++ ) {
+		jInternalArray[i] = (unsigned long) (codecs->data);
+		codecs = codecs->next;
+	}
+
+	env->ReleaseLongArrayElements(jCodecs, jInternalArray, 0);
+
+	return jCodecs;
+}
+
 extern "C" jlong Java_org_linphone_core_LinphoneCoreImpl_enablePayloadType(JNIEnv*  env
 																			,jobject  thiz
 																			,jlong lc
@@ -568,6 +662,12 @@ extern "C" void Java_org_linphone_core_LinphoneCoreImpl_enableEchoCancellation(J
 																			,jboolean enable) {
 	linphone_core_enable_echo_cancellation((LinphoneCore*)lc,enable);
 }
+extern "C" void Java_org_linphone_core_LinphoneCoreImpl_enableEchoLimiter(JNIEnv*  env
+																			,jobject  thiz
+																			,jlong lc
+																			,jboolean enable) {
+	linphone_core_enable_echo_limiter((LinphoneCore*)lc,enable);
+}
 extern "C" jboolean Java_org_linphone_core_LinphoneCoreImpl_isEchoCancellationEnabled(JNIEnv*  env
 																			,jobject  thiz
 																			,jlong lc
@@ -575,11 +675,13 @@ extern "C" jboolean Java_org_linphone_core_LinphoneCoreImpl_isEchoCancellationEn
 	return linphone_core_echo_cancellation_enabled((LinphoneCore*)lc);
 }
 
-extern "C" jlong Java_org_linphone_core_LinphoneCoreImpl_getCurrentCall(JNIEnv*  env
+extern "C" jobject Java_org_linphone_core_LinphoneCoreImpl_getCurrentCall(JNIEnv*  env
 																			,jobject  thiz
 																			,jlong lc
 																			) {
-	return (jlong)linphone_core_get_current_call((LinphoneCore*)lc);
+	LinphoneCoreData *lcdata=(LinphoneCoreData*)linphone_core_get_user_data((LinphoneCore*)lc);
+	
+	return lcdata->getCall(env,linphone_core_get_current_call((LinphoneCore*)lc));
 }
 extern "C" void Java_org_linphone_core_LinphoneCoreImpl_addFriend(JNIEnv*  env
 																			,jobject  thiz
@@ -642,6 +744,14 @@ extern "C" jstring Java_org_linphone_core_LinphoneCoreImpl_getRing(JNIEnv*  env
 		return NULL;
 	}
 }
+extern "C" void Java_org_linphone_core_LinphoneCoreImpl_setRootCA(JNIEnv*  env
+																			,jobject  thiz
+																			,jlong lc
+																			,jstring jpath) {
+	const char* path = jpath?env->GetStringUTFChars(jpath, NULL):NULL;
+	linphone_core_set_root_ca((LinphoneCore*)lc,path);
+	if (path) env->ReleaseStringUTFChars(jpath, path);
+}
 extern "C" void Java_org_linphone_core_LinphoneCoreImpl_enableKeepAlive(JNIEnv*  env
 																,jobject  thiz
 																,jlong lc
@@ -930,16 +1040,11 @@ extern "C" jint Java_org_linphone_core_PayloadTypeImpl_getRate(JNIEnv*  env,jobj
 }
 
 //LinphoneCall
-extern "C" void Java_org_linphone_core_LinphoneCallImpl_ref(JNIEnv*  env
+extern "C" void Java_org_linphone_core_LinphoneCallImpl_finalize(JNIEnv*  env
 																		,jobject  thiz
 																		,jlong ptr) {
-	linphone_call_ref((LinphoneCall*)ptr);
-}
-
-extern "C" void Java_org_linphone_core_LinphoneCallImpl_unref(JNIEnv*  env
-																		,jobject  thiz
-																		,jlong ptr) {
-	linphone_call_unref((LinphoneCall*)ptr);
+	LinphoneCall *call=(LinphoneCall*)ptr;
+	linphone_call_unref(call);
 }
 
 extern "C" jlong Java_org_linphone_core_LinphoneCallImpl_getCallLog(	JNIEnv*  env
@@ -989,10 +1094,23 @@ extern "C" jboolean Java_org_linphone_core_LinphoneCallImpl_isEchoLimiterEnabled
 	return linphone_call_echo_limiter_enabled((LinphoneCall*)ptr);
 }
 
-extern "C" jlong Java_org_linphone_core_LinphoneCallImpl_getReplacedCall(	JNIEnv*  env
+extern "C" jobject Java_org_linphone_core_LinphoneCallImpl_getReplacedCall(	JNIEnv*  env
 																		,jobject  thiz
 																		,jlong ptr) {
-	return (jlong)linphone_call_get_replaced_call((LinphoneCall*)ptr);
+	LinphoneCoreData *lcd=(LinphoneCoreData*)linphone_core_get_user_data(linphone_call_get_core((LinphoneCall*)ptr));	
+	return lcd->getCall(env,linphone_call_get_replaced_call((LinphoneCall*)ptr));
+}
+
+extern "C" jfloat Java_org_linphone_core_LinphoneCallImpl_getCurrentQuality(	JNIEnv*  env
+																		,jobject  thiz
+																		,jlong ptr) {
+	return (jfloat)linphone_call_get_current_quality((LinphoneCall*)ptr);
+}
+
+extern "C" jfloat Java_org_linphone_core_LinphoneCallImpl_getAverageQuality(	JNIEnv*  env
+																		,jobject  thiz
+																		,jlong ptr) {
+	return (jfloat)linphone_call_get_average_quality((LinphoneCall*)ptr);
 }
 
 
@@ -1079,9 +1197,37 @@ extern "C" void Java_org_linphone_core_LinphoneCoreImpl_setVideoWindowId(JNIEnv*
 																		,jobject thiz
 																		,jlong lc
 																		,jobject obj) {
+	jobject oldWindow = (jobject) linphone_core_get_native_video_window_id((LinphoneCore*)lc);
+	if (oldWindow != NULL) {
+		env->DeleteGlobalRef(oldWindow);
+	}
+	if (obj != NULL) {
+		obj = env->NewGlobalRef(obj);
+	}
 	linphone_core_set_native_video_window_id((LinphoneCore*)lc,(unsigned long)obj);
 }
 
+extern "C" void Java_org_linphone_core_LinphoneCoreImpl_setPreviewWindowId(JNIEnv* env
+																		,jobject thiz
+																		,jlong lc
+																		,jobject obj) {
+	jobject oldWindow = (jobject) linphone_core_get_native_preview_window_id((LinphoneCore*)lc);
+	if (oldWindow != NULL) {
+		env->DeleteGlobalRef(oldWindow);
+	}
+	if (obj != NULL) {
+		obj = env->NewGlobalRef(obj);
+	}
+	linphone_core_set_native_preview_window_id((LinphoneCore*)lc,(unsigned long)obj);
+}
+
+extern "C" void Java_org_linphone_core_LinphoneCoreImpl_setDeviceRotation(JNIEnv* env
+																		,jobject thiz
+																		,jlong lc
+																		,jint rotation) {
+	linphone_core_set_device_rotation((LinphoneCore*)lc,rotation);
+}
+
 
 extern "C" void Java_org_linphone_core_LinphoneCoreImpl_setFirewallPolicy(JNIEnv *env, jobject thiz, jlong lc, int enum_value){
 	linphone_core_set_firewall_policy((LinphoneCore*)lc,(LinphoneFirewallPolicy)enum_value);
@@ -1116,6 +1262,11 @@ extern "C" void Java_org_linphone_core_LinphoneCallParamsImpl_enableVideo(JNIEnv
 extern "C" jboolean Java_org_linphone_core_LinphoneCallParamsImpl_getVideoEnabled(JNIEnv *env, jobject thiz, jlong lcp){
 	return linphone_call_params_video_enabled((LinphoneCallParams*)lcp);
 }
+
+extern "C" jboolean Java_org_linphone_core_LinphoneCallParamsImpl_localConferenceMode(JNIEnv *env, jobject thiz, jlong lcp){
+	return linphone_call_params_local_conference_mode((LinphoneCallParams*)lcp);
+}
+
 extern "C" void Java_org_linphone_core_LinphoneCallParamsImpl_destroy(JNIEnv *env, jobject thiz, jlong lc){
 	return linphone_call_params_destroy((LinphoneCallParams*)lc);
 }
@@ -1131,8 +1282,13 @@ extern "C" jlong Java_org_linphone_core_LinphoneCallImpl_enableCamera(JNIEnv *en
 	linphone_call_enable_camera((LinphoneCall *)lc, (bool_t) b);
 }
 
-extern "C" jlong Java_org_linphone_core_LinphoneCoreImpl_inviteAddressWithParams(JNIEnv *env, jobject thiz, jlong lc, jlong addr, jlong params){
-	return (jlong) linphone_core_invite_address_with_params((LinphoneCore *)lc, (const LinphoneAddress *)addr, (const LinphoneCallParams *)params);
+extern "C" jboolean Java_org_linphone_core_LinphoneCallImpl_cameraEnabled(JNIEnv *env, jobject thiz, jlong lc){
+	linphone_call_camera_enabled((LinphoneCall *)lc);
+}
+
+extern "C" jobject Java_org_linphone_core_LinphoneCoreImpl_inviteAddressWithParams(JNIEnv *env, jobject thiz, jlong lc, jlong addr, jlong params){
+	LinphoneCoreData *lcd=(LinphoneCoreData*)linphone_core_get_user_data((LinphoneCore*)lc);
+	return  lcd->getCall(env,linphone_core_invite_address_with_params((LinphoneCore *)lc, (const LinphoneAddress *)addr, (const LinphoneCallParams *)params));
 }
 
 extern "C" jint Java_org_linphone_core_LinphoneCoreImpl_updateAddressWithParams(JNIEnv *env, jobject thiz, jlong lc, jlong call, jlong params){
@@ -1166,6 +1322,15 @@ extern "C" void Java_org_linphone_core_LinphoneCoreImpl_setDownloadBandwidth(JNI
 extern "C" void Java_org_linphone_core_LinphoneCoreImpl_setUploadBandwidth(JNIEnv *env, jobject thiz, jlong lc, jint bw){
 	linphone_core_set_upload_bandwidth((LinphoneCore *)lc, (int) bw);
 }
+
+extern "C" void Java_org_linphone_core_LinphoneCoreImpl_setDownloadPtime(JNIEnv *env, jobject thiz, jlong lc, jint ptime){
+	linphone_core_set_download_ptime((LinphoneCore *)lc, (int) ptime);
+}
+
+extern "C" void Java_org_linphone_core_LinphoneCoreImpl_setUploadPtime(JNIEnv *env, jobject thiz, jlong lc, jint ptime){
+	linphone_core_set_upload_ptime((LinphoneCore *)lc, (int) ptime);
+}
+
 extern "C" int Java_org_linphone_core_LinphoneProxyConfigImpl_getState(JNIEnv*  env,jobject thiz,jlong ptr) {
 	return (int) linphone_proxy_config_get_state((const LinphoneProxyConfig *) ptr);
 }
@@ -1176,7 +1341,7 @@ extern "C" void Java_org_linphone_core_LinphoneProxyConfigImpl_setExpires(JNIEnv
 extern "C" jint Java_org_linphone_core_LinphoneCallImpl_getDuration(JNIEnv*  env,jobject thiz,jlong ptr) {
 	linphone_call_get_duration((LinphoneCall *) ptr);
 }
-	
+
 extern "C" jint Java_org_linphone_core_LinphoneCoreImpl_getSignalingTransportPort(JNIEnv* env,jobject thiz,jlong ptr, jint code) {
 	LCSipTransports tr;
 	linphone_core_get_sip_transports((LinphoneCore *) ptr, &tr);
@@ -1198,6 +1363,163 @@ extern "C" void Java_org_linphone_core_LinphoneCoreImpl_setSignalingTransportPor
 	tr.udp_port = udp;
 	tr.tcp_port = tcp;
 	tr.tls_port = tls;
-	
+
 	linphone_core_set_sip_transports(lc, &tr); // tr will be copied
-}
\ No newline at end of file
+}
+
+extern "C" void Java_org_linphone_core_LinphoneCoreImpl_enableIpv6(JNIEnv* env,jobject  thiz
+              ,jlong lc, jboolean enable) {
+              linphone_core_enable_ipv6((LinphoneCore*)lc,enable);
+}
+
+extern "C" void Java_org_linphone_core_LinphoneCoreImpl_adjustSoftwareVolume(JNIEnv* env,jobject  thiz
+              ,jlong ptr, jint db) {
+	linphone_core_set_playback_gain_db((LinphoneCore *) ptr, db);
+}
+
+extern "C" jint Java_org_linphone_core_LinphoneCoreImpl_pauseCall(JNIEnv *env,jobject thiz,jlong pCore, jlong pCall) {
+	return linphone_core_pause_call((LinphoneCore *) pCore, (LinphoneCall *) pCall);
+}
+extern "C" jint Java_org_linphone_core_LinphoneCoreImpl_pauseAllCalls(JNIEnv *env,jobject thiz,jlong pCore) {
+	return linphone_core_pause_all_calls((LinphoneCore *) pCore);
+}
+extern "C" jint Java_org_linphone_core_LinphoneCoreImpl_resumeCall(JNIEnv *env,jobject thiz,jlong pCore, jlong pCall) {
+	return linphone_core_resume_call((LinphoneCore *) pCore, (LinphoneCall *) pCall);
+}
+extern "C" jboolean Java_org_linphone_core_LinphoneCoreImpl_isInConference(JNIEnv *env,jobject thiz,jlong pCore) {
+	return linphone_core_is_in_conference((LinphoneCore *) pCore);
+}
+extern "C" void Java_org_linphone_core_LinphoneCoreImpl_enterConference(JNIEnv *env,jobject thiz,jlong pCore) {
+	linphone_core_enter_conference((LinphoneCore *) pCore);
+}
+extern "C" void Java_org_linphone_core_LinphoneCoreImpl_leaveConference(JNIEnv *env,jobject thiz,jlong pCore) {
+	linphone_core_leave_conference((LinphoneCore *) pCore);
+}
+extern "C" void Java_org_linphone_core_LinphoneCoreImpl_addAllToConference(JNIEnv *env,jobject thiz,jlong pCore) {
+	linphone_core_add_all_to_conference((LinphoneCore *) pCore);
+}
+extern "C" void Java_org_linphone_core_LinphoneCoreImpl_addToConference(JNIEnv *env,jobject thiz,jlong pCore, jlong pCall) {
+	linphone_core_add_to_conference((LinphoneCore *) pCore, (LinphoneCall *) pCall);
+}
+extern "C" void Java_org_linphone_core_LinphoneCoreImpl_removeFromConference(JNIEnv *env,jobject thiz,jlong pCore, jlong pCall) {
+	linphone_core_remove_from_conference((LinphoneCore *) pCore, (LinphoneCall *) pCall);
+}
+
+extern "C" void Java_org_linphone_core_LinphoneCoreImpl_terminateConference(JNIEnv *env,jobject thiz,jlong pCore) {
+	linphone_core_terminate_conference((LinphoneCore *) pCore);
+}
+extern "C" jint Java_org_linphone_core_LinphoneCoreImpl_getConferenceSize(JNIEnv *env,jobject thiz,jlong pCore) {
+	return linphone_core_get_conference_size((LinphoneCore *) pCore);
+}
+extern "C" void Java_org_linphone_core_LinphoneCoreImpl_terminateAllCalls(JNIEnv *env,jobject thiz,jlong pCore) {
+	linphone_core_terminate_all_calls((LinphoneCore *) pCore);
+}
+extern "C" jobject Java_org_linphone_core_LinphoneCoreImpl_getCall(JNIEnv *env,jobject thiz,jlong pCore,jint position) {
+	LinphoneCoreData *lcd=(LinphoneCoreData*)linphone_core_get_user_data((LinphoneCore*)pCore);
+	LinphoneCall* lCall = (LinphoneCall*) ms_list_nth_data(linphone_core_get_calls((LinphoneCore *) pCore),position);
+	return lcd->getCall(env,lCall);
+}
+extern "C" jint Java_org_linphone_core_LinphoneCoreImpl_getCallsNb(JNIEnv *env,jobject thiz,jlong pCore) {
+	return ms_list_size(linphone_core_get_calls((LinphoneCore *) pCore));
+}
+
+extern "C" void Java_org_linphone_core_LinphoneCoreImpl_transferCall(JNIEnv *env,jobject thiz,jlong pCore, jlong pCall, jstring jReferTo) {
+	const char* cReferTo=env->GetStringUTFChars(jReferTo, NULL);
+	linphone_core_transfer_call((LinphoneCore *) pCore, (LinphoneCall *) pCall, cReferTo);
+	env->ReleaseStringUTFChars(jReferTo, cReferTo);
+}
+extern "C" void Java_org_linphone_core_LinphoneCoreImpl_transferCallToAnother(JNIEnv *env,jobject thiz,jlong pCore, jlong pCall, jlong pDestCall) {
+	linphone_core_transfer_call_to_another((LinphoneCore *) pCore, (LinphoneCall *) pCall, (LinphoneCall *) pDestCall);
+}
+
+extern "C" void Java_org_linphone_core_LinphoneCoreImpl_setZrtpSecretsCache(JNIEnv *env,jobject thiz,jlong pCore, jstring jFile) {
+	if (jFile) {
+		const char* cFile=env->GetStringUTFChars(jFile, NULL);
+		linphone_core_set_zrtp_secrets_file((LinphoneCore *) pCore,cFile);
+		env->ReleaseStringUTFChars(jFile, cFile);
+	} else {
+		linphone_core_set_zrtp_secrets_file((LinphoneCore *) pCore,NULL);
+	}
+}
+
+extern "C" jobject Java_org_linphone_core_LinphoneCoreImpl_findCallFromUri(JNIEnv *env,jobject thiz,jlong pCore, jstring jUri) {
+	const char* cUri=env->GetStringUTFChars(jUri, NULL);
+	LinphoneCall *call= (LinphoneCall *) linphone_core_find_call_from_uri((LinphoneCore *) pCore,cUri);
+	env->ReleaseStringUTFChars(jUri, cUri);
+	LinphoneCoreData *lcdata=(LinphoneCoreData*)linphone_core_get_user_data((LinphoneCore*)pCore);
+	return (jobject) lcdata->getCall(env,call);
+}
+
+
+extern "C" jint Java_org_linphone_core_LinphoneCoreImpl_setVideoDevice(JNIEnv *env,jobject thiz,jlong pCore,jint id) {
+	LinphoneCore* lc = (LinphoneCore *) pCore;
+	const char** devices = linphone_core_get_video_devices(lc);
+	if (devices == NULL) {
+		ms_error("No existing video devices\n");
+		return -1;
+	}
+	int i;
+	for(i=0; i<=id; i++) {
+		if (devices[i] == NULL)
+			break;
+		ms_message("Existing device %d : %s\n", i, devices[i]);
+		if (i==id) {
+			return linphone_core_set_video_device(lc, devices[i]);
+		}
+	}
+	return -1;
+}
+
+extern "C" jint Java_org_linphone_core_LinphoneCoreImpl_getVideoDevice(JNIEnv *env,jobject thiz,jlong pCore) {
+	LinphoneCore* lc = (LinphoneCore *) pCore;
+	const char** devices = linphone_core_get_video_devices(lc);
+	if (devices == NULL) {
+		ms_error("No existing video devices\n");
+		return -1;
+	}
+	const char* cam = linphone_core_get_video_device(lc);
+	if (cam == NULL) {
+		ms_error("No current video device\n");
+	} else {
+		int i=0;
+		while(devices[i] != NULL) {
+			if (strcmp(devices[i], cam) == 0)
+				return i;
+			i++;
+		}
+	}
+	ms_warning("Returning default (0) device\n");
+	return 0;
+}
+
+extern "C" jstring Java_org_linphone_core_LinphoneCallImpl_getAuthenticationToken(JNIEnv*  env,jobject thiz,jlong ptr) {
+	LinphoneCall *call = (LinphoneCall *) ptr;
+	const char* token = linphone_call_get_authentication_token(call);
+	if (token == NULL) return NULL;
+	return env->NewStringUTF(token);
+}
+extern "C" jboolean Java_org_linphone_core_LinphoneCallImpl_isAuthenticationTokenVerified(JNIEnv*  env,jobject thiz,jlong ptr) {
+	LinphoneCall *call = (LinphoneCall *) ptr;
+	return linphone_call_get_authentication_token_verified(call);
+}
+extern "C" jboolean Java_org_linphone_core_LinphoneCallImpl_areStreamsEncrypted(JNIEnv*  env,jobject thiz,jlong ptr) {
+	return linphone_call_are_all_streams_encrypted((LinphoneCall *) ptr);
+}
+
+extern "C" jboolean Java_org_linphone_core_LinphoneCoreImpl_soundResourcesLocked(JNIEnv* env,jobject thiz,jlong ptr){
+	return linphone_core_sound_resources_locked((LinphoneCore *) ptr);
+}
+
+// Needed by Galaxy S (can't switch to/from speaker while playing and still keep mic working)
+// Implemented directly in msandroid.cpp (sound filters for Android).
+extern "C" void msandroid_hack_speaker_state(bool speakerOn);
+
+extern "C" void Java_org_linphone_LinphoneManager_hackSpeakerState(JNIEnv*  env,jobject thiz,jboolean speakerOn){
+	msandroid_hack_speaker_state(speakerOn);
+}
+// End Galaxy S hack functions
+
+
+extern "C" jint Java_org_linphone_core_LinphoneCoreImpl_getMaxCalls(JNIEnv *env,jobject thiz,jlong pCore) {
+	return (jint) linphone_core_get_max_calls((LinphoneCore *) pCore);
+}
