@@ -205,7 +205,11 @@ int linphone_core_enable_payload_type(LinphoneCore *lc, PayloadType *pt, bool_t
 const char *linphone_core_get_payload_type_description(LinphoneCore *lc, PayloadType *pt){
 	if (ms_filter_codec_supported(pt->mime_type)){
 		MSFilterDesc *desc=ms_filter_get_encoder(pt->mime_type);
-		return _(desc->text);
+#ifdef ENABLE_NLS
+		return dgettext("mediastreamer",desc->text);
+#else
+		return desc->text;
+#endif
 	}
 	return NULL;
 }
@@ -396,7 +400,7 @@ static int sendStunRequest(int sock, const struct sockaddr *server, socklen_t ad
 	return 0;
 }
 
-static int parse_stun_server_addr(const char *server, struct sockaddr_storage *ss, socklen_t *socklen){
+int parse_hostname_to_addr(const char *server, struct sockaddr_storage *ss, socklen_t *socklen){
 	struct addrinfo hints,*res=NULL;
 	int ret;
 	const char *port;
@@ -466,7 +470,7 @@ void linphone_core_run_stun_tests(LinphoneCore *lc, LinphoneCall *call){
 		ac=&call->localdesc->streams[0].candidates[0];
 		vc=&call->localdesc->streams[1].candidates[0];
 		
-		if (parse_stun_server_addr(server,&ss,&ss_len)<0){
+		if (parse_hostname_to_addr(server,&ss,&ss_len)<0){
 			ms_error("Fail to parser stun server address: %s",server);
 			return;
 		}
@@ -553,89 +557,6 @@ void linphone_core_run_stun_tests(LinphoneCore *lc, LinphoneCall *call){
 	}
 }
 
-static int extract_sip_port(const char *config){
-	char line[512];
-	char port[12];
-	int ret=-1;
-	FILE *f=fopen(config,"r");
-	if (f){
-		while(fgets(line,sizeof(line),f)!=NULL){
-			if (fmtp_get_value(line,"sip_port",port,sizeof(port))){
-				ret=atoi(port);
-			}
-		}
-		fclose(f);
-	}
-	return ret;
-}
-
-int linphone_core_wake_up_possible_already_running_instance(
-    const char * config_file, const char * addr_to_call)
-{
-	int port=extract_sip_port(config_file);
-	const char *wakeup="WAKEUP sip:127.0.0.1 SIP/2.0\r\n"
-		"Via: SIP/2.0/UDP 127.0.0.1:%i;rport;branch=z9hG4bK%u\r\n"
-		"From: <sip:another_linphone@127.0.0.1>;tag=%u\r\n"
-		"To:   <sip:you@127.0.0.1>\r\n"
-		"CSeq: 1 WAKEUP\r\n"
-		"Call-ID: %u@onsantape\r\n"
-		"Content-length: 0\r\n\r\n";
-	const char * call = "REFER sip:127.0.0.1 SIP/2.0\r\n"
-		"Via: SIP/2.0/UDP 127.0.0.1:%i;rport;branch=z9hG4bK%u\r\n"
-		"From: <sip:another_linphone@127.0.0.1>;tag=%u\r\n"
-		"To:   <sip:you@127.0.0.1>\r\n"
-		"Refer-To: %s\r\n"
-		"CSeq: 1 WAKEUP\r\n"
-		"Call-ID: %u@onsantape\r\n"
-		"Content-length: 0\r\n\r\n";
-
-	/*make sure ortp is initialized (it initializes win32 socket api)*/
-	ortp_init();
-	if (port>0){
-		struct sockaddr_storage ss;
-		socklen_t sslen;
-		char tmp[100];
-		snprintf(tmp,sizeof(tmp),"127.0.0.1:%i",port);
-		if (parse_stun_server_addr(tmp,&ss,&sslen)==0){
-			int locport=57123;
-			ortp_socket_t sock=create_socket(locport);
-			if (sock<0) sock=create_socket(++locport);
-			if (sock>=0){
-				char req[512];
-				if (addr_to_call != NULL)
-					snprintf(req, sizeof(req), call, locport,
-			     		random(), random(), addr_to_call, random());
-				else
-					snprintf(req, sizeof(req), wakeup, locport,
-			     		random(), random(), random());
-				if (connect(sock,(struct sockaddr*)&ss,sslen)<0){
-					fprintf(stderr,"connect failed: %s\n",getSocketError());
-				}else if (send(sock,req,strlen(req),0)>0){
-					/*wait a bit for a response*/
-					int i;
-					for(i=0;i<10;++i){
-						if (recv(sock,req,sizeof(req),0)>0){
-							close_socket(sock);
-							return 0;
-						}else if (getSocketErrorCode()!=EWOULDBLOCK){
-							break;
-						}
-#ifdef WIN32
-						Sleep(100);
-#else
-						usleep(100000);
-#endif
-					}
-				}else{
-					ms_message("sendto() of WAKEUP request failed, nobody to wakeup.");
-				}
-			}
-			close_socket(sock);
-		}
-	}
-	return -1;
-}
-
 #ifdef HAVE_GETIFADDRS
 
 #include <ifaddrs.h>
