@@ -21,7 +21,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
- 
+
 #ifndef _PRIVATE_H
 #define _PRIVATE_H
 
@@ -33,12 +33,13 @@
 #include "config.h"
 #endif
 #include "mediastreamer2/mediastream.h"
+#include "mediastreamer2/msconference.h"
 
-#ifndef LIBLINPHONE_VERSION 
+#ifndef LIBLINPHONE_VERSION
 #define LIBLINPHONE_VERSION LINPHONE_VERSION
 #endif
 
-#ifndef PACKAGE_SOUND_DIR 
+#ifndef PACKAGE_SOUND_DIR
 #define PACKAGE_SOUND_DIR "."
 #endif
 
@@ -62,7 +63,8 @@ struct _LinphoneCallParams{
 	int audio_bw; /* bandwidth limit for audio stream */
 	bool_t has_video;
 	bool_t real_early_media; /*send real media even during early media (for outgoing calls)*/
-	bool_t pad[2];
+	bool_t in_conference; /*in conference mode */
+	bool_t pad;
 };
 
 struct _LinphoneCall
@@ -87,6 +89,7 @@ struct _LinphoneCall
 	int video_port;
 	struct _AudioStream *audiostream;  /**/
 	struct _VideoStream *videostream;
+	MSAudioEndpoint *endpoint; /*used for conferencing*/
 	char *refer_to;
 	LinphoneCallParams params;
 	LinphoneCallParams current_params;
@@ -99,6 +102,13 @@ struct _LinphoneCall
 	bool_t all_muted; /*this flag is set during early medias*/
 	bool_t playing_ringbacktone;
 	bool_t owns_call_log;
+	bool_t ringing_beep; /* whether this call is ringing through an already existent current call*/
+	OrtpEvQueue *audiostream_app_evq;
+	char *auth_token;
+	OrtpEvQueue *videostream_app_evq;
+	bool_t videostream_encrypted;
+	bool_t audiostream_encrypted;
+	bool_t auth_token_verified;
 };
 
 
@@ -108,7 +118,7 @@ void linphone_call_set_state(LinphoneCall *call, LinphoneCallState cstate, const
 
 /* private: */
 LinphoneCallLog * linphone_call_log_new(LinphoneCall *call, LinphoneAddress *local, LinphoneAddress * remote);
-void linphone_call_log_completed(LinphoneCallLog *calllog, LinphoneCall *call, LinphoneCallStatus status);
+void linphone_call_log_completed(LinphoneCall *call);
 void linphone_call_log_destroy(LinphoneCallLog *cl);
 
 void linphone_auth_info_write_config(struct _LpConfig *config, LinphoneAuthInfo *obj, int pos);
@@ -126,7 +136,7 @@ void linphone_friend_notify(LinphoneFriend *lf, LinphoneOnlineStatus os);
 LinphoneFriend *linphone_find_friend_by_inc_subscribe(MSList *l, SalOp *op);
 LinphoneFriend *linphone_find_friend_by_out_subscribe(MSList *l, SalOp *op);
 
-
+int parse_hostname_to_addr(const char *server, struct sockaddr_storage *ss, socklen_t *socklen);
 int set_lock_file();
 int get_lock_file();
 int remove_lock_file();
@@ -200,6 +210,8 @@ int linphone_proxy_config_normalize_number(LinphoneProxyConfig *cfg, const char
 
 void linphone_core_text_received(LinphoneCore *lc, const char *from, const char *msg);
 
+void linphone_core_play_tone(LinphoneCore *lc);
+
 void linphone_call_init_media_streams(LinphoneCall *call);
 void linphone_call_start_media_streams(LinphoneCall *call, bool_t all_inputs_muted, bool_t send_ringbacktone);
 void linphone_call_stop_media_streams(LinphoneCall *call);
@@ -214,6 +226,8 @@ int linphone_core_start_invite(LinphoneCore *lc, LinphoneCall *call, LinphonePro
 void linphone_core_start_refered_call(LinphoneCore *lc, LinphoneCall *call);
 extern SalCallbacks linphone_sal_callbacks;
 void linphone_proxy_config_set_error(LinphoneProxyConfig *cfg, LinphoneReason error);
+bool_t linphone_core_rtcp_enabled(const LinphoneCore *lc);
+
 
 struct _LinphoneProxyConfig
 {
@@ -241,7 +255,7 @@ struct _LinphoneProxyConfig
 	LinphoneReason error;
 };
 
-struct _LinphoneAuthInfo 
+struct _LinphoneAuthInfo
 {
 	char *username;
 	char *realm;
@@ -249,6 +263,7 @@ struct _LinphoneAuthInfo
 	char *passwd;
 	char *ha1;
 	int usecount;
+	time_t last_use_time;
 	bool_t works;
 };
 
@@ -273,7 +288,7 @@ struct _LinphoneFriend{
 	bool_t subscribe_active;
 	bool_t inc_subscribe_pending;
 	bool_t commit;
-};	
+};
 
 
 typedef struct sip_config
@@ -303,7 +318,7 @@ typedef struct rtp_config
 	int audio_jitt_comp;  /*jitter compensation*/
 	int video_jitt_comp;  /*jitter compensation*/
 	int nortp_timeout;
-	bool_t rtp_no_xmit_on_audio_mute;                              
+	bool_t rtp_no_xmit_on_audio_mute;
                               /* stop rtp xmit when audio muted */
 }rtp_config_t;
 
@@ -311,7 +326,8 @@ typedef struct rtp_config
 
 typedef struct net_config
 {
-	char *nat_address;
+	char *nat_address; /* may be IP or host name */
+	char *nat_address_ip; /* ip translated from nat_address */
 	char *stun_server;
 	char *relay;
 	int download_bw;
@@ -380,6 +396,14 @@ typedef struct autoreplier_config
 	const char *message;		/* the path of the file to be played */
 }autoreplier_config_t;
 
+struct _LinphoneConference{
+	MSAudioConference *conf;
+	AudioStream *local_participant;
+	MSAudioEndpoint *local_endpoint;
+	bool_t local_muted;
+};
+
+typedef struct _LinphoneConference LinphoneConference;
 
 struct _LinphoneCore
 {
@@ -395,6 +419,8 @@ struct _LinphoneCore
 	codecs_config_t codecs_conf;
 	ui_config_t ui_conf;
 	autoreplier_config_t autoreplier_conf;
+	MSList *payload_types;
+	int dyn_pt;
 	LinphoneProxyConfig *default_proxy;
 	MSList *friends;
 	MSList *auth_info;
@@ -411,6 +437,7 @@ struct _LinphoneCore
 	VideoPreview *previewstream;
 	struct _MSEventQueue *msevq;
 	RtpTransport *a_rtp,*a_rtcp;
+	RtpTransport *v_rtp,*v_rtcp;
 	MSList *bl_reqs;
 	MSList *subscribers;	/* unknown subscribers */
 	int minutes_away;
@@ -427,6 +454,9 @@ struct _LinphoneCore
 	unsigned long preview_window_id;
 	time_t netup_time; /*time when network went reachable */
 	struct _EcCalibrator *ecc;
+	MSList *hooks;
+	LinphoneConference conf_ctx;
+	char* zrtp_secrets_cache;
 	bool_t use_files;
 	bool_t apply_nat_settings;
 	bool_t initial_subscribes_sent;
@@ -435,7 +465,9 @@ struct _LinphoneCore
 	bool_t auto_net_state_mon;
 	bool_t network_reachable;
 	bool_t use_preview_window;
+	int device_rotation;
 	bool_t ringstream_autorelease;
+	int max_calls;
 };
 
 bool_t linphone_core_can_we_add_call(LinphoneCore *lc);
@@ -468,6 +500,7 @@ struct _EcCalibrator{
 	int sent_count;
 	int64_t acc;
 	int delay;
+	unsigned int rate;
 	LinphoneEcCalibratorStatus status;
 };
 
@@ -477,11 +510,20 @@ LinphoneEcCalibratorStatus ec_calibrator_get_status(EcCalibrator *ecc);
 
 void ec_calibrator_destroy(EcCalibrator *ecc);
 
+void linphone_call_background_tasks(LinphoneCall *call, bool_t one_second_elapsed);
+void linphone_core_preempt_sound_resources(LinphoneCore *lc);
+/*conferencing subsystem*/
+void _post_configure_audio_stream(AudioStream *st, LinphoneCore *lc, bool_t muted);
+void linphone_call_add_to_conf(LinphoneCall *call);
+void linphone_call_remove_from_conf(LinphoneCall *call);
+void linphone_core_conference_check_uninit(LinphoneConference *ctx);
+bool_t linphone_core_sound_resources_available(LinphoneCore *lc);
+
 #define HOLD_OFF	(0)
 #define HOLD_ON		(1)
 
 #ifndef NB_MAX_CALLS
 #define NB_MAX_CALLS	(10)
 #endif
-
+void call_logs_write_to_config_file(LinphoneCore *lc);
 #endif /* _PRIVATE_H */
