@@ -24,10 +24,18 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "private.h"
 #include "offeranswer.h"
 
+#ifdef ANDROID
+// Necessary to make it linked
+static void for_linker() { eXosip_transport_hook_register(NULL); }
+#endif
 static bool_t call_failure(Sal *sal, eXosip_event_t *ev);
 
 static void text_received(Sal *sal, eXosip_event_t *ev);
 
+static void masquerade_via(osip_message_t *msg, const char *ip, const char *port);
+static bool_t fix_message_contact(SalOp *op, osip_message_t *request,osip_message_t *last_answer);
+static void update_contact_from_response(SalOp *op, osip_message_t *response);
+
 void _osip_list_set_empty(osip_list_t *l, void (*freefunc)(void*)){
 	void *data;
 	while(!osip_list_eol(l,0)) {
@@ -185,6 +193,7 @@ void sal_op_release(SalOp *op){
 		eXosip_event_free(op->pending_auth);
 	if (op->rid!=-1){
 		sal_remove_register(op->base.root,op->rid);
+		eXosip_register_remove(op->rid);
 	}
 	if (op->cid!=-1){
 		ms_message("Cleaning cid %i",op->cid);
@@ -269,11 +278,17 @@ Sal * sal_init(){
 	sal=ms_new0(Sal,1);
 	sal->keepalive_period=30;
 	sal->double_reg=TRUE;
+	sal->use_rports=TRUE;
+	sal->use_101=TRUE;
+	sal->reuse_authorization=FALSE;
+	sal->rootCa = 0;
 	return sal;
 }
 
 void sal_uninit(Sal* sal){
 	eXosip_quit();
+	if (sal->rootCa)
+		ms_free(sal->rootCa);
 	ms_free(sal);
 }
 
@@ -323,8 +338,6 @@ void sal_set_callbacks(Sal *ctx, const SalCallbacks *cbs){
 		ctx->callbacks.subscribe_received=(SalOnSubscribeReceived)unimplemented_stub;
 	if (ctx->callbacks.text_received==NULL)
 		ctx->callbacks.text_received=(SalOnTextReceived)unimplemented_stub;
-	if (ctx->callbacks.internal_message==NULL)
-		ctx->callbacks.internal_message=(SalOnInternalMsg)unimplemented_stub;
 	if (ctx->callbacks.ping_reply==NULL)
 		ctx->callbacks.ping_reply=(SalOnPingReply)unimplemented_stub;
 }
@@ -345,14 +358,22 @@ int sal_listen_port(Sal *ctx, const char *addr, int port, SalTransport tr, int i
 	int keepalive = ctx->keepalive_period;
 	
 	switch (tr) {
-	case SalTransportDatagram:
+	case SalTransportUDP:
 		proto=IPPROTO_UDP;
 		eXosip_set_option (EXOSIP_OPT_UDP_KEEP_ALIVE, &keepalive);	
 		break;
-	case SalTransportStream:
+	case SalTransportTCP:
+	case SalTransportTLS:
 		proto= IPPROTO_TCP;
 			keepalive=-1;	
-		eXosip_set_option (EXOSIP_OPT_UDP_KEEP_ALIVE,&keepalive);	
+		eXosip_set_option (EXOSIP_OPT_UDP_KEEP_ALIVE,&keepalive);
+
+		if (ctx->rootCa) {
+			eXosip_tls_ctx_t tlsCtx;
+			memset(&tlsCtx, 0, sizeof(tlsCtx));
+			snprintf(tlsCtx.root_ca_cert, sizeof(tlsCtx.client.cert), "%s", ctx->rootCa);
+			eXosip_set_tls_ctx(&tlsCtx);
+		}
 		break;
 	default:
 		ms_warning("unexpected proto, using datagram");
@@ -362,14 +383,19 @@ int sal_listen_port(Sal *ctx, const char *addr, int port, SalTransport tr, int i
 	eXosip_set_option(13,&err); /*13=EXOSIP_OPT_SRV_WITH_NAPTR, as it is an enum value, we can't use it unless we are sure of the
 					version of eXosip, which is not the case*/
 	/*see if it looks like an IPv6 address*/
+	int use_rports = ctx->use_rports; // Copy char to int to avoid bad alignment
+	eXosip_set_option(EXOSIP_OPT_USE_RPORT,&use_rports);
+	int dont_use_101 = !ctx->use_101; // Copy char to int to avoid bad alignment
+	eXosip_set_option(EXOSIP_OPT_DONT_SEND_101,&dont_use_101);
+
 	ipv6=strchr(addr,':')!=NULL;
 	eXosip_enable_ipv6(ipv6);
 
-	if (is_secure){
-		ms_fatal("SIP over TLS or DTLS is not supported yet.");
+	if (is_secure && tr == SalTransportUDP){
+		ms_fatal("SIP over DTLS is not supported yet.");
 		return -1;
 	}
-	err=eXosip_listen_addr(proto, addr, port, ipv6 ?  PF_INET6 : PF_INET, 0);
+	err=eXosip_listen_addr(proto, addr, port, ipv6 ?  PF_INET6 : PF_INET, is_secure);
 #ifdef HAVE_EXOSIP_GET_SOCKET
 	ms_message("Exosip has socket number %i",eXosip_get_socket(proto));
 #endif
@@ -407,7 +433,20 @@ void sal_use_double_registrations(Sal *ctx, bool_t enabled){
 	ctx->double_reg=enabled;
 }
 
-static int extract_received_rport(osip_message_t *msg, const char **received, int *rportval){
+void sal_use_rport(Sal *ctx, bool_t use_rports){
+	ctx->use_rports=use_rports;
+}
+void sal_use_101(Sal *ctx, bool_t use_101){
+	ctx->use_101=use_101;
+}
+
+void sal_root_ca(Sal* ctx, const char* rootCa) {
+	if (ctx->rootCa)
+		ms_free(ctx->rootCa);
+	ctx->rootCa = ms_strdup(rootCa);
+}
+
+static int extract_received_rport(osip_message_t *msg, const char **received, int *rportval,SalTransport* transport){
 	osip_via_t *via=NULL;
 	osip_generic_param_t *param=NULL;
 	const char *rport=NULL;
@@ -417,10 +456,7 @@ static int extract_received_rport(osip_message_t *msg, const char **received, in
 	osip_message_get_via(msg,0,&via);
 	if (!via) return -1;
 
-	/* it is useless to do that with tcp since client socket might have a different port
-		than the server socket.
-	*/
-	if (strcasecmp(via->protocol,"tcp")==0) return -1;
+	*transport = sal_transport_parse(via->protocol);
 	
 	if (via->port && via->port[0]!='\0')
 		*rportval=atoi(via->port);
@@ -429,7 +465,6 @@ static int extract_received_rport(osip_message_t *msg, const char **received, in
 	if (param) {
 		rport=param->gvalue;
 		if (rport && rport[0]!='\0') *rportval=atoi(rport);
-		else *rportval=5060;
 		*received=via->host;
 	}
 	param=NULL;
@@ -491,6 +526,10 @@ static void sdp_process(SalOp *h){
 	
 }
 
+int sal_call_is_offerer(const SalOp *h){
+	return h->sdp_offering;
+}
+
 int sal_call_set_local_media_description(SalOp *h, SalMediaDescription *desc){
 	if (desc)
 		sal_media_description_ref(desc);
@@ -502,13 +541,16 @@ int sal_call_set_local_media_description(SalOp *h, SalMediaDescription *desc){
 
 int sal_call(SalOp *h, const char *from, const char *to){
 	int err;
+	const char *route;
 	osip_message_t *invite=NULL;
 	sal_op_set_from(h,from);
 	sal_op_set_to(h,to);
 	sal_exosip_fix_route(h);
-	err=eXosip_call_build_initial_invite(&invite,to,from,sal_op_get_route(h),"Phone call");
+	route = sal_op_get_route(h);
+	err=eXosip_call_build_initial_invite(&invite,to,from,route,"Phone call");
 	if (err!=0){
-		ms_error("Could not create call.");
+		ms_error("Could not create call. Error %d (from=%s to=%s route=%s)",
+				err, from, to, route);
 		return -1;
 	}
 	osip_message_set_allow(invite, "INVITE, ACK, CANCEL, OPTIONS, BYE, REFER, NOTIFY, MESSAGE, SUBSCRIBE, INFO");
@@ -535,7 +577,7 @@ int sal_call(SalOp *h, const char *from, const char *to){
 	eXosip_unlock();
 	h->cid=err;
 	if (err<0){
-		ms_error("Fail to send invite !");
+		ms_error("Fail to send invite ! Error code %d", err);
 		return -1;
 	}else{
 		sal_add_call(h->base.root,h);
@@ -545,19 +587,18 @@ int sal_call(SalOp *h, const char *from, const char *to){
 
 int sal_call_notify_ringing(SalOp *h, bool_t early_media){
 	osip_message_t *msg;
-	int err;
 	
 	/*if early media send also 180 and 183 */
 	if (early_media && h->sdp_answer){
 		msg=NULL;
 		eXosip_lock();
-		err=eXosip_call_build_answer(h->tid,180,&msg);
+		eXosip_call_build_answer(h->tid,180,&msg);
 		if (msg){
 			set_sdp(msg,h->sdp_answer);
 			eXosip_call_send_answer(h->tid,180,msg);
 		}
 		msg=NULL;
-		err=eXosip_call_build_answer(h->tid,183,&msg);
+		eXosip_call_build_answer(h->tid,183,&msg);
 		if (msg){
 			set_sdp(msg,h->sdp_answer);
 			eXosip_call_send_answer(h->tid,183,msg);
@@ -723,7 +764,7 @@ int sal_call_refer_with_replaces(SalOp *h, SalOp *other_call_h){
 }
 
 SalOp *sal_call_get_replaces(SalOp *h){
-	if (h->replaces!=NULL){
+	if (h!=NULL && h->replaces!=NULL){
 		int cid;
 		eXosip_lock();
 		cid=eXosip_call_find_by_replaces(h->replaces);
@@ -772,11 +813,12 @@ static void pop_auth_from_exosip() {
 
 int sal_call_terminate(SalOp *h){
 	int err;
+	if (h == NULL) return -1;
 	if (h->auth_info) push_auth_to_exosip(h->auth_info);
 	eXosip_lock();
 	err=eXosip_call_terminate(h->cid,h->did);
 	eXosip_unlock();
-	pop_auth_from_exosip();
+	if (!h->base.root->reuse_authorization) pop_auth_from_exosip();
 	if (err!=0){
 		ms_warning("Exosip could not terminate the call: cid=%i did=%i", h->cid,h->did);
 	}
@@ -785,16 +827,22 @@ int sal_call_terminate(SalOp *h){
 }
 
 void sal_op_authenticate(SalOp *h, const SalAuthInfo *info){
-	if (h->pending_auth){
+    if (h->pending_auth){
 		push_auth_to_exosip(info);
+		
+        /*FIXME exosip does not take into account this update register message*/
+	/*
+        if (fix_message_contact(h, h->pending_auth->request,h->pending_auth->response)) {
+            
+        };
+	*/
+		update_contact_from_response(h,h->pending_auth->response);
 		eXosip_lock();
 		eXosip_default_action(h->pending_auth);
 		eXosip_unlock();
 		ms_message("eXosip_default_action() done");
-		pop_auth_from_exosip();
-		eXosip_event_free(h->pending_auth);
-		sal_remove_pending_auth(sal_op_get_sal(h),h);
-		h->pending_auth=NULL;
+		if (!h->base.root->reuse_authorization) pop_auth_from_exosip();
+		
 		if (h->auth_info) sal_auth_info_delete(h->auth_info); /*if already exist*/
 		h->auth_info=sal_auth_info_clone(info); /*store auth info for subsequent request*/
 	}
@@ -812,8 +860,9 @@ void sal_op_cancel_authentication(SalOp *h) {
 static void set_network_origin(SalOp *op, osip_message_t *req){
 	const char *received=NULL;
 	int rport=5060;
-	char origin[64];
-	if (extract_received_rport(req,&received,&rport)!=0){
+	char origin[64]={0};
+    SalTransport transport;
+	if (extract_received_rport(req,&received,&rport,&transport)!=0){
 		osip_via_t *via=NULL;
 		char *tmp;
 		osip_message_get_via(req,0,&via);
@@ -821,7 +870,11 @@ static void set_network_origin(SalOp *op, osip_message_t *req){
 		tmp=osip_via_get_port(via);
 		if (tmp) rport=atoi(tmp);
 	}
-	snprintf(origin,sizeof(origin)-1,"sip:%s:%i",received,rport);
+    if (transport != SalTransportUDP) {
+        snprintf(origin,sizeof(origin)-1,"sip:%s:%i",received,rport);
+    } else {
+       snprintf(origin,sizeof(origin)-1,"sip:%s:%i;transport=%s",received,rport,sal_transport_to_string(transport)); 
+    }
 	__sal_op_set_network_origin(op,origin);
 }
 
@@ -979,7 +1032,8 @@ static void handle_ack(Sal *sal,  eXosip_event_t *ev){
 static void update_contact_from_response(SalOp *op, osip_message_t *response){
 	const char *received;
 	int rport;
-	if (extract_received_rport(response,&received,&rport)==0){
+	SalTransport transport;
+	if (extract_received_rport(response,&received,&rport,&transport)==0){
 		const char *contact=sal_op_get_contact(op);
 		if (!contact){
 			/*no contact given yet, use from instead*/
@@ -990,8 +1044,10 @@ static void update_contact_from_response(SalOp *op, osip_message_t *response){
 			char *tmp;
 			sal_address_set_domain(addr,received);
 			sal_address_set_port_int(addr,rport);
+			if (transport!=SalTransportUDP)
+				sal_address_set_transport(addr,transport);
 			tmp=sal_address_as_string(addr);
-			ms_message("Contact address updated to %s for this dialog",tmp);
+			ms_message("Contact address updated to %s",tmp);
 			sal_op_set_contact(op,tmp);
 			sal_address_destroy(addr);
 			ms_free(tmp);
@@ -1001,13 +1057,12 @@ static void update_contact_from_response(SalOp *op, osip_message_t *response){
 
 static int call_proceeding(Sal *sal, eXosip_event_t *ev){
 	SalOp *op=find_op(sal,ev);
-	
-	if (op==NULL) {
+
+	if (op==NULL || op->terminated==TRUE) {
 		ms_warning("This call has been canceled.");
 		eXosip_lock();
 		eXosip_call_terminate(ev->cid,ev->did);
 		eXosip_unlock();
-		op->terminated=TRUE;
 		return -1;
 	}
 	if (ev->did>0)
@@ -1042,9 +1097,12 @@ static void call_accepted(Sal *sal, eXosip_event_t *ev){
 	SalOp *op=find_op(sal,ev);
 	const char *contact;
 	
-	if (op==NULL){
-		ms_error("A closed call is accepted ?");
-		return;
+	if (op==NULL || op->terminated==TRUE) {
+		ms_warning("This call has been already terminated.");
+		eXosip_lock();
+		eXosip_call_terminate(ev->cid,ev->did);
+		eXosip_unlock();
+		return ;
 	}
 
 	op->did=ev->did;
@@ -1058,6 +1116,13 @@ static void call_accepted(Sal *sal, eXosip_event_t *ev){
 		if (op->base.local_media) sdp_process(op);
 	}
 	eXosip_call_build_ack(ev->did,&msg);
+	if (msg==NULL) {
+		ms_warning("This call has been already terminated.");
+		eXosip_lock();
+		eXosip_call_terminate(ev->cid,ev->did);
+		eXosip_unlock();
+		return ;
+	}
 	contact=sal_op_get_contact(op);
 	if (contact) {
 		_osip_list_set_empty(&msg->contacts,(void (*)(void*))osip_contact_free);
@@ -1163,10 +1228,14 @@ static bool_t process_authentication(Sal *sal, eXosip_event_t *ev){
 		return TRUE;
 	}
 	if (get_auth_data(ev,&realm,&username)==0){
-		if (op->pending_auth!=NULL)
+		if (op->pending_auth!=NULL){
 			eXosip_event_free(op->pending_auth);
-		op->pending_auth=ev;
-		sal_add_pending_auth (sal,op);
+			op->pending_auth=ev;
+		}else{
+			op->pending_auth=ev;
+			sal_add_pending_auth(sal,op);
+		}
+		
 		sal->callbacks.auth_requested(op,realm,username);
 		return FALSE;
 	}
@@ -1181,6 +1250,11 @@ static void authentication_ok(Sal *sal, eXosip_event_t *ev){
 		ms_warning("No operation associated with this authentication_ok!");
 		return ;
 	}
+	if (op->pending_auth){
+		eXosip_event_free(op->pending_auth);
+		sal_remove_pending_auth(sal,op);
+		op->pending_auth=NULL;
+	}
 	if (get_auth_data(ev,&realm,&username)==0){
 		sal->callbacks.auth_success(op,realm,username);
 	}
@@ -1259,6 +1333,7 @@ static bool_t call_failure(Sal *sal, eXosip_event_t *ev){
 				sr=SalReasonUnknown;
 			}else error=SalErrorNoResponse;
 	}
+	op->terminated=TRUE;
 	sal->callbacks.call_failure(op,error,sr,reason,code);
 	if (computedReason != NULL){
 		ms_free(computedReason);
@@ -1540,11 +1615,6 @@ static void other_request(Sal *sal, eXosip_event_t *ev){
 		eXosip_options_build_answer(ev->tid,200,&options);
 		fill_options_answer(options);
 		eXosip_options_send_answer(ev->tid,200,options);
-	}else if (strcmp(ev->request->sip_method,"WAKEUP")==0
-		&& comes_from_local_if(ev->request)) {
-		eXosip_message_send_answer(ev->tid,200,NULL);
-		ms_message("Receiving WAKEUP request !");
-		sal->callbacks.internal_message(sal,"WAKEUP");
 	}else if (strncmp(ev->request->sip_method, "REFER", 5) == 0){
 		ms_message("Receiving REFER request !");
 		if (comes_from_local_if(ev->request)) {
@@ -1576,39 +1646,20 @@ static void masquerade_via(osip_message_t *msg, const char *ip, const char *port
 	}
 }
 
-static bool_t register_again_with_updated_contact(SalOp *op, osip_message_t *orig_request, osip_message_t *last_answer){
-	osip_message_t *msg;
+
+static bool_t fix_message_contact(SalOp *op, osip_message_t *request,osip_message_t *last_answer) {
+	osip_contact_t *ctt=NULL;
 	const char *received;
 	int rport;
-	osip_contact_t *ctt=NULL;
-	char *tmp;
+	SalTransport transport;
 	char port[20];
-	SalAddress *addr;
-	Sal *sal=op->base.root;
 
-	if (sal->double_reg==FALSE) return FALSE;
-	
-	if (extract_received_rport(last_answer,&received,&rport)==-1) return FALSE;
-	osip_message_get_contact(orig_request,0,&ctt);
-	if (strcmp(ctt->url->host,received)==0){
-		/*ip address matches, check ports*/
-		const char *contact_port=ctt->url->port;
-		if (contact_port==NULL || contact_port[0]=='\0')
-			contact_port="5060";
-		if (atoi(contact_port)==rport){
-			ms_message("Register has up to date contact, doing nothing.");
-			return FALSE;
-		}else ms_message("ports do not match, need to update the register (%s <> %i)", contact_port,rport);
-	}
-	eXosip_lock();
-	msg=NULL;
-	eXosip_register_build_register(op->rid,op->expires,&msg);
-	if (msg==NULL){
-		eXosip_unlock();
-		ms_warning("Fail to create a contact updated register.");
+	if (extract_received_rport(last_answer,&received,&rport,&transport)==-1) return FALSE;
+	osip_message_get_contact(request,0,&ctt);
+	if (ctt == NULL) {
+		/*nothing to update*/
 		return FALSE;
 	}
-	osip_message_get_contact(msg,0,&ctt);
 	if (ctt->url->host!=NULL){
 		osip_free(ctt->url->host);
 	}
@@ -1618,19 +1669,69 @@ static bool_t register_again_with_updated_contact(SalOp *op, osip_message_t *ori
 	}
 	snprintf(port,sizeof(port),"%i",rport);
 	ctt->url->port=osip_strdup(port);
-	if (op->masquerade_via) masquerade_via(msg,received,port);
-	eXosip_register_send_register(op->rid,msg);
-	eXosip_unlock();
+	if (op->masquerade_via) masquerade_via(request,received,port);
+
+	if (transport != SalTransportUDP) {
+		sal_address_set_param((SalAddress *)ctt, "transport", sal_transport_to_string(transport)); 
+	}
+	return TRUE;    
+}
+
+static bool_t register_again_with_updated_contact(SalOp *op, osip_message_t *orig_request, osip_message_t *last_answer){
+	osip_contact_t *ctt=NULL;
+	SalAddress* ori_contact_address=NULL;
+	const char *received;
+	int rport;
+	SalTransport transport;
+	char* tmp;
+	osip_message_t *msg=NULL;
+	Sal* sal=op->base.root;
+
+	if (sal->double_reg==FALSE ) return FALSE; 
+
+	if (extract_received_rport(last_answer,&received,&rport,&transport)==-1) return FALSE;
+	osip_message_get_contact(orig_request,0,&ctt);
 	osip_contact_to_str(ctt,&tmp);
-	addr=sal_address_new(tmp);
+	ori_contact_address = sal_address_new((const char*)tmp);
+	
+	/*check if contact is up to date*/
+	if (strcmp(sal_address_get_domain(ori_contact_address),received) ==0 
+	&& sal_address_get_port_int(ori_contact_address) == rport
+	&& sal_address_get_transport(ori_contact_address) == transport) {
+		ms_message("Register has up to date contact, doing nothing.");
+		osip_free(tmp);     
+		return FALSE;
+	} else ms_message("contact do not match, need to update the register (%s with %s:%i;transport=%s)"
+		      ,tmp
+		      ,received
+		      ,rport
+		      ,sal_transport_to_string(transport));
 	osip_free(tmp);
-	sal_address_clean(addr);
-	tmp=sal_address_as_string(addr);
-	sal_op_set_contact(op,tmp);
-	sal_address_destroy(addr);
-	ms_message("Resending new register with updated contact %s",tmp);
-	ms_free(tmp);
-	return TRUE;
+	sal_address_destroy(ori_contact_address);
+
+	if (transport == SalTransportUDP) {
+		eXosip_lock();
+		eXosip_register_build_register(op->rid,op->expires,&msg);
+		if (msg==NULL){
+		    eXosip_unlock();
+		    ms_warning("Fail to create a contact updated register.");
+		    return FALSE;
+		}
+		if (fix_message_contact(op,msg,last_answer)) {
+			eXosip_register_send_register(op->rid,msg);
+			eXosip_unlock();  
+			ms_message("Resending new register with updated contact");
+			return TRUE;
+		} else {
+		    ms_warning("Fail to send updated register.");
+		    eXosip_unlock();
+		    return FALSE;
+		}
+		eXosip_unlock();
+	}
+
+	update_contact_from_response(op,last_answer);
+	return FALSE;
 }
 
 static void registration_success(Sal *sal, eXosip_event_t *ev){
@@ -1672,6 +1773,22 @@ static bool_t registration_failure(Sal *sal, eXosip_event_t *ev){
 		case 407:
 			return process_authentication(sal,ev);
 			break;
+		case 423: /*interval too brief*/
+			{/*retry with greater interval */
+				osip_header_t *h=NULL;
+				osip_message_t *msg=NULL;
+				osip_message_header_get_byname(ev->response,"min-expires",0,&h);
+				if (h && h->hvalue && h->hvalue[0]!='\0'){
+					int val=atoi(h->hvalue);
+					if (val>op->expires)
+						op->expires=val;
+				}else op->expires*=2;
+				eXosip_lock();
+				eXosip_register_build_register(op->rid,op->expires,&msg);
+				eXosip_register_send_register(op->rid,msg);
+				eXosip_unlock();
+			}
+		break;
 		case 606: /*Not acceptable, workaround for proxies that don't like private addresses
 				 in vias, such as ekiga.net 
 				 On the opposite, freephonie.net bugs when via are masqueraded.
@@ -1894,6 +2011,25 @@ int sal_register(SalOp *h, const char *proxy, const char *from, int expires){
 	return 0;
 }
 
+int sal_register_refresh(SalOp *op, int expires){
+	osip_message_t *msg=NULL;
+	const char *contact=sal_op_get_contact(op);
+	
+	if (op->rid==-1){
+		ms_error("Unexistant registration context, not possible to refresh.");
+		return -1;
+	}
+	eXosip_lock();
+	eXosip_register_build_register(op->rid,expires,&msg);
+	if (msg!=NULL){
+		if (contact) register_set_contact(msg,contact);
+		eXosip_register_send_register(op->rid,msg);
+	}else ms_error("Could not build REGISTER refresh message.");
+	eXosip_unlock();
+	return 0;
+}
+
+
 int sal_unregister(SalOp *h){
 	osip_message_t *msg=NULL;
 	eXosip_lock();
@@ -1907,6 +2043,12 @@ int sal_unregister(SalOp *h){
 SalAddress * sal_address_new(const char *uri){
 	osip_from_t *from;
 	osip_from_init(&from);
+
+	// Remove front spaces
+	while (uri[0]==' ') {
+		uri++;
+	}
+		
 	if (osip_from_parse(from,uri)!=0){
 		osip_from_free(from);
 		return NULL;
@@ -2031,8 +2173,16 @@ char *sal_address_as_string_uri_only(const SalAddress *u){
 	osip_free(tmp);
 	return ret;
 }
-void sal_address_add_param(SalAddress *u,const char* name,const char* value) {
-	osip_uri_uparam_add	(((osip_from_t*)u)->url,ms_strdup(name),ms_strdup(value));
+void sal_address_set_param(SalAddress *u,const char* name,const char* value) {
+	osip_uri_param_t *param=NULL;
+    osip_uri_uparam_get_byname(((osip_from_t*)u)->url,(char*)name,&param);
+    if (param == NULL){
+        osip_uri_uparam_add	(((osip_from_t*)u)->url,ms_strdup(name),ms_strdup(value));
+    } else {
+        osip_free(param->gvalue);
+        param->gvalue=osip_strdup(value);
+    }
+    
 }
 
 void sal_address_destroy(SalAddress *u){
@@ -2060,6 +2210,19 @@ int sal_address_get_port_int(const SalAddress *uri) {
 		return 5060;
 	}
 }
+SalTransport sal_address_get_transport(const SalAddress* addr) {
+    const osip_from_t *u=(const osip_from_t*)addr;
+    osip_uri_param_t *transport_param=NULL;
+    osip_uri_uparam_get_byname(u->url,"transport",&transport_param);
+    if (transport_param == NULL){
+        return SalTransportUDP;
+    }  else {
+        return sal_transport_parse(transport_param->gvalue);
+    }
+}
+void sal_address_set_transport(SalAddress* addr,SalTransport transport) {
+    sal_address_set_param(addr, "transport", sal_transport_to_string(transport));
+}
 
 /* sends a reinvite. Local media description may have changed by application since call establishment*/
 int sal_call_update(SalOp *h, const char *subject){
@@ -2087,3 +2250,6 @@ int sal_call_update(SalOp *h, const char *subject){
 	eXosip_unlock();
 	return err;
 }
+void sal_reuse_authorization(Sal *ctx, bool_t value) {
+	ctx->reuse_authorization=value;
+}
