@@ -42,6 +42,10 @@ struct Sal{
 	void *up;
 	bool_t one_matching_codec;
 	bool_t double_reg;
+	bool_t use_rports;
+	bool_t use_101;
+	bool_t reuse_authorization;
+	char* rootCa; /* File _or_ folder containing root CA */
 };
 
 struct SalOp{
