@@ -20,6 +20,19 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "linphone.h"
 
 
+static void fill_renderers(GtkTreeView *v){
+	GtkTreeViewColumn *c;
+	GtkCellRenderer *r=gtk_cell_renderer_pixbuf_new ();
+
+	g_object_set(r,"stock-size",GTK_ICON_SIZE_BUTTON,NULL);
+	c=gtk_tree_view_column_new_with_attributes("icon",r,"stock-id",0,NULL);
+	gtk_tree_view_append_column (v,c);
+
+	r=gtk_cell_renderer_text_new ();
+	c=gtk_tree_view_column_new_with_attributes("sipaddress",r,"markup",1,NULL);
+	gtk_tree_view_append_column (v,c);
+}
+
 void linphone_gtk_call_log_update(GtkWidget *w){
 	GtkTreeView *v=GTK_TREE_VIEW(linphone_gtk_get_widget(w,"logs_view"));
 	GtkListStore *store;
@@ -30,6 +43,9 @@ void linphone_gtk_call_log_update(GtkWidget *w){
 		store=gtk_list_store_new(3,G_TYPE_STRING,G_TYPE_STRING, G_TYPE_POINTER);
 		gtk_tree_view_set_model(v,GTK_TREE_MODEL(store));
 		g_object_unref(G_OBJECT(store));
+		fill_renderers(GTK_TREE_VIEW(linphone_gtk_get_widget(w,"logs_view")));
+		gtk_button_set_image(GTK_BUTTON(linphone_gtk_get_widget(w,"call_back_button")),
+		                     create_pixmap (linphone_gtk_get_ui_config("callback_button","status-green.png")));
 	}
 	gtk_list_store_clear (store);
 
@@ -40,14 +56,20 @@ void linphone_gtk_call_log_update(GtkWidget *w){
 		char *addr= linphone_address_as_string_uri_only (la);
 		const char *display;
 		gchar *logtxt;
+		gchar quality[20];
+		
 		display=linphone_address_get_display_name (la);
 		if (display==NULL){
 			display=linphone_address_get_username (la);
 			if (display==NULL)
 				display=linphone_address_get_domain (la);
 		}
-		logtxt=g_markup_printf_escaped("<big><b>%s</b></big>\t<small><i>%s</i></small>\n"
-		                               "%s\t%i minutes %i seconds",display, addr, cl->start_date,
+		if (cl->quality!=-1){
+			snprintf(quality,sizeof(quality),"%.1f",cl->quality);
+		}
+		logtxt=g_markup_printf_escaped("<big><b>%s</b></big>\t<small><i>%s</i>\t<i>Quality: %s</i></small>\n"
+		                               "%s\t%i minutes %i seconds\t",display, addr, cl->quality!=-1 ? quality : _("n/a"),
+		                               cl->start_date,
 		                               cl->duration/60,cl->duration%60);
 		gtk_list_store_append (store,&iter);
 		gtk_list_store_set (store,&iter,
@@ -82,7 +104,21 @@ static bool_t put_selection_to_uribar(GtkWidget *treeview){
 }
 
 void linphone_gtk_history_row_activated(GtkWidget *treeview){
-	put_selection_to_uribar(treeview);
+	if (put_selection_to_uribar(treeview)){
+		GtkWidget *mw=linphone_gtk_get_main_window();
+		linphone_gtk_start_call(linphone_gtk_get_widget(mw,"start_call"));
+	}
+}
+
+void linphone_gtk_clear_call_logs(GtkWidget *button){
+	linphone_core_clear_call_logs (linphone_gtk_get_core());
+	linphone_gtk_call_log_update(gtk_widget_get_toplevel(button));
+}
+
+void linphone_gtk_call_log_callback(GtkWidget *button){
+	GtkWidget *mw=linphone_gtk_get_main_window();
+	if (put_selection_to_uribar(linphone_gtk_get_widget(mw,"logs_view")))
+			linphone_gtk_start_call(linphone_gtk_get_widget(mw,"start_call"));
 }
 
 void linphone_gtk_call_log_response(GtkWidget *w, guint response_id){
@@ -100,27 +136,15 @@ void linphone_gtk_call_log_response(GtkWidget *w, guint response_id){
 }
 
 
-static void fill_renderers(GtkTreeView *v){
-	GtkTreeViewColumn *c;
-	GtkCellRenderer *r=gtk_cell_renderer_pixbuf_new ();
-
-	g_object_set(r,"stock-size",GTK_ICON_SIZE_BUTTON,NULL);
-	c=gtk_tree_view_column_new_with_attributes("icon",r,"stock-id",0,NULL);
-	gtk_tree_view_append_column (v,c);
-
-	r=gtk_cell_renderer_text_new ();
-	c=gtk_tree_view_column_new_with_attributes("sipaddress",r,"markup",1,NULL);
-	gtk_tree_view_append_column (v,c);
-}
 
 GtkWidget * linphone_gtk_show_call_logs(void){
 	GtkWidget *mw=linphone_gtk_get_main_window();
+
 	GtkWidget *w=(GtkWidget*)g_object_get_data(G_OBJECT(linphone_gtk_get_main_window()),"call_logs");
 	if (w==NULL){
 		w=linphone_gtk_create_window("call_logs");
 		gtk_button_set_image(GTK_BUTTON(linphone_gtk_get_widget(w,"call_back_button")),
 		                     create_pixmap (linphone_gtk_get_ui_config("callback_button","status-green.png")));
-		fill_renderers(GTK_TREE_VIEW(linphone_gtk_get_widget(w,"logs_view")));
 		g_object_set_data(G_OBJECT(mw),"call_logs",w);
 		g_signal_connect(G_OBJECT(w),"response",(GCallback)linphone_gtk_call_log_response,NULL);
 		gtk_widget_show(w);
