@@ -28,7 +28,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <sys/stat.h>
 #include <unistd.h>
 
-#ifdef __APPLE__
+#ifdef HAVE_GTK_OSX
 #include <gtkosxapplication.h>
 #endif
 
@@ -36,6 +36,14 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #define chdir _chdir
 #endif
 
+#if defined(HAVE_NOTIFY1) || defined(HAVE_NOTIFY4)
+#define HAVE_NOTIFY
+#endif
+
+#ifdef HAVE_NOTIFY
+#include <libnotify/notify.h>
+#endif
+
 #define LINPHONE_ICON "linphone.png"
 
 const char *this_program_ident_string="linphone_ident_string=" LINPHONE_VERSION;
@@ -44,7 +52,6 @@ static LinphoneCore *the_core=NULL;
 static GtkWidget *the_ui=NULL;
 
 static void linphone_gtk_registration_state_changed(LinphoneCore *lc, LinphoneProxyConfig *cfg, LinphoneRegistrationState rs, const char *msg);
-static void linphone_gtk_show(LinphoneCore *lc);
 static void linphone_gtk_notify_recv(LinphoneCore *lc, LinphoneFriend * fid);
 static void linphone_gtk_new_unknown_subscriber(LinphoneCore *lc, LinphoneFriend *lf, const char *url);
 static void linphone_gtk_auth_info_requested(LinphoneCore *lc, const char *realm, const char *username);
@@ -53,9 +60,9 @@ static void linphone_gtk_display_message(LinphoneCore *lc, const char *msg);
 static void linphone_gtk_display_warning(LinphoneCore *lc, const char *warning);
 static void linphone_gtk_display_url(LinphoneCore *lc, const char *msg, const char *url);
 static void linphone_gtk_call_log_updated(LinphoneCore *lc, LinphoneCallLog *cl);
-static void linphone_gtk_refer_received(LinphoneCore *lc, const char  *refer_to);
 static void linphone_gtk_call_state_changed(LinphoneCore *lc, LinphoneCall *call, LinphoneCallState cs, const char *msg);
 static gboolean linphone_gtk_auto_answer(LinphoneCall *call);
+static void linphone_gtk_status_icon_set_blinking(gboolean val);
 
 
 static gboolean verbose=0;
@@ -64,6 +71,7 @@ static gchar * addr_to_call = NULL;
 static gboolean iconified=FALSE;
 static gchar *workingdir=NULL;
 static char *progpath=NULL;
+gchar *linphone_logfile=NULL;
 
 static GOptionEntry linphone_options[]={
 	{
@@ -73,6 +81,13 @@ static GOptionEntry linphone_options[]={
 		.arg_data= (gpointer)&verbose,
 		.description=N_("log to stdout some debug information while running.")
 	},
+	{
+	    .long_name = "logfile",
+	    .short_name = 'l',
+	    .arg = G_OPTION_ARG_STRING,
+	    .arg_data = &linphone_logfile,
+	    .description = N_("path to a file to write logs into.")
+	},
 	{
 		.long_name="iconified",
 		.short_name= '\0',
@@ -105,6 +120,7 @@ static GOptionEntry linphone_options[]={
 };
 
 #define INSTALLED_XML_DIR PACKAGE_DATA_DIR "/linphone"
+#define RELATIVE_XML_DIR 
 #define BUILD_TREE_XML_DIR "gtk"
 
 #ifndef WIN32
@@ -193,7 +209,6 @@ static void linphone_gtk_init_liblinphone(const char *config_file,
 
 	vtable.call_state_changed=linphone_gtk_call_state_changed;
 	vtable.registration_state_changed=linphone_gtk_registration_state_changed;
-	vtable.show=linphone_gtk_show;
 	vtable.notify_presence_recv=linphone_gtk_notify_recv;
 	vtable.new_subscription_request=linphone_gtk_new_unknown_subscriber;
 	vtable.auth_info_requested=linphone_gtk_auth_info_requested;
@@ -249,7 +264,7 @@ static int get_ui_file(const char *name, char *path, int pathsize){
 	if (access(path,F_OK)!=0){
 		snprintf(path,pathsize,"%s/%s.ui",INSTALLED_XML_DIR,name);
 		if (access(path,F_OK)!=0){
-			g_error("Could not locate neither %s/%s.ui and %s/%s.ui .",BUILD_TREE_XML_DIR,name,
+			g_error("Could not locate neither %s/%s.ui nor %s/%s.ui",BUILD_TREE_XML_DIR,name,
 				INSTALLED_XML_DIR,name);
 			return -1;
 		}
@@ -304,6 +319,7 @@ GtkWidget *linphone_gtk_create_widget(const char *filename, const char *widget_n
 		return NULL;
 	}
 	g_object_set_data(G_OBJECT(w),"builder",builder);
+	g_signal_connect_swapped(G_OBJECT(w),"destroy",(GCallback)g_object_unref,builder);
 	gtk_builder_connect_signals(builder,w);
 	return w;
 }
@@ -613,37 +629,29 @@ static void completion_add_text(GtkEntry *entry, const char *text){
 	save_uri_history();
 }
 
+
+void linphone_gtk_show_main_window(){
+	GtkWidget *w=linphone_gtk_get_main_window();
+	LinphoneCore *lc=linphone_gtk_get_core();
+	if (linphone_core_video_enabled(lc)){
+		linphone_core_enable_video_preview(lc,linphone_gtk_get_ui_config_int("videoselfview",
+	    	VIDEOSELFVIEW_DEFAULT));
+	}
+	gtk_widget_show(w);
+	gtk_window_present(GTK_WINDOW(w));
+}
+
 void linphone_gtk_call_terminated(LinphoneCall *call, const char *error){
 	GtkWidget *mw=linphone_gtk_get_main_window();
-	gtk_widget_set_sensitive(linphone_gtk_get_widget(mw,"terminate_call"),FALSE);
-	gtk_widget_set_sensitive(linphone_gtk_get_widget(mw,"start_call"),TRUE);
-	
+	if (linphone_core_get_calls(linphone_gtk_get_core())==NULL){
+	    gtk_widget_set_sensitive(linphone_gtk_get_widget(mw,"terminate_call"),FALSE);
+	    gtk_widget_set_sensitive(linphone_gtk_get_widget(mw,"start_call"),TRUE);
+	}
 	if (linphone_gtk_use_in_call_view() && call)
 		linphone_gtk_in_call_view_terminate(call,error);
 	update_video_title();
 }
 
-static gboolean in_call_timer(){
-	LinphoneCall *call=linphone_core_get_current_call(linphone_gtk_get_core());
-	if (call){
-		linphone_gtk_in_call_view_update_duration(call);
-		return TRUE;
-	}
-	return FALSE;
-}
-
-static bool_t all_other_calls_paused(LinphoneCall *refcall, const MSList *calls){
-	for(;calls!=NULL;calls=calls->next){
-		LinphoneCall *call=(LinphoneCall*)calls->data;
-		LinphoneCallState cs=linphone_call_get_state(call);
-		if (refcall!=call){
-			if (cs!=LinphoneCallPaused  && cs!=LinphoneCallPausing)
-				return FALSE;
-		}
-	}
-	return TRUE;
-}
-
 static void linphone_gtk_update_call_buttons(LinphoneCall *call){
 	LinphoneCore *lc=linphone_gtk_get_core();
 	GtkWidget *mw=linphone_gtk_get_main_window();
@@ -658,21 +666,9 @@ static void linphone_gtk_update_call_buttons(LinphoneCall *call){
 		start_active=TRUE;
 		stop_active=FALSE;
 	}else{
-		stop_active=TRUE;
-		if (all_other_calls_paused(NULL,calls)){
-			start_active=TRUE;
-			add_call=TRUE;
-		}else if (call!=NULL && linphone_call_get_state(call)==LinphoneCallIncomingReceived && all_other_calls_paused(call,calls)){
-			if (call_list_size>1){
-				start_active=TRUE;
-				add_call=TRUE;
-			}else{
-				start_active=TRUE;
-				add_call=FALSE;
-			}
-		}else{
-			start_active=FALSE;
-		}
+		stop_active=TRUE;	
+		start_active=TRUE;
+		add_call=TRUE;
 	}
 	button=linphone_gtk_get_widget(mw,"start_call");
 	gtk_widget_set_sensitive(button,start_active);
@@ -683,12 +679,9 @@ static void linphone_gtk_update_call_buttons(LinphoneCall *call){
 	gtk_widget_set_visible(button,add_call);
 	
 	gtk_widget_set_sensitive(linphone_gtk_get_widget(mw,"terminate_call"),stop_active);
-	if (linphone_core_get_calls(lc)==NULL){
-		linphone_gtk_enable_mute_button(
-				GTK_BUTTON(linphone_gtk_get_widget(linphone_gtk_get_main_window(),"main_mute")),
-			FALSE);
-	}
+
 	linphone_gtk_enable_transfer_button(lc,call_list_size>1);
+	linphone_gtk_enable_conference_button(lc,call_list_size>1);
 	update_video_title();
 }
 
@@ -718,7 +711,7 @@ void linphone_gtk_start_call(GtkWidget *w){
 	GtkWidget *mw=gtk_widget_get_toplevel(w);
 	GtkWidget *uri_bar=linphone_gtk_get_widget(mw,"uribar");
 
-	call=linphone_gtk_get_currently_displayed_call ();
+	call=linphone_gtk_get_currently_displayed_call(NULL);
 	if (call!=NULL && linphone_call_get_state(call)==LinphoneCallIncomingReceived){
 		linphone_core_accept_call(lc,call);
 	}else{
@@ -736,34 +729,40 @@ void linphone_gtk_uri_bar_activate(GtkWidget *w){
 
 
 void linphone_gtk_terminate_call(GtkWidget *button){
-	LinphoneCall *call=linphone_gtk_get_currently_displayed_call ();
-	if (call)
+	gboolean is_conf;
+	LinphoneCall *call=linphone_gtk_get_currently_displayed_call(&is_conf);
+	if (call){
 		linphone_core_terminate_call(linphone_gtk_get_core(),call);
+	}else if (is_conf){
+		linphone_core_terminate_conference(linphone_gtk_get_core());
+	}
 }
 
 void linphone_gtk_decline_clicked(GtkWidget *button){
-	LinphoneCall *call=linphone_gtk_get_currently_displayed_call ();
+	LinphoneCall *call=linphone_gtk_get_currently_displayed_call(NULL);
 	if (call)
 		linphone_core_terminate_call(linphone_gtk_get_core(),call);
 }
 
 void linphone_gtk_answer_clicked(GtkWidget *button){
-	LinphoneCall *call=linphone_gtk_get_currently_displayed_call ();
+	LinphoneCall *call=linphone_gtk_get_currently_displayed_call(NULL);
 	if (call){
-		linphone_core_pause_all_calls(linphone_gtk_get_core());
 		linphone_core_accept_call(linphone_gtk_get_core(),call);
+		linphone_gtk_show_main_window(); /* useful when the button is clicked on a notification */
 	}
 }
 
-void linphone_gtk_set_audio_video(){
-	linphone_core_enable_video(linphone_gtk_get_core(),TRUE,TRUE);
-	linphone_core_enable_video_preview(linphone_gtk_get_core(),
-	    linphone_gtk_get_ui_config_int("videoselfview",VIDEOSELFVIEW_DEFAULT));
-}
-
-void linphone_gtk_set_audio_only(){
-	linphone_core_enable_video(linphone_gtk_get_core(),FALSE,FALSE);
-	linphone_core_enable_video_preview(linphone_gtk_get_core(),FALSE);
+void linphone_gtk_enable_video(GtkWidget *w){
+	gboolean val=gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(w));
+	GtkWidget *selfview_item=linphone_gtk_get_widget(linphone_gtk_get_main_window(),"selfview_item");
+	linphone_core_enable_video(linphone_gtk_get_core(),val,val);
+	gtk_widget_set_sensitive(selfview_item,val);
+	if (val){
+		linphone_core_enable_video_preview(linphone_gtk_get_core(),
+		linphone_gtk_get_ui_config_int("videoselfview",VIDEOSELFVIEW_DEFAULT));
+	}else{
+		linphone_core_enable_video_preview(linphone_gtk_get_core(),FALSE);
+	}
 }
 
 void linphone_gtk_enable_self_view(GtkWidget *w){
@@ -784,21 +783,6 @@ void linphone_gtk_used_identity_changed(GtkWidget *w){
 	if (sel) g_free(sel);
 }
 
-static void linphone_gtk_show_main_window(){
-	GtkWidget *w=linphone_gtk_get_main_window();
-	LinphoneCore *lc=linphone_gtk_get_core();
-	if (linphone_core_video_enabled(lc)){
-		linphone_core_enable_video_preview(lc,linphone_gtk_get_ui_config_int("videoselfview",
-	    	VIDEOSELFVIEW_DEFAULT));
-	}
-	gtk_widget_show(w);
-	gtk_window_present(GTK_WINDOW(w));
-}
-
-static void linphone_gtk_show(LinphoneCore *lc){
-	linphone_gtk_show_main_window();
-}
-
 static void linphone_gtk_notify_recv(LinphoneCore *lc, LinphoneFriend * fid){
 	linphone_gtk_show_friends();
 }
@@ -937,6 +921,94 @@ static void linphone_gtk_display_url(LinphoneCore *lc, const char *msg, const ch
 static void linphone_gtk_call_log_updated(LinphoneCore *lc, LinphoneCallLog *cl){
 	GtkWidget *w=(GtkWidget*)g_object_get_data(G_OBJECT(linphone_gtk_get_main_window()),"call_logs");
 	if (w) linphone_gtk_call_log_update(w);
+	linphone_gtk_call_log_update(linphone_gtk_get_main_window());
+}
+
+#ifdef HAVE_NOTIFY
+static bool_t notify_actions_supported() {
+	bool_t accepts_actions = FALSE;
+	GList *capabilities = notify_get_server_caps();
+	GList *c;
+	if(capabilities != NULL) {
+		for(c = capabilities; c != NULL; c = c->next) {
+			if(strcmp((char*)c->data, "actions") == 0 ) {
+				accepts_actions = TRUE;
+				break;
+			}
+		}
+		g_list_foreach(capabilities, (GFunc)g_free, NULL);
+		g_list_free(capabilities);
+	}
+	return accepts_actions;
+}
+
+static NotifyNotification* build_notification(const char *title, const char *body){
+	 return notify_notification_new(title,body,linphone_gtk_get_ui_config("icon",LINPHONE_ICON)
+#ifdef HAVE_NOTIFY1
+        ,NULL
+#endif
+	);
+}
+
+static void show_notification(NotifyNotification* n){
+	if (n && !notify_notification_show(n,NULL))
+		ms_error("Failed to send notification.");
+}
+
+static void make_notification(const char *title, const char *body){
+	show_notification(build_notification(title,body));
+}
+
+#endif
+
+static void linphone_gtk_notify(LinphoneCall *call, const char *msg){
+#ifdef HAVE_NOTIFY
+	if (!notify_is_initted())
+		if (!notify_init ("Linphone")) ms_error("Libnotify failed to init.");
+#endif
+	if (!call) {
+#ifdef HAVE_NOTIFY
+		if (!notify_notification_show(notify_notification_new("Linphone",msg,NULL
+#ifdef HAVE_NOTIFY1
+	,NULL
+#endif
+),NULL))
+				ms_error("Failed to send notification.");
+#else
+		linphone_gtk_show_main_window();
+#endif
+	} else if (!gtk_window_is_active((GtkWindow*)linphone_gtk_get_main_window())) {
+#ifdef HAVE_NOTIFY
+		char *body=NULL;
+		char *remote=call!=NULL ? linphone_call_get_remote_address_as_string(call) : NULL;
+		NotifyNotification *n;
+		switch(linphone_call_get_state(call)){
+			case LinphoneCallError:
+				make_notification(_("Call error"),body=g_markup_printf_escaped("<span size=\"large\">%s</span>\n%s",msg,remote));
+			break;
+			case LinphoneCallEnd:
+				make_notification(_("Call ended"),body=g_markup_printf_escaped("<span size=\"large\">%s</span>",remote));
+			break;
+			case LinphoneCallIncomingReceived:
+				n=build_notification(_("Incoming call"),body=g_markup_printf_escaped("<span size=\"large\">%s</span>",remote));
+				if (notify_actions_supported()) {
+					notify_notification_add_action (n,"answer", _("Answer"),
+						NOTIFY_ACTION_CALLBACK(linphone_gtk_answer_clicked),NULL,NULL);
+					notify_notification_add_action (n,"decline",_("Decline"),
+						NOTIFY_ACTION_CALLBACK(linphone_gtk_decline_clicked),NULL,NULL);
+				}
+				show_notification(n);
+			break;
+			case LinphoneCallPausedByRemote:
+				make_notification(_("Call paused"),body=g_markup_printf_escaped(_("<span size=\"large\">by %s</span>"),remote));
+			break;
+			default:
+			break;
+		}
+		if (body) g_free(body);
+		if (remote) g_free(remote);
+#endif
+	}
 }
 
 static void linphone_gtk_call_state_changed(LinphoneCore *lc, LinphoneCall *call, LinphoneCallState cs, const char *msg){
@@ -949,20 +1021,18 @@ static void linphone_gtk_call_state_changed(LinphoneCore *lc, LinphoneCall *call
 		break;
 		case LinphoneCallStreamsRunning:
 			linphone_gtk_in_call_view_set_in_call(call);
-			linphone_gtk_enable_mute_button(
-				GTK_BUTTON(linphone_gtk_get_widget(linphone_gtk_get_main_window(),"main_mute")),
-			TRUE);
-			g_timeout_add(250,(GSourceFunc)in_call_timer,NULL);
 		break;
 		case LinphoneCallError:
 			linphone_gtk_in_call_view_terminate (call,msg);
 		break;
 		case LinphoneCallEnd:
 			linphone_gtk_in_call_view_terminate(call,NULL);
+			linphone_gtk_status_icon_set_blinking(FALSE);
 		break;
 		case LinphoneCallIncomingReceived:
-			linphone_gtk_create_in_call_view (call);
-			linphone_gtk_in_call_view_set_incoming(call,!all_other_calls_paused (call,linphone_core_get_calls(lc)));
+			linphone_gtk_create_in_call_view(call);
+			linphone_gtk_in_call_view_set_incoming(call);
+			linphone_gtk_status_icon_set_blinking(TRUE);
 			if (auto_answer)  {
 				linphone_call_ref(call);
 				g_timeout_add(2000,(GSourceFunc)linphone_gtk_auto_answer ,call);
@@ -979,13 +1049,55 @@ static void linphone_gtk_call_state_changed(LinphoneCore *lc, LinphoneCall *call
 		break;
 		case LinphoneCallConnected:
 			linphone_gtk_enable_hold_button (call,TRUE,TRUE);
+			linphone_gtk_status_icon_set_blinking(FALSE);
 		break;
 		default:
 		break;
 	}
+	linphone_gtk_notify(call, msg);
 	linphone_gtk_update_call_buttons (call);
 }
 
+static void update_registration_status(LinphoneProxyConfig *cfg, LinphoneRegistrationState rs){
+	GtkComboBox *box=GTK_COMBO_BOX(linphone_gtk_get_widget(linphone_gtk_get_main_window(),"identities"));
+	GtkTreeModel *model=gtk_combo_box_get_model(box);
+	GtkTreeIter iter;
+	gboolean found=FALSE;
+	const char *stock_id=NULL;
+	
+	if (gtk_tree_model_get_iter_first(model,&iter)){
+		gpointer p;
+		do{
+			gtk_tree_model_get(model,&iter,2,&p,-1);
+			if (p==cfg) {
+				found=TRUE;
+				break;
+			}
+		}while(gtk_tree_model_iter_next(model,&iter));
+	}
+	if (!found) {
+		g_warning("Could not find proxy config in combo box of identities.");
+		return;
+	}
+	switch (rs){
+		case LinphoneRegistrationOk:
+			stock_id=GTK_STOCK_YES;
+		break;
+		case LinphoneRegistrationProgress:
+			stock_id=GTK_STOCK_REFRESH;
+		break;
+		case LinphoneRegistrationCleared:
+			stock_id=NULL;
+		break;
+		case LinphoneRegistrationFailed:
+			stock_id=GTK_STOCK_DIALOG_WARNING;
+		break;
+		default:
+		break;
+	}
+	gtk_list_store_set(GTK_LIST_STORE(model),&iter,1,stock_id,-1);
+}
+
 static void linphone_gtk_registration_state_changed(LinphoneCore *lc, LinphoneProxyConfig *cfg, 
                                                     LinphoneRegistrationState rs, const char *msg){
 	switch (rs){
@@ -1000,12 +1112,7 @@ static void linphone_gtk_registration_state_changed(LinphoneCore *lc, LinphonePr
 		default:
 		break;
 	}
-}
-
-
-static void icon_popup_menu(GtkStatusIcon *status_icon, guint button, guint activate_time, gpointer user_data){
-	GtkWidget *menu=(GtkWidget*)g_object_get_data(G_OBJECT(status_icon),"menu");
-	gtk_menu_popup(GTK_MENU(menu),NULL,NULL,gtk_status_icon_position_menu,status_icon,button,activate_time);
+	update_registration_status(cfg,rs);
 }
 
 void linphone_gtk_open_browser(const char *url){
@@ -1028,6 +1135,13 @@ void linphone_gtk_link_to_website(GtkWidget *item){
 	linphone_gtk_open_browser(home);
 }
 
+#ifndef HAVE_GTK_OSX
+
+static void icon_popup_menu(GtkStatusIcon *status_icon, guint button, guint activate_time, gpointer user_data){
+	GtkWidget *menu=(GtkWidget*)g_object_get_data(G_OBJECT(status_icon),"menu");
+	gtk_menu_popup(GTK_MENU(menu),NULL,NULL,gtk_status_icon_position_menu,status_icon,button,activate_time);
+}
+
 static GtkWidget *create_icon_menu(){
 	GtkWidget *menu=gtk_menu_new();
 	GtkWidget *menu_item;
@@ -1063,20 +1177,85 @@ static GtkWidget *create_icon_menu(){
 
 static GtkStatusIcon *icon=NULL;
 
+static void handle_icon_click() {
+	GtkWidget *mw=linphone_gtk_get_main_window();
+	if (!gtk_window_is_active((GtkWindow*)mw)) {
+		linphone_gtk_show_main_window();
+	} else {
+		gtk_widget_hide(mw);
+	}
+}
+
 static void linphone_gtk_init_status_icon(){
 	const char *icon_path=linphone_gtk_get_ui_config("icon",LINPHONE_ICON);
+	const char *call_icon_path=linphone_gtk_get_ui_config("start_call_icon","startcall-green.png");
 	GdkPixbuf *pbuf=create_pixbuf(icon_path);
 	GtkWidget *menu=create_icon_menu();
 	const char *title;
-	icon=gtk_status_icon_new_from_pixbuf(pbuf);
-	g_object_unref(G_OBJECT(pbuf));
-	g_signal_connect_swapped(G_OBJECT(icon),"activate",(GCallback)linphone_gtk_show_main_window,linphone_gtk_get_main_window());
-	g_signal_connect(G_OBJECT(icon),"popup-menu",(GCallback)icon_popup_menu,NULL);
 	title=linphone_gtk_get_ui_config("title",_("Linphone - a video internet phone"));
+	icon=gtk_status_icon_new_from_pixbuf(pbuf);
+	gtk_status_icon_set_name(icon,title);
+	g_signal_connect_swapped(G_OBJECT(icon),"activate",(GCallback)handle_icon_click,NULL);
+	g_signal_connect(G_OBJECT(icon),"popup-menu",(GCallback)icon_popup_menu,NULL);
 	gtk_status_icon_set_tooltip(icon,title);
 	gtk_status_icon_set_visible(icon,TRUE);
 	g_object_set_data(G_OBJECT(icon),"menu",menu);
 	g_object_weak_ref(G_OBJECT(icon),(GWeakNotify)gtk_widget_destroy,menu);
+	g_object_set_data(G_OBJECT(icon),"icon",pbuf);
+	g_object_weak_ref(G_OBJECT(icon),(GWeakNotify)g_object_unref,pbuf);
+	pbuf=create_pixbuf(call_icon_path);
+	g_object_set_data(G_OBJECT(icon),"call_icon",pbuf);
+}
+
+static gboolean do_icon_blink(GtkStatusIcon *gi){
+	GdkPixbuf *call_icon=g_object_get_data(G_OBJECT(gi),"call_icon");
+	GdkPixbuf *normal_icon=g_object_get_data(G_OBJECT(gi),"icon");
+	GdkPixbuf *cur_icon=gtk_status_icon_get_pixbuf(gi);
+	if (cur_icon==call_icon){
+		gtk_status_icon_set_from_pixbuf(gi,normal_icon);
+	}else{
+		gtk_status_icon_set_from_pixbuf(gi,call_icon);
+	}
+	return TRUE;
+}
+
+#endif
+
+static void linphone_gtk_status_icon_set_blinking(gboolean val){
+#ifdef HAVE_GTK_OSX
+	static gint attention_id;
+	GtkOSXApplication *theMacApp=(GtkOSXApplication*)g_object_new(GTK_TYPE_OSX_APPLICATION, NULL);
+	if (val)
+		attention_id=gtk_osxapplication_attention_request(theMacApp,CRITICAL_REQUEST);
+	else gtk_osxapplication_cancel_attention_request(theMacApp,attention_id);
+#else
+	if (icon!=NULL){
+		guint tout;
+		tout=(unsigned)GPOINTER_TO_INT(g_object_get_data(G_OBJECT(icon),"timeout"));
+		if (val && tout==0){
+			tout=g_timeout_add(500,(GSourceFunc)do_icon_blink,icon);
+			g_object_set_data(G_OBJECT(icon),"timeout",GINT_TO_POINTER(tout));
+		}else if (!val && tout!=0){
+			GdkPixbuf *normal_icon=g_object_get_data(G_OBJECT(icon),"icon");
+			g_source_remove(tout);
+			g_object_set_data(G_OBJECT(icon),"timeout",NULL);
+			gtk_status_icon_set_from_pixbuf(icon,normal_icon);
+		}
+	}
+#endif
+}
+
+static void init_identity_combo(GtkComboBox *box){
+	GtkListStore *store;
+	GtkCellRenderer *r1,*r2;
+	store=gtk_list_store_new(3,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_POINTER);
+	gtk_cell_layout_clear(GTK_CELL_LAYOUT(box));
+	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(box),(r1=gtk_cell_renderer_text_new()),TRUE);
+	gtk_cell_layout_pack_end(GTK_CELL_LAYOUT(box),(r2=gtk_cell_renderer_pixbuf_new()),FALSE);
+	gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(box),r1,"text",0);
+	gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(box),r2,"stock-id",1);
+	g_object_set(G_OBJECT(r1),"ellipsize",PANGO_ELLIPSIZE_END,NULL);
+	gtk_combo_box_set_model(box,GTK_TREE_MODEL(store));
 }
 
 void linphone_gtk_load_identities(void){
@@ -1086,19 +1265,28 @@ void linphone_gtk_load_identities(void){
 	LinphoneProxyConfig *def=NULL;
 	int def_index=0,i;
 	GtkListStore *store;
+	GtkTreeIter iter;
 
 	store=GTK_LIST_STORE(gtk_combo_box_get_model(box));
+	if (gtk_tree_model_get_n_columns(GTK_TREE_MODEL(store))==1){
+		/* model is empty, this is the first time we go here */
+		init_identity_combo(box);
+		store=GTK_LIST_STORE(gtk_combo_box_get_model(box));
+	}
 	gtk_list_store_clear(store);
-
 	linphone_core_get_default_proxy(linphone_gtk_get_core(),&def);
 	def_identity=g_strdup_printf(_("%s (Default)"),linphone_core_get_primary_contact(linphone_gtk_get_core()));
-	gtk_combo_box_append_text(box,def_identity);
+	gtk_list_store_append(store,&iter);
+	gtk_list_store_set(store,&iter,0,def_identity,1,NULL,2,NULL,-1);
 	g_free(def_identity);
 	for(i=1,elem=linphone_core_get_proxy_config_list(linphone_gtk_get_core());
 			elem!=NULL;
 			elem=ms_list_next(elem),i++){
 		LinphoneProxyConfig *cfg=(LinphoneProxyConfig*)elem->data;
-		gtk_combo_box_append_text(box,linphone_proxy_config_get_identity(cfg));
+		gtk_list_store_append(store,&iter);
+		gtk_list_store_set(store,&iter,0,linphone_proxy_config_get_identity(cfg),1,
+		                   linphone_proxy_config_is_registered(cfg) ? GTK_STOCK_YES : NULL,
+		                   2,cfg,-1);
 		if (cfg==def) {
 			def_index=i;
 		}
@@ -1133,13 +1321,14 @@ static void linphone_gtk_connect_digits(void){
 }
 
 static void linphone_gtk_check_menu_items(void){
-	bool_t audio_only=!linphone_core_video_enabled(linphone_gtk_get_core());
+	bool_t video_enabled=linphone_core_video_enabled(linphone_gtk_get_core());
 	bool_t selfview=linphone_gtk_get_ui_config_int("videoselfview",VIDEOSELFVIEW_DEFAULT);
+	GtkWidget *selfview_item=linphone_gtk_get_widget(
+					linphone_gtk_get_main_window(),"selfview_item");
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(linphone_gtk_get_widget(
-					linphone_gtk_get_main_window(),
-					audio_only ? "audio_only_item" : "video_item")), TRUE);
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(linphone_gtk_get_widget(
-					linphone_gtk_get_main_window(),"selfview_item")),selfview);
+					linphone_gtk_get_main_window(),"enable_video_item")), video_enabled);
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(selfview_item),selfview);
+	gtk_widget_set_sensitive(selfview_item,video_enabled);
 }
 
 static gboolean linphone_gtk_can_manage_accounts(){
@@ -1181,13 +1370,6 @@ static void linphone_gtk_configure_main_window(){
 	linphone_gtk_configure_window(w,"main_window");
 	if (title) {
 		gtk_window_set_title(GTK_WINDOW(w),title);
-#if GTK_CHECK_VERSION(2,16,0)
-#ifdef __APPLE__
-		gtk_menu_item_set_label(GTK_MENU_ITEM(linphone_gtk_get_widget(w,"main_menu")),_("Windows"));
-#else
-		gtk_menu_item_set_label(GTK_MENU_ITEM(linphone_gtk_get_widget(w,"main_menu")),title);
-#endif
-#endif
 	}
 	if (start_call_icon){
 		gtk_button_set_image(GTK_BUTTON(linphone_gtk_get_widget(w,"start_call")),
@@ -1219,11 +1401,13 @@ static void linphone_gtk_configure_main_window(){
 		g_object_set_data(G_OBJECT(menu_item),"home",tmp);
 	}
 	{
+		/*
 		GdkPixbuf *pbuf=create_pixbuf("contact-orange.png");
 		if (pbuf) {
 			gtk_image_set_from_pixbuf(GTK_IMAGE(linphone_gtk_get_widget(w,"contact_tab_icon")),pbuf);
 			g_object_unref(G_OBJECT(pbuf));
 		}
+		*/
 	}
 	{
 		GdkPixbuf *pbuf=create_pixbuf("dialer-orange.png");
@@ -1266,7 +1450,7 @@ gboolean linphone_gtk_close(GtkWidget *mw){
 		linphone_core_terminate_all_calls(lc);
 	}
 	linphone_core_enable_video_preview(lc,FALSE);
-#ifdef __APPLE__
+#ifdef __APPLE__ /*until with have a better option*/
 	gtk_window_iconify(GTK_WINDOW(mw));
 #else
 	gtk_widget_hide(mw);
@@ -1274,6 +1458,19 @@ gboolean linphone_gtk_close(GtkWidget *mw){
 	return TRUE;
 }
 
+#ifdef HAVE_GTK_OSX
+static gboolean on_window_state_event(GtkWidget *w, GdkEventWindowState *event){
+        if ((event->new_window_state & GDK_WINDOW_STATE_ICONIFIED) ||(event->new_window_state & GDK_WINDOW_STATE_WITHDRAWN) ){
+                linphone_core_enable_video_preview(linphone_gtk_get_core(),FALSE);
+        }else{
+                linphone_core_enable_video_preview(linphone_gtk_get_core(),
+		linphone_gtk_get_ui_config_int("videoselfview",VIDEOSELFVIEW_DEFAULT) && linphone_core_video_enabled(linphone_gtk_get_core()));
+        }
+        return FALSE;
+}
+#endif
+
+
 static void linphone_gtk_init_main_window(){
 	GtkWidget *main_window;
 
@@ -1284,27 +1481,24 @@ static void linphone_gtk_init_main_window(){
 	linphone_gtk_set_my_presence(linphone_core_get_presence_info(linphone_gtk_get_core()));
 	linphone_gtk_show_friends();
 	linphone_gtk_connect_digits();
-	linphone_gtk_check_menu_items();
 	main_window=linphone_gtk_get_main_window();
-	linphone_gtk_enable_mute_button(GTK_BUTTON(linphone_gtk_get_widget(main_window,
-					"main_mute")),FALSE);
-	if (!linphone_gtk_use_in_call_view()) {
-		gtk_widget_show(linphone_gtk_get_widget(main_window, "main_mute"));
-	}
+	linphone_gtk_call_log_update(main_window);
+	
 	linphone_gtk_update_call_buttons (NULL);
 	/*prevent the main window from being destroyed by a user click on WM controls, instead we hide it*/
 	g_signal_connect (G_OBJECT (main_window), "delete-event",
 		G_CALLBACK (linphone_gtk_close), main_window);
-#ifdef __APPLE__
+#ifdef HAVE_GTK_OSX
 	{
 		GtkWidget *menubar=linphone_gtk_get_widget(main_window,"menubar1");
-		gtk_widget_destroy(linphone_gtk_get_widget(main_window,"imagemenuitem5"));
 		GtkOSXApplication *theMacApp = (GtkOSXApplication*)g_object_new(GTK_TYPE_OSX_APPLICATION, NULL);
 		gtk_osxapplication_set_menu_bar(theMacApp,GTK_MENU_SHELL(menubar));
 		gtk_widget_hide(menubar);
 		gtk_osxapplication_ready(theMacApp);
 	}
+	g_signal_connect(G_OBJECT(main_window), "window-state-event",G_CALLBACK(on_window_state_event), NULL);
 #endif
+	linphone_gtk_check_menu_items();
 }
 
 
@@ -1348,10 +1542,12 @@ void linphone_gtk_log_handler(OrtpLogLevel lev, const char *fmt, va_list args){
 }
 
 
-static void linphone_gtk_refer_received(LinphoneCore *lc, const char *refer_to){
-    GtkEntry * uri_bar =GTK_ENTRY(linphone_gtk_get_widget(
+void linphone_gtk_refer_received(LinphoneCore *lc, const char *refer_to){
+	GtkEntry * uri_bar =GTK_ENTRY(linphone_gtk_get_widget(
 		linphone_gtk_get_main_window(), "uribar"));
-	linphone_gtk_show_main_window();
+	char *text;
+	linphone_gtk_notify(NULL,(text=ms_strdup_printf(_("We are transferred to %s"),refer_to)));
+	g_free(text);
 	gtk_entry_set_text(uri_bar, refer_to);
 	linphone_gtk_start_call(linphone_gtk_get_main_window());
 }
@@ -1360,11 +1556,37 @@ static void linphone_gtk_check_soundcards(){
 	const char **devices=linphone_core_get_sound_devices(linphone_gtk_get_core());
 	if (devices==NULL || devices[0]==NULL){
 		linphone_gtk_display_something(GTK_MESSAGE_WARNING,
-		    _("No sound cards have been detected on this computer.\n"
-			    "You won't be able to send or receive audio calls."));
+			_("No sound cards have been detected on this computer.\n"
+				"You won't be able to send or receive audio calls."));
 	}
 }
 
+static void linphone_gtk_quit(void){
+	linphone_gtk_uninit_instance();
+	linphone_gtk_destroy_log_window();
+	linphone_core_destroy(the_core);
+	linphone_gtk_log_uninit();
+#ifdef HAVE_NOTIFY
+	notify_uninit();
+#endif
+	gdk_threads_leave();
+}
+
+#ifdef HAVE_GTK_OSX
+/*
+This is not the correct way to implement block termination.
+The good way would be to call gtk_main_quit(), and return TRUE.
+Unfortunately this does not work, because if we return TRUE the NSApplication sometimes calls the CFRunLoop recursively, which prevents gtk_main() to exit.
+As a result the program cannot exit at all.
+As a workaround we do all the cleanup (unregistration and config save) within the handler.
+*/
+static gboolean on_block_termination(void){
+	gtk_main_quit();
+	linphone_gtk_quit();
+	return FALSE;
+}
+#endif
+
 int main(int argc, char *argv[]){
 #ifdef ENABLE_NLS
 	void *p;
@@ -1374,6 +1596,7 @@ int main(int argc, char *argv[]){
 	const char *lang;
 	GtkSettings *settings;
 	GdkPixbuf *pbuf;
+	const char *app_name="Linphone";
 
 	g_thread_init(NULL);
 	gdk_threads_init();
@@ -1381,6 +1604,7 @@ int main(int argc, char *argv[]){
 	progpath = strdup(argv[0]);
 	
 	config_file=linphone_gtk_get_config_file();
+	
 
 #ifdef WIN32
 	/*workaround for windows: sometimes LANG is defined to an integer value, not understood by gtk */
@@ -1431,17 +1655,18 @@ int main(int argc, char *argv[]){
 	g_object_set(settings, "gtk-menu-images", TRUE, NULL);
 	g_object_set(settings, "gtk-button-images", TRUE, NULL);
 
-	if (workingdir!=NULL)
-		chdir(workingdir);
+	if (workingdir!=NULL){
+		if (chdir(workingdir)==-1){
+			g_error("Could not change directory to %s : %s",workingdir,strerror(errno));
+		}
+	}
 
 	/* Now, look for the factory configuration file, we do it this late
 		 since we want to have had time to change directory and to parse
 		 the options, in case we needed to access the working directory */
 	factory_config_file = linphone_gtk_get_factory_config_file();
 
-	if (linphone_core_wake_up_possible_already_running_instance(
-		config_file, addr_to_call) == 0){
-		g_message("addr_to_call=%s",addr_to_call);
+	if (linphone_gtk_init_instance(app_name, addr_to_call) == FALSE){
 		g_warning("Another running instance of linphone has been detected. It has been woken-up.");
 		g_warning("This instance is going to exit now.");
 		gdk_threads_leave();
@@ -1451,10 +1676,12 @@ int main(int argc, char *argv[]){
 	add_pixmap_directory("pixmaps");
 	add_pixmap_directory(PACKAGE_DATA_DIR "/pixmaps/linphone");
 
-#ifdef __APPLE__
+#ifdef HAVE_GTK_OSX
 	GtkOSXApplication *theMacApp = (GtkOSXApplication*)g_object_new(GTK_TYPE_OSX_APPLICATION, NULL);
 	g_signal_connect(G_OBJECT(theMacApp),"NSApplicationDidBecomeActive",(GCallback)linphone_gtk_show_main_window,NULL);
 	g_signal_connect(G_OBJECT(theMacApp),"NSApplicationWillTerminate",(GCallback)gtk_main_quit,NULL);
+	/*never block termination:*/
+	g_signal_connect(G_OBJECT(theMacApp),"NSApplicationBlockTermination",(GCallback)on_block_termination,NULL);
 #endif
 	
 	the_ui=linphone_gtk_create_window("main");
@@ -1464,7 +1691,7 @@ int main(int argc, char *argv[]){
 
 	linphone_gtk_init_liblinphone(config_file, factory_config_file);
 	
-	g_set_application_name(linphone_gtk_get_ui_config("title","Linphone"));
+	g_set_application_name(app_name);
 	pbuf=create_pixbuf(linphone_gtk_get_ui_config("icon",LINPHONE_ICON));
 	if (pbuf!=NULL) gtk_window_set_default_icon(pbuf);
 	
@@ -1472,7 +1699,7 @@ int main(int argc, char *argv[]){
 	gtk_timeout_add(30,(GtkFunction)linphone_gtk_iterate,(gpointer)linphone_gtk_get_core());
 	gtk_timeout_add(30,(GtkFunction)linphone_gtk_check_logs,(gpointer)NULL);
 	linphone_gtk_init_main_window();
-#ifndef __APPLE__
+#ifndef HAVE_GTK_OSX
 	linphone_gtk_init_status_icon();
 #endif
 	if (!iconified){
@@ -1483,10 +1710,8 @@ int main(int argc, char *argv[]){
 		linphone_gtk_check_for_new_version();
 
 	gtk_main();
-	gdk_threads_leave();
-	linphone_gtk_destroy_log_window();
-	linphone_core_destroy(the_core);
-#ifndef __APPLE__
+	linphone_gtk_quit();
+#ifndef HAVE_GTK_OSX
 	/*workaround a bug on win32 that makes status icon still present in the systray even after program exit.*/
 	gtk_status_icon_set_visible(icon,FALSE);
 #endif
