@@ -97,7 +97,39 @@ typedef enum MSFilterId{
 	MS_ITC_SINK_ID,
 	MS_ITC_SOURCE_ID,
 	MS_EXT_DISPLAY_ID,
-	MS_H264_DEC_ID
+	MS_H264_DEC_ID,
+	MS_IOUNIT_READ_ID,
+	MS_IOUNIT_WRITE_ID,
+	MS_ANDROID_SOUND_READ_ID,
+	MS_ANDROID_SOUND_WRITE_ID,
+	MS_JPEG_WRITER_ID,
+	MS_X11VIDEO_ID,
+	MS_ANDROID_DISPLAY_ID,
+	MS_ANDROID_VIDEO_READ_ID,
+	MS_ANDROID_VIDEO_WRITE_ID,
+	MS_TONE_DETECTOR_ID,
+	MS_VP8_ENC_ID,
+	MS_VP8_DEC_ID,
+	MS_G722_ENC_ID,
+	MS_G722_DEC_ID,
+	MS_G726_40_ENC_ID,
+	MS_G726_32_ENC_ID,
+	MS_G726_24_ENC_ID,
+	MS_G726_16_ENC_ID,
+	MS_AAL2_G726_40_ENC_ID,
+	MS_AAL2_G726_32_ENC_ID,
+	MS_AAL2_G726_24_ENC_ID,
+	MS_AAL2_G726_16_ENC_ID,
+	MS_G726_40_DEC_ID,
+	MS_G726_32_DEC_ID,
+	MS_G726_24_DEC_ID,
+	MS_G726_16_DEC_ID,
+	MS_AAL2_G726_40_DEC_ID,
+	MS_AAL2_G726_32_DEC_ID,
+	MS_AAL2_G726_24_DEC_ID,
+	MS_AAL2_G726_16_DEC_ID,
+	MS_L16_ENC_ID,
+	MS_L16_DEC_ID
 } MSFilterId;
 
 
