@@ -20,6 +20,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #define mscommon_h
 
 #include <ortp/ortp.h>
+#include <time.h>
 
 #define ms_malloc	ortp_malloc
 #define ms_malloc0	ortp_malloc0
@@ -43,6 +44,18 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #define ms_cond_broadcast	ortp_cond_broadcast
 #define ms_cond_destroy		ortp_cond_destroy
 
+#if defined(_MSC_VER)
+#define MS2_PUBLIC	__declspec(dllexport)
+#else
+#define MS2_PUBLIC
+#endif
+
+#if defined(_WIN32_WCE)
+time_t ms_time (time_t *t);
+#else
+#define ms_time time
+#endif
+
 #ifdef WIN32
 static inline void ms_debug(const char *fmt,...)
 {
@@ -65,8 +78,6 @@ static inline void ms_debug(const char *fmt,...)
 #endif	
 #endif
 
-
-
 #define ms_message	ortp_message
 #define ms_warning	ortp_warning
 #define ms_error	ortp_error
@@ -81,7 +92,11 @@ static inline void ms_debug(const char *fmt,...)
 #define ms_thread_t		ortp_thread_t
 #define ms_thread_create 	ortp_thread_create
 #define ms_thread_join		ortp_thread_join
-#define ms_thread_exit		ortp_thread_exit
+
+typedef struct MSTimeSpec{
+	int64_t tv_sec;
+	int64_t tv_nsec;
+}MSTimeSpec;
 
 struct _MSList {
 	struct _MSList *next;
@@ -95,27 +110,31 @@ typedef struct _MSList MSList;
 #define ms_list_next(elem) ((elem)->next)
 
 
+typedef int (*MSCompareFunc)(const void *a, const void *b);
+
 #ifdef __cplusplus
 extern "C"{
 #endif
 
-MSList * ms_list_append(MSList *elem, void * data);
-MSList * ms_list_prepend(MSList *elem, void * data);
-MSList * ms_list_free(MSList *elem);
-MSList * ms_list_concat(MSList *first, MSList *second);
-MSList * ms_list_remove(MSList *first, void *data);
-int ms_list_size(const MSList *first);
-void ms_list_for_each(const MSList *list, void (*func)(void *));
-void ms_list_for_each2(const MSList *list, void (*func)(void *, void *), void *user_data);
-MSList *ms_list_remove_link(MSList *list, MSList *elem);
-MSList *ms_list_find(MSList *list, void *data);
-MSList *ms_list_find_custom(MSList *list, int (*compare_func)(const void *, const void*), void *user_data);
-void * ms_list_nth_data(const MSList *list, int index);
-int ms_list_position(const MSList *list, MSList *elem);
-int ms_list_index(const MSList *list, void *data);
-MSList *ms_list_insert_sorted(MSList *list, void *data, int (*compare_func)(const void *, const void*));
-MSList *ms_list_insert(MSList *list, MSList *before, void *data);
-MSList *ms_list_copy(const MSList *list);
+void ms_thread_exit(void* ret_val);
+MS2_PUBLIC void ms_get_cur_time(MSTimeSpec *ret);
+MS2_PUBLIC MSList * ms_list_append(MSList *elem, void * data);
+MS2_PUBLIC MSList * ms_list_prepend(MSList *elem, void * data);
+MS2_PUBLIC MSList * ms_list_free(MSList *elem);
+MS2_PUBLIC MSList * ms_list_concat(MSList *first, MSList *second);
+MS2_PUBLIC MSList * ms_list_remove(MSList *first, void *data);
+MS2_PUBLIC int ms_list_size(const MSList *first);
+MS2_PUBLIC void ms_list_for_each(const MSList *list, void (*func)(void *));
+MS2_PUBLIC void ms_list_for_each2(const MSList *list, void (*func)(void *, void *), void *user_data);
+MS2_PUBLIC MSList *ms_list_remove_link(MSList *list, MSList *elem);
+MS2_PUBLIC MSList *ms_list_find(MSList *list, void *data);
+MS2_PUBLIC MSList *ms_list_find_custom(MSList *list, MSCompareFunc compare_func, const void *user_data);
+MS2_PUBLIC void * ms_list_nth_data(const MSList *list, int index);
+MS2_PUBLIC int ms_list_position(const MSList *list, MSList *elem);
+MS2_PUBLIC int ms_list_index(const MSList *list, void *data);
+MS2_PUBLIC MSList *ms_list_insert_sorted(MSList *list, void *data, MSCompareFunc compare_func);
+MS2_PUBLIC MSList *ms_list_insert(MSList *list, MSList *before, void *data);
+MS2_PUBLIC MSList *ms_list_copy(const MSList *list);
 
 #undef MIN
 #define MIN(a,b)	((a)>(b) ? (b) : (a))
@@ -143,7 +162,7 @@ MSList *ms_list_copy(const MSList *list);
  *
  * This must be called once before calling any other API.
  */
-void ms_init(void);
+MS2_PUBLIC void ms_init(void);
 
 /**
  * Load plugins from a specific directory.
@@ -156,20 +175,20 @@ void ms_init(void);
  *
  * Returns: >0 if successfull, 0 if not plugins loaded, -1 otherwise.
  */
-int ms_load_plugins(const char *directory);
+MS2_PUBLIC int ms_load_plugins(const char *directory);
 
 /**
  * Release resource allocated in the mediastreamer2 library.
  *
  * This must be called once before closing program.
  */
-void ms_exit(void);
+MS2_PUBLIC void ms_exit(void);
 
 struct _MSSndCardDesc;
 
-void ms_sleep(int seconds);
+MS2_PUBLIC void ms_sleep(int seconds);
 
-void ms_usleep(uint64_t usec);
+MS2_PUBLIC void ms_usleep(uint64_t usec);
 
 /**
  * The max payload size allowed.
@@ -180,22 +199,22 @@ void ms_usleep(uint64_t usec);
  * value works for both.
  * 
 **/
-int ms_get_payload_max_size();
+MS2_PUBLIC int ms_get_payload_max_size();
 
-void ms_set_payload_max_size(int size);
+MS2_PUBLIC void ms_set_payload_max_size(int size);
 
 /**
  * Returns the network Max Transmission Unit to reach destination_host.
  * This will attempt to send one or more big packets to destination_host, to a random port.
  * Those packets are filled with zeroes.
 **/
-int ms_discover_mtu(const char *destination_host);
+MS2_PUBLIC int ms_discover_mtu(const char *destination_host);
 
 /**
  * Set mediastreamer default mtu, used to compute the default RTP max payload size.
  * This function will call ms_set_payload_max_size(mtu-[ipv6 header size]).
 **/
-void ms_set_mtu(int mtu);
+MS2_PUBLIC void ms_set_mtu(int mtu);
 
 /** @} */
 
@@ -203,4 +222,21 @@ void ms_set_mtu(int mtu);
 }
 #endif
 
+#ifdef MS2_INTERNAL
+#  ifdef HAVE_CONFIG_H
+#  include "mediastreamer-config.h" /*necessary to know if ENABLE_NLS is there*/
+#  endif
+#  if defined(ENABLE_NLS)
+#    include <libintl.h>
+#    define _(String) dgettext (GETTEXT_PACKAGE, String)
+#  else
+#    define _(String) (String)
+#  endif // ENABLE_NLS
+#define N_(String) (String)
+#endif // MS2_INTERNAL
+
+#ifdef ANDROID
+#include "mediastreamer2/msjava.h"
 #endif
+#endif
+
