@@ -31,6 +31,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "mediastreamer2/msfilerec.h"
 #include "mediastreamer2/msvolume.h"
 #include "mediastreamer2/msequalizer.h"
+#include "mediastreamer2/mscodecutils.h"
 
 #ifdef INET6
 	#include <sys/types.h>
@@ -40,14 +41,20 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #endif
 #endif
 
-
 #define MAX_RTP_SIZE	1500
 
-
 /* this code is not part of the library itself, it is part of the mediastream program */
 void audio_stream_free(AudioStream *stream)
 {
-	if (stream->session!=NULL) rtp_session_destroy(stream->session);
+	if (stream->session!=NULL) {
+		rtp_session_unregister_event_queue(stream->session,stream->evq);
+		rtp_session_destroy(stream->session);
+		if (stream->ortpZrtpContext != NULL) {
+			ortp_zrtp_context_destroy(stream->ortpZrtpContext);
+			stream->ortpZrtpContext=NULL;
+		}
+	}
+	if (stream->evq) ortp_ev_queue_destroy(stream->evq);
 	if (stream->rtpsend!=NULL) ms_filter_destroy(stream->rtpsend);
 	if (stream->rtprecv!=NULL) ms_filter_destroy(stream->rtprecv);
 	if (stream->soundread!=NULL) ms_filter_destroy(stream->soundread);
@@ -62,6 +69,9 @@ void audio_stream_free(AudioStream *stream)
 	if (stream->ticker!=NULL) ms_ticker_destroy(stream->ticker);
 	if (stream->read_resampler!=NULL) ms_filter_destroy(stream->read_resampler);
 	if (stream->write_resampler!=NULL) ms_filter_destroy(stream->write_resampler);
+	if (stream->dtmfgen_rtp!=NULL) ms_filter_destroy(stream->dtmfgen_rtp);
+	if (stream->rc) ms_bitrate_controller_destroy(stream->rc);
+	if (stream->qi) ms_quality_indicator_destroy(stream->qi);
 	ms_free(stream);
 }
 
@@ -106,7 +116,35 @@ static void audio_stream_configure_resampler(MSFilter *resampler,MSFilter *from,
 	ms_filter_call_method(to,MS_FILTER_GET_SAMPLE_RATE,&to_rate);
 	ms_filter_call_method(resampler,MS_FILTER_SET_SAMPLE_RATE,&from_rate);
 	ms_filter_call_method(resampler,MS_FILTER_SET_OUTPUT_SAMPLE_RATE,&to_rate);
-	ms_debug("configuring from rate[%i] to rate [%i]",from_rate,to_rate);
+	ms_message("configuring %s-->%s from rate[%i] to rate [%i]",
+	           from->desc->name, to->desc->name, from_rate,to_rate);
+}
+
+static void disable_checksums(ortp_socket_t sock){
+#if defined(DISABLE_CHECKSUMS) && defined(SO_NO_CHECK)
+	int option=1;
+	if (setsockopt(sock,SOL_SOCKET,SO_NO_CHECK,&option,sizeof(option))==-1){
+		ms_warning("Could not disable udp checksum: %s",strerror(errno));
+	}
+#endif
+}
+
+MSTickerPrio __ms_get_default_prio(bool_t is_video){
+	const char *penv;
+	if (is_video){
+		return MS_TICKER_PRIO_NORMAL;
+	}
+	penv=getenv("MS_AUDIO_PRIO");
+	if (penv){
+		if (strcasecmp(penv,"NORMAL")==0)
+			return MS_TICKER_PRIO_NORMAL;
+		if (strcasecmp(penv,"HIGH")==0)
+			return MS_TICKER_PRIO_HIGH;
+		if (strcasecmp(penv,"REALTIME")==0)
+			return MS_TICKER_PRIO_REALTIME;
+		ms_error("Undefined priority %s", penv);
+	}
+	return MS_TICKER_PRIO_REALTIME;
 }
 
 RtpSession * create_duplex_rtpsession( int locport, bool_t ipv6){
@@ -121,6 +159,8 @@ RtpSession * create_duplex_rtpsession( int locport, bool_t ipv6){
 	rtp_session_signal_connect(rtpr,"timestamp_jump",(RtpCallback)rtp_session_resync,(long)NULL);
 	rtp_session_signal_connect(rtpr,"ssrc_changed",(RtpCallback)rtp_session_resync,(long)NULL);
 	rtp_session_set_ssrc_changed_threshold(rtpr,0);
+	rtp_session_set_rtcp_report_interval(rtpr,2500); /*at the beginning of the session send more reports*/
+	disable_checksums(rtp_session_get_rtp_socket(rtpr));
 	return rtpr;
 }
 
@@ -138,18 +178,61 @@ ms_time (time_t *t)
 }
 #endif
 
+static void audio_stream_process_rtcp(AudioStream *stream, mblk_t *m){
+	do{
+		const report_block_t *rb=NULL;
+		if (rtcp_is_SR(m)){
+			rb=rtcp_SR_get_report_block(m,0);
+		}else if (rtcp_is_RR(m)){
+			rb=rtcp_RR_get_report_block(m,0);
+		}
+		if (rb){
+			unsigned int ij;
+			float rt=rtp_session_get_round_trip_propagation(stream->session);
+			float flost;
+			ij=report_block_get_interarrival_jitter(rb);
+			flost=(float)(100.0*report_block_get_fraction_lost(rb)/256.0);
+			ms_message("audio_stream_process_rtcp: interarrival jitter=%u , "
+			           "lost packets percentage since last report=%f, round trip time=%f seconds",ij,flost,rt);
+			if (stream->rc) ms_bitrate_controller_process_rtcp(stream->rc,m);
+			if (stream->qi) ms_quality_indicator_update_from_feedback(stream->qi,m);
+		}
+	}while(rtcp_next_packet(m));
+}
+
+void audio_stream_iterate(AudioStream *stream){
+	if (stream->is_beginning && ms_time(NULL)-stream->start_time>15){
+		rtp_session_set_rtcp_report_interval(stream->session,5000);
+		stream->is_beginning=FALSE;
+	}
+	if (stream->evq){
+		OrtpEvent *ev=ortp_ev_queue_get(stream->evq);
+		if (ev!=NULL){
+			OrtpEventType evt=ortp_event_get_type(ev);
+			if (evt==ORTP_EVENT_RTCP_PACKET_RECEIVED){
+				audio_stream_process_rtcp(stream,ortp_event_get_data(ev)->packet);
+				stream->last_packet_time=ms_time(NULL);
+			}else if (evt==ORTP_EVENT_RTCP_PACKET_EMITTED){
+				/*we choose to update the quality indicator when the oRTP stack decides to emit a RTCP report */
+				ms_quality_indicator_update_local(stream->qi);
+			}
+			ortp_event_destroy(ev);
+		}
+	}
+}
+
 bool_t audio_stream_alive(AudioStream * stream, int timeout){
-	RtpSession *session=stream->session;
-	const rtp_stats_t *stats=rtp_session_get_stats(session);
+	const rtp_stats_t *stats=rtp_session_get_stats(stream->session);
 	if (stats->recv!=0){
 		if (stats->recv!=stream->last_packet_count){
 			stream->last_packet_count=stats->recv;
 			stream->last_packet_time=ms_time(NULL);
-		}else{
-			if (ms_time(NULL)-stream->last_packet_time>timeout){
-				/* more than timeout seconds of inactivity*/
-				return FALSE;
-			}
+		}
+	}
+	if (stats->recv!=0){
+		if (ms_time(NULL)-stream->last_packet_time>timeout){
+			/* more than timeout seconds of inactivity*/
+			return FALSE;
 		}
 	}
 	return TRUE;
@@ -192,19 +275,26 @@ static void payload_type_changed(RtpSession *session, unsigned long data){
 	int pt=rtp_session_get_recv_payload_type(stream->session);
 	audio_stream_change_decoder(stream,pt);
 }
-
-
+/*invoked from FEC capable filters*/
+static  mblk_t* audio_stream_payload_picker(MSRtpPayloadPickerContext* context,unsigned int sequence_number) {
+	return rtp_session_pick_with_cseq(((AudioStream*)(context->filter_graph_manager))->session, sequence_number);
+}
 int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char *remip,int remport,
 	int rem_rtcp_port, int payload,int jitt_comp, const char *infile, const char *outfile,
 	MSSndCard *playcard, MSSndCard *captcard, bool_t use_ec)
 {
 	RtpSession *rtps=stream->session;
-	PayloadType *pt;
+	PayloadType *pt,*tel_ev;
 	int tmp;
 	MSConnectionHelper h;
+	int sample_rate;
+	MSRtpPayloadPickerContext picker_context;
 
 	rtp_session_set_profile(rtps,profile);
 	if (remport>0) rtp_session_set_remote_addr_full(rtps,remip,remport,rem_rtcp_port);
+	if (rem_rtcp_port<=0){
+		rtp_session_enable_rtcp(rtps,FALSE);
+	}
 	rtp_session_set_payload_type(rtps,payload);
 	rtp_session_set_jitter_compensation(rtps,jitt_comp);
 
@@ -217,7 +307,6 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 	stream->dtmfgen=ms_filter_new(MS_DTMF_GEN_ID);
 	rtp_session_signal_connect(rtps,"telephone-event",(RtpCallback)on_dtmf_received,(unsigned long)stream);
 	rtp_session_signal_connect(rtps,"payload_type_changed",(RtpCallback)payload_type_changed,(unsigned long)stream);
-
 	/* creates the local part */
 	if (captcard!=NULL) stream->soundread=ms_snd_card_create_reader(captcard);
 	else {
@@ -237,6 +326,20 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 		ms_error("audiostream.c: undefined payload type.");
 		return -1;
 	}
+	tel_ev=rtp_profile_get_payload_from_mime (profile,"telephone-event");
+
+	if ( (tel_ev==NULL || ( (tel_ev->flags & PAYLOAD_TYPE_FLAG_CAN_RECV) && !(tel_ev->flags & PAYLOAD_TYPE_FLAG_CAN_SEND)))
+	    && ( strcasecmp(pt->mime_type,"pcmu")==0 || strcasecmp(pt->mime_type,"pcma")==0)){
+		/*if no telephone-event payload is usable and pcma or pcmu is used, we will generate
+		  inband dtmf*/
+		stream->dtmfgen_rtp=ms_filter_new (MS_DTMF_GEN_ID);
+	}
+	
+	if (ms_filter_call_method(stream->rtpsend,MS_FILTER_GET_SAMPLE_RATE,&sample_rate)!=0){
+		ms_error("Sample rate is unknown for RTP side !");
+		return -1;
+	}
+	
 	stream->encoder=ms_filter_create_encoder(pt->mime_type);
 	stream->decoder=ms_filter_create_decoder(pt->mime_type);
 	if ((stream->encoder==NULL) || (stream->decoder==NULL)){
@@ -244,34 +347,16 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 		ms_error("mediastream.c: No decoder available for payload %i.",payload);
 		return -1;
 	}
-
-	if (use_ec) {
-		stream->ec=ms_filter_new(MS_SPEEX_EC_ID);
-		ms_filter_call_method(stream->ec,MS_FILTER_SET_SAMPLE_RATE,&pt->clock_rate);
-		if (stream->ec_tail_len!=0)
-			ms_filter_call_method(stream->ec,MS_ECHO_CANCELLER_SET_TAIL_LENGTH,&stream->ec_tail_len);
-		if (stream->ec_delay!=0)
-			ms_filter_call_method(stream->ec,MS_ECHO_CANCELLER_SET_DELAY,&stream->ec_delay);
-		if (stream->ec_framesize!=0)
-			ms_filter_call_method(stream->ec,MS_ECHO_CANCELLER_SET_FRAMESIZE,&stream->ec_framesize);
-	}
-
-	if (stream->el_type!=ELInactive || stream->use_gc || stream->use_ng){
-		stream->volsend=ms_filter_new(MS_VOLUME_ID);
-		stream->volrecv=ms_filter_new(MS_VOLUME_ID);
-		if (stream->el_type!=ELInactive){
-			if (stream->el_type==ELControlFull) {
-				/* also reduce speaker gain when no signal - same parameters as std. noise gate */
-				int tmp=1;
-				ms_filter_call_method(stream->volrecv,MS_VOLUME_ENABLE_NOISE_GATE,&tmp);
-			}
-			ms_filter_call_method(stream->volsend,MS_VOLUME_SET_PEER,stream->volrecv);
-		}
-		if (stream->use_ng){
-			int tmp=1;
-			ms_filter_call_method(stream->volsend,MS_VOLUME_ENABLE_NOISE_GATE,&tmp);
-		}
+	if (ms_filter_has_method(stream->decoder, MS_FILTER_SET_RTP_PAYLOAD_PICKER)) {
+		ms_message(" decoder has FEC capabilities");
+		picker_context.filter_graph_manager=stream;
+		picker_context.picker=&audio_stream_payload_picker;
+		ms_filter_call_method(stream->decoder,MS_FILTER_SET_RTP_PAYLOAD_PICKER, &picker_context);
 	}
+ 	stream->volsend=ms_filter_new(MS_VOLUME_ID);
+	stream->volrecv=ms_filter_new(MS_VOLUME_ID);
+	audio_stream_enable_echo_limiter(stream,stream->el_type);
+	audio_stream_enable_noise_gate(stream,stream->use_ng);
 
 	if (stream->use_agc){
 		int tmp=1;
@@ -280,21 +365,33 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 		ms_filter_call_method(stream->volsend,MS_VOLUME_ENABLE_AGC,&tmp);
 	}
 
+	if (stream->dtmfgen)
+		ms_filter_call_method(stream->dtmfgen,MS_FILTER_SET_SAMPLE_RATE,&sample_rate);
+	if (stream->dtmfgen_rtp)
+		ms_filter_call_method(stream->dtmfgen_rtp,MS_FILTER_SET_SAMPLE_RATE,&sample_rate);
 	/* give the sound filters some properties */
-	if (ms_filter_call_method(stream->soundread,MS_FILTER_SET_SAMPLE_RATE,&pt->clock_rate) != 0) {
+	if (ms_filter_call_method(stream->soundread,MS_FILTER_SET_SAMPLE_RATE,&sample_rate) != 0) {
 		/* need to add resampler*/
 		if (stream->read_resampler == NULL) stream->read_resampler=ms_filter_new(MS_RESAMPLE_ID);
 	}
 
-	if (ms_filter_call_method(stream->soundwrite,MS_FILTER_SET_SAMPLE_RATE,&pt->clock_rate) != 0) {
+	if (ms_filter_call_method(stream->soundwrite,MS_FILTER_SET_SAMPLE_RATE,&sample_rate) != 0) {
 		/* need to add resampler*/
 		if (stream->write_resampler == NULL) stream->write_resampler=ms_filter_new(MS_RESAMPLE_ID);
 	}
 
-
 	tmp=1;
 	ms_filter_call_method(stream->soundwrite,MS_FILTER_SET_NCHANNELS, &tmp);
 
+	/*configure the echo canceller if required */
+	if (!use_ec) {
+		ms_filter_destroy(stream->ec);
+		stream->ec=NULL;
+	}
+	if (stream->ec){
+		ms_filter_call_method(stream->ec,MS_FILTER_SET_SAMPLE_RATE,&sample_rate);
+	}
+
 	/* give the encoder/decoder some parameters*/
 	ms_filter_call_method(stream->encoder,MS_FILTER_SET_SAMPLE_RATE,&pt->clock_rate);
 	ms_message("Payload's bitrate is %i",pt->normal_bitrate);
@@ -319,6 +416,12 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 	if (stream->write_resampler){
 		audio_stream_configure_resampler(stream->write_resampler,stream->rtprecv,stream->soundwrite);
 	}
+
+	if (stream->use_rc){
+		stream->rc=ms_audio_bitrate_controller_new(stream->session,stream->encoder,0);
+	}
+	stream->qi=ms_quality_indicator_new(stream->session);
+	
 	/* and then connect all */
 	/* tip: draw yourself the picture if you don't understand */
 
@@ -331,6 +434,8 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 		ms_connection_helper_link(&h,stream->ec,1,1);
 	if (stream->volsend)
 		ms_connection_helper_link(&h,stream->volsend,0,0);
+	if (stream->dtmfgen_rtp)
+		ms_connection_helper_link(&h,stream->dtmfgen_rtp,0,0);
 	ms_connection_helper_link(&h,stream->encoder,0,0);
 	ms_connection_helper_link(&h,stream->rtpsend,0,-1);
 
@@ -339,10 +444,10 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 	ms_connection_helper_link(&h,stream->rtprecv,-1,0);
 	ms_connection_helper_link(&h,stream->decoder,0,0);
 	ms_connection_helper_link(&h,stream->dtmfgen,0,0);
-	if (stream->equalizer)
-		ms_connection_helper_link(&h,stream->equalizer,0,0);
 	if (stream->volrecv)
 		ms_connection_helper_link(&h,stream->volrecv,0,0);
+	if (stream->equalizer)
+		ms_connection_helper_link(&h,stream->equalizer,0,0);
 	if (stream->ec)
 		ms_connection_helper_link(&h,stream->ec,0,0);
 	if (stream->write_resampler)
@@ -352,12 +457,19 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 	/* create ticker */
 	stream->ticker=ms_ticker_new();
 	ms_ticker_set_name(stream->ticker,"Audio MSTicker");
+	ms_ticker_set_priority(stream->ticker,__ms_get_default_prio(FALSE));
 	ms_ticker_attach(stream->ticker,stream->soundread);
 	ms_ticker_attach(stream->ticker,stream->rtprecv);
 
+	stream->start_time=ms_time(NULL);
+	stream->is_beginning=TRUE;
+
 	return 0;
 }
 
+void audio_stream_enable_adaptive_bitrate_control(AudioStream *st, bool_t enabled){
+	st->use_rc=enabled;
+}
 
 int audio_stream_start_with_files(AudioStream *stream, RtpProfile *prof,const char *remip, int remport,
 	int rem_rtcp_port, int pt,int jitt_comp, const char *infile, const char * outfile)
@@ -431,8 +543,22 @@ void audio_stream_record(AudioStream *st, const char *name){
 
 AudioStream *audio_stream_new(int locport, bool_t ipv6){
 	AudioStream *stream=(AudioStream *)ms_new0(AudioStream,1);
+	MSFilterDesc *ec_desc=ms_filter_lookup_by_name("MSOslec");
+	
+	ms_filter_enable_statistics(TRUE);
+	ms_filter_reset_statistics();
+	
 	stream->session=create_duplex_rtpsession(locport,ipv6);
+	/*some filters are created right now to allow configuration by the application before start() */
 	stream->rtpsend=ms_filter_new(MS_RTP_SEND_ID);
+	
+	if (ec_desc!=NULL)
+		stream->ec=ms_filter_new_from_desc(ec_desc);
+	else
+		stream->ec=ms_filter_new(MS_SPEEX_EC_ID);
+
+	stream->evq=ortp_ev_queue_new();
+	rtp_session_register_event_queue(stream->session,stream->evq);
 	stream->play_dtmfs=TRUE;
 	stream->use_gc=FALSE;
 	stream->use_agc=FALSE;
@@ -453,14 +579,25 @@ void audio_stream_set_relay_session_id(AudioStream *stream, const char *id){
 	ms_filter_call_method(stream->rtpsend, MS_RTP_SEND_SET_RELAY_SESSION_ID,(void*)id);
 }
 
-void audio_stream_set_echo_canceller_params(AudioStream *st, int tail_len_ms, int delay_ms, int framesize){
-	st->ec_tail_len=tail_len_ms;
-	st->ec_delay=delay_ms;
-	st->ec_framesize=framesize;
+void audio_stream_set_echo_canceller_params(AudioStream *stream, int tail_len_ms, int delay_ms, int framesize){
+	if (tail_len_ms!=0)
+		ms_filter_call_method(stream->ec,MS_ECHO_CANCELLER_SET_TAIL_LENGTH,&tail_len_ms);
+	if (delay_ms!=0){
+		ms_filter_call_method(stream->ec,MS_ECHO_CANCELLER_SET_DELAY,&delay_ms);
+	}
+	if (framesize!=0)
+		ms_filter_call_method(stream->ec,MS_ECHO_CANCELLER_SET_FRAMESIZE,&framesize);
 }
 
 void audio_stream_enable_echo_limiter(AudioStream *stream, EchoLimiterType type){
 	stream->el_type=type;
+	if (stream->volsend){
+		bool_t enable_noise_gate = stream->el_type==ELControlFull;
+		ms_filter_call_method(stream->volrecv,MS_VOLUME_ENABLE_NOISE_GATE,&enable_noise_gate);
+		ms_filter_call_method(stream->volsend,MS_VOLUME_SET_PEER,type!=ELInactive?stream->volrecv:NULL);
+	} else {
+		ms_warning("cannot set echo limiter to mode [%i] because no volume send",type);
+	}
 }
 
 void audio_stream_enable_gain_control(AudioStream *stream, bool_t val){
@@ -473,6 +610,13 @@ void audio_stream_enable_automatic_gain_control(AudioStream *stream, bool_t val)
 
 void audio_stream_enable_noise_gate(AudioStream *stream, bool_t val){
 	stream->use_ng=val;
+	if (stream->volsend){
+		ms_filter_call_method(stream->volsend,MS_VOLUME_ENABLE_NOISE_GATE,&val);
+	} else {
+		ms_warning("cannot set noise gate mode to [%i] because no volume send",val);
+	}
+
+
 }
 
 void audio_stream_set_mic_gain(AudioStream *stream, float gain){
@@ -518,6 +662,8 @@ void audio_stream_stop(AudioStream * stream)
 			ms_connection_helper_unlink(&h,stream->ec,1,1);
 		if (stream->volsend!=NULL)
 			ms_connection_helper_unlink(&h,stream->volsend,0,0);
+		if (stream->dtmfgen_rtp)
+			ms_connection_helper_unlink(&h,stream->dtmfgen_rtp,0,0);
 		ms_connection_helper_unlink(&h,stream->encoder,0,0);
 		ms_connection_helper_unlink(&h,stream->rtpsend,0,-1);
 
@@ -526,10 +672,10 @@ void audio_stream_stop(AudioStream * stream)
 		ms_connection_helper_unlink(&h,stream->rtprecv,-1,0);
 		ms_connection_helper_unlink(&h,stream->decoder,0,0);
 		ms_connection_helper_unlink(&h,stream->dtmfgen,0,0);
-		if (stream->equalizer)
-			ms_connection_helper_unlink(&h,stream->equalizer,0,0);
 		if (stream->volrecv!=NULL)
 			ms_connection_helper_unlink(&h,stream->volrecv,0,0);
+		if (stream->equalizer)
+			ms_connection_helper_unlink(&h,stream->equalizer,0,0);
 		if (stream->ec!=NULL)
 			ms_connection_helper_unlink(&h,stream->ec,0,0);
 		if (stream->write_resampler!=NULL)
@@ -538,6 +684,7 @@ void audio_stream_stop(AudioStream * stream)
 
 	}
 	audio_stream_free(stream);
+	ms_filter_log_statistics();
 }
 
 RingStream * ring_start(const char *file, int interval, MSSndCard *sndcard){
@@ -548,34 +695,75 @@ RingStream * ring_start_with_cb(const char *file,int interval,MSSndCard *sndcard
 {
 	RingStream *stream;
 	int tmp;
+	int srcrate,dstrate;
+	MSConnectionHelper h;
+
 	stream=(RingStream *)ms_new0(RingStream,1);
 	stream->source=ms_filter_new(MS_FILE_PLAYER_ID);
-	if (ms_filter_call_method(stream->source,MS_FILE_PLAYER_OPEN,(void*)file)<0){
-		ms_filter_destroy(stream->source);
-		ms_free(stream);
-		return NULL;
-	}
+	if (file)
+		ms_filter_call_method(stream->source,MS_FILE_PLAYER_OPEN,(void*)file);
+	
 	ms_filter_call_method(stream->source,MS_FILE_PLAYER_LOOP,&interval);
 	ms_filter_call_method_noarg(stream->source,MS_FILE_PLAYER_START);
 	if (func!=NULL)
 		ms_filter_set_notify_callback(stream->source,func,user_data);
+	stream->gendtmf=ms_filter_new(MS_DTMF_GEN_ID);
+	
+	
 	stream->sndwrite=ms_snd_card_create_writer(sndcard);
-	ms_filter_call_method(stream->source,MS_FILTER_GET_SAMPLE_RATE,&tmp);
-	ms_filter_call_method(stream->sndwrite,MS_FILTER_SET_SAMPLE_RATE,&tmp);
+	ms_filter_call_method(stream->source,MS_FILTER_GET_SAMPLE_RATE,&srcrate);
+	ms_filter_call_method(stream->gendtmf,MS_FILTER_SET_SAMPLE_RATE,&srcrate);
+	ms_filter_call_method(stream->sndwrite,MS_FILTER_SET_SAMPLE_RATE,&srcrate);
+	ms_filter_call_method(stream->sndwrite,MS_FILTER_GET_SAMPLE_RATE,&dstrate);
+	if (srcrate!=dstrate){
+		stream->write_resampler=ms_filter_new(MS_RESAMPLE_ID);
+		ms_filter_call_method(stream->write_resampler,MS_FILTER_SET_SAMPLE_RATE,&srcrate);
+		ms_filter_call_method(stream->write_resampler,MS_FILTER_SET_OUTPUT_SAMPLE_RATE,&dstrate);
+		ms_message("configuring resampler from rate[%i] to rate [%i]", srcrate,dstrate);
+	}
 	ms_filter_call_method(stream->source,MS_FILTER_GET_NCHANNELS,&tmp);
+	ms_filter_call_method(stream->gendtmf,MS_FILTER_SET_NCHANNELS,&tmp);
 	ms_filter_call_method(stream->sndwrite,MS_FILTER_SET_NCHANNELS,&tmp);
+	
 	stream->ticker=ms_ticker_new();
+	
 	ms_ticker_set_name(stream->ticker,"Audio (ring) MSTicker");
-	ms_filter_link(stream->source,0,stream->sndwrite,0);
+
+	ms_connection_helper_start(&h);
+	ms_connection_helper_link(&h,stream->source,-1,0);
+	ms_connection_helper_link(&h,stream->gendtmf,0,0);
+	if (stream->write_resampler)
+		ms_connection_helper_link(&h,stream->write_resampler,0,0);
+	ms_connection_helper_link(&h,stream->sndwrite,0,-1);
 	ms_ticker_attach(stream->ticker,stream->source);
+	
 	return stream;
 }
 
+void ring_play_dtmf(RingStream *stream, char dtmf, int duration_ms){
+	if (duration_ms>0)
+		ms_filter_call_method(stream->gendtmf, MS_DTMF_GEN_PLAY, &dtmf);
+	else ms_filter_call_method(stream->gendtmf, MS_DTMF_GEN_START, &dtmf);
+}
+
+void ring_stop_dtmf(RingStream *stream){
+	ms_filter_call_method_noarg(stream->gendtmf, MS_DTMF_GEN_STOP);
+}
+
 void ring_stop(RingStream *stream){
+	MSConnectionHelper h;
 	ms_ticker_detach(stream->ticker,stream->source);
-	ms_filter_unlink(stream->source,0,stream->sndwrite,0);
+
+	ms_connection_helper_start(&h);
+	ms_connection_helper_unlink(&h,stream->source,-1,0);
+	ms_connection_helper_unlink(&h,stream->gendtmf,0,0);
+	if (stream->write_resampler)
+		ms_connection_helper_unlink(&h,stream->write_resampler,0,0);
+	ms_connection_helper_unlink(&h,stream->sndwrite,0,-1);
+
 	ms_ticker_destroy(stream->ticker);
 	ms_filter_destroy(stream->source);
+	ms_filter_destroy(stream->gendtmf);
 	ms_filter_destroy(stream->sndwrite);
 	ms_free(stream);
 #ifdef _WIN32_WCE
@@ -587,10 +775,10 @@ void ring_stop(RingStream *stream){
 
 int audio_stream_send_dtmf(AudioStream *stream, char dtmf)
 {
-	if (stream->rtpsend)
+	if (stream->dtmfgen_rtp)
+		ms_filter_call_method(stream->dtmfgen_rtp,MS_DTMF_GEN_PLAY,&dtmf);
+	else if (stream->rtpsend)
 		ms_filter_call_method(stream->rtpsend,MS_RTP_SEND_SEND_DTMF,&dtmf);
-	if (stream->dtmfgen)
-		ms_filter_call_method(stream->dtmfgen,MS_DTMF_GEN_PUT,&dtmf);
 	return 0;
 }
 
@@ -604,10 +792,59 @@ void audio_stream_get_local_rtp_stats(AudioStream *stream, rtp_stats_t *lstats){
 
 void audio_stream_mute_rtp(AudioStream *stream, bool_t val) 
 {
-  if (stream->rtpsend){
-    if (val)
-      ms_filter_call_method(stream->rtpsend,MS_RTP_SEND_MUTE_MIC,&val);
-    else
-      ms_filter_call_method(stream->rtpsend,MS_RTP_SEND_UNMUTE_MIC,&val);
-  }
+	if (stream->rtpsend){
+		if (val)
+			ms_filter_call_method(stream->rtpsend,MS_RTP_SEND_MUTE_MIC,&val);
+		else
+			ms_filter_call_method(stream->rtpsend,MS_RTP_SEND_UNMUTE_MIC,&val);
+	}
+}
+
+float audio_stream_get_quality_rating(AudioStream *stream){
+	if (stream->qi){
+		return ms_quality_indicator_get_rating(stream->qi);
+	}
+	return 0;
+}
+
+MS2_PUBLIC float audio_stream_get_average_quality_rating(AudioStream *stream){
+	if (stream->qi){
+		return ms_quality_indicator_get_average_rating(stream->qi);
+	}
+	return 0;
+}
+
+
+void audio_stream_enable_zrtp(AudioStream *stream, OrtpZrtpParams *params){
+	stream->ortpZrtpContext=ortp_zrtp_context_new(stream->session, params);
+}
+
+bool_t audio_stream_enable_strp(AudioStream* stream, enum ortp_srtp_crypto_suite_t suite, const char* snd_key, const char* rcv_key) {
+	// assign new srtp transport to stream->session
+	// with 2 Master Keys
+	RtpTransport *rtp_tpt, *rtcp_tpt;	
+	
+	if (!ortp_srtp_supported()) {
+		ms_error("ortp srtp support not enabled");
+		return FALSE;
+	}
+	
+	ms_message("%s: stream=%p key='%s' key='%s'", __FUNCTION__,
+		stream, snd_key, rcv_key);
+	 
+	stream->srtp_session = ortp_srtp_create_configure_session(suite, 
+		rtp_session_get_send_ssrc(stream->session), 
+		snd_key, 
+		rcv_key); 
+	
+	if (!stream->srtp_session) {
+		return FALSE;
+	}
+	
+	// TODO: check who will free rtp_tpt ?
+	srtp_transport_new(stream->srtp_session, &rtp_tpt, &rtcp_tpt);
+	
+	rtp_session_set_transports(stream->session, rtp_tpt, rtcp_tpt);
+	
+	return TRUE;
 }
