@@ -1,6 +1,6 @@
 /*
 mediastreamer2 library - modular sound and video processing and streaming
-Copyright (C) 2006  Simon MORLAT (simon.morlat@linphone.org)
+Copyright (C) 2010  Belledonne Communications SARL, Grenoble France.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -24,10 +24,12 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "mediastreamer2/msfilter.h"
 #include "mediastreamer2/msvideo.h"
 
-#include "ffmpeg-priv.h"
+#include "layouts.h"
 
 #define SCALE_FACTOR 4.0f
 #define SELVIEW_POS_INACTIVE -100.0
+#define LOCAL_BORDER_SIZE 2
+
 #include <Vfw.h>
 
 typedef struct Yuv2RgbCtx{
@@ -35,7 +37,7 @@ typedef struct Yuv2RgbCtx{
 	size_t rgblen;
 	MSVideoSize dsize;
 	MSVideoSize ssize;
-	struct ms_SwsContext *sws;
+	MSScalerContext *sws;
 }Yuv2RgbCtx;
 
 static void yuv2rgb_init(Yuv2RgbCtx *ctx){
@@ -55,7 +57,7 @@ static void yuv2rgb_uninit(Yuv2RgbCtx *ctx){
 		ctx->rgblen=0;
 	}
 	if (ctx->sws){
-		ms_sws_freeContext(ctx->sws);
+		ms_scaler_context_free(ctx->sws);
 		ctx->sws=NULL;
 	}
 	ctx->dsize.width=0;
@@ -66,9 +68,9 @@ static void yuv2rgb_uninit(Yuv2RgbCtx *ctx){
 
 static void yuv2rgb_prepare(Yuv2RgbCtx *ctx, MSVideoSize src, MSVideoSize dst){
 	if (ctx->sws!=NULL) yuv2rgb_uninit(ctx);
-	ctx->sws=ms_sws_getContext(src.width,src.height,PIX_FMT_YUV420P,
-			dst.width,dst.height, PIX_FMT_BGR24,
-			SWS_FAST_BILINEAR, NULL, NULL, NULL);
+	ctx->sws=ms_scaler_create_context(src.width,src.height,MS_YUV420P,
+			dst.width,dst.height, MS_RGB24_REV,
+			MS_SCALER_METHOD_BILINEAR);
 	ctx->dsize=dst;
 	ctx->ssize=src;
 	ctx->rgblen=dst.width*dst.height*3;
@@ -94,9 +96,8 @@ static void yuv2rgb_process(Yuv2RgbCtx *ctx, MSPicture *src, MSVideoSize dstsize
 		uint8_t *p;
 
 		p=ctx->rgb+(dstsize.width*3*(dstsize.height-1));
-		if (ms_sws_scale(ctx->sws,src->planes,src->strides, 0,
-           				src->h, &p, &rgb_stride)<0){
-			ms_error("Error in 420->rgb ms_sws_scale().");
+		if (ms_scaler_process(ctx->sws,src->planes,src->strides, &p, &rgb_stride)<0){
+			ms_error("Error in 420->rgb ms_scaler_process().");
 		}
 		if (mirroring) rgb24_mirror(ctx->rgb,dstsize.width,dstsize.height,dstsize.width*3);
 	}
@@ -271,133 +272,6 @@ static void dd_display_preprocess(MSFilter *f){
 }
 
 
-/* compute the ideal placement of the video within a window of size wsize,
-given that the original video has size vsize. Put the result in rect*/
-static void center_with_ratio(MSVideoSize wsize, MSVideoSize vsize, MSRect *rect){
-	int w,h;
-	w=wsize.width & ~0x3;
-	h=((w*vsize.height)/vsize.width) & ~0x1;
-	if (h>wsize.height){
-		/*the height doesn't fit, so compute the width*/
-		h=wsize.height & ~0x1;
-		w=((h*vsize.width)/vsize.height) & ~0x3;
-	}
-	rect->x=(wsize.width-w)/2;
-	rect->y=(wsize.height-h)/2;
-	rect->w=w;
-	rect->h=h;
-}
-
-#define LOCAL_BORDER_SIZE 2
-#define LOCAL_POS_OFFSET 10
-
-static void compute_layout(MSVideoSize wsize, MSVideoSize vsize, MSVideoSize orig_psize, MSRect *mainrect, MSRect *localrect, int localrect_pos, float scalefactor){
-	MSVideoSize psize;
-
-	center_with_ratio(wsize,vsize,mainrect);
-	if (localrect_pos!=-1){
-		psize.width=(int)(wsize.width/scalefactor);
-		psize.height=(int)(wsize.height/scalefactor);
-		center_with_ratio(psize,orig_psize,localrect);
-		if ((wsize.height - mainrect->h < mainrect->h/scalefactor && wsize.width - mainrect->w < mainrect->w/scalefactor) || localrect_pos<=3)
-		{
-			int x_sv;
-			int y_sv;
-			if (localrect_pos%4==1)
-			{
-				/* top left corner */
-				x_sv = LOCAL_POS_OFFSET;
-				y_sv = LOCAL_POS_OFFSET;
-			}
-			else if (localrect_pos%4==2)
-			{
-				/* top right corner */
-				x_sv = (wsize.width-localrect->w-LOCAL_POS_OFFSET);
-				y_sv = LOCAL_POS_OFFSET;
-			}
-			else if (localrect_pos%4==3)
-			{
-				/* bottom left corner */
-				x_sv = LOCAL_POS_OFFSET;
-				y_sv = (wsize.height-localrect->h-LOCAL_POS_OFFSET);
-			}
-			else /* corner = 0: default */
-			{
-				/* bottom right corner */
-				x_sv = (wsize.width-localrect->w-LOCAL_POS_OFFSET);
-				y_sv = (wsize.height-localrect->h-LOCAL_POS_OFFSET);
-			}
-			localrect->x=x_sv; //wsize.width-localrect->w-LOCAL_POS_OFFSET;
-			localrect->y=y_sv; //wsize.height-localrect->h-LOCAL_POS_OFFSET;
-		}
-		else
-		{
-			int x_sv;
-			int y_sv;
-
-			if (wsize.width - mainrect->w < mainrect->w/scalefactor)
-			{
-				// recalculate so we have a selfview taking as
-				// much available space as possible
-				psize.width=wsize.width;
-				psize.height=wsize.height-mainrect->h;
-				center_with_ratio(psize,orig_psize,localrect);
-
-				if (localrect_pos%4==1 || localrect_pos%4==2)
-				{
-					//Self View on Top
-					x_sv = (wsize.width-localrect->w)/2;
-					y_sv = LOCAL_POS_OFFSET;
-
-					mainrect->y = wsize.height-mainrect->h-LOCAL_POS_OFFSET;
-				}
-				else
-				{
-					//Self View on Bottom
-					x_sv = (wsize.width-localrect->w)/2;
-					y_sv = (wsize.height-localrect->h-LOCAL_POS_OFFSET);
-
-					mainrect->y = LOCAL_POS_OFFSET;
-				}
-			}
-			else
-			{
-				// recalculate so we have a selfview taking as
-				// much available space as possible
-				psize.width=wsize.width-mainrect->w;
-				psize.height=wsize.height;
-				center_with_ratio(psize,orig_psize,localrect);
-
-				if (localrect_pos%4==1 || localrect_pos%4==3)
-				{
-					//Self View on left
-					x_sv = LOCAL_POS_OFFSET;
-					y_sv = (wsize.height-localrect->h)/2;
-
-					mainrect->x = wsize.width-mainrect->w-LOCAL_POS_OFFSET;
-				}
-				else
-				{
-					//Self View on right
-					x_sv = (wsize.width-localrect->w-LOCAL_POS_OFFSET);
-					y_sv = (wsize.height-localrect->h)/2;
-
-					mainrect->x = LOCAL_POS_OFFSET;
-				}
-			}
-
-			localrect->x=x_sv; //wsize.width-localrect->w-LOCAL_POS_OFFSET;
-			localrect->y=y_sv; //wsize.height-localrect->h-LOCAL_POS_OFFSET;
-		}
-	}
-/*
-	ms_message("Compute layout result for\nwindow size=%ix%i\nvideo orig size=%ix%i\nlocal size=%ix%i\nlocal orig size=%ix%i\n"
-		"mainrect=%i,%i,%i,%i\tlocalrect=%i,%i,%i,%i",
-		wsize.width,wsize.height,vsize.width,vsize.height,psize.width,psize.height,orig_psize.width,orig_psize.height,
-		mainrect->x,mainrect->y,mainrect->w,mainrect->h,
-		localrect->x,localrect->y,localrect->w,localrect->h);
-*/
-}
 
 static void draw_local_view_frame(HDC hdc, MSVideoSize wsize, MSRect localrect){
 	Rectangle(hdc, localrect.x-LOCAL_BORDER_SIZE, localrect.y-LOCAL_BORDER_SIZE,
@@ -479,14 +353,14 @@ static void dd_display_process(MSFilter *f){
 	obj->wsize=wsize;
 	/*get most recent message and draw it*/
 	if (corner!=-1 && f->inputs[1]!=NULL && (local_im=ms_queue_peek_last(f->inputs[1]))!=NULL) {
-		if (yuv_buf_init_from_mblk(&localpic,local_im)==0){
+		if (ms_yuv_buf_init_from_mblk(&localpic,local_im)==0){
 			obj->lsize.width=localpic.w;
 			obj->lsize.height=localpic.h;
 		}
 	}
 	
 	if (f->inputs[0]!=NULL && (main_im=ms_queue_peek_last(f->inputs[0]))!=NULL) {
-		if (yuv_buf_init_from_mblk(&mainpic,main_im)==0){
+		if (ms_yuv_buf_init_from_mblk(&mainpic,main_im)==0){
 			if (obj->autofit && (obj->vsize.width!=mainpic.w || obj->vsize.height!=mainpic.h)
 				&& (mainpic.w>wsize.width || mainpic.h>wsize.height)){
 				RECT cur;
@@ -503,7 +377,7 @@ static void dd_display_process(MSFilter *f){
 	}
 
 	if (main_im!=NULL || local_im!=NULL || obj->need_repaint){
-		compute_layout(wsize,obj->vsize,obj->lsize,&mainrect,&localrect,corner, scalefactor);
+		ms_layout_compute(wsize,obj->vsize,obj->lsize,corner,scalefactor,&mainrect,&localrect);
 		vsize.width=mainrect.w;
 		vsize.height=mainrect.h;
 		lsize.width=localrect.w;
@@ -631,7 +505,7 @@ static int set_scalefactor(MSFilter *f,void *arg){
 	return 0;
 }
 
-
+#if 0
 static int set_selfview_pos(MSFilter *f,void *arg){
 	DDDisplay *s=(DDDisplay*)f->data;
 	s->sv_posx=((float*)arg)[0];
@@ -647,6 +521,7 @@ static int get_selfview_pos(MSFilter *f,void *arg){
 	((float*)arg)[2]=(float)100.0/s->sv_scalefactor;
 	return 0;
 }
+#endif
 
 static int set_background_color(MSFilter *f,void *arg){
 	DDDisplay *s=(DDDisplay*)f->data;
@@ -665,8 +540,6 @@ static MSFilterMethod methods[]={
 	{	MS_VIDEO_DISPLAY_ENABLE_MIRRORING	,	enable_mirroring},
 	{	MS_VIDEO_DISPLAY_SET_LOCAL_VIEW_MODE	, set_corner },
 	{	MS_VIDEO_DISPLAY_SET_LOCAL_VIEW_SCALEFACTOR	, set_scalefactor },
-	{	MS_VIDEO_DISPLAY_SET_SELFVIEW_POS 	 ,	set_selfview_pos},
-	{	MS_VIDEO_DISPLAY_GET_SELFVIEW_POS    ,  get_selfview_pos},
 	{	MS_VIDEO_DISPLAY_SET_BACKGROUND_COLOR    ,  set_background_color},
 	{	0	,NULL}
 };
