@@ -36,24 +36,29 @@ typedef struct EncState{
 	MSBufferizer *bufferizer;
 } EncState;
 
+static int set_ptime(MSFilter *f, int ptime){
+	EncState *s=(EncState*)f->data;
+	if (ptime<=0 || ptime>140) return -1;
+	s->ptime=(ptime/20)*20;
+	ms_message("MSGsmEnc: got ptime=%i using [%i]",ptime,s->ptime);
+	return 0;
+}
+
 static int enc_add_fmtp(MSFilter *f, void *arg){
 	const char *fmtp=(const char *)arg;
-	EncState *s=(EncState*)f->data;
 	char tmp[30];
+	
 	if (fmtp_get_value(fmtp,"ptime",tmp,sizeof(tmp))){
-		int ptime = atoi(tmp);
-		switch (ptime) {
-		case 20:
-		case 40:
-		case 60:
-		case 80:
-		case 100:
-			s->ptime = atoi(tmp);
-			break;
-		default:
-			ms_warning("MSGsmEnc: unsupported ptime [%i] using default",ptime);
-		}
-		ms_message("MSGsmEnc: got ptime=%i using [%i]",ptime,s->ptime);
+		return set_ptime(f,atoi(tmp));
+	}
+	return 0;
+}
+
+static int enc_add_attr(MSFilter *f, void *arg){
+	const char *attr=(const char *)arg;
+	if (strstr(attr,"ptime:")!=NULL){
+		int ptime = atoi(attr+6);
+		return set_ptime(f,ptime);
 	}
 	return 0;
 }
@@ -88,10 +93,10 @@ static void enc_process(MSFilter *f){
 		ms_bufferizer_put(s->bufferizer,im);
 	}
 	while(ms_bufferizer_get_avail(s->bufferizer) >= buff_size) {
+		mblk_t *om=allocb(33*s->ptime/20,0);
 		buff = (int16_t *)alloca(buff_size);
 		ms_bufferizer_read(s->bufferizer,(uint8_t*)buff,buff_size);
-		mblk_t *om=allocb(33*s->ptime/20,0);
-
+		
 		for (offset=0;offset<buff_size;offset+=unitary_buff_size) {
 			gsm_encode(s->state,(gsm_signal*)&buff[offset/sizeof(int16_t)],(gsm_byte*)om->b_wptr);
 			om->b_wptr+=33;
@@ -103,6 +108,7 @@ static void enc_process(MSFilter *f){
 }
 static MSFilterMethod enc_methods[]={
 	{	MS_FILTER_ADD_FMTP		,	enc_add_fmtp},
+	{    MS_FILTER_ADD_ATTR        ,    enc_add_attr},
 	{	0				,	NULL		}
 };
 
