@@ -19,12 +19,26 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #ifdef HAVE_CONFIG_H
 #include "mediastreamer-config.h"
+#include "gitversion.h"
+#else
+#   ifndef MEDIASTREAMER_VERSION
+#   define MEDIASTREAMER_VERSION "unknown"
+#   endif
+#	ifndef GIT_VERSION
+#	define GIT_VERSION "unknown"
+#	endif
 #endif
 
-extern void __register_ffmpeg_encoders_if_possible(void);
-
 #include "mediastreamer2/mscommon.h"
+#include "mediastreamer2/mscodecutils.h"
 #include "mediastreamer2/msfilter.h"
+#include <ortp/ortp_srtp.h>
+
+extern void __register_ffmpeg_encoders_if_possible(void);
+extern void ms_ffmpeg_check_init();
+extern bool_t libmsandroiddisplay_init(void);
+extern void libmsandroiddisplaybad_init(void);
+extern void libmsandroidopengldisplay_init(void);
 
 #include "alldescs.h"
 #include "mediastreamer2/mssndcard.h"
@@ -38,7 +52,7 @@ extern void __register_ffmpeg_encoders_if_possible(void);
 #else
 #ifndef PACKAGE_PLUGINS_DIR
 #if defined(WIN32) || defined(_WIN32_WCE)
-#define PACKAGE_PLUGINS_DIR "plugins\\"
+#define PACKAGE_PLUGINS_DIR "lib\\mediastreamer\\plugins\\"
 #else
 #define PACKAGE_PLUGINS_DIR "."
 #endif
@@ -50,13 +64,17 @@ extern void __register_ffmpeg_encoders_if_possible(void);
 
 #if defined(__APPLE__) && !defined(__GNUC__)
 #import <Cocoa/Cocoa.h>
-#include <Foundation/Foundation.h> 
+#include <Foundation/Foundation.h>
 #endif
 
 #ifdef ANDROID
 #include <android/log.h>
 #endif
 
+#if defined(WIN32) && !defined(_WIN32_WCE)
+static MSList *ms_plugins_loaded_list;
+#endif
+
 MSList *ms_list_new(void *data){
 	MSList *new_elem=(MSList *)ms_new(MSList,1);
 	new_elem->prev=new_elem->next=NULL;
@@ -162,7 +180,7 @@ MSList *ms_list_find(MSList *list, void *data){
 	return NULL;
 }
 
-MSList *ms_list_find_custom(MSList *list, int (*compare_func)(const void *, const void*), void *user_data){
+MSList *ms_list_find_custom(MSList *list, int (*compare_func)(const void *, const void*), const void *user_data){
 	for(;list!=NULL;list=list->next){
 		if (compare_func(list->data,user_data)==0) return list;
 	}
@@ -252,21 +270,18 @@ MSList *ms_list_copy(const MSList *list){
 }
 
 
-#ifdef __APPLE__
-#define PLUGINS_EXT ".dylib"
-#else
-#define PLUGINS_EXT ".so"
+#ifndef PLUGINS_EXT
+	#define PLUGINS_EXT ".so"
 #endif
-
 typedef void (*init_func_t)(void);
 
 int ms_load_plugins(const char *dir){
 	int num=0;
 #if defined(WIN32) && !defined(_WIN32_WCE)
-	WIN32_FIND_DATA FileData; 
-	HANDLE hSearch; 
-	char szDirPath[1024]; 
-	char szPluginFile[1024]; 
+	WIN32_FIND_DATA FileData;
+	HANDLE hSearch;
+	char szDirPath[1024];
+	char szPluginFile[1024];
 	BOOL fFinished = FALSE;
 	const char *tmp=getenv("DEBUG");
 	BOOL debug=(tmp!=NULL && atoi(tmp)==1);
@@ -283,13 +298,13 @@ int ms_load_plugins(const char *dir){
 	}
 	snprintf(szDirPath, sizeof(szDirPath), "%s", dir);
 
-	while (!fFinished) 
+	while (!fFinished)
 	{
 		/* load library */
 		HINSTANCE os_handle;
 		UINT em;
 		if (!debug) em = SetErrorMode (SEM_FAILCRITICALERRORS);
-		
+
 		snprintf(szPluginFile, sizeof(szPluginFile), "%s\\%s", szDirPath, FileData.cFileName);
 		os_handle = LoadLibraryEx (szPluginFile, NULL, LOAD_WITH_ALTERED_SEARCH_PATH);
 		if (os_handle==NULL)
@@ -299,10 +314,10 @@ int ms_load_plugins(const char *dir){
 		}
 		if (!debug) SetErrorMode (em);
 		if (os_handle==NULL)
-			ms_error("Fail to load plugin %s", szPluginFile); 
+			ms_error("Fail to load plugin %s", szPluginFile);
 		else{
 			init_func_t initroutine;
-			char szPluginName[256]; 
+			char szPluginName[256];
 			char szMethodName[256];
 			char *minus;
 			snprintf(szPluginName, 256, "%s", FileData.cFileName);
@@ -315,6 +330,8 @@ int ms_load_plugins(const char *dir){
 				if (initroutine!=NULL){
 					initroutine();
 					ms_message("Plugin loaded (%s)", szPluginFile);
+					// Add this new loaded plugin to the list (useful for FreeLibrary at the end)
+					ms_plugins_loaded_list=ms_list_append(ms_plugins_loaded_list,os_handle);
 					num++;
 				}else{
 					ms_warning("Could not locate init routine of plugin %s. Should be %s",
@@ -322,16 +339,16 @@ int ms_load_plugins(const char *dir){
 				}
 		}
 		if (!FindNextFile(hSearch, &FileData)) {
-			if (GetLastError() == ERROR_NO_MORE_FILES){ 
-				fFinished = TRUE; 
-			} 
-			else 
-			{ 
-				ms_error("couldn't find next plugin dll."); 
-				fFinished = TRUE; 
-			} 
+			if (GetLastError() == ERROR_NO_MORE_FILES){
+				fFinished = TRUE;
+			}
+			else
+			{
+				ms_error("couldn't find next plugin dll.");
+				fFinished = TRUE;
+			}
 		}
-	} 
+	}
 	/* Close the search handle. */
 	FindClose(hSearch);
 
@@ -339,7 +356,7 @@ int ms_load_plugins(const char *dir){
 	DIR *ds;
 	struct dirent *de;
 	char *fullpath;
-	ds=opendir(dir);	
+	ds=opendir(dir);
 	if (ds==NULL){
 		ms_message("Cannot open directory %s: %s",dir,strerror(errno));
 		return -1;
@@ -350,7 +367,7 @@ int ms_load_plugins(const char *dir){
 			void *handle;
 			fullpath=ms_strdup_printf("%s/%s",dir,de->d_name);
 			ms_message("Loading plugin %s...",fullpath);
-			
+
 			if ( (handle=dlopen(fullpath,RTLD_NOW))==NULL){
 				ms_warning("Fail to load plugin %s : %s",fullpath,dlerror());
 			}else {
@@ -398,6 +415,19 @@ int ms_load_plugins(const char *dir){
 	return num;
 }
 
+void ms_unload_plugins(){
+#if defined(WIN32) && !defined(_WIN32_WCE)
+	 MSList *elem;
+
+	for(elem=ms_plugins_loaded_list;elem!=NULL;elem=elem->next)
+	{
+		HINSTANCE handle=(HINSTANCE )elem->data;
+        FreeLibrary(handle) ;
+	}
+
+	ms_list_free(ms_plugins_loaded_list);
+#endif
+}
 
 #ifdef __ALSA_ENABLED__
 extern MSSndCardDesc alsa_card_desc;
@@ -435,6 +465,14 @@ extern MSSndCardDesc aq_card_desc;
 extern MSSndCardDesc pulse_card_desc;
 #endif
 
+#if TARGET_OS_IPHONE
+extern MSSndCardDesc au_card_desc;
+#endif
+
+#ifdef ANDROID
+extern MSSndCardDesc msandroid_sound_card_desc;
+#endif
+
 static MSSndCardDesc * ms_snd_card_descs[]={
 #ifdef __ALSA_ENABLED__
 	&alsa_card_desc,
@@ -463,12 +501,19 @@ static MSSndCardDesc * ms_snd_card_descs[]={
 #ifdef __PULSEAUDIO_ENABLED__
 	&pulse_card_desc,
 #endif
-	NULL
+
+#if TARGET_OS_IPHONE
+	&au_card_desc,
+#endif
+#ifdef ANDROID
+	&msandroid_sound_card_desc,
+#endif
+NULL
 };
 
 #ifdef VIDEO_ENABLED
 
-#ifdef __linux
+#ifdef HAVE_LINUX_VIDEODEV_H
 extern MSWebCamDesc v4l_desc;
 #endif
 
@@ -479,11 +524,12 @@ extern MSWebCamDesc v4l2_card_desc;
 #ifdef WIN32
 extern MSWebCamDesc ms_vfw_cam_desc;
 #endif
+
 #if defined(WIN32) && defined(HAVE_DIRECTSHOW)
 extern MSWebCamDesc ms_directx_cam_desc;
 #endif
 
-#ifdef __MINGW32__
+#if defined(__MINGW32__) || defined (HAVE_DIRECTSHOW)
 extern MSWebCamDesc ms_dshow_cam_desc;
 #endif
 
@@ -495,31 +541,34 @@ extern MSWebCamDesc ms_v4m_cam_desc;
 extern MSWebCamDesc static_image_desc;
 extern MSWebCamDesc mire_desc;
 #endif
+#ifdef ANDROID
+extern MSWebCamDesc ms_android_video_capture_desc;
+#endif
 
 static MSWebCamDesc * ms_web_cam_descs[]={
 #ifdef HAVE_LINUX_VIDEODEV2_H
 	&v4l2_card_desc,
 #endif
-#ifdef __linux
+#ifdef HAVE_LINUX_VIDEODEV_H
 	&v4l_desc,
 #endif
-#if defined(WIN32) && defined(HAVE_DIRECTSHOW)
-	&ms_directx_cam_desc,
-#endif
 #if defined(WIN32) && defined(HAVE_VFW)
 	&ms_vfw_cam_desc,
 #endif
-#ifdef __MINGW32__
+#if defined(__MINGW32__) || defined (HAVE_DIRECTSHOW)
 	&ms_dshow_cam_desc,
 #endif
 #ifdef __APPLE__
 	&ms_v4m_cam_desc,
 #endif
-
+#if defined (ANDROID)
+	&ms_android_video_capture_desc,
+#endif
 #if !defined(NO_FFMPEG)
 	&mire_desc,
 	&static_image_desc,
 #endif
+
 	NULL
 };
 
@@ -545,6 +594,10 @@ void ms_init(){
 	int i;
 	MSSndCardManager *cm;
 
+#if defined(ENABLE_NLS)
+	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+#endif
+
 #if !defined(_WIN32_WCE)
 	if (getenv("MEDIASTREAMER_DEBUG")!=NULL){
 		ortp_set_log_level_mask(ORTP_DEBUG|ORTP_MESSAGE|ORTP_WARNING|ORTP_ERROR|ORTP_FATAL);
@@ -554,7 +607,7 @@ void ms_init(){
 	ortp_set_log_level_mask(ORTP_MESSAGE|ORTP_WARNING|ORTP_ERROR|ORTP_FATAL);
 	ortp_set_log_handler(ms_android_log_handler);
 #endif
-	ms_message("Registering all filters...");
+	ms_message("Mediastreamer2 " MEDIASTREAMER_VERSION " (git: " GIT_VERSION ") starting.");
 	/* register builtin MSFilter's */
 	for (i=0;ms_filter_descs[i]!=NULL;i++){
 		ms_filter_register(ms_filter_descs[i]);
@@ -575,11 +628,27 @@ void ms_init(){
 		}
 	}
 #if !defined(NO_FFMPEG)
+	ms_ffmpeg_check_init();
 	__register_ffmpeg_encoders_if_possible();
 #endif
 #endif
+#ifdef PACKAGE_PLUGINS_DIR
 	ms_message("Loading plugins");
 	ms_load_plugins(PACKAGE_PLUGINS_DIR);
+#endif
+
+#if defined(ANDROID) && defined (VIDEO_ENABLED)
+	if (1) {
+		libmsandroidopengldisplay_init();
+	} else {
+		if (!libmsandroiddisplay_init()) {
+			libmsandroiddisplaybad_init();
+		}
+	}
+#endif
+
+	ortp_srtp_init();
+
 	ms_message("ms_init() done");
 }
 
@@ -589,6 +658,7 @@ void ms_exit(){
 #ifdef VIDEO_ENABLED
 	ms_web_cam_manager_destroy();
 #endif
+	ms_unload_plugins();
 }
 
 void ms_sleep(int seconds){
@@ -634,6 +704,100 @@ void ms_set_payload_max_size(int size){
 	max_payload_size=size;
 }
 
+extern void _android_key_cleanup(void*);
+
+void ms_thread_exit(void* ref_val) {
+#ifdef ANDROID
+	// due to a bug in old Bionic version
+	// cleanup of jni manually
+	// works directly with Android 2.2
+	_android_key_cleanup(NULL);
+#endif
+	ortp_thread_exit(ref_val);
+}
 
 
+#ifdef __MACH__
+#include <sys/types.h>
+#include <sys/timeb.h>
+#endif
 
+void ms_get_cur_time(MSTimeSpec *ret){
+#if defined(_WIN32_WCE) || defined(WIN32)
+	DWORD timemillis;
+#	if defined(_WIN32_WCE)
+	timemillis=GetTickCount();
+#	else
+	timemillis=timeGetTime();
+#	endif
+	ret->tv_sec=timemillis/1000;
+	ret->tv_nsec=(timemillis%1000)*1000000LL;
+#elif defined(__MACH__) && defined(__GNUC__) && (__GNUC__ >= 3)
+	struct timeval tv;
+	gettimeofday(&tv, NULL);
+	ret->tv_sec=tv.tv_sec;
+	ret->tv_nsec=tv.tv_usec*1000LL;
+#elif defined(__MACH__)
+	struct timeb time_val;
+
+	ftime (&time_val);
+	ret->tv_sec = time_val.time;
+	ret->tv_nsec = time_val.millitm * 1000000LL;
+#else
+	struct timespec ts;
+	if (clock_gettime(CLOCK_MONOTONIC,&ts)<0){
+		ms_fatal("clock_gettime() doesn't work: %s",strerror(errno));
+	}
+	ret->tv_sec=ts.tv_sec;
+	ret->tv_nsec=ts.tv_nsec;
+#endif
+}
+
+struct _MSConcealerContext {
+	uint64_t sample_time;
+	int plc_count;
+	unsigned long total_number_for_plc;
+	unsigned int max_plc_count;
+};
+
+/*** plc context begin***/
+unsigned long ms_concealer_context_get_total_number_of_plc(MSConcealerContext* obj) {
+	return obj->total_number_for_plc;
+}
+
+MSConcealerContext* ms_concealer_context_new(unsigned int max_plc_count){
+	MSConcealerContext *obj=(MSConcealerContext *) ms_new(MSConcealerContext,1);	
+	obj->sample_time=0;
+	obj->plc_count=0;
+	obj->total_number_for_plc=0;
+	obj->max_plc_count=max_plc_count;
+	return obj;
+}
+void ms_concealer_context_destroy(MSConcealerContext* context) {
+	ms_free(context);
+}
+unsigned long ms_concealer_context_get_sampling_time(MSConcealerContext* obj) {
+	return obj->sample_time;
+}
+void ms_concealer_context_set_sampling_time(MSConcealerContext* obj,unsigned long value) {
+	obj->sample_time=value;
+}
+
+
+unsigned int ms_concealer_context_is_concealement_required(MSConcealerContext* obj,uint64_t current_time) {
+	
+	if(obj->sample_time == 0) return 0; /*no valid value*/
+	
+	if (obj->sample_time < current_time && obj->plc_count<obj->max_plc_count) {
+		obj->plc_count++;
+		obj->total_number_for_plc++;
+	} else {
+		obj->plc_count=0;
+		if (obj->plc_count>=obj->max_plc_count) {
+			/*reset sample time*/
+			obj->sample_time=0;
+		}
+	}
+	return obj->plc_count;
+}
+/*** plc context end***/
