@@ -25,19 +25,16 @@ He wrote all the declarations missing to get directshow capture working
 with mingw, and provided a demo code that worked great with minimal code.
 */
 
-
-#include <windows.h>
-#include <winnls.h>
-#include <errors.h>
-#include <initguid.h>
-#include <ocidl.h>
-
-
 #include <mediastreamer2/mswebcam.h>
 #include <mediastreamer2/msfilter.h>
 #include <mediastreamer2/msticker.h>
 #include <mediastreamer2/msvideo.h>
 
+#include <initguid.h>
+#include <ocidl.h>
+
+#undef CINTERFACE
+
 template <typename _ComT>
 class ComPtr{
 	private:
@@ -148,10 +145,46 @@ DEFINE_GUID( MEDIASUBTYPE_UYVY, 0x59565955, 0x0000, 0x0010,
 DEFINE_GUID( MEDIASUBTYPE_RGB24, 0xe436eb7d, 0x524f, 0x11ce,
              0x9f, 0x53, 0x00, 0x20, 0xaf, 0x0b, 0xa7, 0x70 );
 
-using namespace std;
-
 typedef LONGLONG REFERENCE_TIME;
 
+#if 0
+    IEnumPins : public IUnknown
+    {
+    public:
+        virtual HRESULT STDMETHODCALLTYPE Next(
+            /* [in] */ ULONG cPins,
+            /* [size_is][out] */
+            __out_ecount_part(cPins, *pcFetched)  IPin **ppPins,
+            /* [out] */
+            __out_opt  ULONG *pcFetched) = 0;
+
+        virtual HRESULT STDMETHODCALLTYPE Skip(
+            /* [in] */ ULONG cPins) = 0;
+
+        virtual HRESULT STDMETHODCALLTYPE Reset( void) = 0;
+
+        virtual HRESULT STDMETHODCALLTYPE Clone(
+            /* [out] */
+            __out  IEnumPins **ppEnum) = 0;
+
+    };
+#endif
+
+#ifndef DECLARE_ENUMERATOR_
+#define DECLARE_ENUMERATOR_(I,T) \
+    DECLARE_INTERFACE_(I,IUnknown) \
+    { \
+        STDMETHOD(QueryInterface)(THIS_ REFIID,PVOID*) PURE; \
+        STDMETHOD_(ULONG,AddRef)(THIS) PURE; \
+        STDMETHOD_(ULONG,Release)(THIS) PURE; \
+		STDMETHOD(Next)(THIS_ ULONG,T*,ULONG*) PURE;\
+        STDMETHOD(Skip)(THIS_ ULONG) PURE; \
+        STDMETHOD(Reset)(THIS) PURE; \
+        STDMETHOD(Clone)(THIS_ I**) PURE; \
+    }
+#endif
+
+
 typedef struct tagVIDEOINFOHEADER {
   RECT rcSource;
   RECT rcTarget;
@@ -501,9 +534,10 @@ ComPtr< IPin > getPin( IBaseFilter *filter, PIN_DIRECTION direction, int num )
 		ms_error("Error getting pin enumerator" );
 		return retVal;
 	}
-	ULONG found;
+	ULONG found=0;
 	ComPtr< IPin > pin;
-	while ( enumPins->Next( 1, &pin, &found ) == S_OK ) {
+	while ( enumPins->Next( 1, &pin, &found ) == S_OK && found > 0) {
+		found=0; //reset for next loop
 		PIN_DIRECTION pinDirection = (PIN_DIRECTION)( -1 );
 		pin->QueryDirection( &pinDirection );
 		if ( pinDirection == direction ) {
@@ -524,7 +558,8 @@ public:
 	DSCapture(){
 		qinit(&_rq);
 		ms_mutex_init(&_mutex,NULL);
-		_vsize=MS_VIDEO_SIZE_CIF;
+		_vsize.width=MS_VIDEO_SIZE_CIF_W;
+		_vsize.height=MS_VIDEO_SIZE_CIF_H;
 		_fps=15;
 		_start_time=0;
 		_frame_count=0;
@@ -552,7 +587,8 @@ public:
 		return ret;
 	}
 	bool isTimeToSend(uint64_t ticker_time);
-	MSVideoSize getVSize()const{
+	MSVideoSize getVSize(){
+		if (!_ready) createDshowGraph(); /* so that _vsize is updated according to hardware capabilities*/
 		return _vsize;
 	}
 	void setVSize(MSVideoSize vsize){
@@ -673,9 +709,9 @@ static char * fourcc_to_char(char *str, uint32_t fcc){
 	return str;
 }
 
-static int find_best_format(ComPtr<IAMStreamConfig> streamConfig, int count, MSVideoSize *requested_size, MSPixFmt requested_fmt ){
+static int find_best_format(ComPtr<IAMStreamConfig> streamConfig, int count,MSVideoSize *requested_size, MSPixFmt requested_fmt ){
 	int i;
-	MSVideoSize best_found=(MSVideoSize){32768,32768};
+	MSVideoSize best_found={0,0};
 	int best_index=-1;
 	char fccstr[5];
 	char selected_fcc[5];
@@ -696,15 +732,15 @@ static int find_best_format(ComPtr<IAMStreamConfig> streamConfig, int count, MSV
 				MSVideoSize cur;
 				cur.width=infoHeader->bmiHeader.biWidth;
 				cur.height=infoHeader->bmiHeader.biHeight;
-				if (ms_video_size_greater_than(cur,*requested_size)){
-					if (ms_video_size_greater_than(best_found,cur)){
+				if (ms_video_size_greater_than(*requested_size,cur)){
+					if (ms_video_size_greater_than(cur,best_found)){
 						best_found=cur;
 						best_index=i;
 						fourcc_to_char(selected_fcc,infoHeader->bmiHeader.biCompression);
 					}
 				}
 			}
-		};
+		}
 		if ( mediaType->cbFormat != 0 )
 			CoTaskMemFree( (PVOID)mediaType->pbFormat );
 		if ( mediaType->pUnk != NULL ) mediaType->pUnk->Release();
@@ -1022,7 +1058,7 @@ static MSFilter * ms_dshow_create_reader(MSWebCam *obj){
 	return f;
 }
 
-MSWebCamDesc ms_dshow_cam_desc={
+extern "C" MSWebCamDesc ms_dshow_cam_desc={
 	"Directshow capture",
 	&ms_dshow_detect,
 	NULL,
