@@ -18,9 +18,25 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
 #include "mediastreamer2/msfilter.h"
-#include "mediastreamer2/mscommon.h"
 
 static MSList *desc_list=NULL;
+static bool_t statistics_enabled=FALSE;
+static MSList *stats_list=NULL;
+
+static int compare_stats_with_name(const MSFilterStats *stat, const char *name){
+	return strcmp(stat->name,name);
+}
+
+static MSFilterStats *find_or_create_stats(MSFilterDesc *desc){
+	MSList *elem=ms_list_find_custom(stats_list,(MSCompareFunc)compare_stats_with_name,desc->name);
+	MSFilterStats *ret=NULL;
+	if (elem==NULL){
+		ret=ms_new0(MSFilterStats,1);
+		ret->name=desc->name;
+		stats_list=ms_list_append(stats_list,ret);
+	}else ret=(MSFilterStats*)elem->data;
+	return ret;
+}
 
 void ms_filter_register(MSFilterDesc *desc){
 	if (desc->id==MS_FILTER_NOT_SET_ID){
@@ -31,7 +47,15 @@ void ms_filter_register(MSFilterDesc *desc){
 }
 
 void ms_filter_unregister_all(){
-	if (desc_list!=NULL) ms_list_free(desc_list);
+	if (desc_list!=NULL) {
+		ms_list_free(desc_list);
+		desc_list=NULL;
+	}
+	if (stats_list!=NULL){
+		ms_list_for_each(stats_list,ms_free);
+		ms_list_free(stats_list);
+		stats_list=NULL;
+	}
 }
 
 bool_t ms_filter_codec_supported(const char *mime){
@@ -83,6 +107,10 @@ MSFilter *ms_filter_new_from_desc(MSFilterDesc *desc){
 	obj->desc=desc;
 	if (desc->ninputs>0)	obj->inputs=(MSQueue**)ms_new0(MSQueue*,desc->ninputs);
 	if (desc->noutputs>0)	obj->outputs=(MSQueue**)ms_new0(MSQueue*,desc->noutputs);
+
+	if (statistics_enabled){
+		obj->stats=find_or_create_stats(desc);
+	}
 	if (obj->desc->init!=NULL)
 		obj->desc->init(obj);
 	return obj;
@@ -104,18 +132,23 @@ MSFilter *ms_filter_new(MSFilterId id){
 	return NULL;
 }
 
-MSFilter *ms_filter_new_from_name(const char *filter_name){
+MSFilterDesc *ms_filter_lookup_by_name(const char *filter_name){
 	MSList *elem;
 	for (elem=desc_list;elem!=NULL;elem=ms_list_next(elem)){
 		MSFilterDesc *desc=(MSFilterDesc*)elem->data;
 		if (strcmp(desc->name,filter_name)==0){
-			return ms_filter_new_from_desc(desc);
+			return desc;
 		}
 	}
-	ms_error("No such filter with name %s",filter_name);
 	return NULL;
 }
 
+MSFilter *ms_filter_new_from_name(const char *filter_name){
+	MSFilterDesc *desc=ms_filter_lookup_by_name(filter_name);
+	if (desc==NULL) return NULL;
+	return ms_filter_new_from_desc(desc);
+}
+
 
 MSFilterId ms_filter_get_id(MSFilter *f){
 	return f->desc->id;
@@ -136,9 +169,7 @@ int ms_filter_link(MSFilter *f1, int pin1, MSFilter *f2, int pin2){
 
 int ms_filter_unlink(MSFilter *f1, int pin1, MSFilter *f2, int pin2){
 	MSQueue *q;
-	ms_return_val_if_fail(f1, -1);
-	ms_return_val_if_fail(f2, -1);
-	ms_message("ms_filter_unlink: %s:%p,%i-->%s:%p,%i",f1->desc->name,f1,pin1,f2->desc->name,f2,pin2);
+	ms_message("ms_filter_unlink: %s:%p,%i-->%s:%p,%i",f1 ? f1->desc->name : "!NULL!",f1,pin1,f2 ? f2->desc->name : "!NULL!",f2,pin2);
 	ms_return_val_if_fail(pin1<f1->desc->noutputs, -1);
 	ms_return_val_if_fail(pin2<f2->desc->ninputs, -1);
 	ms_return_val_if_fail(f1->outputs[pin1]!=NULL,-1);
@@ -180,6 +211,14 @@ int ms_filter_call_method(MSFilter *f, unsigned int id, void *arg){
 	return -1;
 }
 
+bool_t ms_filter_has_method(MSFilter *f, unsigned int id){
+	MSFilterMethod *methods=f->desc->methods;
+	int i;
+	for(i=0;methods!=NULL && methods[i].method!=NULL; i++)
+		if (methods[i].id==id) return TRUE;
+	return FALSE;
+}
+
 int ms_filter_call_method_noarg(MSFilter *f, unsigned int id){
 	return ms_filter_call_method(f,id,NULL);
 }
@@ -189,10 +228,6 @@ void ms_filter_set_notify_callback(MSFilter *f, MSFilterNotifyFunc fn, void *ud)
 	f->notify_ud=ud;
 }
 
-void ms_filter_enable_synchronous_notifcations(MSFilter *f, bool_t yesno){
-	f->synchronous_notifies=yesno;
-}
-
 void ms_filter_destroy(MSFilter *f){
 	if (f->desc->uninit!=NULL)
 		f->desc->uninit(f);
@@ -202,55 +237,20 @@ void ms_filter_destroy(MSFilter *f){
 	ms_free(f);
 }
 
-#ifdef DEBUG
-
-static long filter_get_cur_time(void *unused)
-{
-#if defined(_WIN32_WCE)
-	DWORD timemillis = GetTickCount();
-	return timemillis;
-#elif defined(WIN32)
-	return timeGetTime() ;
-#elif defined(__MACH__) && defined(__GNUC__) && (__GNUC__ >= 3)
-	struct timeval tv;
-	gettimeofday(&tv, NULL);
-	return (tv.tv_sec*1000LL) + (tv.tv_usec/1000LL);
-#elif defined(__MACH__)
-	struct timespec ts;
-	struct timeb time_val;
-
-	ftime (&time_val);
-	ts.tv_sec = time_val.time;
-	ts.tv_nsec = time_val.millitm * 1000000;
-	return (ts.tv_sec*1000LL) + (ts.tv_nsec/1000000LL);
-#else
-	struct timespec ts;
-	if (clock_gettime(CLOCK_MONOTONIC,&ts)<0){
-		fprintf(stderr, "clock_gettime() doesn't work: %s",strerror(errno));
-	}
-	return (ts.tv_sec*1000LL) + (ts.tv_nsec/1000000LL);
-#endif
-}
-#endif
-
 void ms_filter_process(MSFilter *f){
+	MSTimeSpec start,stop;
 	ms_debug("Executing process of filter %s:%p",f->desc->name,f);
-#ifdef DEBUG
-	long start,stop;
-	start = filter_get_cur_time(NULL);
-#endif
+
+	if (f->stats)
+		ms_get_cur_time(&start);
+
 	f->desc->process(f);
-#ifdef DEBUG
-	stop = filter_get_cur_time(NULL);
-	if(stop-start > 10)
-	{
-		ms_warning("%s take too much time:%ldms\n",f->desc->name,stop-start);
+	if (f->stats){
+		ms_get_cur_time(&stop);
+		f->stats->count++;
+		f->stats->elapsed+=(stop.tv_sec-start.tv_sec)*1000000000LL + (stop.tv_nsec-start.tv_nsec);
 	}
-	else
-	{
-		ms_debug("%s take:%ldms\n",f->desc->name,stop-start);
-	}
-#endif
+
 }
 
 void ms_filter_preprocess(MSFilter *f, struct _MSTicker *t){
@@ -352,4 +352,48 @@ int ms_connection_helper_unlink(MSConnectionHelper *h, MSFilter *f, int inpin, i
 	return err;
 }
 
+void ms_filter_enable_statistics(bool_t enabled){
+	statistics_enabled=enabled;
+}
+
+const MSList * ms_filter_get_statistics(void){
+	return stats_list;
+}
+
+void ms_filter_reset_statistics(void){
+	MSList *elem;
+	
+	for(elem=stats_list;elem!=NULL;elem=elem->next){
+		MSFilterStats *stats=(MSFilterStats *)elem->data;
+		stats->elapsed=0;
+		stats->count=0;
+	}
+}
+
+static int usage_compare(const MSFilterStats *s1, const MSFilterStats *s2){
+	if (s1->elapsed==s2->elapsed) return 0;
+	if (s1->elapsed<s2->elapsed) return 1;
+	return -1;
+}
+
+
+void ms_filter_log_statistics(void){
+	MSList *sorted=NULL;
+	MSList *elem;
+	uint64_t total=1;
+	ms_message("Filter usage statistics:");
+	for(elem=stats_list;elem!=NULL;elem=elem->next){
+		MSFilterStats *stats=(MSFilterStats *)elem->data;
+		sorted=ms_list_insert_sorted(sorted,stats,(MSCompareFunc)usage_compare);
+		total+=stats->elapsed;
+	}
+	ms_message("Name\tCount\tTime/tick (ms)\tCPU Usage");
+	for(elem=sorted;elem!=NULL;elem=elem->next){
+		MSFilterStats *stats=(MSFilterStats *)elem->data;
+		double percentage=100.0*((double)stats->elapsed)/(double)total;
+		double tpt=((double)stats->elapsed*1e-6)/((double)stats->count+1.0);
+		ms_message("%s %i %g %g",stats->name,stats->count,tpt,percentage);
+	}
+	ms_list_free(sorted);
+}
 
