@@ -18,6 +18,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
 #include "mediastreamer2/msfilter.h"
+#include "mediastreamer2/msticker.h"
 
 #include <speex/speex.h>
 
@@ -28,7 +29,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 typedef struct SpeexEncState{
 	int rate;
 	int bitrate;
-	int maxbitrate;
+	int maxbitrate; /*ip bitrate*/
+	int ip_bitrate; /*effective ip bitrate */
 	int ptime;
 	int vbr;
 	int cng;
@@ -44,7 +46,8 @@ static void enc_init(MSFilter *f){
 	s->rate=8000;
 	s->bitrate=-1;
 	s->maxbitrate=-1;
-	s->ptime=0;
+	s->ip_bitrate=-1;
+	s->ptime=20;
 	s->mode=-1;
 	s->vbr=0;
 	s->cng=0;
@@ -65,6 +68,27 @@ static void enc_uninit(MSFilter *f){
 	ms_free(s);
 }
 
+static void apply_max_bitrate(SpeexEncState *s){
+	int pps=1000/s->ptime;
+
+	if (s->maxbitrate>0){
+		/* convert from network bitrate to codec bitrate:*/
+		/* ((nbr/(pps*8)) -20-12-8)*pps*8*/
+		int cbr=(int)( ((((float)s->maxbitrate)/(pps*8))-20-12-8)*pps*8);
+		ms_message("Setting maxbitrate=%i to speex encoder.",cbr);
+		if (speex_encoder_ctl(s->state,SPEEX_SET_BITRATE,&cbr)!=0){
+			ms_error("Could not set maxbitrate %i to speex encoder.",s->bitrate);
+		}
+	}
+	if (speex_encoder_ctl(s->state,SPEEX_GET_BITRATE,&s->bitrate)!=0){
+			ms_error("Could not get bitrate %i to speex encoder.",s->bitrate);
+	}else{
+		/*convert from codec bitrate to network bitrate */
+		s->ip_bitrate=( (s->bitrate/(pps*8))+20+12+8)*8*pps;
+		ms_message("Using bitrate %i for speex encoder, ip bitrate is %i",s->bitrate,s->ip_bitrate);		
+	}
+}
+
 static void enc_preprocess(MSFilter *f){
 	SpeexEncState *s=(SpeexEncState*)f->data;
 	const SpeexMode *mode=NULL;
@@ -176,20 +200,8 @@ static void enc_preprocess(MSFilter *f){
 			ms_error("Could not set quality %i to speex encoder.",q);
 		}
 	}
+	apply_max_bitrate(s);
 
-	if (s->maxbitrate>0){
-		/* convert from network bitrate to codec bitrate:*/
-		/* ((nbr/(50*8)) -20-12-8)*50*8*/
-		int cbr=(int)( ((((float)s->maxbitrate)/(50.0*8))-20-12-8)*50*8);
-		ms_message("Setting maxbitrate=%i to speex encoder.",cbr);
-		if (speex_encoder_ctl(s->state,SPEEX_SET_BITRATE,&cbr)!=0){
-			ms_error("Could not set maxbitrate %i to speex encoder.",s->bitrate);
-		}
-	}
-	if (speex_encoder_ctl(s->state,SPEEX_GET_BITRATE,&s->bitrate)!=0){
-			ms_error("Could not get bitrate %i to speex encoder.",s->bitrate);
-	}
-	else ms_message("Using bitrate %i for speex encoder.",s->bitrate);
 
 	speex_mode_query(mode,SPEEX_MODE_FRAME_SIZE,&s->frame_size);
 }
@@ -204,6 +216,8 @@ static void enc_process(MSFilter *f){
 	if (s->frame_size<=0)
 		return;
 
+	ms_filter_lock(f);
+
 	if (s->ptime>=20)
 	{
 		frame_per_packet = s->ptime/20;
@@ -238,6 +252,7 @@ static void enc_process(MSFilter *f){
 		ms_queue_put(f->outputs[0],om);
 		speex_bits_destroy(&bits);
 	}
+	ms_filter_unlock(f);
 }
 
 static void enc_postprocess(MSFilter *f){
@@ -248,14 +263,28 @@ static void enc_postprocess(MSFilter *f){
 
 static int enc_set_sr(MSFilter *f, void *arg){
 	SpeexEncState *s=(SpeexEncState*)f->data;
-	/* TODO: should be done with fmtp parameter */
 	s->rate=((int*)arg)[0];
 	return 0;
 }
 
+static int enc_get_sr(MSFilter *f, void *arg){
+	SpeexEncState *s=(SpeexEncState*)f->data;
+	((int*)arg)[0]=s->rate;
+	return 0;
+}
+
 static int enc_set_br(MSFilter *f, void *arg){
 	SpeexEncState *s=(SpeexEncState*)f->data;
+	ms_filter_lock(f);
 	s->maxbitrate=((int*)arg)[0];
+	if (s->state) apply_max_bitrate(s);
+	ms_filter_unlock(f);
+	return 0;
+}
+
+static int enc_get_br(MSFilter *f, void *arg){
+	SpeexEncState *s=(SpeexEncState*)f->data;
+	((int*)arg)[0]=s->ip_bitrate;
 	return 0;
 }
 
@@ -303,8 +332,17 @@ static int enc_add_fmtp(MSFilter *f, void *arg){
 		s->mode=atoi(buf);
 	}
 	else {
-		s->mode = -1; /* deault mode */
+		s->mode = -1; /* default mode */
 	}
+	memset(buf, '\0', sizeof(buf));
+	if (fmtp_get_value(fmtp,"ptime",buf,sizeof(buf))){
+		s->ptime=atoi(buf);
+		//if the ptime is not a mulptiple of 20, go to the next multiple
+		if (s->ptime%20)
+			s->ptime = s->ptime - s->ptime%20 + 20; 
+		ms_message("MSSpeexEnc: got ptime=%i",s->ptime);
+	}
+	
 	return 0;
 }
 
@@ -345,7 +383,9 @@ static int enc_add_attr(MSFilter *f, void *arg){
 
 static MSFilterMethod enc_methods[]={
 	{	MS_FILTER_SET_SAMPLE_RATE	,	enc_set_sr	},
+	{	MS_FILTER_GET_SAMPLE_RATE	,	enc_get_sr	},
 	{	MS_FILTER_SET_BITRATE		,	enc_set_br	},
+	{	MS_FILTER_GET_BITRATE		,	enc_get_br	},
 	{	MS_FILTER_ADD_FMTP		,	enc_add_fmtp },
 	{	MS_FILTER_ADD_ATTR		,	enc_add_attr},
 	{	0				,	NULL		}
@@ -389,12 +429,16 @@ MSFilterDesc ms_speex_enc_desc={
 
 #endif
 
+static const int plc_max=10;/*10 frames of plc, no more*/
+
 typedef struct DecState{
 	int rate;
 	int penh;
 	int frsz;
-	uint32_t last_ts;
+	uint64_t sample_time;
 	void *state;
+	int plc_count;
+	bool_t plc;
 } DecState;
 
 static void dec_init(MSFilter *f){
@@ -403,7 +447,9 @@ static void dec_init(MSFilter *f){
 	s->frsz=0;
 	s->state=NULL;
 	s->penh=1;
-	s->last_ts=0;
+	s->sample_time=0;
+	s->plc=1;
+	s->plc_count=0;
 	f->data=s;
 }
 
@@ -441,6 +487,7 @@ static void dec_preprocess(MSFilter *f){
 	speex_mode_query(mode,SPEEX_MODE_FRAME_SIZE,&s->frsz);
 	if (s->penh==1)
 		speex_decoder_ctl (s->state, SPEEX_SET_ENH, &s->penh);
+	s->sample_time=0;
 }
 
 static void dec_postprocess(MSFilter *f){
@@ -449,12 +496,28 @@ static void dec_postprocess(MSFilter *f){
 	s->state=NULL;
 }
 
+static int dec_get_sr(MSFilter *f, void *arg){
+	DecState *s=(DecState*)f->data;
+	((int*)arg)[0]=s->rate;
+	return 0;
+}
+
 static int dec_set_sr(MSFilter *f, void *arg){
 	DecState *s=(DecState*)f->data;
 	s->rate=((int*)arg)[0];
 	return 0;
 }
 
+static int dec_add_fmtp(MSFilter *f, void *arg){
+	DecState *s=(DecState*)f->data;
+	const char *fmtp=(const char *)arg;
+	char buf[32];
+	if (fmtp_get_value(fmtp, "plc", buf, sizeof(buf))){
+		s->plc=atoi(buf);
+	}
+	return 0;
+}
+
 static void dec_process(MSFilter *f){
 	DecState *s=(DecState*)f->data;
 	mblk_t *im;
@@ -462,37 +525,33 @@ static void dec_process(MSFilter *f){
 	int err=-2;
 	SpeexBits bits;
 	int bytes=s->frsz*2;
+	bool_t bits_initd=FALSE;
 	
-	speex_bits_init(&bits);
 	while((im=ms_queue_get(f->inputs[0]))!=NULL){
-		uint32_t curts=mblk_get_timestamp_info(im);
 		int rem_bits=(im->b_wptr-im->b_rptr)*8;
 		
-		speex_bits_reset(&bits);
+		if (!bits_initd) {
+			speex_bits_init(&bits);
+			bits_initd=TRUE;
+		}else speex_bits_reset(&bits);
+
 		speex_bits_read_from(&bits,(char*)im->b_rptr,im->b_wptr-im->b_rptr);
 		
-
-#if 0 /* does not work very well, revisit later*/
-		/* packet loss concealment stuff, for one packet lost 
-			A better algorithm would be needed to accomodate more
-		*/
-		if (s->last_ts!=0){
-			int ts_diff=(int)(curts-s->last_ts);
-			if ( ts_diff>160){
-				ms_message("Packet missing, trying loss concealment, ts_diff=%i",ts_diff);
-				err=speex_decode_int(s->state,NULL,(int16_t*)om->b_wptr);
-				om->b_wptr+=320;
-			}
-		}
-#endif
 		/* support for multiple frame  in one RTP packet */
  		do{
 			om=allocb(bytes,0);
 			err=speex_decode_int(s->state,&bits,(int16_t*)om->b_wptr);
 			om->b_wptr+=bytes;
-			s->last_ts=curts+s->frsz;
+
 			if (err==0){
 				ms_queue_put(f->outputs[0],om);
+				if (s->sample_time==0) s->sample_time=f->ticker->time;
+				s->sample_time+=20;
+				if (s->plc_count>0){
+					ms_warning("Did speex packet loss concealment during %i ms",s->plc_count*20);
+					s->plc_count=0;
+				}
+				
 			}else {
 				if (err==-1)
 					ms_warning("speex end of stream");
@@ -503,11 +562,27 @@ static void dec_process(MSFilter *f){
 		}while((rem_bits= speex_bits_remaining(&bits))>10);
 		freemsg(im);
 	}
-	speex_bits_destroy(&bits);
+	if (s->plc && s->sample_time!=0 && f->ticker->time>=s->sample_time){
+		/* we should output a frame but no packet were decoded
+		 thus do packet loss concealment*/
+		om=allocb(bytes,0);
+		err=speex_decode_int(s->state,NULL,(int16_t*)om->b_wptr);
+		om->b_wptr+=bytes;
+		ms_queue_put(f->outputs[0],om);
+		
+		s->sample_time+=20;
+		s->plc_count++;
+		if (s->plc_count>=plc_max){
+			s->sample_time=0;
+		}
+	}
+	if (bits_initd) speex_bits_destroy(&bits);
 }
 
 static MSFilterMethod dec_methods[]={
 	{	MS_FILTER_SET_SAMPLE_RATE	,	dec_set_sr	},
+	{	MS_FILTER_GET_SAMPLE_RATE	,	dec_get_sr	},
+	{	MS_FILTER_ADD_FMTP	, dec_add_fmtp	},
 	{	0				,	NULL		}
 };
 
@@ -526,7 +601,8 @@ MSFilterDesc ms_speex_dec_desc={
 	dec_process,
 	dec_postprocess,
 	dec_uninit,
-	dec_methods
+	dec_methods,
+	MS_FILTER_IS_PUMP
 };
 
 #else
@@ -544,10 +620,12 @@ MSFilterDesc ms_speex_dec_desc={
 	.postprocess=dec_postprocess,
 	.process=dec_process,
 	.uninit=dec_uninit,
-	.methods=dec_methods
+	.methods=dec_methods,
+	.flags=MS_FILTER_IS_PUMP
 };
 
 #endif
 
 MS_FILTER_DESC_EXPORT(ms_speex_dec_desc)
 MS_FILTER_DESC_EXPORT(ms_speex_enc_desc)
+
