@@ -17,20 +17,37 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
-
-
 #include "mediastreamer2/msticker.h"
 
+#ifndef WIN32
+#include <sys/time.h>
+#include <sys/resource.h>
+#endif
+
+static const double smooth_coef=0.9;
+
+#ifndef TICKER_MEASUREMENTS
+
+#define TICKER_MEASUREMENTS 1
+
+#if defined(__ARM_ARCH__) 
+#	if __ARM_ARCH__ < 7
+/* as MSTicker load computation requires floating point, we prefer to disable it on ARM processors without FPU*/
+#		undef TICKER_MEASUREMENTS
+#		define TICKER_MEASUREMENTS 0 
+#	endif
+#endif
+
+#endif
 
 void * ms_ticker_run(void *s);
-static uint64_t get_cur_time(void *);
+static uint64_t get_cur_time_ms(void *);
 
 void ms_ticker_start(MSTicker *s){
 	s->run=TRUE;
 	ms_thread_create(&s->thread,NULL,ms_ticker_run,s);
 }
 
-
 void ms_ticker_init(MSTicker *ticker)
 {
 	ms_mutex_init(&ticker->lock,NULL);
@@ -40,12 +57,11 @@ void ms_ticker_init(MSTicker *ticker)
 	ticker->interval=10;
 	ticker->run=FALSE;
 	ticker->exec_id=0;
-	ticker->get_cur_time_ptr=&get_cur_time;
+	ticker->get_cur_time_ptr=&get_cur_time_ms;
 	ticker->get_cur_time_data=NULL;
-#ifdef WIN32_TIMERS
-	ticker->TimeEvent=NULL;
-#endif
 	ticker->name=ms_strdup("MSTicker");
+	ticker->av_load=0;
+	ticker->prio=MS_TICKER_PRIO_NORMAL;
 	ms_ticker_start(ticker);
 }
 
@@ -59,7 +75,8 @@ void ms_ticker_stop(MSTicker *s){
 	ms_mutex_lock(&s->lock);
 	s->run=FALSE;
 	ms_mutex_unlock(&s->lock);
-	ms_thread_join(s->thread,NULL);
+	if(s->thread)
+		ms_thread_join(s->thread,NULL);
 }
 
 void ms_ticker_set_name(MSTicker *s, const char *name){
@@ -67,6 +84,10 @@ void ms_ticker_set_name(MSTicker *s, const char *name){
 	s->name=ms_strdup(name);
 }
 
+void ms_ticker_set_priority(MSTicker *ticker, MSTickerPrio prio){
+	ticker->prio=prio;
+}
+
 void ms_ticker_uninit(MSTicker *ticker)
 {
 	ms_ticker_stop(ticker);
@@ -220,36 +241,10 @@ static void run_graphs(MSTicker *s, MSList *execution_list, bool_t force_schedul
 	}
 }
 
-#ifdef __MACH__
-#include <sys/types.h>
-#include <sys/timeb.h>
-#endif
-
-static uint64_t get_cur_time(void *unused){
-#if defined(_WIN32_WCE)
-	DWORD timemillis = GetTickCount();
-	return timemillis;
-#elif defined(WIN32)
-	return timeGetTime() ;
-#elif defined(__MACH__) && defined(__GNUC__) && (__GNUC__ >= 3)
-	struct timeval tv;
-	gettimeofday(&tv, NULL);
-	return (tv.tv_sec*1000LL) + (tv.tv_usec/1000LL);
-#elif defined(__MACH__)
-	struct timespec ts;
-	struct timeb time_val;
-	
-	ftime (&time_val);
-	ts.tv_sec = time_val.time;
-	ts.tv_nsec = time_val.millitm * 1000000;
-	return (ts.tv_sec*1000LL) + (ts.tv_nsec/1000000LL);
-#else
-	struct timespec ts;
-	if (clock_gettime(CLOCK_MONOTONIC,&ts)<0){
-		ms_fatal("clock_gettime() doesn't work: %s",strerror(errno));
-	}
-	return (ts.tv_sec*1000LL) + (ts.tv_nsec/1000000LL);
-#endif
+static uint64_t get_cur_time_ms(void *unused){
+	MSTimeSpec ts;
+	ms_get_cur_time(&ts);
+	return (ts.tv_sec*1000LL) + ((ts.tv_nsec+500000LL)/1000000LL);
 }
 
 static void sleepMs(int ms){
@@ -263,45 +258,73 @@ static void sleepMs(int ms){
 #endif
 }
 
-static int set_high_prio(void){
+static int set_high_prio(MSTicker *obj){
 	int precision=2;
 	int result=0;
+	char* env_prio_c=NULL;
+	int min_prio, max_prio, env_prio;
+	int prio=obj->prio;
+	
+	if (prio>MS_TICKER_PRIO_NORMAL){
 #ifdef WIN32
-	MMRESULT mm;
-	TIMECAPS ptc;
-	mm=timeGetDevCaps(&ptc,sizeof(ptc));
-	if (mm==0){
-		if (ptc.wPeriodMin<(UINT)precision)
-			ptc.wPeriodMin=precision;
-		else
-			precision = ptc.wPeriodMin;
-		mm=timeBeginPeriod(ptc.wPeriodMin);
-		if (mm!=TIMERR_NOERROR){
-			ms_warning("timeBeginPeriod failed.");
+		MMRESULT mm;
+		TIMECAPS ptc;
+		mm=timeGetDevCaps(&ptc,sizeof(ptc));
+		if (mm==0){
+			if (ptc.wPeriodMin<(UINT)precision)
+				ptc.wPeriodMin=precision;
+			else
+				precision = ptc.wPeriodMin;
+			mm=timeBeginPeriod(ptc.wPeriodMin);
+			if (mm!=TIMERR_NOERROR){
+				ms_warning("timeBeginPeriod failed.");
+			}
+			ms_message("win32 timer resolution set to %i ms",ptc.wPeriodMin);
+		}else{
+			ms_warning("timeGetDevCaps failed.");
 		}
-		ms_message("win32 timer resolution set to %i ms",ptc.wPeriodMin);
-	}else{
-		ms_warning("timeGetDevCaps failed.");
-	}
 
-	if(!SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_HIGHEST)){
-		ms_warning("SetThreadPriority() failed (%d)\n", GetLastError());
-	}
+		if(!SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_HIGHEST)){
+			ms_warning("SetThreadPriority() failed (%d)\n", GetLastError());
+		}
 #else
-	struct sched_param param;
-	memset(&param,0,sizeof(param));
-#ifdef TARGET_OS_MAC
-	int policy=SCHED_RR;
-#else
-	int policy=SCHED_OTHER;
+		struct sched_param param;
+		int policy=SCHED_RR;
+		memset(&param,0,sizeof(param));
+
+		if (prio==MS_TICKER_PRIO_REALTIME)
+			policy=SCHED_FIFO;
+		
+		min_prio = sched_get_priority_min(policy);
+		max_prio = sched_get_priority_max(policy);
+		env_prio_c = getenv("MS_TICKER_SCHEDPRIO");
+
+		env_prio = (env_prio_c == NULL)?max_prio:atoi(env_prio_c);
+
+		env_prio = MAX(MIN(env_prio, max_prio), min_prio);
+		ms_message("Priority used: %d", env_prio);
+
+		param.sched_priority=env_prio;
+		if((result=pthread_setschedparam(pthread_self(),policy, &param))) {
+			if (result==EPERM){
+				/*
+					The linux kernel has 
+					sched_get_priority_max(SCHED_OTHER)=sched_get_priority_max(SCHED_OTHER)=0.
+					As long as we can't use SCHED_RR or SCHED_FIFO, the only way to increase priority of a calling thread
+					is to use setpriority().
+				*/
+				if (setpriority(PRIO_PROCESS,0,-20)==-1){
+					ms_message("%s setpriority() failed: %s, nevermind.",obj->name,strerror(errno));
+				}else{
+					ms_message("%s priority increased to maximum.",obj->name);
+				}
+			}else ms_warning("%s: Set pthread_setschedparam failed: %s",obj->name,strerror(result));
+		} else {
+			ms_message("%s priority set to %s and value (%i)",obj->name,
+			           policy==SCHED_FIFO ? "SCHED_FIFO" : "SCHED_RR", param.sched_priority);
+		}
 #endif
-	param.sched_priority=sched_get_priority_max(policy);
-	if((result=pthread_setschedparam(pthread_self(),policy, &param))) {
-		ms_warning("Set sched param failed with error code(%i)\n",result);
-	} else {
-		ms_message("MS ticker priority set to max");
 	}
-#endif
 	return precision;
 }
 
@@ -317,8 +340,7 @@ static void unset_high_prio(int precision){
 #endif
 }
 
-#ifndef WIN32_TIMERS
-
+/*the ticker thread function that executes the filters */
 void * ms_ticker_run(void *arg)
 {
 	uint64_t realtime;
@@ -328,7 +350,7 @@ void * ms_ticker_run(void *arg)
 	int precision=2;
 	int late;
 	
-	precision = set_high_prio();
+	precision = set_high_prio(s);
 
 
 	s->ticks=1;
@@ -337,7 +359,21 @@ void * ms_ticker_run(void *arg)
 
 	while(s->run){
 		s->ticks++;
-		run_graphs(s,s->execution_list,FALSE);
+		{
+#if TICKER_MEASUREMENTS
+			MSTimeSpec begin,end;/*used to measure time spent in processing one tick*/
+			double iload;
+
+			ms_get_cur_time(&begin);
+#endif
+			run_graphs(s,s->execution_list,FALSE);
+#if TICKER_MEASUREMENTS
+			ms_get_cur_time(&end);
+			iload=100*((end.tv_sec-begin.tv_sec)*1000.0 + (end.tv_nsec-begin.tv_nsec)/1000000.0)/(double)s->interval;
+			s->av_load=(smooth_coef*s->av_load)+((1.0-smooth_coef)*iload);
+#endif
+		}
+		
 		s->time+=s->interval;
 		while(1){
 			realtime=s->get_cur_time_ptr(s->get_cur_time_data)-s->orig;
@@ -366,55 +402,8 @@ void * ms_ticker_run(void *arg)
 	return NULL;
 }
 
-#else
-
-void * ms_ticker_run(void *arg)
-{
-	MSTicker *s=(MSTicker*)arg;
-	uint64_t realtime;
-	int precision=2;
-	UINT timerId;
-
-	precision = set_high_prio();
-
-	s->TimeEvent = CreateEvent (NULL, FALSE, FALSE, NULL);
-
-	s->ticks=1;
-	ms_mutex_lock(&s->lock);
-	s->orig=s->get_cur_time_ptr(s->get_cur_time_data);
-
-	timerId = timeSetEvent (s->interval, precision, (LPTIMECALLBACK)s->TimeEvent, 0,
-				  TIME_PERIODIC | TIME_CALLBACK_EVENT_SET);
-	while(s->run){
-		DWORD err;
-
-		s->ticks++;
-		run_graphs(s,s->execution_list,FALSE);
-
-		/* elapsed time since origin */
-		s->time = s->get_cur_time_ptr(s->get_cur_time_data)- s->orig;
-
-		ms_mutex_unlock(&s->lock);
-		err = WaitForSingleObject (s->TimeEvent, s->interval*1000 ); /* wake up each diff */
-		if (err==WAIT_FAILED)
-			ms_message("WaitForSingleObject is failing");
-
-		ms_mutex_lock(&s->lock);
-	}
-	ms_mutex_unlock(&s->lock);
-	timeKillEvent (timerId);
-	CloseHandle (s->TimeEvent);
-	s->TimeEvent=NULL;
-	unset_high_prio(precision);
-	ms_message("MSTicker thread exiting");
-	ms_thread_exit(NULL);
-	return NULL;
-}
-
-#endif
-
 void ms_ticker_set_time_func(MSTicker *ticker, MSTickerTimeFunc func, void *user_data){
-	if (func==NULL) func=get_cur_time;
+	if (func==NULL) func=get_cur_time_ms;
 	/*ms_mutex_lock(&ticker->lock);*/
 	ticker->get_cur_time_ptr=func;
 	ticker->get_cur_time_data=user_data;
@@ -466,3 +455,15 @@ static void print_graphs(MSTicker *s, MSList *execution_list, bool_t force_sched
 void ms_ticker_print_graphs(MSTicker *ticker){
 	print_graphs(ticker,ticker->execution_list,FALSE);
 }
+
+float ms_ticker_get_average_load(MSTicker *ticker){
+#if	!TICKER_MEASUREMENTS
+	static bool_t once=FALSE;
+	if (once==FALSE){
+		ms_warning("ms_ticker_get_average_load(): ticker load measurements disabled for performance reasons.");
+		once=TRUE;
+	}
+#endif
+	return ticker->av_load;
+}
+
