@@ -17,11 +17,22 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
+
 #include "mediastreamer2/msvideo.h"
 #if !defined(NO_FFMPEG)
 #include "ffmpeg-priv.h"
 #endif
 
+#ifdef WIN32
+#include <malloc.h>
+#endif
+
+struct _mblk_video_header {
+	uint16_t w, h;
+	int pad[3];
+};
+typedef struct _mblk_video_header mblk_video_header; 
+
 static void yuv_buf_init(YuvBuf *buf, int w, int h, uint8_t *ptr){
 	int ysize,usize;
 	ysize=w*h;
@@ -39,105 +50,21 @@ static void yuv_buf_init(YuvBuf *buf, int w, int h, uint8_t *ptr){
 }
 
 int ms_yuv_buf_init_from_mblk(YuvBuf *buf, mblk_t *m){
-	int size=m->b_wptr-m->b_rptr;
 	int w,h;
-	if (size==(MS_VIDEO_SIZE_QCIF_W*MS_VIDEO_SIZE_QCIF_H*3)/2){
-		w=MS_VIDEO_SIZE_QCIF_W;
-		h=MS_VIDEO_SIZE_QCIF_H;
-	}else if (size==(MS_VIDEO_SIZE_CIF_W*MS_VIDEO_SIZE_CIF_H*3)/2){
-		w=MS_VIDEO_SIZE_CIF_W;
-		h=MS_VIDEO_SIZE_CIF_H;
-	}else if (size==(MS_VIDEO_SIZE_QVGA_W*MS_VIDEO_SIZE_QVGA_H*3)/2){
-		w=MS_VIDEO_SIZE_QVGA_W;
-		h=MS_VIDEO_SIZE_QVGA_H;
-	}else if (size==(MS_VIDEO_SIZE_VGA_W*MS_VIDEO_SIZE_VGA_H*3)/2){
-		w=MS_VIDEO_SIZE_VGA_W;
-		h=MS_VIDEO_SIZE_VGA_H;
-	}else if (size==(MS_VIDEO_SIZE_4CIF_W*MS_VIDEO_SIZE_4CIF_H*3)/2){
-		w=MS_VIDEO_SIZE_4CIF_W;
-		h=MS_VIDEO_SIZE_4CIF_H;
-	}else if (size==(MS_VIDEO_SIZE_W4CIF_W*MS_VIDEO_SIZE_W4CIF_H*3)/2){
-		w=MS_VIDEO_SIZE_W4CIF_W;
-		h=MS_VIDEO_SIZE_W4CIF_H;
-	}else if (size==(MS_VIDEO_SIZE_SVGA_W*MS_VIDEO_SIZE_SVGA_H*3)/2){
-		w=MS_VIDEO_SIZE_SVGA_W;
-		h=MS_VIDEO_SIZE_SVGA_H;
-	}else if (size==(MS_VIDEO_SIZE_SQCIF_W*MS_VIDEO_SIZE_SQCIF_H*3)/2){
-		w=MS_VIDEO_SIZE_SQCIF_W;
-		h=MS_VIDEO_SIZE_SQCIF_H;
-	}else if (size==(MS_VIDEO_SIZE_QQVGA_W*MS_VIDEO_SIZE_QQVGA_H*3)/2){
-		w=MS_VIDEO_SIZE_QQVGA_W;
-		h=MS_VIDEO_SIZE_QQVGA_H;
-	}else if (size==(MS_VIDEO_SIZE_NS1_W*MS_VIDEO_SIZE_NS1_H*3)/2){
-		w=MS_VIDEO_SIZE_NS1_W;
-		h=MS_VIDEO_SIZE_NS1_H;
-	}else if (size==(MS_VIDEO_SIZE_QSIF_W*MS_VIDEO_SIZE_QSIF_H*3)/2){
-		w=MS_VIDEO_SIZE_QSIF_W;
-		h=MS_VIDEO_SIZE_QSIF_H;
-	}else if (size==(MS_VIDEO_SIZE_SIF_W*MS_VIDEO_SIZE_SIF_H*3)/2){
-		w=MS_VIDEO_SIZE_SIF_W;
-		h=MS_VIDEO_SIZE_SIF_H;
-	}else if (size==(MS_VIDEO_SIZE_4SIF_W*MS_VIDEO_SIZE_4SIF_H*3)/2){
-		w=MS_VIDEO_SIZE_4SIF_W;
-		h=MS_VIDEO_SIZE_4SIF_H;
-	}else if (size==(MS_VIDEO_SIZE_288P_W*MS_VIDEO_SIZE_288P_H*3)/2){
-		w=MS_VIDEO_SIZE_288P_W;
-		h=MS_VIDEO_SIZE_288P_H;
-	}else if (size==(MS_VIDEO_SIZE_432P_W*MS_VIDEO_SIZE_432P_H*3)/2){
-		w=MS_VIDEO_SIZE_432P_W;
-		h=MS_VIDEO_SIZE_432P_H;
-	}else if (size==(MS_VIDEO_SIZE_448P_W*MS_VIDEO_SIZE_448P_H*3)/2){
-		w=MS_VIDEO_SIZE_448P_W;
-		h=MS_VIDEO_SIZE_448P_H;
-	}else if (size==(MS_VIDEO_SIZE_480P_W*MS_VIDEO_SIZE_480P_H*3)/2){
-		w=MS_VIDEO_SIZE_480P_W;
-		h=MS_VIDEO_SIZE_480P_H;
-	}else if (size==(MS_VIDEO_SIZE_576P_W*MS_VIDEO_SIZE_576P_H*3)/2){
-		w=MS_VIDEO_SIZE_576P_W;
-		h=MS_VIDEO_SIZE_576P_H;
-	}else if (size==(MS_VIDEO_SIZE_720P_W*MS_VIDEO_SIZE_720P_H*3)/2){
-		w=MS_VIDEO_SIZE_720P_W;
-		h=MS_VIDEO_SIZE_720P_H;
-	}else if (size==(MS_VIDEO_SIZE_1080P_W*MS_VIDEO_SIZE_1080P_H*3)/2){
-		w=MS_VIDEO_SIZE_1080P_W;
-		h=MS_VIDEO_SIZE_1080P_H;
-	}else if (size==(MS_VIDEO_SIZE_SDTV_W*MS_VIDEO_SIZE_SDTV_H*3)/2){
-		w=MS_VIDEO_SIZE_SDTV_W;
-		h=MS_VIDEO_SIZE_SDTV_H;
-	}else if (size==(MS_VIDEO_SIZE_HDTVP_W*MS_VIDEO_SIZE_HDTVP_H*3)/2){
-		w=MS_VIDEO_SIZE_HDTVP_W;
-		h=MS_VIDEO_SIZE_HDTVP_H;
-	}else if (size==(MS_VIDEO_SIZE_XGA_W*MS_VIDEO_SIZE_XGA_H*3)/2){
-		w=MS_VIDEO_SIZE_XGA_W;
-		h=MS_VIDEO_SIZE_XGA_H;
-	}else if (size==(MS_VIDEO_SIZE_WXGA_W*MS_VIDEO_SIZE_WXGA_H*3)/2){
-		w=MS_VIDEO_SIZE_WXGA_W;
-		h=MS_VIDEO_SIZE_WXGA_H;
-	}else if (size==(MS_VIDEO_SIZE_WQCIF_W*MS_VIDEO_SIZE_WQCIF_H*3)/2){
-		w=MS_VIDEO_SIZE_WQCIF_W;
-		h=MS_VIDEO_SIZE_WQCIF_H;
-	}else if (size==(MS_VIDEO_SIZE_CVD_W*MS_VIDEO_SIZE_CVD_H*3)/2){
-		w=MS_VIDEO_SIZE_CVD_W;
-		h=MS_VIDEO_SIZE_CVD_H;
-	}else if (size==(160*112*3)/2){/*format used by econf*/
-		w=160;
-		h=112;
-	}else if (size==(320*200*3)/2){/*format used by gTalk */
-		w=320;
-		h=200;
-	}else {
-		ms_error("Unsupported image size: size=%i (bug somewhere !)",size);
-		return -1;
-	}
-	if (mblk_get_video_orientation(m)==MS_VIDEO_PORTRAIT){
-		int tmp=h;
-		h=w;
-		w=tmp;
-	}
-	yuv_buf_init(buf,w,h,m->b_rptr);
+
+	// read header
+	mblk_video_header* hdr = (mblk_video_header*)m->b_datap->db_base; 
+	w = hdr->w;
+	h = hdr->h;
+
+	if (m->b_cont == NULL)
+		yuv_buf_init(buf,w,h,m->b_rptr);
+	else
+		yuv_buf_init(buf,w,h,m->b_cont->b_rptr);
 	return 0;
 }
 
+
 int ms_yuv_buf_init_from_mblk_with_size(YuvBuf *buf, mblk_t *m, int w, int h){
   yuv_buf_init(buf,w,h,m->b_rptr);
   return 0;
@@ -174,17 +101,36 @@ int ms_picture_init_from_mblk_with_size(MSPicture *buf, mblk_t *m, MSPixFmt fmt,
 
 mblk_t * ms_yuv_buf_alloc(YuvBuf *buf, int w, int h){
 	int size=(w*h*3)/2;
+	const int header_size =sizeof(mblk_video_header);
 	const int padding=16;
-	mblk_t *msg=allocb(size+padding,0);
+	mblk_t *msg=allocb(header_size + size+padding,0);
+	// write width/height in header
+	mblk_video_header* hdr = (mblk_video_header*)msg->b_wptr; 
+	hdr->w = w;
+	hdr->h = h;
+	msg->b_rptr += header_size;
+	msg->b_wptr += header_size;
 	yuv_buf_init(buf,w,h,msg->b_wptr);
-	if (h>w)
-		mblk_set_video_orientation(msg,MS_VIDEO_PORTRAIT);
-	else
-		mblk_set_video_orientation(msg,MS_VIDEO_LANDSCAPE);
 	msg->b_wptr+=size;
 	return msg;
 }
 
+mblk_t* ms_yuv_buf_alloc_from_buffer(int w, int h, mblk_t* buffer) {
+	const int header_size =sizeof(mblk_video_header);
+	mblk_t *msg=allocb(header_size,0);
+	// write width/height in header
+	mblk_video_header* hdr = (mblk_video_header*)msg->b_wptr; 
+	hdr->w = w;
+	hdr->h = h;
+	msg->b_rptr += header_size;
+	msg->b_wptr += header_size;
+	// append real image buffer
+	msg->b_cont = buffer;
+	buffer->b_datap->db_ref++;
+
+	return msg;
+}
+
 static void plane_copy(const uint8_t *src_plane, int src_stride,
 	uint8_t *dst_plane, int dst_stride, MSVideoSize roi){
 	int i;
@@ -195,7 +141,7 @@ static void plane_copy(const uint8_t *src_plane, int src_stride,
 	}
 }
 
-void ms_yuv_buf_copy(uint8_t *src_planes[], const int src_strides[], 
+void ms_yuv_buf_copy(uint8_t *src_planes[], const int src_strides[],
 		uint8_t *dst_planes[], const int dst_strides[3], MSVideoSize roi){
 	plane_copy(src_planes[0],src_strides[0],dst_planes[0],dst_strides[0],roi);
 	roi.width=roi.width/2;
@@ -204,24 +150,75 @@ void ms_yuv_buf_copy(uint8_t *src_planes[], const int src_strides[],
 	plane_copy(src_planes[2],src_strides[2],dst_planes[2],dst_strides[2],roi);
 }
 
-static void plane_mirror(uint8_t *p, int linesize, int w, int h){
+static void plane_horizontal_mirror(uint8_t *p, int linesize, int w, int h){
 	int i,j;
 	uint8_t tmp;
 	for(j=0;j<h;++j){
 		for(i=0;i<w/2;++i){
+			const int idx_target_pixel = w-1-i;
 			tmp=p[i];
-			p[i]=p[w-1-i];
-			p[w-1-i]=tmp;
+			p[i]=p[idx_target_pixel];
+			p[idx_target_pixel]=tmp;
 		}
 		p+=linesize;
 	}
 }
+static void plane_central_mirror(uint8_t *p, int linesize, int w, int h){
+	int i,j;
+	uint8_t tmp;
+	uint8_t *end_of_image = p + (h-1)*linesize+w-1;
+	uint8_t *image_center=p+(h/2)*linesize + w/2;
+	for(j=0;j<h/2;++j){
+		for(i=0;i<w && p<image_center;++i){
+			tmp=*p;
+			*p=*end_of_image;
+			*end_of_image=tmp;
+			++p;
+			--end_of_image;
+		}
+		p+=linesize-w;
+		end_of_image-=linesize-w;
+	}
+}
+static void plane_vertical_mirror(uint8_t *p, int linesize, int w, int h){
+	int j;
+	uint8_t *tmp=alloca(w*sizeof(int));
+	uint8_t *bottom_line = p + (h-1)*linesize;
+	for(j=0;j<h/2;++j){
+		memcpy(tmp, p, w);
+		memcpy(p, bottom_line, w);
+		memcpy(bottom_line, tmp, w);
+		p+=linesize;
+		bottom_line-=linesize;
+	}
+}
+
+static void plane_mirror(MSMirrorType type, uint8_t *p, int linesize, int w, int h){
+	switch (type){
+		case MS_HORIZONTAL_MIRROR:
+			 plane_horizontal_mirror(p,linesize,w,h);
+			 break;
+		case MS_VERTICAL_MIRROR:
+			plane_vertical_mirror(p,linesize,w,h);
+			break;
+		case MS_CENTRAL_MIRROR:
+			plane_central_mirror(p,linesize,w,h);
+			break;
+		case MS_NO_MIRROR:
+			break;
+	}
+}
+
+/*in place horizontal mirroring*/
+void ms_yuv_buf_mirror(YuvBuf *buf){
+	ms_yuv_buf_mirrors(buf, MS_HORIZONTAL_MIRROR);
+}
 
 /*in place mirroring*/
-void ms_yuv_buf_mirror(YuvBuf *buf){
-	plane_mirror(buf->planes[0],buf->strides[0],buf->w,buf->h);
-	plane_mirror(buf->planes[1],buf->strides[1],buf->w/2,buf->h/2);
-	plane_mirror(buf->planes[2],buf->strides[2],buf->w/2,buf->h/2);
+void ms_yuv_buf_mirrors(YuvBuf *buf, MSMirrorType type){
+	plane_mirror(type, buf->planes[0],buf->strides[0],buf->w,buf->h);
+	plane_mirror(type, buf->planes[1],buf->strides[1],buf->w/2,buf->h/2);
+	plane_mirror(type, buf->planes[2],buf->strides[2],buf->w/2,buf->h/2);
 }
 
 #ifndef MAKEFOURCC
@@ -287,7 +284,7 @@ void rgb24_revert(uint8_t *buf, int w, int h, int linesize){
 		}
 		p+=linesize;
 		pe-=linesize;
-	}	
+	}
 }
 
 void rgb24_copy_revert(uint8_t *dstbuf, int dstlsz,
@@ -398,10 +395,14 @@ static MSScalerContext *ff_create_swscale_context(int src_w, int src_h, MSPixFmt
 	int ff_flags=0;
 	MSFFScalerContext *ctx=ms_new(MSFFScalerContext,1);
 	ctx->src_h=src_h;
+#if __arm__
+	ff_flags|=SWS_FAST_BILINEAR;
+#else
 	if (flags & MS_SCALER_METHOD_BILINEAR)
 		ff_flags|=SWS_BILINEAR;
 	else if (flags & MS_SCALER_METHOD_NEIGHBOUR)
 		ff_flags|=SWS_BILINEAR;
+#endif
 	ctx->ctx=sws_getContext (src_w,src_h,ms_pix_fmt_to_ffmpeg (src_fmt),
 	                                       dst_w,dst_h,ms_pix_fmt_to_ffmpeg (dst_fmt),ff_flags,NULL,NULL,NULL);
 	if (ctx->ctx==NULL){
@@ -448,7 +449,7 @@ typedef struct AndroidScalerCtx{
 	bool_t use_omx;
 }AndroidScalerCtx;
 
-/* for android we use ffmpeg's scaler except for YUV420P-->RGB565, for which we prefer 
+/* for android we use ffmpeg's scaler except for YUV420P-->RGB565, for which we prefer
  another arm neon optimized routine */
 
 static MSScalerContext *android_create_scaler_context(int src_w, int src_h, MSPixFmt src_fmt,
@@ -521,8 +522,8 @@ static MSScalerDesc android_scaler={
 #endif
 
 
-#ifdef ANDROID
-
+#if defined(ANDROID) && defined(__ARM_NEON__)
+#include <arm_neon.h>
 extern MSScalerDesc ms_android_scaler;
 
 static MSScalerDesc *scaler_impl=&ms_android_scaler;
@@ -552,4 +553,210 @@ void ms_video_set_scaler_impl(MSScalerDesc *desc){
 	scaler_impl=desc;
 }
 
+/* Can rotate Y, U or V plane; use step=2 for interleaved UV planes otherwise step=1*/
+static void rotate_plane(int wDest, int hDest, int full_width, uint8_t* src, uint8_t* dst, int step, bool_t clockWise) {
+	int hSrc = wDest;
+	int wSrc = hDest;
+	int src_stride = full_width * step;
 
+	int signed_dst_stride;
+	int incr;
+
+
+
+	if (clockWise) {
+		/* ms_warning("start writing destination buffer from top right");*/
+		dst += wDest - 1;
+		incr = 1;
+		signed_dst_stride = wDest;
+	} else {
+		/* ms_warning("start writing destination buffer from top right");*/
+		dst += wDest * (hDest - 1);
+		incr = -1;
+		signed_dst_stride = -wDest;
+	}
+	int y,x;
+	for (y=0; y<hSrc; y++) {
+		uint8_t* dst2 = dst;
+		for (x=0; x<step*wSrc; x+=step) {
+			/*	Copy a line in source buffer (left to right)
+				Clockwise: Store a column in destination buffer (top to bottom)
+				Not clockwise: Store a column in destination buffer (bottom to top)
+			 */
+			*dst2 = src[x];
+			dst2 += signed_dst_stride;
+		}
+		dst -= incr;
+		src += src_stride;
+	}
+}
+
+
+#ifdef ANDROID
+#include "cpu-features.h"
+static int hasNeon = -1;
+#elif defined (__ARM_NEON__)
+static int hasNeon = 1;
+#else
+static int hasNeon = 0;
+#endif
+
+/* Destination and source images may have their dimensions inverted.*/
+mblk_t *copy_ycbcrbiplanar_to_true_yuv_with_rotation_and_down_scale_by_2(uint8_t* y, uint8_t * cbcr, int rotation, int w, int h, int y_byte_per_row,int cbcr_byte_per_row, bool_t uFirstvSecond, bool_t down_scale) {
+	MSPicture pict;
+	mblk_t *yuv_block = ms_yuv_buf_alloc(&pict, w, h);
+#ifdef ANDROID
+	if (hasNeon == -1) {
+		hasNeon = (android_getCpuFamily() == ANDROID_CPU_FAMILY_ARM && (android_getCpuFeatures() & ANDROID_CPU_ARM_FEATURE_NEON) != 0);
+	}
+#endif
+	if (down_scale && !hasNeon) {
+		ms_error("down scaling by two requires NEON, returning empty block");
+		return yuv_block;
+	}
+
+	if (!uFirstvSecond) {
+		unsigned char* tmp = pict.planes[1];
+		pict.planes[1] = pict.planes[2];
+		pict.planes[2] = tmp;
+	}
+
+	int uv_w = w/2;
+	int uv_h = h/2;
+
+	if (rotation % 180 == 0) {
+		int i,j;
+		uint8_t* u_dest=pict.planes[1], *v_dest=pict.planes[2];
+
+		if (rotation == 0) {
+#ifdef __arm__
+			if (hasNeon) {
+				deinterlace_down_scale_neon(y, cbcr, pict.planes[0], u_dest, v_dest, w, h, y_byte_per_row, cbcr_byte_per_row,down_scale);
+			} else 
+#endif
+			{	
+				// plain copy
+				for(i=0; i<h; i++) {
+					memcpy(&pict.planes[0][i*w], &y[i*y_byte_per_row], w);
+				}
+				// de-interlace u/v
+				for (i=0; i<uv_h; i++) {
+					for(j=0; j<uv_w; j++) {
+						*u_dest++ = cbcr[cbcr_byte_per_row*i + 2*j];
+						*v_dest++ = cbcr[cbcr_byte_per_row*i + 2*j + 1];
+					}
+				}
+			}
+		} else {
+#ifdef __arm__
+			if (hasNeon) {
+				deinterlace_down_scale_and_rotate_180_neon(y, cbcr, pict.planes[0], u_dest, v_dest, w, h, y_byte_per_row, cbcr_byte_per_row,down_scale);
+			} else 
+#endif
+{
+				// 180° y rotation
+				uint8_t* ysrc=y;
+				uint8_t* ydst=&pict.planes[0][h*w-1];
+				for(i=0; i<h*w; i++) {
+					*ydst-- = *ysrc++;
+				}
+				// 180° rotation + de-interlace u/v
+				uint8_t* uvsrc=&cbcr[uv_h*uv_w*2-2];
+				for (i=0; i<uv_h*uv_w*2; i++) {
+					*u_dest++ = *uvsrc--;
+					*v_dest++ = *uvsrc--;
+				}
+			}
+		}
+	} else {
+		bool_t clockwise = rotation == 90 ? TRUE : FALSE;
+		// Rotate Y
+#ifdef __arm__
+		if (hasNeon) {
+			if (clockwise) {
+				rotate_down_scale_plane_neon_clockwise(w,h,y_byte_per_row,(uint8_t*)y,pict.planes[0],down_scale);
+			} else {
+				rotate_down_scale_plane_neon_anticlockwise(w,h,y_byte_per_row,(uint8_t*)y,pict.planes[0], down_scale);
+			}
+		} else 
+#endif
+{
+			uint8_t* dsty = pict.planes[0];
+			uint8_t* srcy = (uint8_t*) y;
+			rotate_plane(w,h,y_byte_per_row,srcy,dsty,1, clockwise);
+		}
+
+#ifdef __arm__
+		if (hasNeon) {
+			rotate_down_scale_cbcr_to_cr_cb(uv_w,uv_h, cbcr_byte_per_row/2, (uint8_t*)cbcr, pict.planes[2], pict.planes[1],clockwise,down_scale);
+		} else 
+#endif
+{
+			// Copying U
+			uint8_t* srcu = cbcr;
+			uint8_t* dstu = pict.planes[1];
+			rotate_plane(uv_w,uv_h,cbcr_byte_per_row/2,srcu,dstu, 2, clockwise);
+			// Copying V
+			uint8_t* srcv = srcu + 1;
+			uint8_t* dstv = pict.planes[2];
+			rotate_plane(uv_w,uv_h,cbcr_byte_per_row/2,srcv,dstv, 2, clockwise);
+		}
+	}
+
+	return yuv_block;
+}
+mblk_t *copy_ycbcrbiplanar_to_true_yuv_with_rotation(uint8_t* y, uint8_t * cbcr, int rotation, int w, int h, int y_byte_per_row,int cbcr_byte_per_row, bool_t uFirstvSecond) {
+	return copy_ycbcrbiplanar_to_true_yuv_with_rotation_and_down_scale_by_2(y, cbcr, rotation, w, h, y_byte_per_row, cbcr_byte_per_row, uFirstvSecond, FALSE);
+}
+void ms_video_init_framerate_controller(MSFrameRateController* ctrl, float fps) {
+	ctrl->start_time = 0;
+	ctrl->th_frame_count = -1;
+	ctrl->fps = fps;
+}
+
+bool_t ms_video_capture_new_frame(MSFrameRateController* ctrl, uint32_t current_time) {
+	int cur_frame;
+	float elapsed;
+
+	/* init controller */
+	if (ctrl->th_frame_count==-1){
+		ctrl->start_time = current_time;
+		ctrl->th_frame_count = 0;
+	}
+
+	elapsed = ((float)(current_time - ctrl->start_time))/1000.0;
+	cur_frame = elapsed * ctrl->fps;
+
+	if (cur_frame>=ctrl->th_frame_count){
+		ctrl->th_frame_count++;
+		return TRUE;
+	} else {
+		return FALSE;
+	}
+}
+
+void ms_video_init_average_fps(MSAverageFPS* afps, float expectedFps) {
+	afps->last_frame_time = -1;
+	afps->last_print_time = -1;
+	afps->mean_inter_frame = 0;
+	afps->expected_fps = expectedFps;
+}
+
+void ms_video_update_average_fps(MSAverageFPS* afps, uint32_t current_time) {
+	if (afps->last_frame_time!=-1){
+		float frame_interval=(float)(current_time - afps->last_frame_time)/1000.0;
+		if (afps->mean_inter_frame==0){
+			afps->mean_inter_frame=frame_interval;
+		}else{
+			afps->mean_inter_frame=(0.8*afps->mean_inter_frame)+(0.2*frame_interval);
+		}
+	} else {
+		afps->last_print_time = current_time;
+	}
+	afps->last_frame_time=current_time;
+
+	if ((current_time - afps->last_print_time > 10000) && afps->mean_inter_frame!=0){
+		ms_message("Captured mean fps=%f, expected=%f",1/afps->mean_inter_frame, afps->expected_fps);
+		afps->last_print_time = current_time;
+	}
+}
