@@ -21,11 +21,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "mediastreamer-config.h"
 #endif
 
-#ifdef HAVE_LIBAVCODEC_AVCODEC_H
-#include <libavcodec/avcodec.h>
-#else
-#include <ffmpeg/avcodec.h>
-#endif
+#include "ffmpeg-priv.h"
 
 #include "mediastreamer2/msfilter.h"
 #include "mediastreamer2/msvideo.h"
@@ -39,6 +35,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include "rfc2429.h"
 
+#define RATE_CONTROL_MARGIN 15000 /*bits/second*/
+
 static bool_t avcodec_initialized=FALSE;
 
 #ifdef ENABLE_LOG_FFMPEG
@@ -65,6 +63,36 @@ void ms_ffmpeg_check_init(){
 	}
 }
 
+/* the goal of this small object is to tell when to send I frames at startup:
+at 2 and 4 seconds*/
+typedef struct VideoStarter{
+	uint64_t next_time;
+	int i_frame_count;
+}VideoStarter;
+
+static void video_starter_init(VideoStarter *vs){
+	vs->next_time=0;
+	vs->i_frame_count=0;
+}
+
+static void video_starter_first_frame(VideoStarter *vs, uint64_t curtime){
+	vs->next_time=curtime+2000;
+}
+
+static bool_t video_starter_need_i_frame(VideoStarter *vs, uint64_t curtime){
+	if (vs->next_time==0) return FALSE;
+	if (curtime>=vs->next_time){
+		vs->i_frame_count++;
+		if (vs->i_frame_count==1){
+			vs->next_time+=2000;
+		}else{
+			vs->next_time=0;
+		}
+		return TRUE;
+	}
+	return FALSE;
+}
+
 typedef struct EncState{
 	AVCodecContext av_context;
 	AVCodec *av_codec;
@@ -77,6 +105,7 @@ typedef struct EncState{
 	int maxbr;
 	int qmin;
 	uint32_t framenum;
+	VideoStarter starter;
 	bool_t req_vfu;
 }EncState;
 
@@ -135,6 +164,7 @@ static bool_t parse_video_fmtp(const char *fmtp, float *fps, MSVideoSize *vsize)
 		}else{
 			ms_warning("unsupported video size %s",tmp);
 			ret=FALSE;
+			goto end;
 		}
 		divider=atoi(equal+1);
 		if (divider!=0){
@@ -145,6 +175,8 @@ static bool_t parse_video_fmtp(const char *fmtp, float *fps, MSVideoSize *vsize)
 			ret=FALSE;
 		}
 	}else ret=FALSE;
+
+end:
 	ms_free(tmp);
 	return ret;
 }
@@ -202,6 +234,12 @@ static void enc_mjpeg_init(MSFilter *f){
 
 static void prepare(EncState *s){
 	AVCodecContext *c=&s->av_context;
+#ifdef ANDROID
+	const int max_br_vbv=128000;
+#else
+	const int max_br_vbv=256000;
+#endif
+
 	avcodec_get_context_defaults(c);
 	if (s->codec==CODEC_ID_MJPEG)
 	{
@@ -217,13 +255,21 @@ static void prepare(EncState *s){
 	}
 
 	/* put codec parameters */
-	c->bit_rate=(float)s->maxbr*0.7;
-	c->bit_rate_tolerance=s->fps!=1?(float)c->bit_rate/(s->fps-1):c->bit_rate;
+	/* in order to take in account RTP protocol overhead and avoid possible
+	 bitrate peaks especially on low bandwidth, we make a correction on the 
+	 codec's target bitrate.
+	*/
+	c->bit_rate=(float)s->maxbr*0.92;
+	if (c->bit_rate>RATE_CONTROL_MARGIN){
+		 c->bit_rate -= RATE_CONTROL_MARGIN;
+	}
+	c->bit_rate_tolerance=s->fps>1?(float)c->bit_rate/(s->fps-1):c->bit_rate;
 
-	if (s->codec!=CODEC_ID_SNOW && s->maxbr<256000){
+	/* ffmpeg vbv rate control consumes too much cpu above a certain target bitrate.
+	We don't use it above max_br_vbv */
+	if (s->codec!=CODEC_ID_SNOW && s->maxbr<max_br_vbv){
 		/*snow does not like 1st pass rate control*/
-		/*and rate control eats too much cpu with CIF high fps pictures*/
-		c->rc_max_rate=(float)s->maxbr*0.8;
+		c->rc_max_rate=c->bit_rate;
 		c->rc_min_rate=0;
 		c->rc_buffer_size=c->rc_max_rate;
 	}else{
@@ -236,13 +282,15 @@ static void prepare(EncState *s){
 	c->height = s->vsize.height;
 	c->time_base.num = 1;
 	c->time_base.den = (int)s->fps;
-	c->gop_size=(int)s->fps*5; /*emit I frame every 5 seconds*/
+	c->gop_size=(int)s->fps*10; /*emit I frame every 10 seconds*/
 	c->pix_fmt=PIX_FMT_YUV420P;
 	s->comp_buf=allocb(c->bit_rate*2,0);
 	if (s->codec==CODEC_ID_SNOW){
 		c->strict_std_compliance=-2;
 	}
 	
+	ms_message("Codec size set to w=%i/h=%i",c->width, c->height);
+
 }
 
 static void prepare_h263(EncState *s){
@@ -270,28 +318,12 @@ static void prepare_h263(EncState *s){
 static void prepare_mpeg4(EncState *s){
 	AVCodecContext *c=&s->av_context;
 	c->max_b_frames=0; /*don't use b frames*/
-	c->flags|=CODEC_FLAG_AC_PRED;
-	c->flags|=CODEC_FLAG_H263P_UMV;
-	/*c->flags|=CODEC_FLAG_QPEL;*/ /*don't enable this one: this forces profile_level to advanced simple profile */
-	c->flags|=CODEC_FLAG_4MV;
-	c->flags|=CODEC_FLAG_GMC;
-	c->flags|=CODEC_FLAG_LOOP_FILTER;
-	c->flags|=CODEC_FLAG_H263P_SLICE_STRUCT;
 }
 
 static void enc_uninit(MSFilter  *f){
 	EncState *s=(EncState*)f->data;
 	ms_free(s);
 }
-#if 0
-static void enc_set_rc(EncState *s, AVCodecContext *c){
-	int factor=c->width/MS_VIDEO_SIZE_QCIF_W;
-	c->rc_min_rate=0;
-	c->bit_rate=400; /* this value makes around 100kbit/s at QCIF=2 */
-	c->rc_max_rate=c->bit_rate+1;
-	c->rc_buffer_size=20000*factor;	/* food recipe */
-}
-#endif
 
 static void enc_preprocess(MSFilter *f){
 	EncState *s=(EncState*)f->data;
@@ -319,8 +351,10 @@ static void enc_preprocess(MSFilter *f){
 		ms_error("avcodec_open() failed: %i",error);
 		return;
 	}
+	video_starter_init(&s->starter);
 	ms_debug("image format is %i.",s->av_context.pix_fmt);
 	ms_message("qmin=%i qmax=%i",s->av_context.qmin,s->av_context.qmax);
+	s->framenum=0;
 }
 
 static void enc_postprocess(MSFilter *f){
@@ -643,7 +677,7 @@ static mblk_t *skip_jpeg_headers(mblk_t *full_frame, mblk_t **lqt, mblk_t **cqt)
 	uint8_t *pbuf_ptr=full_frame->b_rptr;
 	uint8_t *buf_end=full_frame->b_wptr;	
 
-	ms_message("image size: %i)", buf_end-pbuf_ptr);
+	ms_message("image size: %li)", (long)(buf_end-pbuf_ptr));
 
 	*lqt=NULL;
 	*cqt=NULL;
@@ -651,7 +685,7 @@ static mblk_t *skip_jpeg_headers(mblk_t *full_frame, mblk_t **lqt, mblk_t **cqt)
 	err = find_marker(&pbuf_ptr, buf_end);
 	while (err!=-1)
 	{
-		ms_message("marker found: %x (offset from beginning%i)", err, pbuf_ptr-full_frame->b_rptr);
+		ms_message("marker found: %x (offset from beginning %li)", err, (long)(pbuf_ptr-full_frame->b_rptr));
 		if (err==0xdb)
 		{
 			/* copy DQT table */
@@ -734,15 +768,17 @@ static void process_frame(MSFilter *f, mblk_t *inm){
 	int error;
 	mblk_t *comp_buf=s->comp_buf;
 	int comp_buf_sz=comp_buf->b_datap->db_lim-comp_buf->b_datap->db_base;
+	YuvBuf yuv;
 
+	ms_yuv_buf_init_from_mblk(&yuv, inm);
 	/* convert image if necessary */
 	avcodec_get_frame_defaults(&pict);
-	avpicture_fill((AVPicture*)&pict,(uint8_t*)inm->b_rptr,c->pix_fmt,c->width,c->height);
+	avpicture_fill((AVPicture*)&pict,yuv.planes[0],c->pix_fmt,c->width,c->height);
 	
 	/* timestamp used by ffmpeg, unset here */
 	pict.pts=AV_NOPTS_VALUE;
 		
-	if (s->framenum==(int)(s->fps*2.0) || s->framenum==(int)(s->fps*4.0)){
+	if (video_starter_need_i_frame (&s->starter,f->ticker->time)){
 		/*sends an I frame at 2 seconds and 4 seconds after the beginning of the call*/
 		s->req_vfu=TRUE;
 	}
@@ -758,10 +794,15 @@ static void process_frame(MSFilter *f, mblk_t *inm){
 		comp_buf->b_wptr+=4;
 		comp_buf_sz-=4;
 	}
+
 	error=avcodec_encode_video(c, (uint8_t*)comp_buf->b_wptr,comp_buf_sz, &pict);
+
 	if (error<=0) ms_warning("ms_AVencoder_process: error %i.",error);
 	else{
 		s->framenum++;
+		if (s->framenum==1){
+			video_starter_first_frame (&s->starter,f->ticker->time);
+		}
 		if (c->coded_frame->pict_type==FF_I_TYPE){
 			ms_message("Emitting I-frame");
 		}
@@ -796,22 +837,35 @@ static int enc_set_br(MSFilter *f, void *arg){
 	EncState *s=(EncState*)f->data;
 	bool_t snow=s->codec==CODEC_ID_SNOW;
 	s->maxbr=*(int*)arg;
+	if (s->av_context.codec!=NULL){
+		/*when we are processing, apply new settings immediately*/
+		ms_filter_lock(f);
+		enc_postprocess(f);
+		enc_preprocess(f);
+		ms_filter_unlock(f);
+		return 0;
+	}
 	if (s->maxbr>=1024000 && s->codec!=CODEC_ID_H263P){
 		s->vsize.width = MS_VIDEO_SIZE_SVGA_W;
 		s->vsize.height = MS_VIDEO_SIZE_SVGA_H;
-		s->fps=17;
+		s->fps=25;
 	}else if (s->maxbr>=800000 && s->codec!=CODEC_ID_H263P){
 		s->vsize.width = MS_VIDEO_SIZE_VGA_W;
 		s->vsize.height = MS_VIDEO_SIZE_VGA_H;
-		s->fps=17;
+		s->fps=25;
 	}else if (s->maxbr>=512000){
 		s->vsize.width=MS_VIDEO_SIZE_CIF_W;
 		s->vsize.height=MS_VIDEO_SIZE_CIF_H;
-		s->fps=17;
+		s->fps=25;
 	}else if (s->maxbr>=256000){
 		s->vsize.width=MS_VIDEO_SIZE_CIF_W;
 		s->vsize.height=MS_VIDEO_SIZE_CIF_H;
-		s->fps=10;
+		s->fps=17;
+		s->qmin=3;
+	}else if (s->maxbr>=170000){
+		s->vsize.width=MS_VIDEO_SIZE_QVGA_W;
+		s->vsize.height=MS_VIDEO_SIZE_QVGA_H;
+		s->fps=15;
 		s->qmin=3;
 	}else if (s->maxbr>=128000){
 		s->vsize.width=MS_VIDEO_SIZE_QCIF_W;
@@ -829,13 +883,6 @@ static int enc_set_br(MSFilter *f, void *arg){
 		s->fps=5;
 		s->qmin=5;
 	}
-	if (s->av_context.codec!=NULL){
-		/*apply new settings dynamically*/
-		ms_filter_lock(f);
-		enc_postprocess(f);
-		enc_preprocess(f);
-		ms_filter_unlock(f);
-	}
 	return 0;
 }
 
