@@ -17,6 +17,10 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
+#ifdef HAVE_CONFIG_H
+#include "mediastreamer-config.h"
+#endif
+
 #include "mediastreamer2/mediastream.h"
 #include "mediastreamer2/msfilter.h"
 #include "mediastreamer2/msvideo.h"
@@ -24,9 +28,9 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "mediastreamer2/msvideoout.h"
 #include "mediastreamer2/msextdisplay.h"
 
-#ifdef HAVE_CONFIG_H
-#include "mediastreamer-config.h"
-#endif
+#include <ortp/zrtp.h>
+
+
 
 extern RtpSession * create_duplex_rtpsession( int locport, bool_t ipv6);
 
@@ -41,7 +45,7 @@ void video_stream_free (VideoStream * stream)
 	}
 	if (stream->rtprecv != NULL)
 		ms_filter_destroy (stream->rtprecv);
-	if (stream->rtpsend!=NULL) 
+	if (stream->rtpsend!=NULL)
 		ms_filter_destroy (stream->rtpsend);
 	if (stream->source != NULL)
 		ms_filter_destroy (stream->source);
@@ -69,7 +73,10 @@ void video_stream_free (VideoStream * stream)
 		ortp_ev_queue_destroy(stream->evq);
 	if (stream->display_name!=NULL)
 		ms_free(stream->display_name);
-	
+	if (stream->rc!=NULL){
+		ms_bitrate_controller_destroy(stream->rc);
+	}
+
 	ms_free (stream);
 }
 
@@ -93,7 +100,7 @@ void video_stream_change_decoder(VideoStream *stream, int payload){
 	PayloadType *pt=rtp_profile_get_payload(prof,payload);
 	if (pt!=NULL){
 		MSFilter *dec;
-		
+
 		if (stream->decoder!=NULL && stream->decoder->desc->enc_fmt!=NULL &&
 		    strcasecmp(pt->mime_type,stream->decoder->desc->enc_fmt)==0){
 			/* same formats behind different numbers, nothing to do */
@@ -104,7 +111,7 @@ void video_stream_change_decoder(VideoStream *stream, int payload){
 			ms_filter_unlink(stream->rtprecv, 0, stream->decoder, 0);
 			if (stream->tee2)
 				ms_filter_unlink(stream->decoder,0,stream->tee2,0);
-			else 
+			else
 				ms_filter_unlink(stream->decoder,0,stream->output,0);
 			ms_filter_postprocess(stream->decoder);
 			ms_filter_destroy(stream->decoder);
@@ -126,30 +133,6 @@ void video_stream_change_decoder(VideoStream *stream, int payload){
 	}
 }
 
-static void video_stream_adapt_bitrate(VideoStream *stream, int jitter, float lost){
-	if (stream->encoder!=NULL){
-		if (lost>10){
-			int bitrate=0;
-			int new_bitrate;
-			ms_warning("Remote reports bad receiving experience, trying to reduce bitrate of video encoder.");
-			
-			ms_filter_call_method(stream->encoder,MS_FILTER_GET_BITRATE,&bitrate);
-			if (bitrate==0){
-				ms_error("Video encoder does not implement MS_FILTER_GET_BITRATE.");
-				return;
-			}
-			if (bitrate>=20000){
-				new_bitrate=bitrate-10000;
-				ms_warning("Encoder bitrate reduced from %i to %i b/s.",bitrate,new_bitrate);
-				ms_filter_call_method(stream->encoder,MS_FILTER_SET_BITRATE,&new_bitrate);
-			}else{
-				ms_warning("Video encoder bitrate already at minimum.");
-			}
-			
-		}
-	}
-}
-
 static void video_steam_process_rtcp(VideoStream *stream, mblk_t *m){
 	do{
 		if (rtcp_is_SR(m)){
@@ -158,11 +141,13 @@ static void video_steam_process_rtcp(VideoStream *stream, mblk_t *m){
 			rb=rtcp_SR_get_report_block(m,0);
 			if (rb){
 				unsigned int ij;
+				float rt=rtp_session_get_round_trip_propagation(stream->session);
 				float flost;
 				ij=report_block_get_interarrival_jitter(rb);
 				flost=(float)(100.0*report_block_get_fraction_lost(rb)/256.0);
-				ms_message("interarrival jitter=%u , lost packets percentage since last report=%f ",ij,flost);
-				if (stream->adapt_bitrate) video_stream_adapt_bitrate(stream,ij,flost);
+				ms_message("video_steam_process_rtcp: interarrival jitter=%u , lost packets percentage since last report=%f, round trip time=%f seconds",ij,flost,rt);
+				if (stream->rc)
+					ms_bitrate_controller_process_rtcp(stream->rc,m);
 			}
 		}
 	}while(rtcp_next_packet(m));
@@ -175,9 +160,10 @@ void video_stream_iterate(VideoStream *stream){
 			MS_VIDEO_OUT_HANDLE_RESIZING);
 	*/
 	if (stream->evq){
-		OrtpEvent *ev=ortp_ev_queue_get(stream->evq);
-		if (ev!=NULL){
-			if (ortp_event_get_type(ev)==ORTP_EVENT_RTCP_PACKET_RECEIVED){
+		OrtpEvent *ev;
+		while (NULL != (ev=ortp_ev_queue_get(stream->evq))) {
+			OrtpEventType evt=ortp_event_get_type(ev);
+			if (evt == ORTP_EVENT_RTCP_PACKET_RECEIVED){
 				OrtpEventData *evd=ortp_event_get_data(ev);
 				video_steam_process_rtcp(stream,evd->packet);
 			}
@@ -199,6 +185,8 @@ static void choose_display_name(VideoStream *stream){
 	stream->display_name=ms_strdup("MSAndroidDisplay");
 #elif defined (HAVE_X11_EXTENSIONS_XV_H)
 	stream->display_name=ms_strdup("MSX11Video");
+#elif defined(__ios)
+	stream->display_name=ms_strdup("IOSDisplay");	
 #else
 	stream->display_name=ms_strdup("MSVideoOut");
 #endif
@@ -236,7 +224,7 @@ void video_stream_enable_self_view(VideoStream *stream, bool_t val){
 }
 
 void video_stream_enable_adaptive_bitrate_control(VideoStream *s, bool_t yesno){
-	s->adapt_bitrate=yesno;
+	s->use_rc=yesno;
 }
 
 void video_stream_set_render_callback (VideoStream *s, VideoStreamRenderCallback cb, void *user_pointer){
@@ -277,7 +265,7 @@ static MSVideoSize get_compatible_size(MSVideoSize maxsize, MSVideoSize wished_s
 	int max_area=maxsize.width*maxsize.height;
 	int whished_area=wished_size.width*wished_size.height;
 	if (whished_area>max_area){
-		return ms_video_size_change_orientation(maxsize,ms_video_size_get_orientation(wished_size));
+		return maxsize;
 	}
 	return wished_size;
 }
@@ -286,9 +274,26 @@ static void configure_video_source(VideoStream *stream){
 	MSVideoSize vsize,cam_vsize;
 	float fps=15;
 	MSPixFmt format;
-	
+
+	/* transmit orientation to source filter */
+	ms_filter_call_method(stream->source,MS_VIDEO_CAPTURE_SET_DEVICE_ORIENTATION,&stream->device_orientation);
+	/* transmit its preview window id if any to source filter*/
+	if (stream->preview_window_id!=0){
+		video_stream_set_native_preview_window_id(stream, stream->preview_window_id);
+	}
+
 	ms_filter_call_method(stream->encoder,MS_FILTER_GET_VIDEO_SIZE,&vsize);
 	vsize=get_compatible_size(vsize,stream->sent_vsize);
+	ms_filter_call_method(stream->source,MS_FILTER_SET_VIDEO_SIZE,&vsize);
+	/*the camera may not support the target size and suggest a one close to the target */
+	ms_filter_call_method(stream->source,MS_FILTER_GET_VIDEO_SIZE,&cam_vsize);
+	if (cam_vsize.width*cam_vsize.height<=vsize.width*vsize.height &&
+			cam_vsize.width != vsize.width){
+		vsize=cam_vsize;
+		ms_message("Output video size adjusted to match camera resolution (%ix%i)\n",vsize.width,vsize.height);
+	} else if (cam_vsize.width*cam_vsize.height>vsize.width*vsize.height){
+		ms_warning("Camera video size greater than encoder one. A scaling filter will be used!\n");
+	}
 	ms_filter_call_method(stream->encoder,MS_FILTER_SET_VIDEO_SIZE,&vsize);
 	ms_filter_call_method(stream->encoder,MS_FILTER_GET_FPS,&fps);
 	ms_message("Setting sent vsize=%ix%i, fps=%f",vsize.width,vsize.height,fps);
@@ -296,20 +301,26 @@ static void configure_video_source(VideoStream *stream){
 	if (ms_filter_get_id(stream->source)!=MS_STATIC_IMAGE_ID) {
 		ms_filter_call_method(stream->source,MS_FILTER_SET_FPS,&fps);
 	}
-	ms_filter_call_method(stream->source,MS_FILTER_SET_VIDEO_SIZE,&vsize);
 	/* get the output format for webcam reader */
 	ms_filter_call_method(stream->source,MS_FILTER_GET_PIX_FMT,&format);
+
 	if (format==MS_MJPEG){
 		stream->pixconv=ms_filter_new(MS_MJPEG_DEC_ID);
 	}else{
 		stream->pixconv = ms_filter_new(MS_PIX_CONV_ID);
 		/*set it to the pixconv */
 		ms_filter_call_method(stream->pixconv,MS_FILTER_SET_PIX_FMT,&format);
-		ms_filter_call_method(stream->source,MS_FILTER_GET_VIDEO_SIZE,&cam_vsize);
 		ms_filter_call_method(stream->pixconv,MS_FILTER_SET_VIDEO_SIZE,&cam_vsize);
 	}
 	stream->sizeconv=ms_filter_new(MS_SIZE_CONV_ID);
 	ms_filter_call_method(stream->sizeconv,MS_FILTER_SET_VIDEO_SIZE,&vsize);
+	if (stream->rc){
+		ms_bitrate_controller_destroy(stream->rc);
+		stream->rc=NULL;
+	}
+	if (stream->use_rc){
+		stream->rc=ms_av_bitrate_controller_new(NULL,NULL,stream->session,stream->encoder);
+	}
 }
 
 int video_stream_start (VideoStream *stream, RtpProfile *profile, const char *remip, int remport,
@@ -324,7 +335,7 @@ int video_stream_start (VideoStream *stream, RtpProfile *profile, const char *re
 
 	if (cam==NULL){
 		cam=ms_web_cam_manager_get_default_cam (
-		      ms_web_cam_manager_get());                                
+		      ms_web_cam_manager_get());
 	}
 
 	pt=rtp_profile_get_payload(profile,payload);
@@ -332,7 +343,7 @@ int video_stream_start (VideoStream *stream, RtpProfile *profile, const char *re
 		ms_error("videostream.c: undefined payload type.");
 		return -1;
 	}
-	
+
 	rtp_session_set_profile(rtps,profile);
 	if (remport>0) rtp_session_set_remote_addr_full(rtps,remip,remport,rem_rtcp_port);
 	rtp_session_set_payload_type(rtps,payload);
@@ -348,7 +359,7 @@ int video_stream_start (VideoStream *stream, RtpProfile *profile, const char *re
 	rtp_session_set_jitter_buffer_params(stream->session,&jbp);
 	rtp_session_set_rtp_socket_recv_buffer_size(stream->session,socket_buf_size);
 	rtp_session_set_rtp_socket_send_buffer_size(stream->session,socket_buf_size);
-	
+
 	if (stream->dir==VideoStreamSendRecv || stream->dir==VideoStreamSendOnly){
 		/*plumb the outgoing stream */
 
@@ -363,7 +374,7 @@ int video_stream_start (VideoStream *stream, RtpProfile *profile, const char *re
 		stream->cam=cam;
 		stream->source = ms_web_cam_create_reader(cam);
 		stream->tee = ms_filter_new(MS_TEE_ID);
-		
+
 		if (pt->normal_bitrate>0){
 			ms_message("Limiting bitrate of video encoder to %i bits/s",pt->normal_bitrate);
 			ms_filter_call_method(stream->encoder,MS_FILTER_SET_BITRATE,&pt->normal_bitrate);
@@ -376,7 +387,7 @@ int video_stream_start (VideoStream *stream, RtpProfile *profile, const char *re
 				stream->output2=ms_filter_new_from_name (stream->display_name);
 			}
 		}
-		
+
 		configure_video_source (stream);
 			/* and then connect all */
 		ms_filter_link (stream->source, 0, stream->pixconv, 0);
@@ -385,7 +396,7 @@ int video_stream_start (VideoStream *stream, RtpProfile *profile, const char *re
 		ms_filter_link (stream->tee, 0 ,stream->encoder, 0 );
 		ms_filter_link (stream->encoder,0, stream->rtpsend,0);
 		if (stream->output2){
-			if (stream->window_id!=0){
+			if (stream->preview_window_id!=0){
 				ms_filter_call_method(stream->output2, MS_VIDEO_DISPLAY_SET_NATIVE_WINDOW_ID,&stream->preview_window_id);
 			}
 			ms_filter_link(stream->tee,1,stream->output2,0);
@@ -405,7 +416,7 @@ int video_stream_start (VideoStream *stream, RtpProfile *profile, const char *re
 		stream->rtprecv = ms_filter_new (MS_RTP_RECV_ID);
 		ms_filter_call_method(stream->rtprecv,MS_RTP_RECV_SET_SESSION,stream->session);
 
-		
+
 		stream->jpegwriter=ms_filter_new(MS_JPEG_WRITER_ID);
 		if (stream->jpegwriter)
 			stream->tee2=ms_filter_new(MS_TEE_ID);
@@ -422,7 +433,7 @@ int video_stream_start (VideoStream *stream, RtpProfile *profile, const char *re
 		}
 		if (pt->recv_fmtp!=NULL)
 			ms_filter_call_method(stream->decoder,MS_FILTER_ADD_FMTP,(void*)pt->recv_fmtp);
-	
+
 		/*force the decoder to output YUV420P */
 		format=MS_YUV420P;
 		ms_filter_call_method(stream->decoder,MS_FILTER_SET_PIX_FMT,&format);
@@ -470,6 +481,8 @@ void video_stream_update_video_params(VideoStream *stream){
 }
 
 void video_stream_change_camera(VideoStream *stream, MSWebCam *cam){
+	bool_t keep_source=(cam==stream->cam);
+	
 	if (stream->ticker && stream->source){
 		ms_ticker_detach(stream->ticker,stream->source);
 		/*unlink source filters and subsequent post processin filters */
@@ -477,18 +490,19 @@ void video_stream_change_camera(VideoStream *stream, MSWebCam *cam){
 		ms_filter_unlink (stream->pixconv, 0, stream->sizeconv, 0);
 		ms_filter_unlink (stream->sizeconv, 0, stream->tee, 0);
 		/*destroy the filters */
-		ms_filter_destroy(stream->source);
+		if (!keep_source) ms_filter_destroy(stream->source);
 		ms_filter_destroy(stream->pixconv);
 		ms_filter_destroy(stream->sizeconv);
 
 		/*re create new ones and configure them*/
-		stream->source = ms_web_cam_create_reader(cam);
+		if (!keep_source) stream->source = ms_web_cam_create_reader(cam);
 		stream->cam=cam;
 		configure_video_source(stream);
+		
 		ms_filter_link (stream->source, 0, stream->pixconv, 0);
 		ms_filter_link (stream->pixconv, 0, stream->sizeconv, 0);
 		ms_filter_link (stream->sizeconv, 0, stream->tee, 0);
-		
+
 		ms_ticker_attach(stream->ticker,stream->source);
 	}
 }
@@ -508,7 +522,7 @@ video_stream_stop (VideoStream * stream)
 			ms_ticker_detach(stream->ticker,stream->source);
 		if (stream->rtprecv)
 			ms_ticker_detach(stream->ticker,stream->rtprecv);
-	
+
 		rtp_stats_display(rtp_session_get_stats(stream->session),"Video session's RTP statistics");
 
 		if (stream->source){
@@ -564,6 +578,12 @@ void video_stream_set_native_window_id(VideoStream *stream, unsigned long id){
 
 void video_stream_set_native_preview_window_id(VideoStream *stream, unsigned long id){
 	stream->preview_window_id=id;
+	if (stream->output2){
+		ms_filter_call_method(stream->output2,MS_VIDEO_DISPLAY_SET_NATIVE_WINDOW_ID,&id);
+	}
+	if (stream->source){
+		ms_filter_call_method(stream->source,MS_VIDEO_DISPLAY_SET_NATIVE_WINDOW_ID,&id);
+	}
 }
 
 unsigned long video_stream_get_native_preview_window_id(VideoStream *stream){
@@ -572,6 +592,11 @@ unsigned long video_stream_get_native_preview_window_id(VideoStream *stream){
 		if (ms_filter_call_method(stream->output2,MS_VIDEO_DISPLAY_GET_NATIVE_WINDOW_ID,&id)==0)
 			return id;
 	}
+	if (stream->source){
+		if (ms_filter_has_method(stream->source,MS_VIDEO_DISPLAY_GET_NATIVE_WINDOW_ID) 
+		    && ms_filter_call_method(stream->source,MS_VIDEO_DISPLAY_GET_NATIVE_WINDOW_ID,&id)==0)
+			return id;
+	}
 	return stream->preview_window_id;
 }
 
@@ -579,6 +604,17 @@ void video_stream_use_preview_video_window(VideoStream *stream, bool_t yesno){
 	stream->use_preview_window=yesno;
 }
 
+void video_stream_set_device_rotation(VideoStream *stream, int orientation){
+	if (stream == 0)
+		return;
+
+	stream->device_orientation = orientation;
+	MSFilter* target_filter=stream->source;
+	if (target_filter){
+		ms_filter_call_method(target_filter,MS_VIDEO_CAPTURE_SET_DEVICE_ORIENTATION,&orientation);
+	}
+}
+
 VideoPreview * video_preview_new(void){
 	VideoPreview *stream = (VideoPreview *)ms_new0 (VideoPreview, 1);
 	stream->sent_vsize.width=MS_VIDEO_SIZE_CIF_W;
@@ -599,7 +635,6 @@ void video_preview_start(VideoPreview *stream, MSWebCam *device){
 
 	stream->source = ms_web_cam_create_reader(device);
 
-	stream->output2=ms_filter_new_from_name (displaytype);
 
 	/* configure the filters */
 	ms_filter_call_method(stream->source,MS_FILTER_SET_VIDEO_SIZE,&vsize);
@@ -616,18 +651,20 @@ void video_preview_start(VideoPreview *stream, MSWebCam *device){
 	}
 
 	format=MS_YUV420P;
+
+	stream->output2=ms_filter_new_from_name (displaytype);
 	ms_filter_call_method(stream->output2,MS_FILTER_SET_PIX_FMT,&format);
 	ms_filter_call_method(stream->output2,MS_FILTER_SET_VIDEO_SIZE,&disp_size);
 	ms_filter_call_method(stream->output2,MS_VIDEO_DISPLAY_ENABLE_MIRRORING,&mirroring);
 	ms_filter_call_method(stream->output2,MS_VIDEO_DISPLAY_SET_LOCAL_VIEW_MODE,&corner);
-	if (stream->preview_window_id!=0){
-		ms_filter_call_method(stream->output2, MS_VIDEO_DISPLAY_SET_NATIVE_WINDOW_ID,&stream->preview_window_id);
-	}
 	/* and then connect all */
 
 	ms_filter_link(stream->source,0, stream->pixconv,0);
 	ms_filter_link(stream->pixconv, 0, stream->output2, 0);
 
+	if (stream->preview_window_id!=0){
+		video_stream_set_native_preview_window_id(stream, stream->preview_window_id);
+	}
 	/* create the ticker */
 	stream->ticker = ms_ticker_new();
 	ms_ticker_set_name(stream->ticker,"Video MSTicker");
@@ -638,7 +675,6 @@ void video_preview_stop(VideoStream *stream){
 	ms_ticker_detach(stream->ticker, stream->source);
 	ms_filter_unlink(stream->source,0,stream->pixconv,0);
 	ms_filter_unlink(stream->pixconv,0,stream->output2,0);
-	
 	video_stream_free(stream);
 }
 
@@ -648,7 +684,7 @@ int video_stream_recv_only_start(VideoStream *videostream, RtpProfile *profile,
 }
 
 int video_stream_send_only_start(VideoStream *videostream,
-				RtpProfile *profile, const char *addr, int port, int rtcp_port, 
+				RtpProfile *profile, const char *addr, int port, int rtcp_port,
 				int used_pt, int  jitt_comp, MSWebCam *device){
 	video_stream_set_direction (videostream,VideoStreamSendOnly);
 	return video_stream_start(videostream,profile,addr,port,rtcp_port,used_pt,jitt_comp,device);
@@ -662,3 +698,40 @@ void video_stream_recv_only_stop(VideoStream *vs){
 void video_stream_send_only_stop(VideoStream *vs){
 	video_stream_stop(vs);
 }
+
+/* enable ZRTP on the video stream using information from the audio stream */
+void video_stream_enable_zrtp(VideoStream *vstream, AudioStream *astream, OrtpZrtpParams *param){
+	if (astream->ortpZrtpContext != NULL) {
+		vstream->ortpZrtpContext=ortp_zrtp_multistream_new(astream->ortpZrtpContext, vstream->session, param);
+	}
+}
+
+bool_t video_stream_enable_strp(VideoStream* stream, enum ortp_srtp_crypto_suite_t suite, const char* snd_key, const char* rcv_key) {
+	// assign new srtp transport to stream->session
+	// with 2 Master Keys
+	RtpTransport *rtp_tpt, *rtcp_tpt;	
+	
+	if (!ortp_srtp_supported()) {
+		ms_error("ortp srtp support not enabled");
+		return FALSE;
+	}
+	
+	ms_message("%s: stream=%p key='%s' key='%s'", __FUNCTION__,
+		stream, snd_key, rcv_key);
+	 
+	stream->srtp_session = ortp_srtp_create_configure_session(suite, 
+		rtp_session_get_send_ssrc(stream->session), 
+		snd_key,
+		rcv_key); 
+	
+	if (!stream->srtp_session) {
+		return FALSE;
+	}
+	
+	// TODO: check who will free rtp_tpt ?
+	srtp_transport_new(stream->srtp_session, &rtp_tpt, &rtcp_tpt);
+	
+	rtp_session_set_transports(stream->session, rtp_tpt, rtcp_tpt);
+	
+	return TRUE;
+}
