@@ -17,10 +17,22 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
+#ifdef HAVE_CONFIG_H
+#include "mediastreamer-config.h"
+#endif
+
+#include <signal.h>
+
 #include "mediastreamer2/mediastream.h"
 #include "mediastreamer2/msvideoout.h"
 #include "mediastreamer2/msv4l.h"
 
+static int stopped=FALSE;
+
+static void stop(int signum){
+	stopped=TRUE;
+}
+
 int main(int argc, char *argv[]){
 	VideoStream *vs;
 	MSWebCam *cam;
@@ -34,12 +46,17 @@ int main(int argc, char *argv[]){
 	ortp_set_log_level_mask(ORTP_MESSAGE|ORTP_WARNING|ORTP_ERROR|ORTP_FATAL);
 	ms_init();
 	cam=ms_web_cam_manager_get_default_cam(ms_web_cam_manager_get());
+
+	signal(SIGINT,stop);
 	/* this is to test the sequence start/stop */
 	for(i=0;i<1;++i){
 		int n;
-		vs=video_preview_start(cam,vsize);
+		vs=video_preview_new();
+		/*video_preview_set_display_filter_name(vs,"MSVideoOut");*/
+		video_preview_set_size(vs,vsize);
+		video_preview_start(vs, cam);
 
-        	for(n=0;n<1000;++n){
+        for(n=0;n<60000 && !stopped;++n){
 #ifdef WIN32
 			MSG msg;
     		Sleep(100);
@@ -63,11 +80,11 @@ int main(int argc, char *argv[]){
 
 			    vs->tee = ms_filter_new(MS_TEE_ID);
 
-			    ms_filter_unlink(vs->pixconv,0, vs->output,0);
+			    ms_filter_unlink(vs->pixconv,0, vs->output2,0);
 
 			    ms_filter_link(vs->pixconv,0,vs->tee,0);
-			    ms_filter_link(vs->tee,0,vs->output,0);
-			    ms_filter_link(vs->tee,1,vs->output,1);
+			    ms_filter_link(vs->tee,0,vs->output2,0);
+			    ms_filter_link(vs->tee,1,vs->output2,1);
 			    
 			    //ms_filter_unlink(vs->tee,0,vs->output,0);
 			    ms_ticker_attach (vs->ticker, vs->source);
@@ -76,34 +93,34 @@ int main(int argc, char *argv[]){
 			if (n==500)
 			  {
 			    int corner=1;
-			    ms_filter_call_method(vs->output,MS_VIDEO_DISPLAY_SET_LOCAL_VIEW_MODE,&corner);
+			    ms_filter_call_method(vs->output2,MS_VIDEO_DISPLAY_SET_LOCAL_VIEW_MODE,&corner);
 			  }
 			if (n==600)
 			  {
 			    int corner=2;
-			    ms_filter_call_method(vs->output,MS_VIDEO_DISPLAY_SET_LOCAL_VIEW_MODE,&corner);
+			    ms_filter_call_method(vs->output2,MS_VIDEO_DISPLAY_SET_LOCAL_VIEW_MODE,&corner);
 			  }
 			if (n==700)
 			  {
 			    int corner=3;
-			    ms_filter_call_method(vs->output,MS_VIDEO_DISPLAY_SET_LOCAL_VIEW_MODE,&corner);
+			    ms_filter_call_method(vs->output2,MS_VIDEO_DISPLAY_SET_LOCAL_VIEW_MODE,&corner);
 			  }
 			if (n==800)
 			  {
 			    int corner=-1;
-			    ms_filter_call_method(vs->output,MS_VIDEO_DISPLAY_SET_LOCAL_VIEW_MODE,&corner);
+			    ms_filter_call_method(vs->output2,MS_VIDEO_DISPLAY_SET_LOCAL_VIEW_MODE,&corner);
 			  }
 			if (n==900)
 			  {
 			    ms_ticker_detach (vs->ticker, vs->source);
 
 			    ms_filter_unlink(vs->pixconv,0,vs->tee,0);
-			    ms_filter_unlink(vs->tee,0,vs->output,0);
-			    ms_filter_unlink(vs->tee,1,vs->output,1);
+			    ms_filter_unlink(vs->tee,0,vs->output2,0);
+			    ms_filter_unlink(vs->tee,1,vs->output2,1);
 			    ms_filter_destroy(vs->tee);
 			    vs->tee=NULL;
 
-			    ms_filter_link(vs->pixconv,0, vs->output,0);
+			    ms_filter_link(vs->pixconv,0, vs->output2,0);
 
 			    
 			    ms_ticker_attach (vs->ticker, vs->source);
@@ -113,3 +130,4 @@ int main(int argc, char *argv[]){
 	}
 	return 0;
 }
+
