@@ -235,6 +235,50 @@ PayloadType payload_type_g726_16={
 	CHANNELS(1)
 };
 
+PayloadType payload_type_aal2_g726_40={
+	TYPE( PAYLOAD_AUDIO_PACKETIZED),
+	CLOCK_RATE(8000),
+	BITS_PER_SAMPLE( 0),
+	ZERO_PATTERN(NULL),
+	PATTERN_LENGTH( 0),
+	NORMAL_BITRATE( 8000),
+	MIME_TYPE ("AAL2-G726-40"),
+	CHANNELS(1)
+};
+
+PayloadType payload_type_aal2_g726_32={
+	TYPE( PAYLOAD_AUDIO_PACKETIZED),
+	CLOCK_RATE(8000),
+	BITS_PER_SAMPLE( 0),
+	ZERO_PATTERN(NULL),
+	PATTERN_LENGTH( 0),
+	NORMAL_BITRATE( 8000),
+	MIME_TYPE ("AAL2-G726-32"),
+	CHANNELS(1)
+};
+
+PayloadType payload_type_aal2_g726_24={
+	TYPE( PAYLOAD_AUDIO_PACKETIZED),
+	CLOCK_RATE(8000),
+	BITS_PER_SAMPLE( 0),
+	ZERO_PATTERN(NULL),
+	PATTERN_LENGTH( 0),
+	NORMAL_BITRATE( 8000),
+	MIME_TYPE ("AAL2-G726-24"),
+	CHANNELS(1)
+};
+
+PayloadType payload_type_aal2_g726_16={
+	TYPE( PAYLOAD_AUDIO_PACKETIZED),
+	CLOCK_RATE(8000),
+	BITS_PER_SAMPLE( 0),
+	ZERO_PATTERN(NULL),
+	PATTERN_LENGTH( 0),
+	NORMAL_BITRATE( 8000),
+	MIME_TYPE ("AAL2-G726-16"),
+	CHANNELS(1)
+};
+
 PayloadType payload_type_mpv=
 {
 	TYPE( PAYLOAD_VIDEO),
@@ -303,6 +347,7 @@ void av_profile_init(RtpProfile *profile)
 	rtp_profile_set_payload(profile,7,&payload_type_lpc);
 	rtp_profile_set_payload(profile,4,&payload_type_g7231);
 	rtp_profile_set_payload(profile,8,&payload_type_pcma8000);
+	rtp_profile_set_payload(profile,9,&payload_type_g722);
 	rtp_profile_set_payload(profile,10,&payload_type_l16_stereo);
 	rtp_profile_set_payload(profile,11,&payload_type_l16_mono);
 	rtp_profile_set_payload(profile,18,&payload_type_g729);
@@ -489,17 +534,37 @@ PayloadType payload_type_jpeg={
 	CHANNELS(0)
 };
 
+PayloadType payload_type_vp8={
+	TYPE( PAYLOAD_VIDEO),
+	CLOCK_RATE(90000),
+	BITS_PER_SAMPLE(0),
+	ZERO_PATTERN(NULL),
+	PATTERN_LENGTH(0),
+	NORMAL_BITRATE(256000),
+	MIME_TYPE ("VP8-DRAFT-0-3-2"),
+	CHANNELS(0)
+};
 
 PayloadType	payload_type_t140={
-	PAYLOAD_AUDIO_PACKETIZED, /*type */
-	1000,
-	0,
-	NULL,
-	0,
-	0,
-	"t140",
-	0,
-	0
+	TYPE( PAYLOAD_TEXT),
+	CLOCK_RATE(1000),
+	BITS_PER_SAMPLE(0),
+	ZERO_PATTERN(NULL),
+	PATTERN_LENGTH(0),
+	NORMAL_BITRATE(0),
+	MIME_TYPE ("t140"),
+	CHANNELS(0)
+};
+
+PayloadType payload_type_t140_red={
+	TYPE(PAYLOAD_TEXT),
+	CLOCK_RATE(1000),
+	BITS_PER_SAMPLE(0),
+	ZERO_PATTERN(NULL),
+	PATTERN_LENGTH(0),
+	NORMAL_BITRATE(0),
+	MIME_TYPE("red"),
+	CHANNELS(0)
 };
 
 PayloadType	payload_type_x_udpftp={
@@ -514,3 +579,58 @@ PayloadType	payload_type_x_udpftp={
 	0
 };
 
+PayloadType payload_type_g722={
+       TYPE( PAYLOAD_AUDIO_PACKETIZED),
+       CLOCK_RATE(8000),
+       BITS_PER_SAMPLE( 0),
+       ZERO_PATTERN(NULL),
+       PATTERN_LENGTH( 0),
+       NORMAL_BITRATE( 64000),
+       MIME_TYPE ("G722"),
+       CHANNELS(1)
+};
+
+PayloadType payload_type_silk_nb={
+	TYPE( PAYLOAD_AUDIO_PACKETIZED),
+	CLOCK_RATE(8000),
+	BITS_PER_SAMPLE( 0),
+	ZERO_PATTERN(NULL),
+	PATTERN_LENGTH( 0),
+	NORMAL_BITRATE(10000),
+	MIME_TYPE ("SILK"),
+	CHANNELS(1)
+};
+
+PayloadType payload_type_silk_mb={
+	TYPE( PAYLOAD_AUDIO_PACKETIZED),
+	CLOCK_RATE(12000),
+	BITS_PER_SAMPLE( 0),
+	ZERO_PATTERN(NULL),
+	PATTERN_LENGTH( 0),
+	NORMAL_BITRATE(15000),
+	MIME_TYPE ("SILK"),
+	CHANNELS(1)
+};
+
+PayloadType payload_type_silk_wb={
+	TYPE( PAYLOAD_AUDIO_PACKETIZED),
+	CLOCK_RATE(16000),
+	BITS_PER_SAMPLE( 0),
+	ZERO_PATTERN(NULL),
+	PATTERN_LENGTH( 0),
+	NORMAL_BITRATE(20000),
+	MIME_TYPE ("SILK"),
+	CHANNELS(1)
+};
+
+PayloadType payload_type_silk_swb={
+	TYPE( PAYLOAD_AUDIO_PACKETIZED),
+	CLOCK_RATE(24000),
+	BITS_PER_SAMPLE( 0),
+	ZERO_PATTERN(NULL),
+	PATTERN_LENGTH( 0),
+	NORMAL_BITRATE(30000),
+	MIME_TYPE ("SILK"),
+	CHANNELS(1)
+};
+
