@@ -47,8 +47,8 @@ extern int rtcp_rr_init(RtpSession *session, char *buf, int size);
 /* this function initialize all session parameter's that depend on the payload type */
 static void payload_type_changed(RtpSession *session, PayloadType *pt){
 	jitter_control_set_payload(&session->rtp.jittctl,pt);
-	session->rtp.rtcp_report_snt_interval=RTCP_DEFAULT_REPORT_INTERVAL*pt->clock_rate;
 	rtp_session_set_time_jump_limit(session,session->rtp.time_jump);
+	rtp_session_set_rtcp_report_interval(session,session->rtcp.interval);
 	if (pt->type==PAYLOAD_VIDEO){
 		session->permissive=TRUE;
 		ortp_message("Using permissive algorithm");
@@ -284,6 +284,7 @@ rtp_session_init (RtpSession * session, int mode)
 	rtp_session_set_jitter_buffer_params(session,&jbp);
 	rtp_session_set_time_jump_limit(session,5000);
 	rtp_session_enable_rtcp(session,TRUE);
+	rtp_session_set_rtcp_report_interval(session,RTCP_DEFAULT_REPORT_INTERVAL);
 	session->recv_buf_size = UDP_MAX_SIZE;
 	session->symmetric_rtp = FALSE;
 	session->permissive=FALSE;
@@ -401,6 +402,28 @@ void rtp_session_enable_rtcp(RtpSession *session, bool_t yesno){
 	session->rtcp.enabled=yesno;
 }
 
+/**
+ * Sets the default interval in milliseconds for RTCP reports emitted by the session
+ *
+**/
+void rtp_session_set_rtcp_report_interval(RtpSession *session, int value_ms){
+	int recvpt=rtp_session_get_recv_payload_type(session);
+	int sendpt=rtp_session_get_send_payload_type(session);
+	if (recvpt!=-1){
+		PayloadType *pt=rtp_profile_get_payload(session->rcv.profile,recvpt);
+		if (pt!=NULL){
+			session->rtcp.rtcp_report_snt_interval_r=(value_ms*pt->clock_rate)/1000;
+		}
+	}
+	if (sendpt!=-1){
+		PayloadType *pt=rtp_profile_get_payload(session->snd.profile,sendpt);
+		if (pt!=NULL){
+			session->rtcp.rtcp_report_snt_interval_s=(value_ms*pt->clock_rate)/1000;
+		}
+	}
+	session->rtcp.interval=value_ms;
+}
+
 /**
  *	Set the RTP profile to be used for the sending by this session. By default, all session are created by
  *	rtp_session_new() are initialized with the AV profile, as defined in RFC 3551. The application
@@ -598,6 +621,13 @@ rtp_session_set_ssrc (RtpSession * session, uint32_t ssrc)
 }
 
 
+uint32_t
+rtp_session_get_send_ssrc (RtpSession* session)
+{
+	return session->snd.ssrc;
+}
+
+
 void rtp_session_update_payload_type(RtpSession *session, int paytype){
 	/* check if we support this payload type */
 	PayloadType *pt=rtp_profile_get_payload(session->rcv.profile,paytype);
@@ -933,7 +963,30 @@ static void payload_type_changed_notify(RtpSession *session, int paytype){
 		rtp_signal_table_emit (&session->on_payload_type_changed);
 	}
 }
+/**
+ *	Try to get an rtp packet presented as a mblk_t structure from the rtp session at a given sequence number.
+ *	This function is very usefull for codec with Forward error correction capabilities
+ *
+ *	This function returns the entire packet (with header).
+ *
+ *	 *
+ * @param session a rtp session.
+ * @param sequence_number a sequence number.
+ *
+ * @return a rtp packet presented as a mblk_t, or NULL if not found.
+ **/
 
+mblk_t *
+rtp_session_pick_with_cseq (RtpSession * session, const uint16_t sequence_number) {
+	queue_t* q= &session->rtp.rq;
+	mblk_t* mb;
+	for (mb=qbegin(q); !qend(q,mb); mb=qnext(q,mb)){
+		if (rtp_get_seqnumber(mb)==sequence_number) {
+			return mb;
+		}
+	}
+	return NULL;
+}
 
 /**
  *	Try to get a rtp packet presented as a mblk_t structure from the rtp session.
@@ -1088,8 +1141,6 @@ rtp_session_recvm_with_ts (RtpSession * session, uint32_t user_ts)
 	else
 	{
 		ortp_debug ("No mp for timestamp queried");
-		stream->stats.unavaillable++;
-		ortp_global_stats.unavaillable++;
 	}
 	rtp_session_rtcp_process_recv(session);
 	
@@ -1271,8 +1322,8 @@ void rtp_session_set_time_jump_limit(RtpSession *session, int milisecs){
  * Closes the rtp and rtcp sockets.
 **/
 void rtp_session_release_sockets(RtpSession *session){
-	if (session->rtp.socket>=0) close_socket (session->rtp.socket);
-	if (session->rtcp.socket>=0) close_socket (session->rtcp.socket);
+	if (session->rtp.socket!=(ortp_socket_t)-1) close_socket (session->rtp.socket);
+	if (session->rtcp.socket!=(ortp_socket_t)-1) close_socket (session->rtcp.socket);
 	session->rtp.socket=-1;
 	session->rtcp.socket=-1;
 	
@@ -1340,6 +1391,9 @@ void rtp_session_uninit (RtpSession * session)
 	session->signal_tables = o_list_free(session->signal_tables);
 	msgb_allocator_uninit(&session->allocator);
 
+	if (session->net_sim_ctx)
+		ortp_network_simulator_destroy(session->net_sim_ctx);
+
 #if (_WIN32_WINNT >= 0x0600)
 	if (session->rtp.QoSFlowID != 0)
     {
@@ -1439,6 +1493,63 @@ const rtp_stats_t * rtp_session_get_stats(const RtpSession *session){
 	return &session->rtp.stats;
 }
 
+/**
+ * Retrieves the session's jitter specific statistics.
+**/
+const jitter_stats_t * rtp_session_get_jitter_stats( const RtpSession *session ) {
+	return &session->rtp.jitter_stats;
+}
+
+/**
+ * @brief For <b>test purpose only</b>, sets a constant lost packet value within <b>all</b> RTCP output packets.@n
+ *
+ * The SR or RR RTCP packet contain a lost packet field. After this procedure is called, the lost packet field will be set to a constant value in all output SR or RR packets. This parameter will overridden the actual number of lost packets in the input RTP stream that the RTCP stack had previously processed.
+ * @param s : the rtp session.
+ * @param value : the lost packets test vector value.
+**/
+void rtp_session_rtcp_set_lost_packet_value( struct _RtpSession *s, const unsigned int value ) {
+	s->lost_packets_test_vector = value;
+	s->flags|=RTCP_OVERRIDE_LOST_PACKETS;
+}
+
+/**
+ * @brief For <b>test purpose only</b>, sets a constant interarrival_jitter value within <b>all</b> RTCP output packets.@n
+ *
+ * The SR or RR RTCP packet contain an interarrival jitter field. After this procedure is called, the interarrival jitter field will be set to a constant value in all output SR or RR packets. This parameter will overridden the actual interarrival jitter value that was processed by the RTCP stack.
+ * @param s : the rtp session.
+ * @param value : the interarrival jitter test vector value.
+**/
+void rtp_session_rtcp_set_jitter_value( struct _RtpSession *s, const unsigned int value ) {
+	s->interarrival_jitter_test_vector = value;
+	s->flags|=RTCP_OVERRIDE_JITTER;
+}
+
+/**
+ * @brief For <b>test purpose only</b>, simulates a constant RTT (Round Trip Time) value by setting the LSR field within <b>all</b> returned RTCP output packets.@n
+ *
+ * The RTT processing involves two RTCP packets exchanged between two different devices.@n
+ * In a <b>normal</b> operation the device 1 issues a SR packets at time T0, hence this packet has a timestamp field set to T0.
+ * The LSR and DLSR fiels of that packet are not considered here. This packet is received by the Device 2 at T1. 
+ * In response, the Device 2 issues another SR or RR packets at T2 with the following fields;
+ * - a timestamp set to T2.
+ * - a LSR (Last SR packet timestamp) field set to T0 ( this value has been extracted from the first packet).
+ * - a DLSR (Delay since Last SR packet) field set to (T2 - T1).
+ * .
+ * This packet is received by The Device 1 at T3. So the Device 1 is now able to process the RTT using the formula :
+ * RTT = T3 - LSR - DLSR = (T1 - T0) - (T3 - T2).@n
+ * This way of processing is described in par. 6.4 of the RFC3550 standard.
+ *
+ * In the <b>test</b> mode that is enabled by this procedure, the RTCP stack is considered as beeing part of the device 2. For setting the RTT to a constant RTT0 value, the Device 2 artificially sets the LSR field of the second packet to (T1 - RTT0), instead of T0 in normal mode. The two other fields (timestamp and DLSR) are set as in the normal mode. So the Device 1 will process :
+ * RTT = T3 - LSR - DLSR = RTT0 + (T3 - T2) that is near to RTT0 is T3 - T2 is small enough.
+ * @note It is impossible to actually make the mesured RTT strictly equal to RTT0, as the packet trip time (T3 - T2) is unknown when this packet is issued by the Device 2.
+ * @param s : the rtp session.
+ * @param value : The desired RTT test vector value (RTT0).
+**/
+void rtp_session_rtcp_set_delay_value( struct _RtpSession *s, const unsigned int value ) {
+	s->delay_test_vector= value;
+	s->flags|=RTCP_OVERRIDE_DELAY;
+}
+
 void rtp_session_reset_stats(RtpSession *session){
 	memset(&session->rtp.stats,0,sizeof(rtp_stats_t));
 }
@@ -1536,6 +1647,21 @@ void rtp_session_clear_recv_error_code(RtpSession *session){
 	session->rtp.send_errno=0;
 }
 
+/**
+ * Returns the last known round trip propagation delay.
+ *
+ * This value is known after successful RTCP SR or RR exchanged between a sender and a receiver.
+ * oRTP automatically takes care of sending SR or RR packets.
+ * You might want to call this function when you receive an RTCP event (see rtp_session_register_event_queue() ).
+ * This value might not be known: at the beginning when no RTCP packets have been exchanged yet, or simply because the
+ * rtcp channel is broken due to firewall problematics, or because the remote implementation does not support RTCP.
+ *
+ * @returns the round trip propagation time in seconds if known, -1 if unknown. 
+**/
+float rtp_session_get_round_trip_propagation(RtpSession *session){
+	return session->rtt;
+}
+
 /**
  * Destroys a rtp session.
  * All memory allocated for the RtpSession is freed.
@@ -1584,10 +1710,47 @@ int rtp_get_payload(mblk_t *packet, unsigned char **start){
 		ortp_warning("Invalid RTP packet");
 		return -1;
 	}
+	if (rtp_get_extbit(packet)){
+		int extsize=rtp_get_extheader(packet,NULL,NULL);
+		if (extsize>=0){
+			tmp+=4+extsize;
+		}
+	}
 	*start=tmp;
 	return packet->b_wptr-tmp;
 }
 
+/**
+ * Obtain the extension header if any.
+ * @param packet the RTP packet.
+ * @param profile the profile field of the extension header
+ * @param start_ext_header pointer that will be set to the beginning of the payload of the extension header.
+ * @return the size of the extension in bytes (the payload size, it can be 0), -1 if parsing of the extension header failed or if no extension is present.
+**/
+int rtp_get_extheader(mblk_t *packet, uint16_t *profile, uint8_t **start_ext){
+	int size=0;
+	uint8_t *ext_header;
+	if (rtp_get_extbit(packet)){
+		ext_header=packet->b_rptr+RTP_FIXED_HEADER_SIZE+(rtp_get_cc(packet)*4);
+		if (ext_header+4 <= packet->b_wptr){
+			uint32_t h=ntohl(*(uint32_t*)ext_header);
+			size=(int)(h & 0xFFFF);
+			if (profile) *profile=(h>>16);
+			size=(size*4); /*the size is given in the packet as multiple of 32 bit words, excluding the 4 byte header*/
+			if ((ext_header+4+size)> packet->b_wptr){
+				ortp_warning("Inconsistent size for rtp extension header");
+				return -1;
+			}
+			if (start_ext) *start_ext=ext_header+4;
+			return size;
+		}else{
+			ortp_warning("Insufficient size for rtp extension header.");
+			return -1;
+		}
+	}
+	return -1;
+}
+
 
 /**
  *  Gets last time a valid RTP or RTCP packet was received.
