@@ -252,7 +252,7 @@ rtp_session_set_local_addr (RtpSession * session, const char * addr, int port)
 	ortp_socket_t sock;
 	int sockfamily;
 	bool_t reuse_addr;
-	if (session->rtp.socket>=0){
+	if (session->rtp.socket!=(ortp_socket_t)-1){
 		/* don't rebind, but close before*/
 		rtp_session_release_sockets(session);
 	}
@@ -273,7 +273,7 @@ rtp_session_set_local_addr (RtpSession * session, const char * addr, int port)
 		session->rtp.loc_port=port;
 		/*try to bind rtcp port */
 		sock=create_and_bind(addr,port+1,&sockfamily,reuse_addr);
-		if (sock!=-1){
+		if (sock!=(ortp_socket_t)-1){
 			session->rtcp.sockfamily=sockfamily;
 			session->rtcp.socket=sock;
 		}else{
@@ -287,6 +287,7 @@ rtp_session_set_local_addr (RtpSession * session, const char * addr, int port)
 
 		return 0;
 	}
+	ortp_error("Could not bind RTP socket on port to %s port %i",addr,port);
 	return -1;
 }
 
@@ -309,7 +310,7 @@ int rtp_session_set_multicast_ttl(RtpSession *session, int ttl)
     if (ttl>0) session->multicast_ttl = ttl;
     
     // Don't do anything if socket hasn't been created yet
-    if (session->rtp.socket < 0) return 0;
+    if (session->rtp.socket == (ortp_socket_t)-1) return 0;
 
     switch (session->rtp.sockfamily) {
         case AF_INET: {
@@ -385,7 +386,7 @@ int rtp_session_set_multicast_loopback(RtpSession *session, int yesno)
     }
      
     // Don't do anything if socket hasn't been created yet
-    if (session->rtp.socket < 0) return 0;
+    if (session->rtp.socket == (ortp_socket_t)-1) return 0;
 
     switch (session->rtp.sockfamily) {
         case AF_INET: {
@@ -457,7 +458,7 @@ int rtp_session_set_dscp(RtpSession *session, int dscp){
 	if (dscp>=0) session->dscp = dscp;
 	
 	// Don't do anything if socket hasn't been created yet
-	if (session->rtp.socket < 0) return 0;
+	if (session->rtp.socket == (ortp_socket_t)-1) return 0;
 
 #if (_WIN32_WINNT >= 0x0600)
 	memset(&ovi, 0, sizeof(ovi));
@@ -745,11 +746,11 @@ rtp_session_set_remote_addr_and_port(RtpSession * session, const char * addr, in
 
 void rtp_session_set_sockets(RtpSession *session, int rtpfd, int rtcpfd)
 {
-	if (rtpfd>=0) set_non_blocking_socket(rtpfd);
-	if (rtcpfd>=0) set_non_blocking_socket(rtcpfd);
+	if (rtpfd!=-1) set_non_blocking_socket(rtpfd);
+	if (rtcpfd!=-1) set_non_blocking_socket(rtcpfd);
 	session->rtp.socket=rtpfd;
 	session->rtcp.socket=rtcpfd;
-	if (rtpfd>=0 || rtcpfd>=0 )
+	if (rtpfd!=-1 || rtcpfd!=-1 )
 		session->flags|=(RTP_SESSION_USING_EXT_SOCKETS|RTP_SOCKET_CONNECTED|RTCP_SOCKET_CONNECTED);
 	else session->flags&=~(RTP_SESSION_USING_EXT_SOCKETS|RTP_SOCKET_CONNECTED|RTCP_SOCKET_CONNECTED);
 }
@@ -800,10 +801,10 @@ void rtp_session_flush_sockets(RtpSession *session){
 	    return;
 	  }
 
-	if (session->rtp.socket>=0){
+	if (session->rtp.socket!=(ortp_socket_t)-1){
 		while (recvfrom(session->rtp.socket,(char*)trash,sizeof(trash),0,(struct sockaddr *)&from,&fromlen)>0){};
 	}
-	if (session->rtcp.socket>=0){
+	if (session->rtcp.socket!=(ortp_socket_t)-1){
 		while (recvfrom(session->rtcp.socket,(char*)trash,sizeof(trash),0,(struct sockaddr*)&from,&fromlen)>0){};
 	}
 }
@@ -836,19 +837,30 @@ static int rtp_sendmsg(int sock,mblk_t *m, struct sockaddr *rem_addr, int addr_l
 #endif	
 
 #define IP_UDP_OVERHEAD (20+8)
+#define IP6_UDP_OVERHEAD (40+8)
 
 static void update_sent_bytes(RtpSession*s, int nbytes){
+#ifdef ORTP_INET6
+	int overhead=(s->rtp.sockfamily==AF_INET6) ? IP6_UDP_OVERHEAD : IP_UDP_OVERHEAD;
+#else
+	int overhead=IP_UDP_OVERHEAD;
+#endif
 	if (s->rtp.sent_bytes==0){
 		gettimeofday(&s->rtp.send_bw_start,NULL);
 	}
-	s->rtp.sent_bytes+=nbytes+IP_UDP_OVERHEAD;
+	s->rtp.sent_bytes+=nbytes+overhead;
 }
 
 static void update_recv_bytes(RtpSession*s, int nbytes){
+#ifdef ORTP_INET6
+	int overhead=(s->rtp.sockfamily==AF_INET6) ? IP6_UDP_OVERHEAD : IP_UDP_OVERHEAD;
+#else
+	int overhead=IP_UDP_OVERHEAD;
+#endif
 	if (s->rtp.recv_bytes==0){
 		gettimeofday(&s->rtp.recv_bw_start,NULL);
 	}
-	s->rtp.recv_bytes+=nbytes+IP_UDP_OVERHEAD;
+	s->rtp.recv_bytes+=nbytes+overhead;
 }
 
 int
@@ -913,7 +925,7 @@ rtp_session_rtcp_send (RtpSession * session, mblk_t * m)
 	}
 
 	if (session->rtcp.enabled &&
-		( (sockfd>=0 && (session->rtcp.rem_addrlen>0 ||using_connected_socket))
+		( (sockfd!=(ortp_socket_t)-1 && (session->rtcp.rem_addrlen>0 ||using_connected_socket))
 			|| rtp_session_using_transport(session, rtcp) ) ){
 		if (rtp_session_using_transport(session, rtcp)){
 			error = (session->rtcp.tr->t_sendto) (session->rtcp.tr, m, 0,
@@ -955,7 +967,7 @@ rtp_session_rtp_recv (RtpSession * session, uint32_t user_ts)
 	socklen_t addrlen = sizeof (remaddr);
 	mblk_t *mp;
 	
-	if ((sockfd<0) && !rtp_session_using_transport(session, rtp)) return -1;  /*session has no sockets for the moment*/
+	if ((sockfd==(ortp_socket_t)-1) && !rtp_session_using_transport(session, rtp)) return -1;  /*session has no sockets for the moment*/
 
 	while (1)
 	{
@@ -986,30 +998,39 @@ rtp_session_rtp_recv (RtpSession * session, uint32_t user_ts)
 						session->flags|=RTP_SOCKET_CONNECTED;
 				}
 			}
-			/* then parse the message and put on queue */
 			mp->b_wptr+=error;
-			rtp_session_rtp_parse (session, mp, user_ts, (struct sockaddr*)&remaddr,addrlen);
+			if (session->net_sim_ctx)
+				mp=rtp_session_network_simulate(session,mp);
+			/* then parse the message and put on jitter buffer queue */
+			if (mp){
+				rtp_session_rtp_parse(session, mp, user_ts, (struct sockaddr*)&remaddr,addrlen);
+				update_recv_bytes(session,mp->b_wptr-mp->b_rptr);
+			}
 			session->rtp.cached_mp=NULL;
 			/*for bandwidth measurements:*/
-			update_recv_bytes(session,error);
 		}
 		else
 		{
-		 	int errnum=getSocketErrorCode();
-			if (error == 0){
-				/*0 can be returned by RtpTransport functions in case of EWOULDBLOCK*/
-				/*we ignore it*/
-				/*ortp_warning
-					("rtp_recv: strange... recv() returned zero.");*/
-			}
-			else if (!is_would_block_error(errnum))
+			int errnum;
+			if (error==-1 && !is_would_block_error((errnum=getSocketErrorCode())) )
 			{
 				if (session->on_network_error.count>0){
 					rtp_signal_table_emit3(&session->on_network_error,(long)"Error receiving RTP packet",INT_TO_POINTER(getSocketErrorCode()));
-				}else ortp_warning("Error receiving RTP packet: %s.",getSocketError());
+				}else ortp_warning("Error receiving RTP packet: %s, err num  [%i],error [%i]",getSocketError(),errnum,error);
+			}else{
+				/*EWOULDBLOCK errors or transports returning 0 are ignored.*/
+				if (session->net_sim_ctx){
+					/*drain possible packets queued in the network simulator*/
+					mp=rtp_session_network_simulate(session,NULL);
+					if (mp){
+						/* then parse the message and put on jitter buffer queue */
+						rtp_session_rtp_parse(session, mp, user_ts, (struct sockaddr*)&session->rtp.rem_addr,session->rtp.rem_addrlen);
+						update_recv_bytes(session,msgdsize(mp));
+					}
+				}
 			}
 			/* don't free the cached_mp, it will be reused next time */
-			return -1;	/* avoids an infinite loop ! */
+			return -1;
 		}
 	}
 	return error;
@@ -1025,6 +1046,84 @@ void rtp_session_notify_inc_rtcp(RtpSession *session, mblk_t *m){
 	else freemsg(m);  /* avoid memory leak */
 }
 
+static void compute_rtt(RtpSession *session, const struct timeval *now, const report_block_t *rb){
+	uint64_t curntp=ortp_timeval_to_ntp(now);
+	uint32_t approx_ntp=(curntp>>16) & 0xFFFFFFFF;
+	uint32_t last_sr_time=report_block_get_last_SR_time(rb);
+	uint32_t sr_delay=report_block_get_last_SR_delay(rb);
+	/*ortp_message("rtt curntp=%u, last_sr_time=%u, sr_delay=%u",approx_ntp,last_sr_time,sr_delay);*/
+	if (last_sr_time!=0 && sr_delay!=0){
+		double rtt_frac=approx_ntp-last_sr_time-sr_delay;
+		rtt_frac/=65536.0;
+		/*take into account the network simulator */
+		if (session->net_sim_ctx && session->net_sim_ctx->params.max_bandwidth>0){
+			double sim_delay=(double)session->net_sim_ctx->qsize/(double)session->net_sim_ctx->params.max_bandwidth;
+			rtt_frac+=sim_delay;
+		}
+		session->rtt=rtt_frac;
+		/*ortp_message("rtt estimated to %f ms",session->rtt);*/
+	}
+}
+
+/*
+ * @brief : for SR packets, retrieves their timestamp, gets the date, and stores these information into the session descriptor. The date values may be used for setting some fields of the report block of the next RTCP packet to be sent.
+ * @param session : the current session descriptor.
+ * @param block : the block descriptor that may contain a SR RTCP message.
+ * @note a basic parsing is done on the block structure. However, if it fails, no error is returned, and the session descriptor is left as is, so it does not induce any change in the caller procedure behaviour.
+ */
+static void process_rtcp_packet( RtpSession *session, mblk_t *block ) {
+	rtcp_common_header_t *rtcp;
+	RtpStream * rtpstream = &session->rtp;
+
+	int msgsize = (int) ( block->b_wptr - block->b_rptr );
+	if ( msgsize < RTCP_COMMON_HEADER_SIZE ) {
+		ortp_debug( "Receiving a too short RTCP packet" );
+		return;
+	}
+
+	rtcp = (rtcp_common_header_t *)block->b_rptr;
+	/* compound rtcp packet can be composed by more than one rtcp message */
+	do{
+		struct timeval reception_date;
+
+		/* Getting the reception date from the main clock */	
+		gettimeofday( &reception_date, NULL );
+
+		if (rtcp_is_SR(block) ) {
+			rtcp_sr_t *sr = (rtcp_sr_t *) rtcp;
+			
+			/* The session descriptor values are reset in case there is an error in the SR block parsing */
+			rtpstream->last_rcv_SR_ts = 0;
+			rtpstream->last_rcv_SR_time.tv_usec = 0;
+			rtpstream->last_rcv_SR_time.tv_sec = 0;
+
+			
+			if ( ntohl( sr->ssrc ) != session->rcv.ssrc ) {
+				ortp_debug( "Receiving a RTCP SR packet from an unknown ssrc" );
+				return;
+			}
+
+			if ( msgsize < RTCP_COMMON_HEADER_SIZE + RTCP_SSRC_FIELD_SIZE + RTCP_SENDER_INFO_SIZE + ( RTCP_REPORT_BLOCK_SIZE * sr->ch.rc ) ) {
+				ortp_debug( "Receiving a too short RTCP SR packet" );
+				return;
+			}
+
+			/* Saving the data to fill LSR and DLSR field in next RTCP report to be transmitted */
+			/* This value will be the LSR field of the next RTCP report (only the central 32 bits are kept, as described in par.4 of RC3550) */
+			rtpstream->last_rcv_SR_ts = ( ntohl( sr->si.ntp_timestamp_msw ) << 16 ) | ( ntohl( sr->si.ntp_timestamp_lsw ) >> 16 );
+			/* This value will help in processing the DLSR of the next RTCP report ( see report_block_init() in rtcp.cc ) */
+			rtpstream->last_rcv_SR_time.tv_usec = reception_date.tv_usec;
+			rtpstream->last_rcv_SR_time.tv_sec = reception_date.tv_sec;
+			compute_rtt(session,&reception_date,&sr->rb[0]);
+		}else if ( rtcp_is_RR(block)){
+			const report_block_t *rb=rtcp_RR_get_report_block(block,0);
+			if (rb) compute_rtt(session,&reception_date,rb);
+		}
+	}while (rtcp_next_packet(block));
+	rtcp_rewind(block);
+}
+
+
 int
 rtp_session_rtcp_recv (RtpSession * session)
 {
@@ -1037,7 +1136,7 @@ rtp_session_rtcp_recv (RtpSession * session)
 	socklen_t addrlen=0;
 	mblk_t *mp;
 
-	if (session->rtcp.socket<0 && !rtp_session_using_transport(session, rtcp)) return -1;  /*session has no rtcp sockets for the moment*/
+	if (session->rtcp.socket==(ortp_socket_t)-1 && !rtp_session_using_transport(session, rtcp)) return -1;  /*session has no rtcp sockets for the moment*/
 	
 
 	while (1)
@@ -1065,6 +1164,7 @@ rtp_session_rtcp_recv (RtpSession * session)
 		if (error > 0)
 		{
 			mp->b_wptr += error;
+			process_rtcp_packet( session, mp );
 			/* post an event to notify the application*/
 			{
 				rtp_session_notify_inc_rtcp(session,mp);
@@ -1084,10 +1184,11 @@ rtp_session_rtcp_recv (RtpSession * session)
 		{
 			int errnum=getSocketErrorCode();
 
-			if (error == 0)
+			if (error == 0 || (error=-1 && errnum==0))
 			{
-				ortp_warning
-					("rtcp_recv: strange... recv() returned zero.");
+				/*ortp_warning
+					("rtcp_recv: strange... recv() returned zero.");*/
+				/*(error == -1 && errnum==0) for buggy drivers*/
 			}
 			else if (!is_would_block_error(errnum))
 			{
