@@ -43,6 +43,7 @@ typedef struct _OList OList;
 OList * o_list_append(OList *elem, void * data);
 OList * o_list_remove(OList *list, void *data);
 OList * o_list_free(OList *elem);
+OList *o_list_remove_link(OList *list, OList *elem);
 
 #ifndef MIN
 #define MIN(a,b) (((a)>(b)) ? (b) : (a))
@@ -80,7 +81,7 @@ typedef union{
 #endif
 #define ntoh24(x) hton24(x)
 
-#ifdef _WIN32
+#if defined(WIN32) || defined(_WIN32_WCE)
 #define is_would_block_error(errnum)	(errnum==WSAEWOULDBLOCK)
 #else
 #define is_would_block_error(errnum)	(errnum==EWOULDBLOCK || errnum==EAGAIN)
@@ -88,4 +89,6 @@ typedef union{
 
 void ortp_ev_queue_put(OrtpEvQueue *q, OrtpEvent *ev);
 
+uint64_t ortp_timeval_to_ntp(const struct timeval *tv);
+
 #endif
